// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/tools/closure_container.c
// Description: Der Closure-Container
// Author:	Freaky (23.12.93)
// Modified by:	Freaky (22.05.2000) do_call eingebaut

inherit "/i/tools/description";

#include <deklin.h>
#include <description.h>
#include <v_item.h>

nosave mapping current_debuginfo;
nosave int current_debuginfo_time;
nosave object current_debuginfo_ob;

private void set_current_debuginfos(mapping m)
{
    current_debuginfo = m;
    if(m)
    {
        current_debuginfo_time = time();
        current_debuginfo_ob = caller_stack(1)[<1];
    }
}

mapping query_debug_info()
{
    if(current_debuginfo && current_debuginfo_time == time() && current_debuginfo_ob == caller_stack(1)[<1])
        return current_debuginfo;
}

nomask void create()
{
}

nomask closure do_bind(closure c)
{
    current_debuginfo = 0;
    return bind_lambda(c);
}

nomask mixed do_apply(closure c, varargs mixed *parameter)
{
    current_debuginfo = 0;
    return apply(bind_lambda(c),parameter);
}

nomask mixed do_call(mixed ob, string funktion, varargs mixed *parameter)
{
    current_debuginfo = 0;
    return apply(#'call_other,ob,funktion,parameter);
}

nomask mixed do_efun_call(string funktion, varargs mixed *parameter)
{
    current_debuginfo = 0;
    return apply(symbol_function(funktion),parameter);
}

#ifdef __LPC_INLINE_CLOSURES__

nomask closure do_bind_with_args(closure c, varargs mixed* args)
{
    closure cl = bind_lambda(c);
    current_debuginfo = 0;
    return function mixed (varargs mixed* vargs)
               { return funcall(cl, args..., vargs...); };
}

nomask closure do_bind_varargs(closure c)
{
    closure cl = bind_lambda(c);
    current_debuginfo = 0;
    return function mixed (varargs mixed* vargs)
               { return funcall(cl, vargs); };
}

// Fuer Controller...
nomask closure do_bind_with_args_ignore_n(closure c, int num, varargs mixed* args)
{
    closure cl = bind_lambda(c);
    current_debuginfo = 0;
    return function mixed (varargs mixed* vargs)
    {
        return funcall(cl, args..., vargs[num..]...);
    };
}

#endif

/* ============== STRING_PARSER FUNCTIONS ====================== */

#define PREVIOUS_OBJECT()	(strstr(object_name(previous_object()),"/secure/simul_efun/")?previous_object():previous_object(1))
#define DEKLIN_OBJ		(find_object("/secure/simul_efun/deklin")||find_object("/secure/simul_efun/backup/deklin"))

// Kleine Subroutine, um einen Wert zu bestimmen. Dies ist notwendig,
// um sich (vor allem bei Arrays/Mappings) rekursiv aufrufen zu koennen.
// Parameter:
//   str - Der zu parsende String
//   pos - An dieser Stelle muss geparst werden, wird per Referenz
//         uebergeben. -1 als Ergebnis bedeutet einen Fehler.
//   abbruch - Eine Regexp, die angibt, wo bei einem String (dem ja die
//             Begrenzungszeichen "" fehlen) abgebrochen werden soll
//   use_tp - Anstatt OBJ_TP wird 'tp genutzt
//   deep - 1, wenn dieser Wert sich innerhalb eines Arrays oder Mappings
//          befindet. (Dann gehen 'symbol und #'funktion nicht mehr.
//          Arrays werden dann auch nicht mehr gequotet.)
private mixed value_parser(string str, int pos, string abbruch, mapping variables, int deep)
{
   if(str[pos..pos+1]=="({")  // Ein Array
   {
      mixed tmp=({});
      pos+=2;
      while(str[pos..pos+1]!="})")
      {                                              // Naechste Element holen
         tmp+=({value_parser(str, &pos, "}\\)|,", variables, 1)});
         if(pos<0) return 0;                         // Bei Fehler -> Ende
         if(str[pos..pos]==",")                      // Komma uebergehen
            pos++;
         else if(str[pos..pos+1]!="})")              // Kein Komma und kein })
         {                                           // -> Feher
            pos=-1;
            return 0;
         }
      }
      pos+=2;                                        // Fuer "})"
      return deep?tmp:quote(tmp);                    // Quoten fuer die Lambda
   }
   else if(str[pos..pos+1]=="([")                    // Ein Mapping
   {
      mapping tmp=([]);
      pos+=2;
      while(str[pos..pos+1]!="])")
      {                                              // Erster Wert, der Schluessel
         string key = value_parser(str, &pos, ":|,|\\]\\)", variables, 1);
         if(pos<0) return 0;
         if(str[pos..pos]==":")                      // Ein Paar Key:Value
         {
            pos++;
	    tmp+=([key:value_parser(str, &pos, ",|\\]\\)", variables, 1) ]);
	    if(pos<0) return 0;
         }
	 else if(str[pos..pos]==",")                 // Nur ein Schluessel
	    pos++;                                   // ohne Wert
         else if(str[pos..pos+1]!="])")              // -> Fehler
         {
            pos=-1;
            return 0;
         }
      }
      pos+=2;                                        // Fuer "])"
      return tmp;
   }
   else if(str[pos..pos]=="(")                       // normale Klammerung
   {
      mixed tmp;                                     // Einfach rekursiv
      pos++;                                         // aufrufen,
      tmp=value_parser(str, &pos, "\\)", variables, deep);
      if(str[pos..pos]!=")")                         // um danach die
      {                                              // Klammerung testen
         pos=-1;                                     // zu koennen.
	 return 0;
      }
      pos++;
      return tmp;
   }
   else if(str[pos..pos]=="\"")                      // Ein String in ""
   {
      int i;
      string tmp;
      i=strstr(str,"\"",pos+1);
      if(i<0)
      {
         pos=-1;
	 return 0;
      }
      tmp=str[pos+1..i-1];
      pos=i+1;
      return tmp;
   }
   else                                              // Zahlen oder Strings
   {
      mixed tmp=regexplode(str[pos..<1],abbruch)[0]; // Ende ermitteln
      int k;
      pos+=strlen(tmp);
      if(!deep && tmp[0..1]=="#'")                   // #'fun ?
         return ({#'call_other,PREVIOUS_OBJECT(),tmp[2..<1]});
      if(to_string(k=to_int(tmp))==tmp)              // Eine Zahl?
         return k;
      if(!deep && tmp[0..0]=="'")                    // 'symbol?
         return quote(tmp[1..<1]);
      if(member(variables, tmp))
          return variables[tmp];
      switch(tmp)
      {
         case "OBJ_TO":
           return PREVIOUS_OBJECT();
	 case "OBJ_PO":
	   return OBJ_PO;
	 case "OBJ_TP":
	   return OBJ_TP;
	 case "OBJ_OW":
	    return OBJ_OW;
	 case "OBJ_TI":
	    return OBJ_TI;
	 default:
	    return tmp;
      }
   }
   // Hierher kommt man nicht.
}

varargs mixed string_parser(string str, mixed default_value, mixed variables, mapping functions)
{
   string *parts, fun, element;
   int i,j;
   mixed head, op;

   if (!str)
      return 0;
   if (str=="")
      return "";

   if(!functions)
      functions = ([]);
   if(!variables)
      variables = ([]);
   else if(symbolp(variables))
      variables = ([ "OBJ_TP": variables ]);
   else if(!mappingp(variables))
      variables = ([ "OBJ_TP": 'tp ]);

   if(intp(default_value))
   {
      if(default_value)
         default_value = variables["OBJ_TP"] || OBJ_TP;
      else
         default_value = variables["OBJ_TO"] || PREVIOUS_OBJECT();;
   }

   // Wir teilen nach $Funktion.
   parts = regexplode(str,"\\$[^\\(]*\\(");
   // Also haben wir abwechselnd "$fun(" und "par) Text"

   // Am Anfang steht reiner Text (oder "")
   if(parts[0]!="")
      head = parts[0];

   for (i = 1; i < sizeof(parts); i+=2)
   {
      fun = parts[i][1..<2]; // $ und ( abschneiden
      element = parts[i+1]; // spart staendige Array-Indizierungen
      
      if (fun == "dir" || fun == "Dir" || 
//          function_exists(fun, this_object())==__FILE__[0..<3] ||
	  function_exists(fun, DEKLIN_OBJ) ||
          member(functions, fun))
      {
         op = 0;
	 switch(fun)
	 {
	    case "dir" :
               if(!member(functions, "dir"))         // Das haette Vorrang
               {
	          j=strstr(element,")");             // schliessende Klammer
	          if(j<0) break;                     // suchen, ansonsten Fehler
	          j++;
	          op = 'richtung;
	          break;
               }
	    case "Dir" :
               if(!member(functions,fun))
               {
	          j=strstr(element,")");
	          if(j<0) break;
	          j++;
	          op = ({ #'capitalize, 'richtung });
	          break;
               }
	    default:
               j=0;
	       op = ({});
	       while(element[j..j]!=")")	     // Parameter parsen   
	       {
	          if(element[j..j]!=",")             // leer -> ignorieren
		     op+= ({ value_parser(element,&j,",|\\)", variables, 0) });
		  if(element[j..j]==",")
		     j++;
		  else if(element[j..j]!=")")
		     break;
	       }
	       if(j<0) break;
	       j++;
	       if(!sizeof(op))                       // Kein Parameter?
	          op = ({ default_value });
               if(member(functions, fun))
                  op = ({#'funcall, functions[fun] }) + op;
               else
	          op = ({ symbol_function(fun) }) + op;
         }
	 if(j>=0)
	 {
	    if(head)
	       head = ({#'+,head,op});
	    else
	       head = op;
	 }
	 else // Fehler, die Funktion als String aufnehmen
	 {
	    element=parts[i]+element;
	    j=0;
	 }
	 if(strlen(element[j..<1]))
	 {
	    if(head)
	       head = ({#'+,head,element[j..<1]});
	    else
	       head = element[j..<1];
	 }
      }
      else if((j=strstr(parts[i],"$",1))>0) // Funktion gibt es nicht.
      {                                     // Einfach nur ein einzelnes '$'?
          if(head)
              head = ({#'+, head, parts[i][0..j-1]});
          else
              head = parts[i][0..j-1];
          parts[i]=parts[i][j..<1];
          i-=2;
      }
      else if(head)                         // Dann Text einfach nur hinzufuegen.
          head = ({#'+,head,parts[i]+element});
      else
          head = parts[i]+element;
   }
   return head;
}

varargs closure mixed_to_closure(mixed mix, mixed *symbols, mixed default_value, mixed variables, mapping functions)
{
   int a;
   mixed ret;

   if(stringp(mix))
      ret = string_parser(mix, default_value, variables, functions);
   else if(pointerp(mix))
      for(a = sizeof(mix)-2,
          ret = (stringp(mix[<1])?string_parser(mix[<1], default_value, variables,
                  functions):mix[<1]);
          a >= 0;
          a--)
         ret = ({ #'+, stringp(mix[a])?string_parser(mix[a], default_value, variables,
             functions):mix[a], ret });
   else if(closurep(mix))
      return mix;
   else
      ret = "";

   if(pointerp(ret))
   {
       int simul = strstr(object_name(previous_object(0)), "/secure/simul_efun/") ? 0 : 1;
       mapping debuginfo = ([
            "TO":   object_name(previous_object(0 + simul)),
            "PO":   object_name(previous_object(1 + simul)),
            "Orig": stringp(mix) ? mix : sprintf("%Q\n", mix),
        ]);

       ret = ({ #',,
               ({ #'set_current_debuginfos, debuginfo }),
               ({ #'=, 'mixed_to_closure_result, ret }),
               ({ #'set_current_debuginfos, 0 }),
               'mixed_to_closure_result
             });
   }

   return unbound_lambda(symbols?symbols:({}), ret);
}

public string|closure compile_description(mixed desc, mapping tags)
{
    return compile_desc(desc, &tags);
}

protected int|closure compile_condition(mixed cond, mapping tags)
{
    return compile_cond(cond, &tags);
}

/* Fuer die Descriptions muessen wir die desc_* - Aufrufe an das jeweilige
 * Objekt weiterleiten.
 */
private object desc_object(mapping info)
{
    object|mapping item = info[TI_ITEM]||info[TI_OBJECT]||info[TI_ROOM];

    if (objectp(item))
        return item;
    if (mappingp(item))
        return item[VI_MASTER]||item[VI_OBJECT];
    return 0;
}

protected mixed desc_condition(string name, mixed info, mixed* par)
{
    mixed res = ::desc_condition(name, info, par);
    if (res)
        return res;
    else
    {
        object ob = desc_object(info);

        return ob && call_other(ob, "desc_condition_"+name, info, par...);
    }
}

protected mixed desc_filter(string name, mixed info, mixed orig, mixed* par)
{
    mixed res = ::desc_filter(name, info, orig, par);
    if (res)
        return res;
    else
    {
        object ob = desc_object(info);

        return ob && call_other(ob, "desc_filter_"+name, info, orig, par...);
    }
}

protected mixed desc_text(string name, mapping info, mixed* par)
{
    mixed res = ::desc_text(name, info, par);
    if (res)
        return res;
    else
    {
        object ob = desc_object(info);

        return ob ? call_other(ob, "desc_text_"+name, info, par...) : "";
    }
}

protected int desc_number(string name, mixed info, mixed* par)
{
    int res = ::desc_number(name, info, par);
    if (res)
        return res;
    else
    {
        object ob = desc_object(info);

        return ob && call_other(ob, "desc_number_"+name, info, par...);
    }
}
