// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/clothes/kleidung.c
// Description: Kleidung zum Anziehen
// Modified by: Kurdel (24.02.97) Kosmetik, mehrere Sachen an-/ausziehen
//		Freaky (22.09.97) notify in do_* rein, da sonst das nicht
//			aufgerufen wird, wenn man die Kleidung bewegt
//		Freaky (08.02.1999) notify aus remove() raus (ist in do_remove)
//			do_wear, do_remove: notify nur, wenn es noch nicht
//			aufgerufen war.
//		Freaky (22.02.1999) worn_adjektiv ist ein String (kein mixed)
//		Freaky (07.03.1999) worn_adjektiv ist jetzt richtig mixed
//		Freaky (04.06.1999) das forbidden wird jetzt nach dem Check,
//			ob man die Kleidung ueberhaupt anziehen kann,aufgerufen
//              Sissi  (05.06.2000) Kleidungs anzieh- und auszieh add actions
//                     einschliesslich catch command auskommentiert

#pragma save_types

virtual inherit "/i/item";
virtual inherit "/i/move";
virtual inherit "/i/value";
#if __VERSION__ > "3.6.3"
private functions inherit "/i/tools/kleidung";
#else
protected functions inherit "/i/tools/kleidung";
#endif

#include <control.h>
#include <deklin.h>
#include <description.h>
#include <config.h>
#include <object_stats.h>
#include <apps.h>
#include <parse_com.h>
#include <move.h>
#include <message.h>
#include <simul_efuns.h>
#include <error.h>


#define AUFSETZBAR member( \
    ({ "rucksack", "muetze", "brille" }), clothes_class ) != -1

#define FAIL(x)  return notify_fail(x)
#define ERR(x)   { write(x); return 1; }
#define RET(x)   { if (err) ERR(x) else FAIL(x); }
#define IST      (ist(this_object(),IST_SPACE_BEFORE|IST_SPACE_AFTER))
#define GRUND_WORN_ADJ (pointerp(worn_adjektiv) ? worn_adjektiv[0] : worn_adjektiv)
#define PREFIX(x) (prefix? \
                   wrap_say(sprintf("%-25s",Ein(this_object(),({}))+":"), \
                            x,0,26): \
                   wrap(x))

#define	KLEIDUNGSINFOS "/static/adm/KLEIDUNGEN"

#define CONSERVATION_HANDLE_XYZ(s,x) \
    if (stringp(x) || !x) \
        this_object()->add_setter_conservation((s),({x})); \
    else \
        this_object()->set_conservation_constraint((s),1);
#define CONSERVATION_HANDLE_MSG(s) \
    if (stringp(msg) || !msg) \
        this_object()->add_setter_conservation((s),({msg})); \
    else \
        this_object()->set_conservation_constraint((s),1);


private static int is_worn;
private int temp_schutz;
private string clothes_class;
private static closure wear_msg, wear_msg_other, remove_msg, remove_msg_other;
private mixed worn_adjektiv = "angezogen";
private int content_visible_when_worn;
private string long_owner_when_worn, long_other_when_worn;
closure look_msg_when_worn;

private string grundform(mixed adj)
{
    return pointerp(adj) ? adj[0] : adj;
}

/*
FUNKTION: set_worn_adjektiv
DEKLARATION: void set_worn_adjektiv(mixed worn_adjektiv)
BESCHREIBUNG:
Hiermit gibt man ein Adjektiv an, welches angezeigt wird,
wenn ein Kleidungsstueck angezogen wurde. Man kann nur
ein einziges Adjektiv angeben. Default ist "angezogen".
worn_adjektiv kann entweder ein String oder
ein Array ({Grundform, Stamm}) sein.
VERWEISE: query_worn_adjektiv
GRUPPEN: kleidung
*/
void set_worn_adjektiv(mixed adj)
{
    if (adj)
    {
        if (is_worn && worn_adjektiv)
            this_object()->delete_adjektiv(worn_adjektiv);
        worn_adjektiv = adj;
        this_object()->add_setter_conservation("set_worn_adjektiv",
            ({worn_adjektiv}));
        if (is_worn && !this_object()->adjektiv(GRUND_WORN_ADJ))
            this_object()->add_adjektiv(worn_adjektiv,1);
    }
}

/*
FUNKTION: query_worn_adjektiv
DEKLARATION: mixed query_worn_adjektiv()
BESCHREIBUNG:
Liefert das Adjektiv zurueck, das gesetzt wird, wenn ein Kleidungsstueck
angezogen wird.
Achtung: Hier kann natuerlich auch ein Adjektiv der Form
({ Grundform, unregelmaessiger_wortstamm }) geliefert werden.
VERWEISE: set_worn_adjektiv
GRUPPEN: kleidung
*/
mixed query_worn_adjektiv()
{
    return worn_adjektiv;
}

/*
FUNKTION: do_wear
DEKLARATION: void do_wear()
BESCHREIBUNG:
Damit zieht das umgebende Lebewesen diese Kleidung ohne die Ausgabe
irgendwelcher Meldungen an.
VERWEISE: do_remove
GRUPPEN: kleidung
*/
void do_wear()
{
    if(!is_worn)
    {
	is_worn = 1;
	if (environment())
	    environment()->add_temperatur_schutz(temp_schutz);

	do_notifies(C_OMIT_OBJ, "wear", ({"","_me"}),
	    ({environment(), this_object()}));
    }

    if(is_worn && worn_adjektiv && !this_object()->adjektiv(GRUND_WORN_ADJ))
       this_object()->add_adjektiv(worn_adjektiv,1);
}

/*
FUNKTION: do_remove
DEKLARATION: void do_remove()
BESCHREIBUNG:
Damit zieht das umgebende Lebewesen diese Kleidung ohne die Ausgabe
irgendwelcher Meldungen aus.
VERWEISE: do_wear
GRUPPEN: kleidung
*/
void do_remove()
{
    if (is_worn)
    {
	is_worn = 0;
	if (environment())
	{
	    environment()->add_temperatur_schutz(-temp_schutz);
	}

	do_notifies(C_OMIT_OBJ, "undress", ({"","_me"}),
	    ({environment(), this_object()}));
    }
    if (worn_adjektiv && this_object())
	this_object()->delete_adjektiv(worn_adjektiv);
}

int aufsetzbar() { return AUFSETZBAR; }

/*
FUNKTION: already_worn
DEKLARATION: varargs mixed already_worn(int all)
BESCHREIBUNG:
Prueft, ob diese Kleidung getragen werden darf, oder ob derjenige Kleidung
gleichen Typs traegt. Darf er diese Kleidung tragen, so liefert diese
Funktion 0 zurueck. Bei all=0 wird ansonsten eine Kleidung gleichen Typs
geliefert, welche das Anziehen verhindert. Bei all=1 werden alle Kleidungen
diesen Typs in einem Array geliefert (oder evntl. aus
Kompatibilitaetsproblemen eine Kleidung als object).

Ueberlagerbar, z.B. fuer Ruestungsklassen, oder magische Dinge, die man
nicht gemeinsam mit metallener Kleidung/Ruestung tragen kann.
VERWEISE: do_wear
GRUPPEN: kleidung
*/
varargs mixed already_worn(int all)
{
    if (clothes_class)
    {
	object *inv;
	struct clothes_type typinfos = query_clothes_info(clothes_class);
	mapping obs=([]);
	mapping wrong=([:0]);

	inv = all_inventory(environment());
	foreach(object o:inv)
	{
	    string t;
	    if (o->query_worn() && (t=o->query_typ()))
	    {
		if(obs[t])
		    obs[t]+=({o});
		else
		    obs[t]=({o});
	    }
	}

        if (typinfos)
	{
	    object *typobs = obs[clothes_class] || ({});
	    int max_num = typinfos.quantity;

	    foreach(string typ: typinfos.count_types)
		if(obs[typ])
		    typobs+=obs[typ];

	    if(typinfos.quantity_per_hand)
		max_num *= environment()->query_num_hands();
	    if(sizeof(typobs)>=max_num)
		wrong+=mkmapping(typobs);
	}

	if(sizeof(wrong))
	{
	    inv = filter(inv, wrong);
	    if(all)
		return inv;
	    else
		return inv[0];
	}
    }
}

/*
FUNKTION: set_wear_msg
DEKLARATION: void set_wear_msg(mixed wear_msg)
BESCHREIBUNG:
Setzt die Meldung, die derjenige erhaelt, der diese Kleidung anzieht.
Die Meldung wird automatisch umgebrochen und kann auch eine Pseudeclosure
sein. Weitere Informationen dazu gibt es in /doc/funktionsweisen/messages
VERWEISE: query_wear_msg,
          set_wear_msg_other, set_remove_msg, set_remove_msg_other
GRUPPEN: kleidung
*/
void set_wear_msg(mixed msg)
{
    CONSERVATION_HANDLE_MSG("set_wear_msg");
    wear_msg=msg?mixed_to_closure(msg,({'tp}),0,1):0;
}

/*
FUNKTION: query_wear_msg
DEKLARATION: string query_wear_msg()
BESCHREIBUNG:
Liefert die Meldung, die derjenige erhaelt, der diese Kleidung anzieht.
Diese Meldung wird vor der Ausgabe noch umgebrochen.
VERWEISE: set_wear_msg,
          query_wear_msg_other, query_remove_msg, query_remove_msg_other
GRUPPEN: kleidung
*/
string query_wear_msg()
{
  return wear_msg?closure_to_string(wear_msg,({environment()}))
                 :AUFSETZBAR?("Du setzt " + deinen(0,"") + " auf.")
                            :("Du ziehst " + deinen(0,"") + " an.");
}

/*
FUNKTION: set_wear_msg_other
DEKLARATION: void set_wear_msg_other(mixed wear_msg_other)
BESCHREIBUNG:
Setzt die Meldung, die alle anderen im Raum erhalten, wenn jemand diese
Kleidung anzieht. Die Meldung wird automatisch umgebrochen und kann auch
eine Pseudeclosure sein.
Weitere Informationen dazu gibt es in /doc/funktionsweisen/messages
VERWEISE: query_wear_msg_other,
          set_wear_msg, set_remove_msg, set_remove_msg_other
GRUPPEN: kleidung
*/
void set_wear_msg_other(mixed msg)
{
    CONSERVATION_HANDLE_MSG("set_wear_msg_other");
    wear_msg_other=msg?mixed_to_closure(msg,({'tp}),0,1):0;
}

/*
FUNKTION: query_wear_msg_other
DEKLARATION: string query_wear_msg_other()
BESCHREIBUNG:
Liefert die Meldung, die alle anderen Anwesenden im Raum erhalten, wenn
jemand diese Kleidung anzieht. Diese Meldung wird vor der Ausgabe noch
umgebrochen.
VERWEISE: set_wear_msg,
          query_wear_msg_other, query_remove_msg, query_remove_msg_other
GRUPPEN: kleidung
*/
string query_wear_msg_other()
{
  return wear_msg_other?closure_to_string(wear_msg_other,({environment()}))
	    :AUFSETZBAR?(Der(environment()) + " setzt " + seinen(0,"") + " auf.")
	               :(Der(environment()) + " zieht " + seinen(0,"") + " an.");
}

/*
FUNKTION: set_remove_msg
DEKLARATION: void set_remove_msg(mixed remove_msg)
BESCHREIBUNG:
Setzt die Meldung, die derjenige erhaelt, der diese Kleidung auszieht.
Die Meldung wird automatisch umgebrochen und kann auch eine Pseudeclosure
sein. Weitere Informationen dazu gibt es in /doc/funktionsweisen/messages
VERWEISE: query_remove_msg,
          set_wear_msg, set_wear_msg_other, set_remove_msg_other
GRUPPEN: kleidung
*/
void set_remove_msg(mixed msg)
{
    CONSERVATION_HANDLE_MSG("set_remove_msg");
    remove_msg=msg?mixed_to_closure(msg,({'tp}),0,1):0;
}

/*
FUNKTION: query_remove_msg
DEKLARATION: string query_remove_msg()
BESCHREIBUNG:
Liefert die Meldung, die derjenige erhaelt, der diese Kleidung auszieht.
Diese Meldung wird vor der Ausgabe noch umgebrochen.
VERWEISE: set_remove_msg,
          query_wear_msg, query_wear_msg_other, query_remove_msg_other
GRUPPEN: kleidung
*/
string query_remove_msg()
{
  return remove_msg?closure_to_string(remove_msg,({environment()}))
		   :AUFSETZBAR?("Du setzt " + deinen(0,"") + " ab.")
			      :("Du ziehst " + deinen(0,"") + " aus.");
}

/*
FUNKTION: set_remove_msg_other
DEKLARATION: void set_remove_msg_other(mixed remove_msg_other)
BESCHREIBUNG:
Setzt die Meldung, die alle anderen im Raum erhalten, wenn jemand diese
Kleidung auszieht. Die Meldung wird automatisch umgebrochen und kann auch
eine Pseudeclosure sein.
Weitere Informationen dazu gibt es in /doc/funktionsweisen/messages
VERWEISE: query_remove_msg_other,
          set_wear_msg, set_wear_msg_other, set_remove_msg
GRUPPEN: kleidung
*/
void set_remove_msg_other(mixed msg)
{
    CONSERVATION_HANDLE_MSG("set_remove_msg_other");
    remove_msg_other=msg?mixed_to_closure(msg,({'tp}),0,1):0;
}

/*
FUNKTION: query_remove_msg_other
DEKLARATION: string query_remove_msg_other()
BESCHREIBUNG:
Liefert die Meldung, die alle anderen Anwesenden im Raum erhalten, wenn
jemand diese Kleidung auszieht. Diese Meldung wird vor der Ausgabe noch
umgebrochen.
VERWEISE: set_remove_msg,
          query_wear_msg, query_wear_msg_other, query_remove_msg_other
GRUPPEN: kleidung
*/
string query_remove_msg_other()
{
  return remove_msg_other?closure_to_string(remove_msg_other,({environment()}))
	    :AUFSETZBAR?(Der(environment()) + " setzt " + seinen(0,"") + " ab.")
		       :(Der(environment()) + " zieht " + seinen(0,"") + " aus.");
}

private int is_magie_vs_magie(mixed alreadyworn)
{
    // Wenn man eine Kleidung vom Rüstungs-Typ magic anziehen will,
    // aber schon etwas vom selben Rüstungstyp an hat, dann kann die
    // Meldung (für Spieler) total verwirrend sein. Im konkreten Fall
    // waren es ein magischer Mantel und ein magisches Kettchen.

    // > ziehe kettchen an
    // Du hast bereits einen Mantel angezogen!

    // Daher finden wie hier raus, ob es genau dieser Fall ist und
    // geben dafür eine spezielle Meldung aus.

    if (this_object()->query_armour_class() != "magie")
        return 0;

    object other;
    if (pointerp(alreadyworn) && (sizeof(alreadyworn) == 1))
    {
        other = alreadyworn[0];
    }
    else if (objectp(alreadyworn))
    {
        other = alreadyworn;
    }
    else
    {
        return 0;
    }

    if (other->query_armour_class() != "magie")
        return 0;

    return 1;
}

varargs int wear_command(int err, int prefix)
{
    mixed worn_obs;
    string wmo, wm;

    if (this_player()->free_hand() == -1)
	RET(PREFIX("Du hast keine Hand frei."))
    if (is_worn)
	RET(PREFIX(Der(0,"") + IST+ "bereits " + GRUND_WORN_ADJ + "."))
    if (environment() != this_player())
	RET(PREFIX(Den() + " kann man nur anziehen, wenn man " + ihn() +
	    " bei sich hat."))
    if (worn_obs = this_object()->already_worn(1))
    {
        if (is_magie_vs_magie(worn_obs))
            RET(PREFIX("Die magischen Strukturen mögen es nicht, "
            "dass du "+den(worn_obs)+" und "+den(this_object())+
            " zusammen trägst."));

        RET(PREFIX("Du hast bereits " +
        liste(map(objectp(worn_obs)?(worn_obs=({worn_obs})):worn_obs,#'einen,"")) +
        " " + grundform(worn_obs[<1]->query_worn_adjektiv()) + "!"))
    }
    if (do_forbiddens(C_OMIT_OBJ, "wear", ({"","_me"}),
	    ({this_player(), this_object()})))
	return 1;
    wmo = this_object()->query_wear_msg_other();
    wm = this_object()->query_wear_msg();
    this_player()->send_message(MT_LOOK,MA_WEAR,
	strlen (wmo) ? wrap(wmo) : "",
	strlen (wm) ? PREFIX(wm) : PREFIX(""),
	this_player());
    do_wear();
    return 1;
}

varargs int remove_command(int err, int prefix)
{
    string rmo, rm;
    if (this_player()->free_hand() == -1)
	RET(PREFIX("Du hast keine Hand frei."))
    if (!is_worn)
	RET(PREFIX(Der() + IST + "nicht " + GRUND_WORN_ADJ + "."))
    if (do_forbiddens(C_OMIT_OBJ, "undress", ({"","_me"}),
	    ({this_player(), this_object()})))
	return 1;
    rmo = this_object()->query_remove_msg_other();
    rm = this_object()->query_remove_msg();
    this_player()->send_message(MT_LOOK,MA_UNWEAR,
	strlen (rmo) ? wrap(rmo) : "",
	strlen (rm) ? PREFIX(rm) : PREFIX(""),
	this_player());
    do_remove();
    return 1;
}

/*
FUNKTION: query_worn
DEKLARATION: int query_worn()
BESCHREIBUNG:
Liefert 1 zurueck, wenn ein Kleidungsstueck angezogen ist, sonst 0.
VERWEISE:
GRUPPEN: kleidung
*/
int query_worn() { return is_worn; }

/*
FUNKTION: set_schutz
DEKLARATION: void set_schutz(int schutz)
BESCHREIBUNG:
Setzt den Temperaturschutz eines Kleidungsstueck. Die von einem Spieler
empfundene Tempeatur ist die Aussentemperatur + dem Temperaturschutz aller
angezogenen Kleidungsstuecke. D.h. ein Wert von schutz<0 kuehlt, ein
Wert >0 waermt. -> /doc/richtlinien/kleidung/temperatur.
VERWEISE: query_schutz
GRUPPEN: kleidung
*/
void set_schutz(int schutz) 
{
    temp_schutz = schutz;
    this_object()->add_setter_conservation("set_schutz",({temp_schutz}));
}

/*
FUNKTION: set_typ
DEKLARATION: void set_typ(string typ)
BESCHREIBUNG:
Setzt den Kleidungstyp. Ein Spieler kann jeweils nur soviel Kleidungsstuecke
des gleichen Typs anhaben, wie es in /static/adm/KLEIDUNGEN eingetragen ist.
Dort stehen auch alle erlaubten Typen drin.
VERWEISE: query_typ
GRUPPEN: kleidung
*/
void set_typ(string class)
{
    clothes_class = class = convert_umlaute(class);
    this_object()->add_setter_conservation("set_typ",({class}));
#ifdef UNItopia
    if(class && !query_clothes_info(class))
	do_warning2("Der Kleidungstyp '"+class+"' ist unbekannt.\n", __FILE__,
	  object_name(extern_call()?previous_object():this_object()),__LINE__);
#endif
}

/*
FUNKTION: query_schutz
DEKLARATION: int query_schutz()
BESCHREIBUNG:
Liefert den Temperaturschutz eines Kleidungsstuecks zurueck.
VERWEISE: set_schutz
GRUPPEN: kleidung
*/
int query_schutz() { return temp_schutz; }

/*
FUNKTION: query_typ
DEKLARATION: string query_typ()
BESCHREIBUNG:
Liefert den Kleidungstyp eines Kleidungsstuecks zurueck.
VERWEISE: set_typ
GRUPPEN: kleidung
*/
string query_typ() { return clothes_class; }

/*
FUNKTION: query_cloth
DEKLARATION: int query_cloth()
BESCHREIBUNG:
Liefert bei Kleidungsstuecken 1 zurueck. Dient dazu, im Programm ein Objekt
als Kleidungsstueck zu identifizieren.
VERWEISE:
GRUPPEN: kleidung
*/
int query_cloth() { return 1; }

// Verwendet fuer (angezogen) , (beschaedigt) ...
string additional_string(mixed adj, string def)
{
    if (!adj || query_short_string())
	return " (" + (adj ? (pointerp(adj) ? adj[0] : adj) : def) + ")";
    return "";
}

string query_short(object betrachter)
{
    if (query_worn())
	return ::query_short(betrachter) +
		additional_string(GRUND_WORN_ADJ,"angezogen");
    return ::query_short(betrachter);
}

string query_long(object viewer)
{
    if (query_worn())
    {
	// Alte Funktionen beachten...
	string long;
        if (viewer == environment())
            long = this_object()->query_long_owner_when_worn (viewer);
        else
            long = this_object()->query_long_other_when_worn (viewer);
        if (long)
	    return long + Er()+ IST + GRUND_WORN_ADJ + ".\n";
    }
    return ::query_long(viewer);
}

protected string query_long_postprocess(string msg, mapping info)
{
    msg = ::query_long_postprocess(msg, info);

    if(query_worn() && !query_long_has_tag(T_ATOM_TAG_WORN_TEXT))
	msg += wrap(desc_text(T_ATOM_WORN_TEXT, info, ({})));

    return msg;
}

protected mixed desc_condition(string name, mixed info, mixed* par)
{
    object ob = info[TI_OBJECT] || (objectp(info[TI_ITEM]) && info[TI_ITEM]) || this_object();

    switch(name)
    {
        case T_ATOM_WORN:
            return ob && ob->query_worn();
    }

    return ::desc_condition(name, info, par);
}

protected mixed desc_text(string name, mixed info, mixed* par)
{
    object ob = info[TI_OBJECT] || (objectp(info[TI_ITEM]) && info[TI_ITEM]) || this_object();

    switch(name)
    {
        case T_ATOM_WORN_TEXT:
            if(ob)
                return Er(ob) + (ist(ob,IST_SPACE_BEFORE|IST_SPACE_AFTER)) + grundform(ob->query_worn_adjektiv()) + ".";
    }

    return ::desc_text(name, info, par);
}

/*
FUNKTION: T_LISTE
DEKLARATION: Liste der T-Defines fuer Kleidungen
BESCHREIBUNG:

Vordefinierte Bedingungen:
 - T_WORN		Die Kleidung ist angezogen.

Vordefinierte Texte:
 - T_WORN_TEXT		"Die Kleidung ist angezogen." oder so.

Hinweise fuer die Meldungsgeneration:
 - T_HAS_WORN_TEXT	Keine eigene Meldung darueber, ob angezogen.

GRUPPEN: kleidung
*/
/*
FUNKTION: set_long_owner_when_worn
DEKLARATION: deprecated void set_long_owner_when_worn(string long)
BESCHREIBUNG:
Setzt das Long des anziehbaren Objektes im angezogenen Zustand fuer den
Traeger. Eine passende Anschaumeldung kann mit set_look_msg_when_worn
gesetzt werden.

Es wird empfohlen, diese Funktion nicht mehr einzusetzen und stattdessen
diese Beschreibung mit Hilfe von T_OWNER und T_WORN in die normale
Raumbeschreibung zu integrieren.

VERWEISE: query_long_owner_when_worn, set_long_other_when_worn,
          query_long_other_when_worn, set_content_visible_when_worn,
          query_content_visible_when_worn, set_look_msg_when_worn
GRUPPEN: kleidung
*/
/*
FUNKTION: query_long_owner_when_worn
DEKLARATION: deprecated string query_long_owner_when_worn(object viewer)
BESCHREIBUNG:
Liefert das Long des anziehbaren Objektes im angezogenen Zustand fuer den
Traeger.
VERWEISE: set_long_owner_when_worn, set_long_other_when_worn,
          query_long_other_when_worn, set_content_visible_when_worn,
          query_content_visible_when_worn, set_look_msg_when_worn
GRUPPEN: kleidung
*/
/*
FUNKTION: set_long_other_when_worn
DEKLARATION: deprecated void set_long_other_when_worn(string long)
BESCHREIBUNG:
Setzt das Long des anziehbaren Objektes im angezogenen Zustand fuer die
Umstehenden, also die Leute, die das Teil nicht anhaben.

Es wird empfohlen, diese Funktion nicht mehr einzusetzen und stattdessen
diese Beschreibung mit Hilfe von T_OWNER und T_WORN in die normale
Raumbeschreibung zu integrieren.

VERWEISE: query_long_other_when_worn, set_long_owner_when_worn,
          query_long_owner_when_worn, set_content_visible_when_worn,
          query_content_visible_when_worn
GRUPPEN: kleidung
*/
/*
FUNKTION: set_look_msg_when_worn
DEKLARATION: void set_look_msg_when_worn(mixed look)
BESCHREIBUNG:
Setzt die look - Meldung (siehe query_look_msg) fuer ein Kleidungsstueck
im Angezogenen Fall.
Standardmaessig liefert diese fuer Brillen, Halsketten, Muetzen und Ohrringe
die Meldung, dass man vergeblich versucht, xzy anzusehen.
BEISPIEL: set_look_msg_when_worn("$Der(OBJ_TP) verrenkt sich vergeblich,
    um $den(OBJ_TO) anzusehen, aber das wird natuerlich nix, weil $er(OBJ_TO)
    angezogen ist.")
VERWEISE: query_long_other_when_worn, set_long_owner_when_worn,
          query_long_owner_when_worn, set_content_visible_when_worn,
          query_content_visible_when_worn
GRUPPEN: kleidung
*/

string query_look_msg ()
{
    if (!query_worn())
	return 0;
    if (look_msg_when_worn)
        return closure_to_string (look_msg_when_worn);
#if 0 // Vergeblich? Unsinn.
    if (member (({"brille","halskette","muetze","ohrringe"}), clothes_class)
        != -1)
    return wrap (Der(this_player())+" versucht vergeblich, "
       +den()+" anzusehen, aber "+der()+IST+GRUND_WORN_ADJ+".");
#endif
}

string set_look_msg_when_worn (mixed look)
{
    CONSERVATION_HANDLE_XYZ("set_look_msg_when_worn",look);
    look_msg_when_worn = mixed_to_closure (look);
}

string query_long_owner_when_worn (object viewer)
{
    return long_owner_when_worn;
}

deprecated void set_long_owner_when_worn (string long)
{
    long_owner_when_worn = long ? wrap (long) : long;
}

string query_long_other_when_worn (object viewer)
{
    return long_other_when_worn;
}

deprecated void set_long_other_when_worn (string long)
{
    long_other_when_worn = long ? wrap (long) : long;
}

<int|string> kleidung_my_forbidden_move(string ctrl,mapping mv_infos)
{
    if (query_worn())
    {
        if ((!objectp(mv_infos[MOVE_DEST_STR]||mv_infos[MOVE_NEW_ROOM]) ||
            strstr(object_name(mv_infos[MOVE_NEW_ROOM]),
                    PLAYER_INVENTORY_CONTAINER)) &&
                (environment() 
                &&environment()->forbidden("undress", this_object()) ||
                 this_object()->forbidden("undress_me", environment())))
           return 1;
        do_remove();
    }
    return 0;
}

int remove()
{
    if (query_worn())
	do_remove();
    return ::remove();
}


void create()
{
    set_id(({"kleidung"}));
    set_class_id(({"kleidung"}));
    set_name("kleidung");
    set_material( ({"textil"}) );
    set_gender("weiblich");
    set_long("Du siehst nichts Besonderes.\n");
    set_value(10);
    set_weight(1);
    set_schutz(0);
    add_controller("forbidden_move",#'kleidung_my_forbidden_move);
    if (program_name(this_object())==__FILE__) // fuer replace_program-Objekte
        clear_initial_conservation_data();
}


#ifdef LOG_OBJECT_STATS
void log_object_stats()
{
   OBJECT_STATS->add_object_stats(OS_CLOTH, this_object(),
      ({
	 query_name(),
	 query_weight(),
	 query_value(),
	 query_schutz(),
	 query_typ(),
      }));
}
#endif

/*
FUNKTION: forbidden_wear
DEKLARATION: int forbidden_wear(object clothes, object who)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes anziehen kann, wird
who->forbidden("wear", clothes) aufgerufen, liefert dieser Aufruf einen
Wert ungleich 0 zurueck, wird die Kleidung nicht angezogen.

Die Funktion forbidden ruft in allen mit who->add_controller("forbidden_wear",
other) angemeldeten Objekten other die Funktionen other->forbidden_wear(
clothes, who) auf.
Liefert auch nur eine dieser Funktionen einen Wert ungleich 0, dann returnt
forbidden diesen und die Kleidung kann nicht angezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_wear oder forbidden, falls er diese Funktion
ueberlagern will, sorgen.
Beispielanwendung: Kleidung, die das Anziehen weiterer Sachen verhindert,
an einem Badestrand darf nur nackt gebadet werden

Bemerkung: Es wird auch clothes->forbidden("wear_me",who) aufgerufen.
VERWEISE: forbidden, notify, notify_wear, attack, forbidden_undress
GRUPPEN: kleidung
*/

/*
FUNKTION: forbidden_wear_me
DEKLARATION: int forbidden_wear_me(object who, object clothes)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes anziehen kann, wird
clothes->forbidden("wear_me", who) aufgerufen, liefert dieser Aufruf einen
Wert ungleich 0 zurueck, wird die Kleidung nicht angezogen.

Die Funktion forbidden ruft in allen mit clothes->add_controller(
"forbidden_wear_me", other) angemeldeten Objekten other die Funktionen
other->forbidden_wear_me(who, clothes) auf. Liefert auch nur eine dieser
Funktionen einen Wert ungleich 0, dann returnt forbidden diesen und die
Kleidung kann nicht angezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_wear_me oder forbidden, falls er diese Funktion
ueberlagern will, sorgen.
Beispielanwendung: Auf Monster oder Traeger 'spezialisierte' Kleidungen,
Damenschuhe, Herrenschuhe

Bemerkung: Es wird auch who->forbidden("wear",clothes) aufgerufen.
VERWEISE: forbidden, notify, notify_wear, attack, forbidden_undress
GRUPPEN: kleidung
*/

/*
FUNKTION: forbidden_wear_here
DEKLARATION: int forbidden_wear_here(object who, object clothes)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes anziehen kann, wird im Raum
room->forbidden("wear_here", who, clothes) aufgerufen, liefert dieser
Aufruf einen Wert ungleich 0 zurueck, wird die Kleidung nicht angezogen.

Die Funktion forbidden ruft in allen mit room->add_controller(
"forbidden_wear_here", other) angemeldeten Objekten other die Funktionen
other->forbidden_wear_here(who, clothes) auf. Liefert auch nur eine dieser
Funktionen einen Wert ungleich 0, dann returnt forbidden diesen und die
Kleidung kann nicht angezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_wear_here oder forbidden, falls er diese
Funktion ueberlagern will, sorgen.

Bemerkung: Es wird auch who->forbidden("wear", clothes) und
           clothes->forbidden("wear_me", who) aufgerufen.
VERWEISE: forbidden, notify, notify_wear, attack, forbidden_undress
GRUPPEN: kleidung
*/

/*
FUNKTION: notify_wear
DEKLARATION: void notify_wear(object clothes, object who)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes angezogen hat, wird who->notify(
"wear", clothes) aufgerufen.

Die Funktion notify ruft in allen mit who->add_controller("notify_wear", other)
angemeldeten Objekten other die Funktionen other->notify_wear(clothes, who)
auf. Sowohl who als auch other haben dann eine Moeglichkeit, auf das Anziehen
der Kleidung clothes zu reagieren.

Bemerkung: Es wird auch clothes->notify("wear_me",who) aufgerufen.
VERWEISE: forbidden, notify, forbidden_wear, notify_undress
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: notify_wear_me
DEKLARATION: void notify_wear_me(object who, object clothes)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes angezogen hat, wird clothes->
notify("wear_me", who) aufgerufen.

Die Funktion notify ruft in allen mit clothes->add_controller("notify_wear_me",
other) angemeldeten Objekten other die Funktionen other->notify_wear_me(who,
clothes) auf.
Sowohl clothes als auch other haben dann eine Moeglichkeit, auf das Anziehen
durch das Lebewesen who zu reagieren.
Zum Beispiel durch Ueberwerfen eines Shadows oder Veraendern des Aussehens.

Bemerkung: Es wird auch who->notify("wear",clothes) aufgerufen.
VERWEISE: forbidden, notify, forbidden_wear, notify_undress
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: notify_wear_here
DEKLARATION: void notify_wear_here(object who, object clothes)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes angezogen hat, wird im Raum
room->notify("wear_me", who, clothes) aufgerufen.

Die Funktion notify ruft in allen mit room->add_controller(
"notify_wear_here", other) angemeldeten Objekten other die Funktion
other->notify_wear_here(who, clothes) auf. Sowohl room als auch other
haben dann eine Moeglichkeit, auf das Anziehen der Kleidung durch das
Lebewesen zu reagieren.

Bemerkung: Es wird auch who->notify("wear", clothes) und
           clothes->notify("wear_me", who) aufgerufen.
VERWEISE: forbidden, notify, forbidden_wear, notify_undress
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: forbidden_undress
DEKLARATION: int forbidden_undress(object clothes, object who)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes ausziehen kann, wird
who->forbidden("undress", clothes) aufgerufen, liefert dieser Aufruf einen
Wert ungleich 0 zurueck, wird die Kleidung nicht ausgezogen.

Die Funktion forbidden ruft in allen mit who->add_controller(
"forbidden_undress", other) angemeldeten Objekten other die Funktionen
other->forbidden_undress(clothes, who) auf.
Liefert auch nur eine dieser Funktionen einen Wert ungleich 0, dann returnt
forbidden diesen und die Kleidung kann nicht ausgezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_undress oder forbidden, falls er diese Funktion
ueberlagern will, sorgen.
Beispielanwendung: Auf einem Friedhof darf man weder nackt gehen, noch darf man
sich dort pietaetlos ausziehen.

Bemerkung: Es wird auch clothes->forbidden("undress_me",who) aufgerufen.
VERWEISE: forbidden, notify, notify_undress, attack, forbidden_wear
GRUPPEN: kleidung
*/

/*
FUNKTION: forbidden_undress_me
DEKLARATION: int forbidden_undress_me(object who, object clothes)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes ausziehen kann, wird
clothes->forbidden("undress_me", who) aufgerufen, liefert dieser Aufruf einen
Wert ungleich 0 zurueck, wird die Kleidung nicht ausgezogen.

Die Funktion forbidden ruft in allen mit clothes->add_controller(
"forbidden_undress_me", other) angemeldeten Objekten other die Funktionen
other->forbidden_undress_me(who, clothes) auf. Liefert auch nur eine dieser
Funktionen einen Wert ungleich 0, dann returnt forbidden diesen und die
Kleidung kann nicht ausgezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_undress_me oder forbidden, falls er diese
Funktion ueberlagern will, sorgen.
Beispielanwendung: Ein magischer Ring, der sich nicht mehr vom Finger ziehen
laesst.

Bemerkung: Es wird auch who->forbidden("undress",clothes) aufgerufen.
VERWEISE: forbidden, notify, notify_undress, attack, forbidden_wear
GRUPPEN: kleidung
*/

/*
FUNKTION: forbidden_undress_here
DEKLARATION: int forbidden_undress_here(object who, object clothes)
BESCHREIBUNG:
Bevor ein Lebewesen who die Kleidung clothes ausziehen kann, wird im Raum
room->forbidden("undress_here", who, clothes) aufgerufen, liefert dieser
Aufruf einen Wert ungleich 0 zurueck, wird die Kleidung nicht ausgezogen.

Die Funktion forbidden ruft in allen mit room->add_controller(
"forbidden_undress_here", other) angemeldeten Objekten other die Funktionen
other->forbidden_undress_here(who, clothes) auf. Liefert auch nur eine dieser
Funktionen einen Wert ungleich 0, dann returnt forbidden diesen und die
Kleidung kann nicht ausgezogen werden.

Fuer die Ausgabe einer Meldung an das Lebewesen who und evtl. den Raum muss
der Programmierer in forbidden_undress_here oder forbidden, falls er diese
Funktion ueberlagern will, sorgen.

Bemerkung: Es wird auch who->forbidden("undress", clothes) und
           clothes->forbidden("undress_me", who) aufgerufen.
VERWEISE: forbidden, notify, notify_undress, attack, forbidden_wear
GRUPPEN: kleidung
*/

/*
FUNKTION: notify_undress
DEKLARATION: void notify_undress(object clothes, object who)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes ausgezogen hat, wird who->
notify("undress", clothes) aufgerufen.

Die Funktion notify ruft in allen mit who->add_controller("notify_undress",
other) angemeldeten Objekten other die Funktionen other->notify_undress(
clothes, who) auf. Sowohl who als auch other haben dann eine Moeglichkeit,
auf das Ausziehen der Kleidung clothes zu reagieren.
Beispielanwendung: Ein schamhafter NPC...

Bemerkung: Es wird auch clothes->notify("undress_me",who) aufgerufen.
VERWEISE: forbidden, notify, forbidden_undress, notify_wear
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: notify_undress_me
DEKLARATION: void notify_undress_me(object who, object clothes)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes ausgezogen hat, wird clothes->
notify("undress_me", who) aufgerufen.

Die Funktion notify ruft in allen mit clothes->add_controller(
"notify_undress_me", other) angemeldeten Objekten other die Funktionen
other->notify_undress_me(who, clothes) auf.
Sowohl clothes als auch other haben dann eine Moeglichkeit, auf das
Ausziehen durch das Lebewesen who zu reagieren. Zum Beispiel einen Shadow
wieder zu entfernen oder das Aussehen der Kleidung zu veraendern.

Bemerkung: Es wird auch who->notify("undress",clothes) aufgerufen.
VERWEISE: forbidden, notify, forbidden_undress, notify_wear
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: notify_undress_here
DEKLARATION: void notify_undress_here(object who, object clothes)
BESCHREIBUNG:
Nachdem das Lebewesen who die Kleidung clothes ausgezogen hat, wird im Raum
room->notify("undress_me", who, clothes) aufgerufen.

Die Funktion notify ruft in allen mit room->add_controller(
"notify_undress_here", other) angemeldeten Objekten other die Funktion
other->notify_undress_here(who, clothes) auf. Sowohl room als auch other
haben dann eine Moeglichkeit, auf das Ausziehen der Kleidung durch das
Lebewesen zu reagieren.

Bemerkung: Es wird auch who->notify("undress", clothes) und
           clothes->notify("undress_me", who) aufgerufen.
VERWEISE: forbidden, notify, forbidden_undress, notify_wear
GRUPPEN: spieler, monster, haende, waffen
*/

/*
FUNKTION: query_content_visible_when_worn
DEKLARATION: int query_content_visible_when_worn()
BESCHREIBUNG:
Liefert fuer ein anziehbares Kleidungsstueck, welches Inhalt aufnehmen
kann (Rucksack, Handtasche), ob man im angezogenen Zustand den Inhalt
sehen kann.
VERWEISE: set_content_visible_when_worn
GRUPPEN: kleidung
*/

int query_content_visible_when_worn()
{
    return content_visible_when_worn;
}

/*
FUNKTION: set_content_visible_when_worn
DEKLARATION: void set_content_visible_when_worn(int visible)
BESCHREIBUNG:
Liefert fuer ein anziehbares Kleidungsstueck, welches Inhalt aufnehmen
kann (Rucksack, Handtasche), ob man im angezogenen Zustand den Inhalt
sehen kann.
VERWEISE: set_content_visible_when_worn
GRUPPEN: kleidung
*/

void set_content_visible_when_worn (int isvisible)
{
    content_visible_when_worn = isvisible;
    this_object()->add_setter_conservation("set_content_visible_when_worn",
        ({content_visible_when_worn}));
}
