// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /i/item/licht.c
// Description: Licht-System von UNItopia.
//              Aus den verschiedensten Files zusammengesammelt,
//              von Driver-Light-EFuns unabhaengig gemacht.
// Author:      Menaures (15.04.02)

#define TO this_object()

/* --- Globale Variablen: --- */

private int own_light;

/*
FUNKTION: query_own_light
DEKLARATION: int query_own_light()
BESCHREIBUNG:
Mit dieser Funktion kann man den Licht-Level abfragen, der von dem Objekt
selbst ausgeht.
VERWEISE: add_own_light, set_own_light, query_light
GRUPPEN: licht
*/
int query_own_light()
{
    return own_light;
}

/*
FUNKTION: set_own_light
DEKLARATION: void set_own_light(int value)
BESCHREIBUNG:
Mit dieser Funktion setzt man den Licht-Level eines Objektes auf value.
Der gesetzte Licht-Level laesst sich dann mit query_own_light() abfragen.
VERWEISE: add_own_light, query_own_light
GRUPPEN: licht
*/
void set_own_light(int value)
{
    own_light = value;
    this_object()->add_setter_conservation("set_own_light",({own_light}) );
}

/*
FUNKTION: add_own_light
DEKLARATION: void add_own_light(int amount)
BESCHREIBUNG:
Mit dieser Funktion kann man den Licht-Level eines Objektes um amount anheben,
oder, wenn amount negativ ist, absenken.
Der geaenderte Licht-Level laesst sich dann mit query_own_light() abfragen.
VERWEISE: set_own_light, query_own_light
GRUPPEN: licht
*/
void add_own_light(int value)
{
    own_light += value;
    this_object()->add_setter_conservation("set_own_light",({own_light}) );
}

/*
FUNKTION: query_light
DEKLARATION: int query_light()
BESCHREIBUNG:
Liefert den Lichtlevel zurueck, der sich in der aeussersten Umgebung
des Objektes befindet.
VERWEISE: query_own_light, query_inner_light, query_outer_light
GRUPPEN: licht
*/
int query_light()
{
    if(environment())
        return all_environment()[<1]->query_light();
    else
        return TO->query_inner_light() + TO->query_own_light();
}

/*
Aus Kompatibilitaetsgruenden:

TODO:  Nach allen set_light-Aufrufen greppen und
TODO:: auf add_own_light(i) umstellen.

TODO:  Wenn umgestellt, Funktion entfernen
*/
int set_light(int value)
{
    TO->add_own_light(value);
    return TO->query_light();
}

/*
FUNKTION: query_inner_light
DEKLARATION: int query_inner_light()
BESCHREIBUNG:
Liefert den Lichtlevel, den die Objekte in dem Container nach aussen hin
produzieren.

Beispiel:

 tisch->query_inner_light()   liefert den Lichtlevel zurueck,
                              den der Inhalt des Tischs nach aussen hin abgibt.

ACHTUNG: Das Eigenlicht des Objektes (hier also des Tischs) ist in diesem Wert
         nicht enthalten. Dieses muss mit query_own_light() abgefragt werden.

         Handelt es sich bei dem Container um ein nicht transparentes Objekt,
         wird query_inner_light() nicht an die Umgebung weitergegeben.

VERWEISE: query_outer_light, query_light, query_own_light
GRUPPEN: licht
*/
int query_inner_light()
{
    int sum, light;
    int * lights;

    // Der Lichtlevel der inneren Objekte wird berechnet und zurueckgeliefert.

    lights = all_inventory(TO)->query_outer_light() - ({0});

    foreach(light : lights)
        sum += light;

    return sum;
}

/*
FUNKTION: query_outer_light
DEKLARATION: int query_outer_light()
BESCHREIBUNG:
Liefert den Lichtlevel, den die Objekte in dem Container sowie der Container
selbst nach aussen hin abgeben. Fuer den Fall, dass es sich um einen
lichtundurchlaessigen Container handelt, entspricht dies query_own_light().

Beispiel:

 tisch->query_outer_light()    liefert zurueck, wieviel Licht von
                               dem Tisch samt Inhalt ausgeht.

VERWEISE: query_inner_light, query_light, query_own_light
GRUPPEN: licht
*/
int query_outer_light()
{
    if(!TO->query_transparent())
    {
        // Das Objekt ist undurchsichtig.
        // Nur das eigene Licht wird nach aussen hin abgegeben.

        return TO->query_own_light();
    }

    else
    {
        // Der innere Lichtlevel plus das eigene Licht wird nach
        // aussen hin abgegeben.

        return TO->query_inner_light() + TO->query_own_light();
    }
}

/* --- End of file. --- */
