// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/living/body.c
// Description:	Der Koerper mit den Koerperattributen

#pragma save_types
#pragma strong_types

#include <apps.h>

private string koerperform;
private string race;
private int koerpergroesse;
private int abilities;

/*
FUNKTION: set_koerperform
DEKLARATION: void set_koerperform(string form)
BESCHREIBUNG:
Mit dieser Funktion setzt man die Koerperform des Lebewesens.
Folgende Koerperformen sind moeglich:
  humanoid                     Mensch, Ork, Elf, ...
  vierbeiner                   Fliege, Hund, Katze, Baer, ...
  vogel                        Vogel, ...
  fisch                        Delphin, Hai, Wal, ...
  schlange                     Wurm, Schlange, ...
  insekt                       Fliege, Muecke, Kaefer, ...
  gegenstand                   Lebende Truhen, Steine, ...
  pflanze                      Baum, Blume, Unkraut, ...
VERWEISE: query_koerperform, set_koerpergroesse, query_koerpergroesse
GRUPPEN: monster
*/
void set_koerperform(string form)
{
    if (MONSTER_MASTER->valid_koerperform(form))
	koerperform = form;
}

/*
FUNKTION: query_koerperform
DEKLARATION: string query_koerperform()
BESCHREIBUNG:
Diese Funktion liefert die Koerperform des Lebewesens.
VERWEISE: set_koerperform, set_koerpergroesse, query_koerpergroesse
GRUPPEN: monster
*/
string query_koerperform()
{
    return koerperform;
}

/*
FUNKTION: set_koerpergroesse
DEKLARATION: void set_koerpergroesse(int groesse)
BESCHREIBUNG:
Mit dieser Funktion setzt man die Koerpergroesse des Lebewesens.
Folgende Koerpergroessen sind moeglich:

  1:   0 cm bis   5 cm         Fliege
  2:   5 cm bis  30 cm         Maus, Ratte
  3:  30 cm bis  70 cm         Katze
  4:  70 cm bis 1,2 m          Hund
  5: 1,2 m  bis 1,7 m          Wolf
  6: 1,7 m  bis 2,2 m          Mensch
  7: 2,2 m  bis 3,7 m          Baer
  8: 3,7 m  bis   7 m          Elefant
  9:   7 m  bis  15 m          kleiner Drache
 10: groesser 15 m             grosser Drache

VERWEISE: set_koerperform, query_koerperform, query_koerpergroesse
GRUPPEN: monster
*/
void set_koerpergroesse(int groesse)
{
    koerpergroesse = groesse;
}

/*
FUNKTION: query_koerpergroesse
DEKLARATION: int query_koerpergroesse()
BESCHREIBUNG:
Diese Funktion liefert die Koerpergroesse des Lebewesens.
VERWEISE: set_koerpergroesse, set_koerperform, query_koerperform
GRUPPEN: monster
*/
int query_koerpergroesse()
{
    return koerpergroesse;
}

/*
FUNKTION: set_abilities
DEKLARATION: void set_abilities(int flag)
BESCHREIBUNG:
Mit dieser Funktion setzt man die Faehigkeiten eines Lebewesens.
Folgende Faehigkeiten sind moeglich:
  MD_AB_WALK (1): laufen             Kann sich an Land fortbewegen
  MD_AB_SWIM (2): schwimmen          Kann ohne AP-Abzug schwimmen
  MD_AB_FLY (4):  fliegen            Kann fliegen

Die Faehigkeiten werden mit einem Bitweise ODER verknuepft.
Die Konstanten dazu sind in monster_master.h definiert.
VERWEISE: query_abilities
GRUPPEN: monster
*/
void set_abilities(int flag)
{
    abilities = flag;
}

/*
FUNKTION: query_abilities
DEKLARATION: int query_abilities()
BESCHREIBUNG:
Diese Funktion liefert die Faehigkeiten des Lebewesens.
VERWEISE: set_abilities
GRUPPEN: monster
*/
int query_abilities()
{
    return abilities;
}


/*
FUNKTION: set_race
DEKLARATION: void set_race(string rasse)
BESCHREIBUNG:
Damit setzt man die Rasse des Lebewesen. Es wird empfohlen anstatt dieser
Funktion die Funktion initialize zu verwenden, damit das Lebewesen gleich
fuer die Rasse korrekte Werte erhaelt.
VERWEISE: query_race, initialize
GRUPPEN: monster
*/
void set_race(string rasse)
{
    race = rasse;
}

/*
FUNKTION: query_race
DEKLARATION: string query_race()
BESCHREIBUNG:
Diese Funktion liefert die Rasse dieses Lebewesens zurueck.
VERWEISE: initialize
GRUPPEN: monster
*/
string query_race()
{
    return race;
}
