// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/object/countunits.c
// Description:	Einheiten fuer CountObs und MultiObs
// Author:	Gnomi

#ifdef UNItopia
private functions inherit "/p/Misc/i/zahlen";
#endif

#include <deklin.h>

int query_count();

private mapping units = ([ "": 1 ]);
private mixed *backunits = ({ ({ 1, ""}) });

/*
FUNKTION: set_units
DEKLARATION: void set_units(mapping hin, mapping back)
BESCHREIBUNG:
Mit dieser Funktion setzt man die Einheiten, welche erkannt
und ausgegeben werden sollen. Das erste Mapping 'hin' enthaelt
die zu erkennenden Einheiten, welche im Mapping klein geschrieben
werden muessen. Das zweite Mapping 'back' enthaelt die anzuzeigenden
Einheiten mit Singular als ersten und Plural als zweiten Wert.

Beispiel:

    set_units(([
         "g":		1,
	 "gramm":	1,
	 "kg":		1000,
	 "kilo":	1000,
	 "kilogramm":	1000,
	 "t":		1000000,
	 "tonne":	1000000,
	 "tonnen":	1000000
    ]),
    ([
	1:		" Gramm";" Gramm",
	1000:		" Kilo";" Kilo",
	1000000:	" Tonne";" Tonnen",
    ]));

Im Beispiel werden sowohl "1g" als auch "1 gramm" oder "3 kilo"
erkannt. Angezeigt werden sie jedoch als "1 Gramm" oder "3 Kilo".

ACHTUNG: Damit die Anzeige ordnungsgemaess funktioniert, muss
         /i/object/countunits entweder vor /i/object/countob
	 inheritet werden, oder query_menge muss ueberlagert
	 und an countunits weitergeleitet werden (return
	 countunits::query_menge()).
GRUPPEN: countob
*/
void set_units(mapping hin, mapping back)
{
    units = hin;
    backunits = sort_array(transpose_array(unmkmapping(back)),
		    (: $1[0] < $2[0] :));
}

int parse_count(string* words, int result)
{
    result = 0;
    
    for(int i=0;i<sizeof(words);i++)
    {
	int lastidx = i;
	int zahl;
	string word = lower_case(words[i]);
	string rest;
	
	if((sscanf(word,"%d%s", zahl, rest) && zahl>0)
#ifdef UNItopia
	    || (zahl=wort2zahl(word))>0
#endif
	  )
	{
	    rest = trim(rest||"");
	    if(!sizeof(rest) && i+1<sizeof(words) && stringp(words[i+1]))
	    {
	        rest = lower_case(words[i+1]);
		if(!member(units,rest) && member(units, ""))
		    rest = "";
		else
	    	    i++;
	    }
	    
	    if(!member(units, rest))
		return lastidx;
	    
	    result += zahl * units[rest];
	}
	else if(word == "und")
	    continue;
	else
	    return i;
    }
    
    return sizeof(words);
}

mixed query_menge()
{
    int rest = query_count();
    string* str = ({});
    
    foreach(mixed u: backunits)
	if(rest >= u[0])
	{
	    if(rest < 2*u[0])
	        str += ({ to_string(rest / u[0]) + u[1] });	// Singular
	    else	
	        str += ({ to_string(rest / u[0]) + u[2] });	// Plural
	    rest %= u[0];
	}
    
    if(sizeof(str))
    {
	string res = liste(str);
	return ([ "name": res, "gender": "saechlich", "personal": 1,
		  "genitiv": res, "dativ": res, "akkusativ": res ]);
    }
//	return ({ liste(str), PRON_NICHT_DEKLIN });
}

int query_plural()
{
    int rest = query_count();

    foreach(mixed u: backunits)
	if(rest > u[0])
	    return 1;
	else if(rest == u[0])
	    return 0;

    return rest>1;
}
