// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/object/pfeil.c
// Description: Pfeil auf multiob basierend.
// Author:	Francis
// Modified by: Garthan (11.03.96) auf countob basierender Pfeil
// Modified by: Garthan (23.03.96) auf multiob basierender Pfeil

/*
 * Ein (Beispiel-) Pfeil fuer /obj/schuss_waffe. Hier findet man
 * alle Moeglichkeiten, die /obj/schuss_waffe seinen Pfeilen bietet.
 */

#pragma save_types

inherit "/i/object/multiob";

#include <error.h>

private int life;
private string* damage_type;

/*
FUNKTION: set_life
DEKLARATION: void set_life(int life)
BESCHREIBUNG:
Damit setzt man die durchschnittliche Lebensdauer eines Pfeiles.
Nach jeder Benutzung wird einfach getestet, ob random(life)!=0 ist,
und falls nicht, der Pfeil zerstoert.

Dies spielt nur dann eine Rolle, wenn bei der Schusswaffe nicht mittels
set_destroy_pfeil gesetzt wurde, dass das Geschoss sofort zerstoert
werden soll.
VERWEISE: query_life, check_life, set_destroy_pfeil, query_destroy_pfeil
GRUPPEN: waffen
*/
void set_life(int i)
{
    life = i;
    add_setter_conservation("set_life",({life}) );
}

/*
FUNKTION: add_life
DEKLARATION: deprecated int add_life(int i)
BESCHREIBUNG:
Ist veraltet und macht nichts mehr.

Aufgrund der Tatsache, dass das beim Geschoss gesetzte Leben nur eine
durchschnittliche Lebensdauer (also im Endeffekt nur eine Wahrscheinlichkeit)
darstellt, sollte dies bei der Benutzung nicht verringert werden.
Um eine Abnutzung anzuzeigen, kann man check_life() aufrufen.
VERWEISE: set_life, query_life, check_life
GRUPPEN: waffen
*/
deprecated int add_life(int i) {}

/*
FUNKTION: query_life
DEKLARATION: int query_life()
BESCHREIBUNG:
Diese Funktion liefert die durchschnittliche(!) Lebensdauer des Geschosses
zurueck.
VERWEISE: set_life, check_life
GRUPPEN: waffen
*/
int query_life()
{
    return life;
}

/*
FUNKTION: check_life
DEKLARATION: void check_life()
BESCHREIBUNG:
Bei Aufruf dieser Funktion wird das Geschoss abgenutzt.
Dabei kann passieren, dass dieses Objekt zerstoert wird.
VERWEISE: set_life, query_life
GRUPPEN: waffen
*/
void check_life()
{
    if(!random(life))
	add_count(-1);	// Zerstoert notfalls das Objekt
}

/*
FUNKTION: query_damage_type
DEKLARATION: string* query_damage_type()
BESCHREIBUNG:
Liefert die Schadensart des Pfeils zurueck.
Diese hat Vorrang vor der Schadensart des Bogens und wird
beim AP-Abzug durch add_hp als AH_DAMAGE_TYPE uebergeben.
VERWEISE: set_damage_type, set_damage
GRUPPEN: waffen, kampf
*/
string* query_damage_type()
{
    return damage_type;
}

/*
FUNKTION: set_damage_type
DEKLARATION: void set_damage_type(string* damage_type)
BESCHREIBUNG:
Damit wird die Schadensart des Pfeils festgelegt.
Diese hat Vorrang vor der Schadensart des Bogens und wird
beim AP-Abzug durch add_hp als AH_DAMAGE_TYPE uebergeben.
VERWEISE: query_damage_type, set_damage
GRUPPEN: waffen, kampf
*/
void set_damage_type(string* dtype)
{
    damage_type = dtype;
    add_setter_conservation("set_damage_type",({damage_type}) );
}

/*
FUNKTION: init_geschoss
DEKLARATION: void init_geschoss(string kategorie)
BESCHREIBUNG:
Setzt alle relevanten Werte fuer die entsprechende Geschoss-Kategorie.

Geschoss-Kategorie  passend fuer die Waffen-Kategorie
-----------------------------------------------------
blasrohrpfeil       blasrohr
bolzen              armbrust
kieselstein         schleuder
pfeil               bogen
VERWEISE: set_pfeil_id, query_pfeil_id
GRUPPEN: waffen, geschoss
*/

void init_geschoss(string kategorie)
{
    switch(kategorie)
    {
	case "pfeil":
	    set_id("pfeil");
	    set_plural_id(({"pfeile"}));
	    set_singular_name("pfeil");
	    set_plural_name("pfeile");
	    set_gender("maennlich");
	    set_count_type("pfeil");
	    set_smell("Du nimmst den Geruch des Todes wahr.\n");
	    set_material("holz");
	    set_life(1);
	    set_damage_type(({"stich"})); 
	    break;

    case "bolzen":
	    set_singular_name("bolzen");
	    set_plural_name("bolzen");
	    set_gender("maennlich");
	    set_id("bolzen");
	    set_plural_id(({"bolzen"}));
	    set_count_type("bolzen");
	    set_material("metall");
	    set_smell("Du nimmst den Geruch des Todes wahr.\n");
	    set_life(4);
	    set_damage_type(({"stich"}));
	    break;

	case "blasrohrpfeil":
	    set_singular_name("blasrohrpfeil");
	    set_plural_name("blasrohrpfeile");
	    set_gender("maennlich");
	    set_id(({"blasrohrpfeil", "pfeil", }));
	    set_plural_id(({"blasrohrpfeile", "pfeile", }));
	    set_count_type("blasrohrpfeil");
	    set_material("holz");
	    set_smell("Du nimmst den Geruch des Todes wahr.\n");
	    set_life(2);
	    set_damage_type(({"stich"}));
	    break;

	case "kieselstein":
	    set_singular_name("kieselstein");
	    set_plural_name("kieselsteine");
	    set_gender("maennlich");
	    set_id(({"kieselstein", "stein", }));
	    set_plural_id(({"kieselsteine", "steine", }));
	    set_count_type("kieselstein");
	    set_material("stein");
	    set_life(10);
	    set_damage_type(({"stumpf"}));
	    break;

	default:
    	    do_error("Unbekannte Geschoss-Kategorie '"+kategorie+"'!\n");
    	    return;
    }
}

private void update_life(string cname, object alt, object neu)
{
    int ac,nc;
    // neu == this_object()
    set_life((query_life()*(nc=query_count())+alt->query_life()*(ac=alt->query_count()))/(ac+nc));
}

void create()
{
    ::create();
    init_geschoss("pfeil");
    add_controller("notify_incorporated_countob", #'update_life);
    if (program_name(this_object())==__FILE__) // fuer replace_program-Objekte
        clear_initial_conservation_data();
}

/*
FUNKTION: query_extra_damage
DEKLARATION: int query_extra_damage(object feind, object besitzer, object waffe)
BESCHREIBUNG:
Hier kann noch zusaetzlich Schaden abhaengig vom Pfeil, dem Feind, Besitzer
oder seiner Waffe angegeben werden. Er wird auf den Schaden der Waffe
aufaddiert.
VERWEISE: query_extra_damage, real_lost_hp
GRUPPEN: waffen
*/

#if EXAMPLE_CODE
int query_extra_damage(object feind, object besitzer, object waffe)
{
   return 0;
}
#endif

/*
FUNKTION: real_lost_hp
DEKLARATION: void real_lost_hp(object feind, object besitzer, int verlorene_hp)
BESCHREIBUNG:
Hier bekommt man die tatsaechliche Anzahl an im Feind abgezogenen
Ausdauerpunkten zurueck. Man kann z.B. einen vergifteten Pfeil simulieren,
indem man hier dem Feind einen Virus verpasst.
VERWEISE: query_extra_damage, real_lost_hp
GRUPPEN: waffen
*/
#if EXAMPLE_CODE
void real_lost_hp(object feind, object besitzer, int verlorene_hp)
{
}
#endif

/*
FUNKTION: query_geschoss
DEKLARATION: int query_geschoss()
BESCHREIBUNG:
Liefert 1, wenn es sich um ein Geschoss fuer Schusswaffen handelt, sonst 0.

query_geschoss() wird allerdings von Schusswaffen nicht ausgewertet; sofern
query_pfeil_id() einer Schusswaffe in einem Objekt als ID gesetzt ist, kann
es mit dieser Schusswaffe verschossen werden.
VERWEISE: set_pfeil_id, query_pfeil_id
GRUPPEN: waffen
*/
int query_geschoss ()
{
    return 1;
}

object split_object(int i)
{
    object ob;

    if((ob = ::split_object(i)) && ob != this_object())
        ob->set_life(query_life());
    return ob;
}
