// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /i/object/soul.c
// Description: Ueberarbeitete deutsche Seele. Teile von Prospero
//              uebernommen.
// Created by : Reiga
//
// Neue Seele-Befehle bitte in
//     /p/Doc/Lehre/Demo/room/soul-test-room.c
// testen.
//
// Modified by: fast alle ham da schon dran rumgepfriemelt, siehe Versionen
// Version 1.0      Reiga     April 93
// Version 1.1      Francis   16.07.93
// Version 1.2      Garthan   29.07.93
// Version 1.3      Monty     Feb.  94
// Version 1.4      Pulami    02.03.94
// Version 1.4Beta  Monty     25.05.94
// Version 1.4Beta  Freaky    28.06.94
// Version 1.4Gamma Viper     10.08.94
// Version 2.0      Monty     11.11.94
//                Grosse Ueberarbeitung, Neue Emotes.
// Version 2.0a     Monty     18.07.95
//                Neue Emotes, kleine Bugfixes.
// Version 2.0.b    Sissi      6.08.95
//                Ausfuehrlichere Texte bei ein paar Befehlen
// Version 2.0.c    Sissi      5.10.95
//                Frage - und schuettle - Befehle verschlimmbessert
//                wuschle und kringle eingebaut
// Version 2.1      Monty     16.02.96
//                notify_soul() eingebaut bei allen Kommandos, die einen
//                Partner betreffen
// Version 2.2      Monty (29.04 1996)
//                neue Seele-Befehle: deute, erbleiche, friere, maule, roechle
// version 2.21     Monty (28.06 96)
//                neue Seele-Befehle: kneife, haue, schwitze, kreische, japse,
//                bruddle,
//                notify_soul in deute, kleine Bugfixes.
// Version 3.0      Zorro (16.08.96)
//                - soul_command() eingebaut vor allen Kommandos, die einen
//                  Partner betreffen
//                - Alle Funktionen nach dem Schema:
//                  soul_command, write, say, tell_object, notify_soul
//                - Alle du's, dein's & dir's  auf Grossschreibung umgestellt
// Version 3.01    Sissi (14.12.96)
//                - entschuldige
// Version 3.1      Zorro (30.07.97)
//                - NO_GHOST, IS_GHOST, Komandos auch fuer Geister
//                - SOUL_COMMAND erst nach Test auf (opfer!=owner)
// Freaky (20.07.1998) groehle raus (schreibt sich groele)
// Parsec (23.11.1998)
//    - query_long() ueberlagert
//    - Seele sucht Partner auch in Besitzer
//    - Viele Aktionen erlauben jetzt auch Praepostitonen,
//       z.B. huepfe UM, knabber AN, ... (anschmiegen, knickse, kotze, krieche,
//            popel, quengle, stolper, taenzle, tanze, verneige, verabschiede,
//            verbeuge, zupfe
//    - weinen fuer Nicht-Lebewesen-Partner korrigiert
//    - neu: verbeugen, schimpfen, tobe, vogel, zunge
//    - stark ueberarbeitet tuschle, pinkle
//    - wuete beruecksichtig Landschaft
//    - pinkle beruecksichtig Landschaft und angezogene Hosen
//    - taenzle aktiviert
//    - kitzle-Reaktion beruecksichtigt soul_command
//    - forbidden/notify_seele/comm + allowed_seele eingefuehrt
//    - druecken, knuddeln, kuschel, schmusen, umarmen mit Seele
//    - Du, Dich, ... klein um einheitlich mit restlicher MUD-Lib zu sein
//    - Neues Message-System
//    - Bewertung der Seeleaktionen (MIES, NEUTRAL, NETT)
//    - sag, frag, schmoll mit Partner
//    - Bei Fehlgeschlagenen Seele-Befehlen ggf. auch den Grund ausgeben
//      (z.B.anschmiege xx)


#pragma save_types
#pragma strong_types


#define SAGE_VERWENDEN         1
#define FRAGE_MIT_PARTNER      1

// #define MAKE_HELP

inherit "/i/install";
inherit "/i/item";
inherit "/i/tools/dein_name";        // Funktion des ich-Kommandos
inherit "/i/tools/soul_hlp";   // Parser und Ausgabegenerator
inherit "/i/tools/room_types";


#include <config.h>
#include <invis.h>
#include <deklin.h>
#include <landschaft.h>
#include <parse_com.h>
#include <more.h>
#include <message.h>
#include <level.h>
#include <commands.h>
#include <soul.h>

#define ENV(t)          environment(t)
#define CAP(t)          capitalize(t)
#define TO              this_object()
#define TP              this_player()
#define GHOST(x)        if (x->query_ghost()) return 0
#define SEELE 0
#define TIGER 1
#define TEDDY 2
#define HP              HELP_PATH+"/soul/"
#define K               HP+"komm"
#define KA              HP+"komma"
#define KPA             HP+"kommap"
#define KP              HP+"kommp"
#define TABLE(x)        explode(sprintf("%-79#s\n",read_file(x)), "\n")
#define LINE            "----------------------------------------"\
                                                "---------------------------------------"
#define SAVEFILE        "/save/seele"


#define IS_GHOST    (owner->query_ghost())
#define NO_GHOST    if( IS_GHOST ) \
 { notify_fail("Als Geist ist dir das leider nicht moeglich!\n"); return 0; }



private object  owner;
private int     zustand;


void set_seele()
{
    set_id(({"seele"}));
    set_name("seele");
    set_gender("weiblich");
    set_adjektiv(({}));
    set_feel( 0) ;
    // set_invis( V_NOLIST);
    set_invis( V_INVIS);
    zustand = SEELE;
}


void set_tiger()
{
    set_name("stofftiger");
    set_gender("maennlich");
    set_id(({"tiger","stofftiger","seele"}));
    set_feel( "Dein Stofftiger ist ganz besonders weich und flauschig.") ;
    set_invis( V_VIS);
    zustand = TIGER;
}


void set_teddy()
{
    set_name("teddybär");
    set_gender("maennlich");
    set_id(({"teddybär","bär","teddy","seele"}));
    set_feel( "Dein Teddybär ist ganz besonders weich und flauschig.") ;
    set_invis( V_VIS);
    zustand = TEDDY;
}


void create()
{
    set_seele() ;

    set_long( Wer( 0, ART_EIN | ART_VIS)+".");
    set_weight(0);
    add_controller("allowed_seele",this_object());
    
    seteuid( getuid());
}


mixed *query_auto_load()
{
    return ({zustand,query_adjektiv(),this_object()->query_gruppenumarmung_bekannt()});
}


string query_long(object viewer)
{
    switch ( zustand )
    {
        case TIGER :
            return wrap(
                ( sizeof( query_adjektiv()) ?
                  Ein() : "Ein niedlicher, kleiner Stofftiger")+
                ". Er gehört "+
                (( ((playerp( previous_object())) ?
                    previous_object() : this_player()) == owner ) ?
                 "dir" : dem( owner))+
                ".") ;

        case TEDDY :
            return wrap(
                Ein( 0, ( sizeof( query_adjektiv()) ? 0 : "süß"))+
                ", der vom vielen "
                "Knuddeln und Drücken schon ganz abgegriffen ist.");

        default :
            return Wer( 0, ART_ER|ART_VIS)+" ist durchsichtig.\n" ;
    }
}


void init_arg(mixed *args)
{
    if(!pointerp(args) || sizeof(args)<=2) return;
    switch(args[0])
    {
        case TIGER : set_tiger(); break;
        case TEDDY : set_teddy(); break;
    }
    if (args[0] != SEELE)
        set_adjektiv(args[1]);
    if (sizeof (args) >= 3)
       this_object()->set_gruppenumarmung_bekannt(args[2]);
}

void init()
{
    if ( owner  ||
         this_player() != environment() ) // Sonst koennen AL-Monster
                                          // die Kommandos bekommen
        return;

    owner = environment();
    soul_hlp::set_owner(owner); // damit soul_hlp auch tut!
    dein_name::init();          // Das <dein_name>-Kommando.
    add_action ("verwandle",             "verwandele", -9);
    add_action ("verwandle",             "verwandle");
    add_action ("beschreibe",            "beschreibe", -9);
    add_action ("hilfe",                 "hilfe");


/*
Funktionen ohne Partner und Adverb sind mit "*" gekennzeichnet, wenn sie
eine Kommndozeile beruecksichtigen, ist noch ein "+" dran...
Wenn ein "P" dransteht, wird ein Partner beruecksichtigt, bei "A" ein
Adverb.
*/

    add_action( "achsel",        "achselzucken",-10);// A
    add_action( "achsel",        "zucke",-4);
    add_action( "achsel",        "schulterzucken",-12);
    add_action( "achsel",        "schulter");
    add_action( "aechze",        "ächze",-4);     // A
    add_action( "anschmiege",    "anschmiegen",-9);// AP
    add_action( "anschmiege",    "schmiegen",-7);
    add_action( "applaudiere",   "applaudiere",-7);// AP
    add_action( "argl",		 "argle", -4);
    add_action( "baggere",       "bagger",AA_SHORT);    // AP
    add_action( "bedauere",      "bedauere",-7);      // AP
    add_action( "bedauere",      "bedaure");
    add_action( "begruesse",     "begrüße",-6);  // AP
    add_action( "bewundere",     "bewundere",-8);  // AP
    add_action( "blinzle",       "blinzel");       // AP
    add_action( "blinzle",       "blinzle");
    add_action( "boxe",          "boxe",-3);       // AP
    add_action( "brumme",        "brumme",-5);     // AP
    add_action( "daeumle",       "däumle");       // A
    add_action( "daeumle",       "däumel");
    add_action( "danke",         "danke",-4);      // AP
    add_action( "deute",         "deute",-4);      // AP
    add_action( "drohe",         "drohe",-4);      // P
    add_action( "druecke",       "drücke",-5);    // AP
    add_action( "entmanne",      "entmanne",-7);   // AP
    add_action( "entschuldige",  "entschuldige",-11); // AP
    add_action( "erbleiche",     "erbleiche",-8);  // A
    add_action( "erroete",       "erröte",-5);    // A
//    add_action( "erschreck",     "erschreck",-8);  // AP
//    add_action( "erschreck",     "erschrick");
    add_action( "feixe",         "feixe",-4);      // AP
    add_action( "flippe",        "flippe",-5);     // A
    add_action( "fluche",        "fluche",-5);     // A
    add_action( "freue",         "freue",-4);      // A
    add_action( "friere",        "friere",-5);     // *
    add_action( "fuchtle",       "fuchtel");       // A
    add_action( "fuchtle",       "fuchtle");
    add_action( "furze",         "furze",-4);      // A
    add_action( "gackere",       "gackere",-6);    // A
    add_action( "gaehne",        "gähne",-4);     // AP
    add_action( "gluckse",       "gluckse",-6);    // A
    add_action( "gratuliere",    "gratuliere",-9); // AP
    add_action( "grinse",        "grinse",-5);     // AP
    add_action( "grueble",       "grübele",-6);   // A
    add_action( "grueble",       "grüble");
    add_action( "grummle",       "grummle");       // AP
    add_action( "grummle",       "grummele",-7);
    add_action( "grumpfgrummle", "grumpfe", -6);
    add_action( "grumpfgrummle", "grumpfgrummele", -7);
    add_action( "grumpfgrummle", "grumpfgrummle");
    add_action( "grunze",        "grunze",-5);     // A
    add_action( "gruppenumarmung","gruppenumarmung"); // *
    add_action( "gruppenumarmung","gu");           // *
    add_action( "haetschle",     "hätschele",-8); // AP
    add_action( "haetschle",     "hätschle");
    add_action( "haue",          "haue",-3);       // AP
    add_action( "heule",         "heule",-4);      // A
    add_action( "hechle",        "hechle",-5);     // AP
    add_action( "hechle",        "hechel");
    add_action( "hickse",        "hickse",-5);     // A
    add_action( "huch",		 "huche", -4);
    add_action( "huepfe",        "hüpfe",-4);     // A
    add_action( "huestle",       "hüstle");       // A
    add_action( "huestle",       "hüstele",-6);   // A
    add_action( "huste",         "huste",-4);      // AP
    add_action( "ieks",		 "iekse", -4);
    add_action( "ja",            "ja");            // *
    add_action( "jammer",        "jammere",-6);    // AP
    add_action( "japse",         "japse",-4);      // A
    add_action( "jaule",         "jaule",-4);      // A
    add_action( "jubiliere",     "jubiliere",-8);  // A
    add_action( "juble",         "juble");         // A
    add_action( "juble",         "jubel");
    add_action( "keife",         "keife",-4);      // AP
    add_action( "keuche",        "keuche",-5);     // A
    add_action( "kichere",       "kichere",-6);    // A
    add_action( "kitzeln",       "kitzele",-6);    // AP
    add_action( "kitzeln",       "kitzle");
    add_action( "klatsche",      "klatsche",-7);   // AP
    add_action( "knabbere",      "knabbere",-7);   // AP
    add_action( "kneife",        "kneife",-5);     // AP
    add_action( "knickse",       "knickse",-6);    // AP
    add_action( "knirsche",      "knirsche",-7);   // A
    add_action( "knuddle",       "knuddle");       // AP
    add_action( "knuddle",       "knuddele",-7);
    add_action( "knurre",        "knurre",-5);     // AP
    add_action( "knutsche",      "knutsche",-7);   // P
    add_action( "kotze",         "kotze",-4);      // AP
    add_action( "kraule",	 "kraule",-5);
    add_action( "kratze",        "kratze",-5);     // AP
    add_action( "krieche",       "krieche",-6);    // AP
    add_action( "kringle",       "kringel");       // *
    add_action( "kringle",       "kringle");
    add_action( "kuesse",        "küsse",-4);     // AP
    add_action( "kuschle",       "kuschele",-7);   // P
    add_action( "kuschle",       "kuschle");
    add_action( "lache",         "lache",-4);      // AP
    add_action( "lache",         ":-D");
    add_action( "lache",         ":D");
    add_action( "laechle",       "lächele",-6);   // AP
    add_action( "laechle",       "lächle");
    add_action( "laechle",       ":-)");
    add_action( "laechle",       ":)");
    add_action( "laechle",       "(-:");
    add_action( "laechle",       "(:");
    add_action( "lecke",         "lecke",-4);      // AP
    add_action( "liebe",         "liebe");         // P
    add_action( "maule",         "maule",-4);      // AP
    add_action( "moshe",         "moshe",-4);      // A
    add_action( "moshe",         "mosche",-5);
    add_action( "nerve",         "nerve",-4);      // AP
    add_action( "nicke",         "nicke",-4);      // AP
    add_action( "niese",         "nies",AA_SHORT);        // AP
    add_action( "noergle",       "nörgel");       // AP
    add_action( "noergle",       "nörgle");
    add_action( "ohrfeige",      "ohrfeige");      // AP
    add_action( "patsch",        "patsch",AA_SHORT);      // AP
    add_action( "patsch",        "m)");      // AP
    add_action( "patsch",        "m(");      // AP
    add_action( "patsch",        "(m");      // AP
    add_action( "patsch",        ")m");      // AP
    add_action( "pfeife",        "pfeif",AA_SHORT);       // AP
    add_action( "piepse",        "piepse",-5);     // A
    add_action( "pinkle",        "pinkle");        // AP
    add_action( "pinkle",        "pinkele",-6);
    add_action( "pople",         "popel");         // AP
    add_action( "pople",         "pople");
    add_action( "pruste",        "pruste",-5);     // AP
    add_action( "quaele",        "quäle",-4);     // AP
    add_action( "quengle",       "quengle");       // AP
    add_action( "quengle",       "quengel");
    add_action( "raeuspere",     "räuspere",-7);  // A
    add_action( "reibe",         "reibe",-4);      // AP
    add_action( "roechle",       "röchel");       // A
    add_action( "roechle",       "röchle");
    add_action( "ruelpse",       "rülpse",-5);    // AP
    add_action( "ruempfe",       "rümpfe",-5);    // A
    add_action( "runzle",        "runzele",-6);    // A
    add_action( "runzle",        "runzle");
    add_action( "sabbere",       "sabbere",-6);    // AP
    add_action( "schaeme",       "schäme",-5);    // A
    add_action( "schimpfe",      "schimpfe",-7);   // AP
    add_action( "schluchze",     "schluchz",AA_SHORT);    // A
    add_action( "schlucke",      "schlucke",-7);   // A
    add_action( "schmatze",	 "schmatze",-7);
    add_action( "schmolle",      "schmolle",-7);   // AP
    add_action( "schmunzle",     "schmunzele",-9); // AP
    add_action( "schmunzle",     "schmunzle");
    add_action( "schmuse",       "schmuse",-6);    // AP
    add_action( "schnarche",     "schnarche",-8);  // *
    add_action( "schnaube",      "schnaube",-7);   // AP
    add_action( "schnaufe",      "schnaufe",-7);   // AP
    add_action( "schneuze",      "schneuze",-7);   // A
    add_action( "schniefe",      "schniefe",-7);   // AP
    add_action( "schniefe",      ":-(");
    add_action( "schniefe",      ":(");
    add_action( "schniefe",      ")-:");
    add_action( "schniefe",      "):");
    add_action( "schnippe",      "schnippe",-7);   // A
    add_action( "schnuckel",	 "schnuckele",-9);
    add_action( "schnuckel",	 "schnuckle");
    add_action( "schnurre",      "schnurre",-7);   // AP
    add_action( "schreie",       "schreie",-6);    // AP
    add_action( "schuettle",     "schüttele",-8); // AP
    add_action( "schuettle",     "schüttle");
    add_action( "schwitze",      "schwitze",-7);   // A
    add_action( "seufze",        "seufze",-5);     // A
    add_action( "spucke",        "spucke",-5);     // AP
    add_action( "starre",        "starre",-5);     // AP
    add_action( "staune",        "staune",-5);     // AP
    add_action( "stiere",        "stiere",-5);     // AP
    add_action( "stoehne",       "stöhne",-5);    // A
    add_action( "stolpere",      "stolpere",-7);   // AP
    add_action( "stolziere",     "stolziere",-8);  // A
    add_action( "stosse",        "stoße",-4);     // AP
    add_action( "strahle",       "strahle",-6);    // AP
    add_action( "streichle",     "streichele",-9); // AP
    add_action( "streichle",     "streichle");
    add_action( "stupse",        "stupse",-5);     // P
    add_action( "taenzle",       "tänzle");       // P
    add_action( "taenzle",       "tänzel");
    add_action( "taetschel",     "tätschele",-8); // P
    add_action( "taetschel",     "tätschle");
    add_action( "tanze",         "tanze",-4);      // AP
    add_action( "taumle",        "taumele",-6);    // A
    add_action( "taumle",        "taumle");
    add_action( "tobe",          "tobe",-3);       // *
    add_action( "torkel",	 "torkele",-6);
    add_action( "torkel",	 "torkle");
    add_action( "traeume",       "träume",-5);    // A
    add_action( "trete",         "trete",-4);      // AP
    add_action( "trete",         "tritt");
    add_action( "troeste",       "tröste",-5);    // AP
    add_action( "umarme",        "umarme",-5);     // AP
    add_action( "verabschiede",  "verabschiede");  // AP
    add_action( "verbeuge",      "verbeuge",-7);   // AP
    add_action( "verneige",      "verneige",-7);   // AP
    add_action( "verneine",      "nein");
    add_action( "verneine",      "verneine",-7);   // *
    add_action( "verzweifel",	 "verzweifele",-10);
    add_action( "verzweifel",	 "verzweifle");
    add_action( "vogel",         "vogel");         // P
    add_action( "wackle",        "wackele",-6);    // A
    add_action( "wackle",        "wackle");
    add_action( "waelze",        "wälze",-4);     // *
    add_action( "weine",         "weine",-4);      // A
    add_action( "weine",         ":'-(");      // A
    add_action( "weine",         ":'(");      // A
    add_action( "weine",         ")-':");      // A
    add_action( "weine",         ")':");      // A
    add_action( "wiehere",       "wiehere",-6);    // A
    add_action( "winke",         "winke",-4);      // AP
    add_action( "wonke",         "wonke",-4);      // AP
    add_action( "winsle",        "winsel");
    add_action( "winsle",        "winsle");        // AP
    add_action( "wuerge",        "würge",-4);     // P
    add_action( "wuerge",        "erwürge",-6);   // AP
    add_action( "wuete",         "wüte",-3);      // *
    add_action( "wundere",       "wundere",-6);    // *
    add_action( "wuschle",       "wusch",AA_SHORT);     // P
    add_action( "zittere",       "zittere",-6);    // A
    add_action( "zunge",         "zunge");         // AP
    add_action( "zunge",         ":-P");         // AP
    add_action( "zunge",         ":-p");         // AP
    add_action( "zunge",         ":P");         // AP
    add_action( "zunge",         ":p");         // AP
    add_action( "zunge",         "q-:");         // AP
    add_action( "zunge",         "q:");         // AP
    add_action( "zupfe",         "zupfe",-4);      // AP
    add_action( "zwicke",        "zwicke",-5);     // AP
    add_action( "zwinkere",      "zwinkere",-7);   // AP
    add_action( "zwinkere",      ";-)");   // AP
    add_action( "zwinkere",      ";)");   // AP
    add_action( "zwinkere",      "(-;");   // AP
    add_action( "zwinkere",      "(;");   // AP

#ifdef UNItopia
    add_action( "nasel",         "nasele",-5);     // AP
    add_action( "nasel",         "nasle");         // AP
#endif
}

/* erst die Befehle, mit denen man die Seele direkt beeinflussen kann */

int verwandle( string str)
{
    string  wasn, was;

    notify_fail( "Verwandle was in was?\n");
    if( !str || str=="" )
        return 0;
    str=lower_case( space(str));
    if( sscanf( str,"%s in %s", wasn, was)!=2 || !me( wasn) )
        return 0;

    was = convert_umlaute(was);

    if( member( ({ "tiger", "stofftiger", "einen tiger", "einen stofftiger" }),
                was) != -1 )
    {
        if ( zustand == TIGER )
            msg_notify( "Aber du hast doch schon "+wen( 0, ART_EIN | ART_VIS,
                                                        ({}))+".");
        else
        {
            msg_soul_action( MA_EMOTE,
                             Wer( 0, ART_DEIN | ART_VIS)+" verwandelt sich.",
                             Der( owner)+" verwandelt "+
                             wen( 0, ART_SEIN | ART_VIS)+".",
                             MT_LOOK) ;
            set_tiger();
        }
        return 1;
    }
    if( member( ({ "teddybaer", "einen teddybaer",  "teddy", "einen teddy",
                   "baer", "einen baer"}), was) != -1 )
    {
        if ( zustand == TEDDY )
            msg_notify( "Aber du hast doch schon "+wen( 0, ART_EIN | ART_VIS,
                                                        ({}))+".");
        else
        {
            msg_soul_action( MA_EMOTE,
                             Wer( 0, ART_DEIN | ART_VIS)+" verwandelt sich.",
                             Der( owner)+" verwandelt "+
                             wen( 0, ART_SEIN | ART_VIS)+".",
                             MT_LOOK) ;
            set_teddy();
        }
        return 1;
    }
    if( member( ({ "seele", "eine seele" }), was) != -1 )
    {
        if ( zustand == SEELE )
            msg_notify( "Aber "+wer( 0, ART_DEIN | ART_VIS)+" ist doch schon "+
                        wer( 0, ART_EIN | ART_VIS)+".") ;
        else
        {
            msg_soul_action( MA_EMOTE,
                             Wer( 0, ART_DEIN | ART_VIS)+" verwandelt sich.",
                             Der( owner)+" verwandelt "+
                             wen( 0, ART_SEIN | ART_VIS)+".",
                             MT_LOOK) ;
            set_seele();
        }
        return 1;
    }
    msg_notify( "Du kannst deine Seele nur in einen Teddy, einen Tiger oder "
                "halt in eine Seele verwandeln.");
    return 1;
}

int allowed_seele()
{
    return 1;
}

int beschreibe(string str)
{
    string was, wie;
    notify_fail("Beschreibe was wie?\n");
    if(!str || str=="") return 0;
    str=lower_case(str);
    if(sscanf(str,"%s %s",was,wie)!=2 || !me(was)) return 0;

    if( zustand == SEELE )
    {
        msg_notify(
            "Du musst deine Seele erst verwandeln, bevor du sie beschreiben "
            "kannst!\n");
        return 1;
    }
    if(strlen(wie)>15)
    {
        msg_notify("Sorry, das ist einfach zu lang!\n");
        return 1;
    }
    msg_soul_action( MA_EMOTE,
                     "Du beschreibst "+wen( 0, ART_DEIN | ART_VIS)+".",
                     Der(owner)+" beschreibt "+
                     wen( 0, ART_SEIN | ART_VIS)+".",
                     MT_NOTIFY, MT_LOOK) ;

    set_adjektiv(({wie}));
    return 1;
}

#ifdef MAKE_HELP
/*
Die Hilfefunktion bastelt aus verschiedenen Hilfefiles einen huebsch
Formatierten Hilfetext zusammen. Damit entfaellt das laestige
Aendern des Hilfetexts, wenn man mal eine neue Funktion in der Seele
einfuehrt.
*/

string write_hilfe()
{
    rm (SAVEFILE);
    write_file(SAVEFILE, LINE+"\n"+
        wrap("Die Kommandos werden OHNE [] eingegeben. Buchstaben in [] "
        "können einfach weggelassen werden. Probiere einfach alle Befehle "
        "der Seele aus.")+
        LINE+"\nKommandos, um Gefühle auszudrücken:\n\n"
        "        ich                 siehe 'hilfe ich'\n"
        "        mein, mir, mich     analog\n"+LINE+"\n"
        "Kommandos ohne Partner und Adverb:\n"
        "(Kommandos mit '+' können wie 'sag' funktionieren)\n\n"+
        implode(TABLE(K),"\n")+
        LINE+"\nKommandos nur mit Partner:\n\n"+
        implode(TABLE(KP),"\n")+
        LINE+"\nKommandos nur mit Adverben:\n\n"+
        implode(TABLE(KA), "\n")+
        LINE+"\nKommandos mit Partner und möglichem Adverb:\n\n"+
        implode(TABLE(KPA),"\n")+
        LINE+"\n"+
        wrap("Man kann die Seele mit 'verwandle' in einen Teddybär "
                "oder einen Stofftiger (oder wieder zurück in die unsichbare "
                "seele) verwandeln und mit 'beschreibe' ein "
                "(nicht zu langes) Adjektiv anhängen. Ein paar Kommandos "
                "funktionieren mit einer sichtbaren Seele anders...")+
        LINE+"\n");
    return "Hilfefile "+SAVEFILE+" geschrieben.";
}
#endif

int hilfe(string who)
{
    if (!this_object()->me(who)) return 0;
    this_player()->more(HELP_PATH+"/seele", "--- Mehr ---", 0, M_AUTO_END);
    return 1;
}



/*
----------------------------------------------------------------------------
Jetzt alle Emotions, die man mit der Seele ausdruecken kann, in
alphabetischer Reihenfolge
----------------------------------------------------------------------------
*/

#include "/i/object/soul_commands.inc"
#ifdef UNItopia
#include "/z/Raetsel/Suedpol/sys/nasel_seele.inc"
#endif
