// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /i/player/fehler.c
// Description: Fehler/Idee/Typo/Wuerdige - Befehl
// Author:      Parsec
// Modified by: Parsec (18.05.98) Es duerfen maximal 6 Woerter zum beschreiben
//                                eines Objektes verwendet werden, da sonst 
//                                TLEs auftreten koennen
//              Parsec (22.06.98) Idee .../..,  idee hier bla,  und
//                                Objektnachfrage vergisst 1-Objekt
//                                ausgebessert
//              Parsec (08.04.99) Goetter koennen Ideen zu Dateinamen setzen
//              Rompler (16.8.99) Lob/Typo eingefuehrt
//              Parsec (20.08.99) Deklination von Idee/Lob/Fehler/Typo
//                                ueberall richtig
//              Parsec (24.09.99) Fehler zu Verzeichnissen + finden von
//                                .c-Dateien

// -------------------------------------------------------------------
//  idee/fehler - add_action
//  Syntax:
//
//   idee|fehler  [hier|raum|<objekt>|<v-item>]  [$]  [<text>]
//   idee|fehler  $<objekt>|<v-item>  [<text>]
//
//  Fehlende oder falsche Angaben werden interaktiv erfragt.
//  Fuer Goetter ist auch die Verwendung eines Objektstrings wie
//  beim Zauberstab moeglich.
//  Beispiele:
//        idee  Parsec:kuscheli
//    - Verwendung von Dateinamen
//        idee  /w/pars*/work*
//    - Pfadkuerzel
//        pk w /w/parsec
//        idee  w/wo*
//    - Markierungen
//        zm Parsec:kuscheli
//        idee  @0
//
//  Nuetzlich ist das vorallem bei Objekten die keine Umgebung haben,
//  wie Inherits oder irgendwelche Master-Objekte.
// -------------------------------------------------------------------

#pragma save_types

#include <config.h>
#include <deklin.h>
#include <level.h>
#include <parse_com.h>
#include <error_db.h>
#include <message.h>
#include <editor.h>
#include <strings.h>
#include <input_to.h>
#include <move.h>
#include <invis.h>
#include "player.h"

#define WIZ_SEARCHES_WIZ_OBJECTS                1
#define LIMIT_NUMBER_OF_WORDS_IN_OBJ_STRING     1

// Das wird auch vom Anfaengerpraktikum genutzt, daher diese Abfrage
#define FEHLER_WER	(playerp(this_object())?this_object():this_player())


// Falls eingeschaltet wird nach maximal 6 Woertern gesucht, die das
// gesuchte Objekt beschreiben (meine 2. gruene sproede kaputte fackel)
// Ohne das Limit kann es bei langen Ideetexten die direkt in der Komando-
// zeile eingegeben werden zu zu vielen Evals kommen (parse_com mit vielen
// woertern ist teuer wenn es nix findet)
// Aber wer beschreibt sein Objekt schon mit mehr als 6 Woertern ... also
// ist das eigentlich keine Einschraenkung.
#if LIMIT_NUMBER_OF_WORDS_IN_OBJ_STRING
#define LIMIT_WORDS(str)    implode(explode(space(str), " ")[0..5], " ")
#else
#define LIMIT_WORDS(str)    space(str)
#endif


private static mapping *feedback;
private static int feedback_nr;

/* prototypes */
int fehler(string str);
void fehler_absetzen(string text, mixed objekt);
int fehler_an_datei(string text, string datei, mapping dinfo);
void set_fehler_ohne_dollar_hier(int i);
int query_fehler_ohne_dollar_hier();
private varargs string fehler_objekt_deinen(mixed ob, int no_extra_file_name);
private string fehler_objekt_deinem(mixed ob);
private mixed wiz_get_object(string str);
private mixed *expand_wildcards(string str);
private varargs void fehler_nachfragen(int nur_prompt);
private void fehler_prompt_hilfe();
void get_fehler_objekt(string str);
private void fehler_schreibe_text();
private varargs void detail_ed(object who, string key, mixed text, string erg, mapping vitem);
/* end prototypes */



private static mixed   fehler_objekt;
private static string  fehler_text;
private static mapping fehler_mwas;
private static int     fehler_objekt_auswahl;
private static int     fehler_ohne_dollar_hier;
private static int     fehler_art;
private static object  fehler_initial_umgebung;

private int keine_gummigoettchen;

#define fehler_was  (fehler_mwas["name"])

#define FEHLER_EINGABE_UNVOLLSTAENDIG                                 \
    write(Dein(fehler_mwas, 0, FEHLER_WER)+" konnte keinem "        \
           "Raum oder Objekt eindeutig zugeordnet werden.\n")

#define FEHLER_ABBRUCH                                                \
    write(wrap("Schade! Vielleicht waere "+                         \
                 dein(fehler_mwas, 0, FEHLER_WER)+                   \
                 " hilfreich gewesen."))

//
// idee/fehler - add_action  von Parsec
// Syntax:
//
//   idee|fehler  [hier|raum|<objekt>|<v-item>]  [$]  [<text>]
//   idee|fehler  $<objekt>|<v-item>  [<text>]
//
int fehler(string str)
{
    mixed   parsed;
    int     pos;
    string  objekt_str, objekt_str2, strip_str;

    switch(regreplace(lower_case(query_verb_ascii()), "#|echte(r|s|)_", "", 1))
    {
        case "detail":
            fehler_mwas= ([ "name":"Detail", "gender":"saechlich" ]);
            fehler_art = EDB_DETAIL;
            break;
        case "idee":
            fehler_mwas= ([ "name":"Idee", "gender":"weiblich" ]);
            fehler_art = EDB_IDEE;
            break;
        case "toll" :
        case "wuerdige" :
        case "wuerdigung" :
            fehler_mwas= ([ "name":"Würdigung", "gender":"weiblich" ]);
            fehler_art = EDB_LOB;
            break;
        case "typo":
            fehler_mwas= ([ "name":"Typo", "gender":"maennlich" ]);
            fehler_art = EDB_TYPO;
            break;
        default:
            fehler_mwas= ([ "name":"Fehlermeldung", "gender":"weiblich" ]);
            fehler_art = EDB_FEHLER;
            break;
    }

    fehler_text = "";
    fehler_objekt = 0;
    fehler_objekt_auswahl = 0;
    fehler_initial_umgebung = environment(FEHLER_WER);

    if (!str)
        fehler_nachfragen();
    else if ((pos = strstr(str, "$")) == -1)
    {
        // Kein $ in der Eingabe
        objekt_str = LIMIT_WORDS(str);

        parsed = parse_com(objekt_str);
        if (parsed[PARSE_RET_CODE] == PARSE_OK)
        {
            // Objekt/V-Item am Anfang des Textes gefunden

            fehler_objekt = parsed[PARSE_OBS][0];
            if (strlen(strip(parsed[PARSE_REST])))
            {
                if (fehler_ohne_dollar_hier)
                    fehler_absetzen(str, 0);
                else
                {
                    FEHLER_EINGABE_UNVOLLSTAENDIG;
                    fehler_objekt_auswahl = 1;
                    fehler_text = str;
                    fehler_nachfragen();
                }
            }
            else
                // Nur Objekt/V-Item angegeben
                fehler_nachfragen();
        }
#if WIZ_SEARCHES_WIZ_OBJECTS
        else if (wizp(FEHLER_WER)  &&
                  fehler_objekt = wiz_get_object(objekt_str))
        {
            objekt_str2 = regreplace(objekt_str, " *[^ ]* *$", "", 1);
            if (fehler_objekt == wiz_get_object(objekt_str2))
            {
                if (fehler_ohne_dollar_hier)
                    fehler_absetzen(str, 0);
                else
                {
                    FEHLER_EINGABE_UNVOLLSTAENDIG;
                    fehler_objekt_auswahl = 1;
                    fehler_text = str;
                    fehler_nachfragen();
                }
            }
            else
                // Nur Objekt angegeben
                fehler_nachfragen();
        }
#endif
        else if (sizeof(regexp(({ lower_case(str) }),
                                  "^ *(hier|raum)([^:]|$)")))
        {
            // Text faengt mit "hier" oder "raum" an

            if (strlen(regreplace(lower_case(str), "^ *(hier|raum) *", "", 1)))
                fehler_absetzen(str, 0);
            else
                fehler_schreibe_text();
        }
        else if (fehler_ohne_dollar_hier)
            fehler_absetzen(str, 0);
        else
        {
            // Kein $ angegeben und nix vernuenftiges gefunden
            FEHLER_EINGABE_UNVOLLSTAENDIG;
            fehler_text = str;
            fehler_nachfragen();
        }
    }
    else
    {
        // $ war in der Eingabe
        objekt_str = LIMIT_WORDS(str[0..pos-1]);
        strip_str = strip(str = str[pos+1..]);

        parsed = parse_com(objekt_str);
        if (parsed[PARSE_RET_CODE] == PARSE_OK)
        {
            // Objekt/V-Item vor $ gefunden
            if (strlen(strip_str))
                fehler_absetzen(strip_str, parsed[PARSE_OBS][0]);
            else
            {
                fehler_objekt = parsed[PARSE_OBS][0];
                fehler_schreibe_text();
            }
        }
#if WIZ_SEARCHES_WIZ_OBJECTS
        else if (wizp(FEHLER_WER)  &&
                  fehler_objekt = wiz_get_object(objekt_str))
        {
            if (strlen(strip_str))
                fehler_absetzen(strip_str, fehler_objekt);
            else
                fehler_schreibe_text();
        }
#endif
        else
        {
            // Zeug vor $ laesst sich nicht parsen

            if (objekt_str == "" && str[0] != ' ')
            {
                // Unmittelbar nach dem $ steht ein Wort (ohne Lehrzeichen)
                objekt_str2 = explode(str, " ")[0];

                parsed = parse_com(objekt_str2);
                if (parsed[PARSE_RET_CODE] == PARSE_OK)
                {
                    // Objekt/V-Item direkt nach $ gefunden (ohne Leerstelle)
                    // vor dem $ steht nix

                    strip_str = strip(str[strlen(objekt_str2)..]);
                    if (strlen(strip_str))
                        fehler_absetzen(strip_str, parsed[PARSE_OBS][0]);
                    else
                    {
                        fehler_objekt = parsed[PARSE_OBS][0];
                        fehler_schreibe_text();
                    }
                    return 1;
                }
#if WIZ_SEARCHES_WIZ_OBJECTS
                else if (wizp(FEHLER_WER)  &&
                          fehler_objekt = wiz_get_object(objekt_str2))
                {
                    // Objekt direkt nach $ gefunden (ohne Leerstelle)
                    // vor dem $ steht nix

                    strip_str = strip(str[strlen(objekt_str2)..]);
                    if (strlen(strip_str))
                        fehler_absetzen(strip_str, fehler_objekt);
                    else
                        fehler_schreibe_text();
                    return 1;
                }
#endif
            }

            if (lower_case(objekt_str) == "hier" ||
                 lower_case(objekt_str) == "raum")
            {
                // "hier" oder "raum" vor dem $
                if (strlen(strip_str))
                    fehler_absetzen(strip_str, 0);
                else
                    fehler_schreibe_text();
            }
            else if (strlen(strip_str))
            {
                write(wrap(
                    ((strlen(objekt_str)) ?
                     capitalize(objekt_str)+" nicht gefunden." :
                     "Kein Objekt angegeben!")));
                fehler_text = strip_str;
                fehler_nachfragen();
            }
            else
            {
                write(wrap(
                    ((strlen(objekt_str)) ?
                     capitalize(objekt_str)+" nicht gefunden" :
                     "Kein Objekt")+
                    " und kein Text angegeben."));
                fehler_nachfragen();
            }
        }
    }
    return 1;
}

// Fehler absetzen aus more heraus:
int fehler_an_datei(string text, string datei, mapping dinfo)
{
    if (file_size(datei||"")<0 || space(text)=="") {
        return 0;
    }
    MASTER_OB->log_fehler_new(space(text), datei,
                               FEHLER_WER, EDB_FEHLER, dinfo);
    return 1;
}

int fehler_an_po(string text, object po, mapping dinfo)
{
    if (!objectp(po) || space(text)=="") {
        return 0;
    }
    MASTER_OB->log_fehler_new(space(text), po,
                               FEHLER_WER, EDB_FEHLER, dinfo);
    return 1;
}

/*
FUNKTION: set_keine_gummigoettchen
DEKLARATION: void set_keine_gummigoettchen(int flag)
BESCHREIBUNG:
Schaltet die Gummigoettchen fuer abgesetzte Fehler ein (flag == 0, 
default) oder aus (flag != 0).
VERWEISE: query_keine_gummigoettchen
GRUPPEN: spieler
*/
void set_keine_gummigoettchen(int flag) 
{ 
    keine_gummigoettchen = flag != 0 ? 1 : 0; 
    return;
}

/*
FUNKTION: query_keine_gummigoettchen
DEKLARATION: int query_keine_gummigoettchen()
BESCHREIBUNG:
Liefert 1, wenn der Spieler keine Gummigoettchen fuer abgesetzte Fehler
erhaelt, sonst 0.
VERWEISE: set_keine_gummigoettchen
GRUPPEN: spieler
*/
int query_keine_gummigoettchen() 
{ 
    return keine_gummigoettchen; 
}


// Setzt den Fehler in die Fehler-DB
void fehler_absetzen(string text, mixed objekt)
{
    object bonbon;

    if (!objekt)
        objekt = fehler_initial_umgebung||environment(FEHLER_WER);

    MASTER_OB->log_fehler_new(text, objekt,
                               FEHLER_WER, fehler_art);/*(fehler_was==1));*/
    if (wizp(FEHLER_WER) && objectp(objekt))
        write(wrap("Objekt: "+object_name(objekt)));

    write(wrap(
        Dein(fehler_mwas, 0, FEHLER_WER)+" für "+
        fehler_objekt_deinen(objekt, 1)+
        " wurde gespeichert. Vielen Dank für Deine Hilfe."));
    
    // seit 01.04.2011: Gummigoettchen
    if (!this_player()->query_keine_gummigoettchen()) 
    {
        bonbon=clone_object("/room/rathaus/obj/gummigott");
#ifdef UNItopia
        if (this_player()->query_gilde() == "Vampyrgilde") {
            bonbon->set_vamp_nahrung(-1);
        }
#endif
        if(MOVE_OK!=bonbon->move(this_player()))
        {
            if (this_player()->query_invis() == V_INVIS) 
            {
               this_player()->send_message_to(this_player(), 
                   MT_NOTIFY, MA_REMOVE,
                   wrap("Aus einem harmlosen Loch im Raum-Zeit-Gefüge tritt "
                        "ein livrierter Diener und will Dir ein leckeres "
                        "Gummigöttchen in die Hand drücken. Da Du leider "
                        "keinen Platz mehr hast und außerdem unsichtbar"
                        "bist, verschwindet er lieber schnell wieder, bevor "
                        "ihn jemand anderes bemerken kann."));
                bonbon->remove();
            }
            else if (MOVE_OK!=bonbon->move(environment(this_player())))
                bonbon->remove();
            else 
                bonbon->send_message(MT_LOOK, MA_PUT, wrap("Aus einem "
                "harmlosen Loch im Raum-Zeit-Gefüge tritt ein livrierter "
                "Diener und will "+dem(this_player())+" ein leckeres "
                "Gummigöttchen in die Hand drücken. Aber weil "+
                der(this_player())+" wohl keinen Platz mehr hat, legt er "
                "es vor "+ihm(this_player())+" auf den Boden."), 
                wrap("Aus einem harmlosen Loch im Raum-Zeit-Gefüge tritt "
                "ein livrierter Diener und will Dir ein leckeres "
                "Gummigöttchen in die Hand drücken. Da Du leider "
                "keinen Platz mehr hast, legt er es vor Dich hin."), 
                this_player());
        }
        else 
        {
            if (this_player()->query_invis() == V_INVIS)
                bonbon->send_message_to(this_player(),
                    MT_LOOK, MA_PUT, 
                    wrap("Aus einem harmlosen Loch im Raum-Zeit-Gefüge "
                         "tritt ein livrierter Diener und drückt Dir "
                         "schnell und unauffällig ein leckeres "
                         "Gummigöttchen in die Hand, so dass ihn niemand "
                         "anderes bemerkt."));
            else 
                bonbon->send_message(MT_LOOK, MA_PUT, wrap("Aus einem "
                "harmlosen Loch im Raum-Zeit-Gefüge tritt ein livrierter "
                "Diener und drückt "+dem(this_player())+" ein leckeres "
                "Gummigöttchen in die Hand."), wrap("Aus einem harmlosen "
                "Loch im Raum-Zeit-Gefüge tritt ein livrierter Diener und "
                "drückt Dir ein leckeres Gummigöttchen in die Hand."),
                this_player());
        }
    }
}


void set_fehler_ohne_dollar_hier(int i)
{
    fehler_ohne_dollar_hier = i;
}


int query_fehler_ohne_dollar_hier()
{
    return fehler_ohne_dollar_hier;
}


private varargs string fehler_objekt_deinen(mixed ob, int no_extra_file_name)
{
    return
        ((ob && ob != environment(FEHLER_WER))
         ?    ((stringp(ob))
            ? ((wizp(FEHLER_WER)) ? ob : "Etwas")
            : ((objectp(ob) && ob->query_cap_name() && (!wizp(FEHLER_WER) || ob->query_gender()) ||
                mappingp(ob) && ob["name"])
            ? (((objectp(ob) && environment(ob) ||
                 mappingp(ob) && ob["environment"]) == FEHLER_WER)
		? deinen(ob) : ihren(ob)) +
               ((wizp(FEHLER_WER))
                ? ((objectp(ob))
                   ? ((no_extra_file_name) ? "" :" "+object_name(ob))
                   : " (Detail)")
                : "")
            : ((wizp(FEHLER_WER))
                ? ((objectp(ob)) ? object_name(ob) : "Detail")
                : "dieses namelose Etwas")))
         : "diesen Raum");
}


private string fehler_objekt_deinem(mixed ob)
{
    return
        ((ob && ob != environment(FEHLER_WER))
         ? ((stringp(ob))
            ? ((wizp(FEHLER_WER)) ? ob : "Etwas")
            : ((objectp(ob) && ob->query_cap_name() ||
                 mappingp(ob) && ob["name"])
               ? (((objectp(ob) && environment(ob) ||
                     mappingp(ob) && ob["environment"]) == FEHLER_WER) ?
                  deinem(ob) : ihrem(ob)) +
               ((wizp(FEHLER_WER))
                ? ((objectp(ob)) ? " "+object_name(ob) : " (Detail)")
                : "")
               : ((wizp(FEHLER_WER))
                  ? ((objectp(ob)) ? object_name(ob) : "Detail")
                  : "diesem namelosen Etwas")))
         : "diesem Raum");
}


#if WIZ_SEARCHES_WIZ_OBJECTS
// Versucht aus  str  ein Objekt zu generieren nach Art des Zauberstabes.
// Hauptanwendung in der Art  /w/parsec/wo*,
// Parsec:kuscheli  oder  Markierungen @0, @1
private mixed wiz_get_object(string str)
{
    mixed   res;
    string  tmp, *tmp2;
    int     size;

    str = space(str);

    // Damit nicht faelchlicherweise die @0 Marierung verwendet wird
    if (str == "")
        return 0;
    else if (!catch(res = search_object(str)) && res)
        return res;

    // search_object() findet keine nicht im Raum awesenden Objekte
    // wenn danach noch was in  str  steht z.B.
    // search_object("detlef ist dumm"), deswegen nochmal die nachfrage:
    else if ((tmp = explode(str, " ")[0]) != str  &&
        !catch(res = search_object(tmp))  &&  res)
        return res;

    // Ok, ein Objekt scheint es nicht zu sein, testen wir mal, ob es
    // ein Dateiname ist
    else if (sizeof(tmp2 = expand_wildcards(tmp))  &&
              !catch(size = file_size(tmp2[0]))     &&
              (size >= 0 || size == -2))
        return tmp2[0];
    else if (sizeof(tmp2= expand_wildcards(tmp+".c"))  &&
              !catch(size = file_size(tmp2[0]))         &&
              size >= 0)
        return tmp2[0];
    else
        return 0;
}


// geklaut aus dem Zaeuberstab
private mixed *expand_wildcards(string str) {
    string *wild_files, *files, *ret;
    string path, *com_path, last;
    int i;

    if (str)
        wild_files=explode(str," ")-({""});
    else
        wild_files=({""});
    ret=({});
    for (i=0; i<sizeof(wild_files); i++) {
        com_path=FEHLER_WER->compose_path(wild_files[i],2);
        if (sizeof(com_path) &&
                (member(last=com_path[<1],'*')>=0 || member(last,'?')>=0)) {
            if (sizeof(com_path)>1)
                path="/"+implode(com_path[0..<2],"/")+"/";
            else
                path="/";

            if ((files=get_dir(path+last)) && sizeof(files-=({".",".."})))
                ret+=map(files,lambda(({'x}),({#'+,path,'x})));
            else
                ret+=({path+last});
            }
        else
            ret+=({"/"+implode(com_path,"/")});
        }

    return ret;
}

#endif


// Nachfragen, falls das Objekt/V-Item nicht klar ist
private varargs void fehler_nachfragen(int nur_prompt)
{
    if (fehler_objekt)
    {
        if (fehler_objekt_auswahl)
        {
            if (!nur_prompt)
                write(wrap(
                    fehler_was+
                    ": Bitte gib entweder\n"
                    " - ein Objekt an  oder\n"+
                    sprintf(" - %-=70s\n",
                             "'1' für "+fehler_objekt_deinen(fehler_objekt)+
                             "  oder")+
                    " - <return>, falls Du diesen Raum meinst.\n"
                    "Abbruch mit 'q'."));
            input_to("get_fehler_objekt", INPUT_PROMPT,
        	"[<Objektangabe>,1,<return>,q,?] ");
        }
        else
            fehler_schreibe_text();
    }
    else
    {
        if (!nur_prompt)
            write(wrap(
                fehler_was+
                ": Bitte gib ein Objekt an oder "
                "<return>, falls Du diesen Raum meinst. "
                "Abbruch mit 'q'."));
        input_to("get_fehler_objekt", INPUT_PROMPT,
    	    "[<Objektangabe>,<return>,q,?] ");
    }
}


// Erklaerung zum Objekt-Nachfrage-Prompt
private void fehler_prompt_hilfe()
{
    write(wrap(
        "Wo soll "+dein(fehler_mwas, 0, FEHLER_WER)+" zugeordnet werden:\n"
        " - Einem Objekt (also alles was man so anschauen kann, "
        "wie Gegenstände,\n"
        "   Monster, Detailbeschreibungen), so gib dessen Bezeichnung ein, "
        "also\n"
        "   z.B. Ork, mein Schwert oder Fenster.\n"+
#if WIZ_SEARCHES_WIZ_OBJECTS
        ((wizp(FEHLER_WER)) ?
         "        Für Götter ist auch die Verwendung von Objektstrings wie\n"
         "        beim Zauberstab möglich:\n"
         "          /obj/fackel, /w/pars*/work*, Parsec:kuscheli\n" : "")+
#endif
        ((fehler_objekt_auswahl) ?
         sprintf(" - %-=70s\n",
                  capitalize(fehler_objekt_deinem(fehler_objekt))+
                  ", so gib eine '1' ein.") : "")+
        " - Dem Raum in dem Du Dich gerade befindest, dann drücke nur "
        "<return>.\n"
        " - Ist es ein allgemeiner Fehler und Du weißt nicht wohin damit,\n"
        "   dann gib 'mich' ein.\n"
        " - Willst Du lieber "+den(fehler_mwas)+" abbrechen, dann tippe 'q' "
        "oder '~q'."
       ));
    fehler_nachfragen(1);
}


// Antwort auf Nachfragen, falls das Objekt/V-Item nicht klar ist
void get_fehler_objekt(string str)
{
    mixed   parsed;
    string  tmp;

    if (!str || (str = space(str)) == "")
    {
        fehler_objekt = 0;
        fehler_schreibe_text();
    }
    else if (fehler_objekt_auswahl && str == "1")
        fehler_schreibe_text();
    else if (str == "~q")
        FEHLER_ABBRUCH;
    else if (str == "?")
        fehler_prompt_hilfe();
    else
    {
        parsed = parse_com(str);
        if (parsed[PARSE_RET_CODE] == PARSE_OK)
        {
            fehler_objekt = parsed[PARSE_OBS][0];
            fehler_schreibe_text();
        }
#if WIZ_SEARCHES_WIZ_OBJECTS
        else if (wizp(FEHLER_WER)  &&
                  (parsed = wiz_get_object(str)))
        {
            fehler_objekt = parsed;
            fehler_schreibe_text();
        }
#endif
        else
        {
            if ((tmp = lower_case(str)) == "q")
                FEHLER_ABBRUCH;
            else if (tmp == "hier" || tmp == "raum")
            {
                fehler_objekt = 0;
                fehler_schreibe_text();
            }
            else
            {
                write(wrap(capitalize(str)+
                             " nicht gefunden."));
                fehler_nachfragen();
            }
        }
    }
}


// Fehlertext schreiben (aehnlich zum Newsreader)
private void fehler_ed_fertig(string *text)
{
    if(!text)
    {
        FEHLER_ABBRUCH;
	return;
    }

    if (!sizeof(filter(text, (: strlen(space($1)) :))))
    {
        write(  "Du hast keinen Text eingegeben. "
                "Versuch's nochmal.\n");
        fehler_schreibe_text();
    }
    else
        fehler_absetzen(implode(text,"\n"), fehler_objekt);
}

private void fehler_schreibe_text()
{
    if(fehler_art == EDB_DETAIL)
    {
        detail_ed(FEHLER_WER);
    }
    else
    {
        if (this_object()->uses_webmud() &&
              !this_object()->query_client_option(CLIENT_WEBMUD_NO_EDIT))
        {
            if (wizp(FEHLER_WER) && objectp(fehler_objekt))
                write(wrap("Objekt: "+object_name(fehler_objekt)));

            write(wrap(
                Dein(fehler_mwas, 0, FEHLER_WER)+" für "+
                fehler_objekt_deinen(fehler_objekt, 1)+
                " wird über den WebMud-Editor nun eingegeben."));
        }
        FEHLER_WER->mini_ed(#'fehler_ed_fertig, 0, 0,
            ([
            MINI_ED_START_TEXT:
                wrap("Gib bitte nun "+deinen(fehler_mwas)+
                ((!fehler_objekt)?" zum Raum":
                 (fehler_objekt==FEHLER_WER)?" zu dir":
                 stringp(fehler_objekt)?sprintf(" zu %s", fehler_objekt):
                 (objectp(fehler_objekt) && wizp(FEHLER_WER))?sprintf(" zu %Q", fehler_objekt):
                 (auto_owner_search(fehler_objekt)==this_player())?(" zu "+deinem(fehler_objekt)):
                 (" zu "+ihrem(fehler_objekt)))+" ein. Mit '**' oder '.' "
                 "beenden, mit '~q' abbrechen."),
             MINI_ED_TITLE : Der(fehler_mwas),
            ]),
            ([
            MINI_ED_FORCE_WRAP: 1,
            ]), 
            sizeof(fehler_text) && explode(fehler_text,"\n"));
    }
}

// Detailtext schreiben:
private varargs void detail_ed(object who, string key, mixed text, string erg, mapping vitem)
{
#define DETAIL_ED(x, y, z, m, t) \
    who->mini_ed( \
    lambda( ({'text}), ({#'detail_ed, who, x, 'text, z, m}) ), \
    0, 0, \
    ([ MINI_ED_START_TEXT: wrap(y), \
       MINI_ED_PLAYER_INFO: "", MINI_ED_WIZ_INFO: "",  \
       MINI_ED_TITLE: t, ]),\
    ([ MINI_ED_WRAP_LEN:   -1 ]) )

#define DETAIL_PROMPT(x, y, p, z, m) \
    who->send_message_to(who, 0, 0, wrap(y) ); \
    input_to( lambda( ({'text}), ({#'detail_ed, who, x, 'text, z, m}) ), INPUT_PROMPT, p );


#define DETAIL_TEXT(x, y) \
    (!sizeof(y - ({""})) ? "" : \
    sprintf("%s%s", (x[<1]=='\n'?x:left(x, 12)), wrap(implode(y, "\n"))))

    // Spielerobjekt verschwunden...
    if(!playerp(who))
        return;

    mixed hilfstext;
    if(stringp(text))
    {
        text = trim(text);

        if(text == "~q")
            text = 0;

        else if(text == "**" || text == ".")
            text = ({});

        else
            text = ({text});
    }

    if(key && !text)
    {
        FEHLER_ABBRUCH;
        return;
    }

    switch(key)
    {
        case 0:
            who->send_message_to(who, 0, 0,
                wrap("Du beginnst, ein neues Detail zu schreiben.\n"
                     "Eigenschaften, die du nicht beschreiben möchtest, "
                     "überspringst du mit **. Abbruch mit ~q."));
            DETAIL_PROMPT("name",
                "\nGib nun den Namen des Details ein.\n",
                "Name: ",
                0, ([ ]) );
            break;

        case "name":
            text -= ({""});
            if (sizeof(text)) {
                vitem["name"] = text[0];
            }
            DETAIL_PROMPT("gender",
                "\nGib nun an, welches Geschlecht das Detail hat "
                "(maennlich, weiblich oder saechlich).",
                "Geschlecht: ",
                DETAIL_TEXT("Name:", text), vitem);
            break;

        case "gender":
            text -= ({""});
            if (sizeof(text)) {
                vitem["gender"] = text[0];
            }
            DETAIL_PROMPT("plural",
                "\nSteht das Detail im Plural? Das ist der Fall, "
                "wenn du gerade Türme, Berge, Strohhalme, Dünen "
                "oder ähnliche Pluralnamen beschreibst.\n"
                "Bitte antworte nur mit Ja oder Nein.",
                "Plural: ",
                erg+DETAIL_TEXT("Geschlecht:", text),vitem);
            break;

        case "plural":
            text -= ({""});
            if (sizeof(text)) {
                if (lower_case(space(text[0])) == "ja")
                    vitem["plural"] = 1;
            }
            DETAIL_PROMPT("id",
                "\nGib nun die IDs ein, mit denen das Detail "
                "angesprochen werden können soll.\n"
                "Beispiel: Ein Füllfederhalter könnte die IDs "
                "federhalter, halter, und füller haben und mit "
                "diesen Begriffen angesprochen werden.",
                "IDs: ",
                erg+DETAIL_TEXT("Plural:", text),vitem);
            break;

        case "id":
            text -= ({""});
            if (sizeof(text)) {
                hilfstext = regexplode(text[0],"[, ]") -({ "",","," "});
                if (sizeof(hilfstext)) {
                    vitem["id"] = hilfstext;
                }
            }
            DETAIL_PROMPT("adjektiv",
                "\nFalls das Detail ein oder mehrere Adjektive hat, "
                "kannst du diese hier angeben.\n"
                "Beispiele: groß, rund, schwarz, lang, flauschig, ...\n"
                "Mehrere Adjektive bitte durch Komma und Leerzeichen getrennt.",
                "Adjektiv: ",
                erg+DETAIL_TEXT("IDs:", text),vitem);
            break;

        case "adjektiv":
            text -= ({""});
            if (sizeof(text)) {
                hilfstext = regexplode(text[0],"[, ]") -({ "",","," "});
                if (sizeof(hilfstext)) {
                    vitem["adjektiv"] = hilfstext;
                }
            }
            DETAIL_ED("long",
                "\nGib nun die Beschreibung ein, die man beim Betrachten "
                "des Details erhält.\n"
                "Bitte durchgehend schreiben und nur bei Absätzen "
                "eine neue Zeile beginnen.",
                erg+DETAIL_TEXT("Adjektiv:", text),vitem,"Betrachte Detail");
            break;

        case "long":
            if (sizeof(text)) {
                vitem["long"] = implode(text,"\n");
            }
            DETAIL_ED("feel",
                "\nGib nun den Text ein, den man beim Befühlen "
                "des Details erhält.\n"
                "Bitte durchgehend schreiben und nur bei Absätzen "
                "eine neue Zeile beginnen.",
                erg+DETAIL_TEXT("\n--- Beschreibung: ---\n", text),vitem,
                "Fühle Detail");
            break;

        case "feel":
            if (sizeof(text)) {
                vitem["feel"] = implode(text,"\n");
            }
            DETAIL_ED("noise",
                "\nGib nun den Text ein, den man beim Lauschen "
                "an dem Detail erhält.\n"
                "Bitte durchgehend schreiben und nur bei Absätzen "
                "eine neue Zeile beginnen.",
                erg+DETAIL_TEXT("\n--- Gefühl: ---\n", text),vitem,
                "Lausche am Detail");
            break;

        case "noise":
            if (sizeof(text)) {
                vitem["noise"] = implode(text,"\n");
            }
            DETAIL_ED("smell",
                "\nGib nun den Text ein, den man beim Riechen "
                "an dem Detail erhält.\n"
                "Bitte durchgehend schreiben und nur bei Absätzen "
                "eine neue Zeile beginnen.",
                erg+DETAIL_TEXT("\n--- Geräusch: ---\n", text),vitem,
                "Rieche am Detail");
            break;

        case "smell":
            if (sizeof(text)) {
                vitem["smell"] = implode(text,"\n");
            }
            DETAIL_ED("kommentar",
                "\nFüge deinem Detail noch einen Kommentar hinzu.\n"
                "Hier kannst du beispielsweise erwähnen, welche "
                "Hinweise es auf dein Detail gibt oder welche weiteren "
                "Eigenschaften es haben soll.",
                erg+DETAIL_TEXT("\n--- Geruch: ---\n", text),vitem,
                "Kommentar zum Detail");
            break;

        case "kommentar":
            erg += DETAIL_TEXT("\n--- Kommentar: ---\n", text);
            erg = trim(erg, TRIM_BOTH, " \n");

            if(!strlen(erg))
            {
                FEHLER_ABBRUCH;
                break;
            }
#define VITEM_KEYS1 ({ "name","gender","plural","id","adjektiv" })
#define VITEM_KEYS2 ({ "long","feel","noise","smell" })
            hilfstext = filter(VITEM_KEYS1, (: $2[$1] != 0 :), vitem);
            hilfstext = map(hilfstext, 
     (: wrap_say("    \""+$1+"\": ",mixed2str($2[$1])+",",72)[..<2] :), vitem);
            erg += "\n  add_v_item( ([\n"+implode(hilfstext, "\n");
            hilfstext = filter(VITEM_KEYS2, (: $2[$1] != 0 :), vitem);
            hilfstext = map(hilfstext, 
     (: wrap_say("    \""+$1+"\": ",mixed2str($2[$1])+",",72)[..<2] :), vitem);
            hilfstext = map(map(explode(implode(hilfstext,"\n"),"\n"),
                (: ($1[<2..<1]=="\",")? $1 : $1+" \"" :)),
                (: (space($1)[0..0] != "\"")?"        \""+space($1):$1 :) );
            erg += "\n"+implode(hilfstext, "\n")+"\n  ]));";
            fehler_absetzen(erg, fehler_objekt);
            break;

        default:
            who->send_message_to(who, 0, 0,
                wrap("Oh je, da ist wohl was schiefgelaufen, bitte "
                     "setze einen Fehler zu dir ab."));
    }
}


void feedback_an (string s);
void feedback_nb (string s);
void feedback_out ();

void feedbacktest ()
{
    mixed *m;
    if ((m = ERROR_DB->get_feedback(this_object()->query_real_name()))
        && (sizeof (m)))
        this_object()->send_message_to(this_object(),MT_NOTIFY,MA_UNKNOWN,
             "Für Dich liegt Fehlerfeedback vor.\n"
             "Mit dem Befehl \"feedback\" kannst Du es lesen.\n\n");
}

int feedback ()
{
    if (query_input_pending (this_object()) || query_editing (this_object()))
        return notify_fail("Jetzt geht das nicht. Eines nach dem anderen.\n");
    feedback = ERROR_DB->get_feedback(this_object()->query_real_name());
    if (!feedback || !sizeof (feedback))
        return notify_fail ("Kein Fehlerfeedback vorhanden.\n");
    feedback_nr = 0;
    write ("Feedback auf Deine Fehlermeldungen:");
    feedback_out();
    return 1;
}

void feedback_out()
{
    write ("\nDeine Fehlermeldung:\n    vom: "
        +timestr(feedback[feedback_nr]["date_err"])
        +(feedback[feedback_nr]["objekt"]
          ?"\n    zum "+feedback[feedback_nr]["objekt"]:"")
        +":\n\n"
        +wrap(feedback[feedback_nr]["text"],79)+"\n");

    if (feedback[feedback_nr]["typ"] == "r")
        write ("    Rückfrage vom "+timestr(feedback[feedback_nr]["date_fb"])+":\n");
    else
        write ("    Antwort vom "+timestr(feedback[feedback_nr]["date_fb"])+":\n");

    write (wrap(feedback[feedback_nr]["fb"],0,8));
    if (feedback[feedback_nr]["typ"] == "r")
        feedback_an ("");
    else {
        ERROR_DB->del_feedback(this_object()->query_real_name(),feedback_nr);
        feedback = arr_delete (feedback,feedback_nr);
        if (feedback_nr == sizeof (feedback))
            write ("Alle Feedbacks angeschaut.\n");
        else feedback_nb("");
    }
}

void feedback_nb (string s)
{
    if ((s == "q") || (s == "Q"))
        write ("Dann eben ein anderes mal.\n");
    else if ((s == "n") || (s == "N") || (s == "+")) {
        feedback_out();
    }
    else {
        write ("Möchtest Du: n) nächstes Feedback sehen, q) abbrechen?\n");
        input_to ("feedback_nb", INPUT_PROMPT, "Feedback: ");
    }
}

void feedback_an (string s)
{
    if ((s == "a") || (s == "A")) {
        write ("Gib bitte Deine Antwort ein, '**' oder '.' wenn Du fertig bist,\n"
            "'~q' zum Abbrechen, '~r' um den bisher eingegebenen Text anzuzeigen:\n");
        fehler_text = "";
        input_to ("feedback_reply", INPUT_PROMPT, "Antwort: ");

    }
    else if ((s == "n") || (s == "N") || (s == "+")) {
        feedback_nr++;
        if (feedback_nr == sizeof (feedback))
            write ("Alle Feedbacks angeschaut.\n");
        else
            feedback_out();
    }
    else if ((s == "q") || (s == "Q"))
        write ("Dann eben ein anderes mal.\n");
    else {
        write ("Möchtest Du: a) antworten, n) nächstes Feedback, q) abbrechen?\n");

        input_to ("feedback_an", INPUT_PROMPT, "Feedback: ");
    }
}

// eine Zeile des Fehlertextes schreiben (aehnlich zum Newsreader)
void feedback_reply (string s)
{
    if (s == "~q") {
        write ("Du brichst ab.\n");
        return;
    }
    if (s == "." || s == "**") {
        if (space(fehler_text)[0..<2] == "") {
            write("Du hast keinen Text eingegeben. Versuch's nochmal.\n");
            
            input_to ("feedback_reply", INPUT_PROMPT, "Antwort: ");
        }
        else
        {
#if 0
            ERROR_DB->add_comment (feedback[feedback_nr]["nr"],
                ({this_object()->query_real_name(),time(),
                "Rückantwort von Spieler:\n"+fehler_text[0..<2]}));
#else
            ERROR_DB->add_history(feedback[feedback_nr]["nr"], this_object(),
                EDB_EHT_ANSWER, fehler_text[0..<2]);
#endif
            ERROR_DB->del_feedback(this_object()->query_real_name(),feedback_nr);
            feedback = arr_delete (feedback,feedback_nr);
            write ("Vielen Dank für Deine Antwort.\n");
            if (feedback_nr == sizeof (feedback))
                write ("Keine weiteren Feedback-Einträge vorhanden.\n");
            else
                feedback_out();
        }
    }
    else if (s == "~r") {
        write ("Deine bisherige Antwort:\n"+wrap(fehler_text));
        input_to ("feedback_reply", INPUT_PROMPT, "Antwort: ");
    } else {
        if (s && strlen (s))
            fehler_text += wrap (s);

        input_to ("feedback_reply", INPUT_PROMPT, "Antwort: ");
    }
}

