// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /i/tools/build_table.c
// Description: Erstellen einer einfachen Tabelle aus einem zweidimensionalen
//              Array
// Author:      Sissi (04.05.2001)

#pragma save_types

/*
FUNKTION: build_table
DEKLARATION: varargs string* build_table(mixed s, string *trenn, string *ausr)
BESCHREIBUNG:
Mit build_table kann auf einfache Weise aus einem zweidimensionalen Array
aus Strings eine Tabelle erzeugt werden.
Eine Angabe einer maximalen Tabellenbreite oder maximaler Spaltenbreite
ist nicht moeglich.
Enthaelt ein oder mehrere Eintraege in einer Zeile ein \n so fuehrt dies
direkt darunter zu einer neuen Zeile mit diesen Eintraegen.

BEISPIEL:
    this_player()->more (
        build_table ( ({ ({"Ueberschrift 1","-","text 1","text 2"}),
                         ({"Ueberschrift 2","-","text 3","text 4"}),
                         ({"Ueberschrift 3","-","text 5","text 6"}) }),
                      ({"|","~"}),
                      ({"r","l","z"}) ) );
    erzeugt:
        Ueberschrift 1 | Ueberschrift 2 | Ueberschrift 3
        ~~~~~~~~~~~~~~~|~~~~~~~~~~~~~~~~|~~~~~~~~~~~~~~~
                text 1 | text 3         |     text 5
                text 2 | text 4         |     text 6

PARAMETER:

    S:       ein zweidimensionales String - Array, jedes Element dieses Arrays
             entspricht einer Spalte der Tabelle; jede Spalte besteht aus einem
             Stringarray mit den Elementen dieser Spalte.
             Besonderheit: Ist ein Element ein "-" oder identisch mit dem als
             horizontale Linie festgelegten Muster (siehe unten), so wird ein
             horizontaler Strich gezogen.

    Trenner: ein Stringarray, mit dem man besondere Muster fuer einzelne
             Tabellenelemente festlegen kann. Diese Muster koennen alle laenger
             als nur ein Zeichen sein. Nur die Elemente, die man wirklich
             aendern will, muessen angegeben werden.
             Der ganze Parameter kann weggelassen werden.
             Die Reihenfolge der Elemente dieses Arrays:
              - vertikaler Trenner: standardmaessig ein |
              - horizontale Linie:  standardmaessig ein -
              - kreuz, wenn vertikale und horizontale Linie sich treffen,
                standardmaessig ein |
              - linker Rand der Tabelle
              - rechter Rand der Tabelle
              - oberer Rand der Tabelle
              - unterer Rand der Tabelle
              - linke obere Ecke der Tabelle
              - rechte obere Ecke der Tabelle
              - linke untere Ecke der Tabelle
              - rechte untere Ecke der Tabelle

    Ausrichtung:
	     ein Stringarray, in dem fuer jede Spalte eine Ausrichtung
             festgelegt werden kann:
             l oder -     fuer linksbuendig, dies ist die Standardeinstellung
             r oder +     fuer rechtsbuendig, 
             z, c oder |  fuer zentriert,
             Dieser Parameter kann weggelassen werden.
  
ALTERNATIVE:
In /p/Misc/i/table existiert eine deutlich maechtigere Alternative, die
allerdings dafuer auch um ein Vielfaches groesser und aufwendiger zu
benutzen ist.
VERWEISE: /p/Misc/i/table
GRUPPEN: tool, text
*/


varargs string* build_table (mixed s, string *trenner, string *format)
{
    int i, j, k, max, nlflag, trennstelle;
    string* res, pat, temp, vert_trenner, hor_line, cross,
        rand_links, rand_rechts, rand_oben, rand_unten,
        links_oben, rechts_oben, links_unten, rechts_unten;

    vert_trenner = "|";
    hor_line = "-";
    cross = "|";
    if (trenner && (sizeof(trenner) >=  1) && trenner[0]) vert_trenner = trenner[0];
    if (trenner && (sizeof(trenner) >=  2) && trenner[1]) hor_line = trenner[1];
    if (hor_line == "") hor_line = " ";
    if (trenner && (sizeof(trenner) >=  3) && trenner[2]) cross = trenner[2];
    if (trenner && (sizeof(trenner) >=  4)) rand_links = trenner[3];
    if (trenner && (sizeof(trenner) >=  5)) rand_rechts = trenner[4];
    if (trenner && (sizeof(trenner) >=  6)) rand_oben = trenner[5];
    if (trenner && (sizeof(trenner) >=  7)) rand_unten = trenner[6];
    if (trenner && (sizeof(trenner) >=  8)) links_oben = trenner[7];
    if (trenner && (sizeof(trenner) >=  9)) rechts_oben = trenner[8];
    if (trenner && (sizeof(trenner) >= 10)) links_unten = trenner[9];
    if (trenner && (sizeof(trenner) >= 11)) rechts_unten = trenner[10];
    for (i = 0; i < sizeof (s[0]); i++) {
        for (j = 0, nlflag = 0; j < sizeof (s); j++)
            if (sizeof (s[j]) > i) {
                if (strstr (s[j][i],"\n") != -1)
                    nlflag = 1;
            }
        if (nlflag)
           for (j = 0; j < sizeof (s); j++)
               if (sizeof (s[j]) > i) {
                   trennstelle = strstr (s[j][i],"\n");
                   if (trennstelle == -1) trennstelle = strlen (s[j][i]);
                   s[j][i..i] = ({s[j][i][0..trennstelle-1],s[j][i][trennstelle+1..]});
               }
    }
    for (j = 0, res = ({}); j < sizeof (s); j++) {
        for (i = max = 0; i < sizeof (s[j]); i++)
            if ((k = strlen (s[j][i])) > max) max = k;
        for (i = 0; i < sizeof (s[j]); i++) {
            if (s[j][i] == "-")
                pat = s[j][i] = hor_line;
            else if (s[j][i] == hor_line)
                pat = hor_line;
            else pat = " ";
            if (format && (sizeof (format) > j))
                switch (format[j]) {
                    case "-": case "l":
                        temp = left (s[j][i],max,pat);
                        break;
                    case "+": case "r":
                        temp = right (s[j][i],max,pat);
                        break;
                    case "|": case "z": case "c":
                        temp = center (s[j][i],max,pat);
                        break;
                }
                else
                    temp = left (s[j][i],max,pat);
            if (sizeof (res) <= i)
                res += ({temp});
            else
                res[i] += temp;
            if (j < sizeof (s)-1)
                if (pat == hor_line)
                    res[i] += hor_line+cross+hor_line;
                else
                    res[i] += " "+vert_trenner+" ";
        }
    }
    if (rand_links)
        for (i = 0; i < sizeof (res); i++)
            res[i] = rand_links + " "+res[i];
    if (rand_rechts)
        for (i = 0; i < sizeof (res); i++)
            res[i] = res[i] + " "+rand_rechts;
    if (rand_oben) {
        res = ({left (links_oben || "",
                   strlen (res[0])-(rechts_oben?strlen (rechts_oben):0),
                   rand_oben) + (rechts_oben ? rechts_oben : "")})
              + res;
    }
    if (rand_unten) {
        res += ({left (links_unten || "",
                   strlen (res[0])-(rechts_unten?strlen (rechts_unten):0),
                   rand_unten) + (rechts_unten ? rechts_unten : "")});
    }
    return res;
}
