// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/tools/check_save
// Description:	Prueft beliebige Datentypen auf Speicherbarkeit.
// Author:	Myonara nach einer Vorlage von mixed2str

#pragma strict_types
#pragma save_types

#include <lpctypes.h>

/*
FUNKTION: check_save
DEKLARATION: int check_save(mixed arr)
BESCHREIBUNG:
Prueft beliebigen Datentyp auf Speicherbarkeit.
GRUPPEN: grundlegendes
VERWEISE: mixed2str, save_value
*/
int check_save(mixed arr)
{
    switch(typeof(arr))
    {
      default:
      case T_INVALID:
	return 0; // Unbekannt,Invalid => nicht speicherbar
      case T_LVALUE:
	return 1; // speicherbar??
      case T_STRING:
      case T_NUMBER:
      case T_FLOAT:
	return 1;
      case T_POINTER:
      {
	int i;
	if (sizeof(arr) <= 0)
	    return 1; // leeres Array ist speicherbar.
	for (i = 0; i < sizeof(arr) - 1; i++)
	{
	    if (!check_save(arr))
	        return 0; // erster Fehlschlag => nicht speicherbar.
	}
	return 1; // alle ok => speicherbar
      }
      case T_OBJECT:
	return 0; // noe.
      case T_MAPPING:
      {
	mixed *ind;
	int a, end, j;
	if (sizeof(arr) <= 0)
	    return 1; // Leeres Mapping ist speicherbar.
	ind = m_indices(arr);
	end = get_type_info(arr,1)-1;
	for(j = 0; j < sizeof(ind); j++)
	{
	    if (!check_save(ind[j]))
	        return 0; // Ungueltiger Index => nicht speicherbar.
	    if (end >= 0)
	    {
		for (a = 0; a < end; a++)
		{
		    if (!check_save(arr[ind[j],a]))
		        return 0; // erster Fehlschlag bei den Werten.
		}
	    }
	}
	return 1;
      }
      case T_CLOSURE:
	return 0; // Closures nicht speicherbar.
      case T_SYMBOL:
	return 1; // ??? sprintf("SYMBOL(%O)",arr);
      case T_QUOTED_ARRAY:
	return 1; // ??? sprintf("QUOTED-ARRAY(%O)",arr);
      case T_STRUCT:
        for(int i = 0; i < sizeof(arr); i++)
        {
            if (!check_save(arr->(i)))
                return 0;
        }
        return 1;
    }
}
