// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/i/tools/mail.c
// Description:	Funktionen zum Bearbeiten des Postarrays, Aliase
// Author:	Garthan(28.03.95)

#pragma save_types

#include <apps.h>
#include <config.h>
#include <news.h>

#define FOLDER_NAME_MIN 03
#define FOLDER_NAME_MAX 25

private string test_folder_name(string folder)
{
   if(strlen(folder) < FOLDER_NAME_MIN || strlen(folder) > FOLDER_NAME_MAX)
      return "Verzeichnisname muss Länge zwischen "+FOLDER_NAME_MIN+" und "+
	     FOLDER_NAME_MAX+" haben.\n";
   if(member(folder, ' ') >= 0)
      return "Der Verzeichnisname darf keine Leerzeichen enthalten.\n";
   if(sscanf(folder, "%~d"))
      return "Der Verzeichnisname darf nicht numerisch beginnen.\n";
}

static string add_folder(mapping post, string folder)
{
   string err;

   if(!mappingp(post))
      return "Ungültiges Postformat\n";
   if(!stringp(folder))
      return "Kein Name für das neue Verzeichnis angegeben.\n";
   if(err = test_folder_name(folder))
      return err;
   if(member(post, folder))
      return "Ein Verzeichnis mit diesem Namen gibt es bereits.\n";
   post[folder] = ({});
   return 0;
}

static string delete_folder(mapping post, string folder)
{
   if(!mappingp(post))
      return "Ungültiges Postformat\n";
   if(!stringp(folder))
      return "Kein Name für das zu löschende Verzeichnis angegeben.\n";
   if(!member(post, folder))
      return "Diese Verzeichnis gibt es nicht!\n";
   if(sizeof(post[folder]))
      return "Das angegebene Verzeichnis enthält noch Mails.\n";
   m_delete(post, folder);
   return 0;
}

static string remove_folder(mapping post, string folder)
{
   if(!mappingp(post))
      return "Ungültiges Postformat\n";
   if(!stringp(folder))
      return "Kein Name für das zu löschende Verzeichnis angegeben.\n";
   if(!member(post, folder))
      return "Diese Verzeichnis gibt es nicht!\n";
   m_delete(post, folder);
   return 0;
}

static string move_mail(mapping post, int von, int bis,
     		        string from_folder, string to_folder)
{
   if(!mappingp(post))
      return "Ungültiges Postformat\n";
   if(!stringp(from_folder))
      return "Kein Name für das Quellverzeichnis angegeben.\n";
   if(!stringp(to_folder))
      return "Kein Name für das Zielverzeichnis angegeben.\n";
   if(!member(post, from_folder))
      return "Quellverzeichnis existiert nicht.\n";
   if(!member(post, to_folder))
      return "Zielverzeichnis existiert nicht.\n";
   if(!intp(von) || von < 0 || von >= sizeof(post[from_folder]) ||
      !intp(bis) || bis < 0 || bis >= sizeof(post[from_folder]) || von > bis)
      return "Nachricht[en] existieren nicht im Quellverzeichnis.\n";
   if(!post[to_folder])
      post[to_folder] = ({});
   post[to_folder] += post[from_folder][von..bis];
   post[from_folder][von..bis] = ({});
   return 0;
}

static string rename_folder(mapping post, string from_folder, string to_folder)
{
   string err;
   if(!mappingp(post))
      return "Ungültiges Postformat\n";
   if(!stringp(from_folder))
      return "Kein Name für das Quellverzeichnis angegeben.\n";
   if(!stringp(to_folder))
      return "Kein Name für das Zielverzeichnis angegeben.\n";
   if(!member(post, from_folder))
      return "Quellverzeichnis existiert nicht.\n";
   if(member(post, to_folder))
      return "Zielverzeichnis existiert bereits.\n";
   if(err = test_folder_name(to_folder))
      return err;
   post[to_folder] = post[from_folder];
   m_delete(post, from_folder);
   return 0;
}


// --------------------- Aliase --------------------------------------------

static mapping read_alias(string aliasfile)
{
   mapping aliases;
   string file, *lines, alias, expand;
   int i;

   aliases = m_allocate(60,2);
   if(file = read_file(aliasfile))
      for(i = sizeof(lines = regexp(explode(file,"\n") - ({""}),"^[^#]")); i--;)
         if(sscanf(lines[i], "%s:%s", alias, expand) == 2)
            if(member(aliases, alias))
               aliases[alias] += map(explode(expand,",")-({""}),#'trim);
	    else
	    {
	       aliases[alias] = map(explode(expand,",")-({""}),#'trim);
	       aliases[alias,1] = 1;
	    }
	 else if(sscanf(lines[i], "%s=%s", alias, expand) == 2)
            if(member(aliases, alias))
               aliases[alias] += map(explode(expand,",")-({""}),#'trim);
	    else
	       aliases[alias] = map(explode(expand,",")-({""}),#'trim);

   return aliases;
}

// ----

private string *lords_of(string domain)
{
    int i;
    string *domains, *ret;

    if(domain && domain != "")
       ret = (DOMAIN_INFOS->query_domain_lords(domain) || ({}))+
             (DOMAIN_INFOS->query_domain_helfer(domain) || ({}));
    else
       for(ret = ({}),i = sizeof(domains = DOMAIN_INFOS->query_domains());
           i--;)
          ret += lords_of(domains[i]);
   return ret;
}

private string *expand_function(string arg)
{
   string fun, parm, *ret, *sr;
   ret = ({});
   if(sscanf(arg, "$%s(%s)", fun, parm) == 2)
   {
      switch(fun)
      {
#        ifdef ADMINS
         case "mudadm":
	       ret += ADMINS;
            break;
#        endif
#	 ifdef VORSTAND
	 case "vorstand":
		ret += VORSTAND;
	    break;
#	 endif
         case "auth":
            ret += FILED->query_auth(parm);
            break;
	 case "coders":
	    ret += filter(FILED->query_coders(parm), #'player_exists);
            break;
         case "lords":
            ret += lords_of(parm);
            break;
         case "members":
            ret += (DOMAIN_INFOS->query_domain_members(parm) || ({}));
            break;
         case "news":
            ret += NEWSD->query_owners()["/"] || ({});
            break;
         case "spielerrat":
            if ((sr = SPIELERRAT->query_spielerrat()) && sizeof (sr))
                ret += sr;
            break;
#	 ifdef UNItopia
         case "freunde":
         {
            object schnuller;
            string *sf;

	    if (this_player() && (schnuller=present("schnuller",this_player()))
	        && (sf=schnuller->query_existing_friends()) && pointerp(sf)
	        && sizeof(sf))
	        ret += sf;
	    break;
	}
#	 endif
      }
   }
   return ret;
}

static string *recurse_aliases(mixed arg, mapping aliases,
                               string owner, int depth)
{
   string *expand, *ret;
   int i;

   ret = ({});
   if(stringp(arg))
      arg = map(explode(lower_case(arg), ","), #'trim)-({""});
   if(pointerp(arg))
      for(i = 0; i < sizeof(arg); i++)
         if((expand = aliases[lower_case(arg[i])]) && depth < 20)
            ret += recurse_aliases(expand - ({owner}), aliases, owner, depth+1);
         else if(arg[i][0..0] == "$")
            ret += expand_function(arg[i]) - ({owner});
         else
            ret += ({ lower_case(arg[i]) });
   return ret;
}

// --------------------- Utils --------------------------------------------

static string klist(string *list)
{
    string ret;

    ret = "";
    if (sizeof(list) > 1)
	ret += implode(list[0..<2], ", ") + " und ";
    return ret + list[<1];
}
