// ----------------------------------------------------------------
// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /i/tools/soul_hlp.c
// Description: Hilfsfunktionen fuer die Seele
// Created by : Monty 11.11.94 im Zug der Fileverkleinerung von soul.c
//  Die folgenden Seele-Funktionen sind nach hier gewandert:
//  Say         gewrapptes say()
//  Write       gewrapptes write()
//  plus_adv    aktion mit adverb zusammenbasteln *geaendert*
//  soul_parser die richtigen Objekte ausfiltern *vereinfacht*
//  adv_part    aktion mit adverb und oder partner zusammenbasteln
// Modified by:
//  Monty (18.07.95):   in add_str() mit Brachialgewalt die
//                      Probleme in plus_adv() mit den Spaces geloest...
//  Monty (16.02.96):   in plus_part den Aufruf von notify_soul
//                      eingebaut, Doku zu notify_soul eingebaut.
//  Monty (26.04 96):   Tuning an adv_part, optionales Satzzeichen.
//  Zorro (16.08 96):   soul_command() eingebaut
//  Sissi (12.11.96):   lebende v-items ermoeglicht, Tell_object
//  Zorro (30.07.97):   Kommandos gehen auch fuer Geister
//  Parsec (23.11.98):
//     - bei "living"-V-Items wurde "soul"-Eintrag nur ausgegeben wenn auch
//       nach Lebewesen geparsed wurde -> Test jetzt auch in adv_part.
//     - Hilfsfunktionen  query_innenraum(raum), query_im_wasser(raum)
//     - adv_part: unsichtbare Partner bekommen 'dich' statt
//       'jemamnd' (ART_VIS)
//     - soul_parser nimmt jetzt auch eine optionale Praepostiton als
//       3. Argument
//     - Nichtlebende Objekte koennen jetzt auch von Seele-Befehlen die ein
//       Lebewesen als Partner verlangen gesselt werden.
//       Dazu muessen sie auf  allowed("seele")  mit 1 antworten.
//  Parsec (11.12.98):  Jetzt auch V-Items an V-Items beseelbar.
//                      (Probleme mit ["environment"] weg)
//  Parsec (24.09.99):  feel() hier her umgezogen, static owner
//  Parsec (14.10.99):  Fehlermeldung wenn exec_command()-Id oder
//                      name# nicht gefunden
//  Parsec (04.01.00):  parse_text() gibt Fehler bei nicht gefunden
//                      exec_command-Ids.
//                      parse_text() "auf|in|von|.. "+ALLES verwirrt
//                      parse_text() nicht mehr.

#define ALTLASTEN_RAUS        0

#define EXEC_COMMAND_ID_REGEXP   " *##[0-9][0-9]*#[0-9][0-9]*\(\|$\)"

#pragma save_types

#define ENV(t)          environment(t)
#define CAP(t)          capitalize(t)
#define DU      0
#define ER      1
#if 0
#   define GHOST(x)    if (x->query_ghost()) return 0
#else
#   define GHOST(x)    do{}while(0)
#endif
#define FEEL(x)         this_object()->feel(x)

#include <deklin.h>
#include <message.h>
#include <more.h>
#include <parse_com.h>
#include <stats.h>
#include <soul.h>


private static object owner;


#if 1
/* prototypes */
void msg_notify(string str);
void msg_write(int msg_type, int msg_action, string str);
varargs void msg_say(int msg_type, int msg_action, string str, mixed excludes);
void msg_to(mixed who, int msg_type, int msg_action, string str);
varargs void msg_soul_action(int msg_action,
                    string str_owner, string str_other,
                    int msg_type_owner, int msg_type_other);
private varargs void msg_soul_action_partner(mixed partner, int msg_action,
                    string str_owner, string str_partner, string str_other,
                    int msg_type_owner, int msg_type_partner, int msg_type_other);
varargs int forbidden_msg_soul_action_partner_notify(
    mixed partner, string aktion, string adverb, int align, int flags,
    string str_owner, string str_partner, string str_other,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other);
varargs int forbidden_msg_soul_action_notify(
    string aktion, string adverb, int align, int flags,
    string str_owner, string str_other,
    int msg_action,
    int msg_type_owner, int msg_type_other);
string add_space(mixed str);
// int query_innenraum(object raum);
// int query_im_wasser(object raum);
string conv_akt(string aktion, int flag);
varargs mixed *soul_parser(string str, int flags, string preposition);
varargs int soul_plus_adv(
    string str, string aktion, int align, int flags,
    int msg_action, int msg_type_owner, int msg_type_other,
    mixed fill, string fail,
    string default_adv, int reflexiv);
varargs int soul_adv_part(
    mixed res, string aktion,
    int align, int flags, int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other,
    mixed fill1, mixed fill,
    int fall, string fail, string default_adv, string dot);
#endif
private int soul_forbidden(
    mixed partner, string aktion, string adverb, int align, int flags,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other);
private void soul_notify(
    mixed partner, string aktion, string adverb, int align, int falgs,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other);
/* end prototypes */


// Nicht in der Seele definiert
/*


/*
FUNKTION: notify_seele
DEKLARATION: void notify_seele(object wer, mixed wen, string what, string adverb, int align, int flags, int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
BESCHREIBUNG:
Die Seele ruft jedesmal, NACHDEM ein Seele-Befehl ausgefuehrt wurde,
in allen Lebewesen im Raum und im Raum selbst notify("seele", ...) auf.
Ist ein Objekt bei einem dieser Lebewesen oder dem Raum als Controller
fuer "notify_seele" angemeldet
 (lebewesen_oder_raum->add_controller("notify_seele", object);),
so wird dann in diesem Objekt die Funktion
notify_seele(wer, wen, what, adverb, align, flags, msg_typ_wer, msg_typ_wen,
msg_typ_andere) aufgerufen.

Damit kann z.B. ein Monster erkennen, dass gerade ein Spieler mit der Seele
auf ihn eingewirkt hat.
Wenn man eine Seele-Aktion vorher behandeln oder gar verhindern will, verwendet
man forbidden_seele()!
Achtung: Die Kommunikationsbefehle der Seele (antworte, frage, ...) verwenden
         statt notify_seele, notify_comm.

Die Parameter bedeuten:
    wer     Das ist der, der "geseelt" hat (oder geknuddelt oder so).
    wen     Der "geseelte" (z.B. der Geknuddelte, ist 0 falls kein
            Partner verwendet wurde).
    what    Das ist (in etwa) der Seele-Befehl, der benutzt wurde. Wenn
            auf spezielle Befehle reagiert werden soll, muss man eben
            ausprobieren, was die returnen.
    adverb  Das ist das benutzte Adverb, oder eben 0, wenn keins verwendet
            wurde.
    align   Ist Seelekommando eher neutral (NEUTRAL == 0), mies (MIES == -1)
            oder nett (NETT == 1) gemeint (Konstanten aus soul.h).
    flags   Andere Eigenschaften der Seele-Aktion. Mit bitweisem-&
            zu testen. Momentan implementierte Eigenschaften:
            flags & PSEUDO_MOVE   Bewegt man sich bei der Aktion
                                  (z.B. waelzen, tanzen, huepfen, ...)
    msg_typ_wer     Welche Sinne des Seelenden werden durch die Aktion
                    angesprochen (z.B. knuddle: msg_typ_wer & MT_FEEL - der
                    Seelende spuert die Aktion). Konstanten aus message.h
    msg_typ_wen     Welche Sinne des Geseelten werden angesprochen.
    msg_typ_andere  Welche Sinne der Beobachter werden angesprochen.

BEISPIEL 1:  zg /p/Doc/Lehre/Demo/room/soul-demo
BEISPIEL 2:
In einem Monster:
void create()
{
   ...
   add_controller("notify_seele", this_object());
}

void notify_seele(object wer, mixed wen, string what, string adverb,
                   int align, int flags,
                   int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
{
    if (wer != this_object()  &&  wen == this_object())
    {
        do_command("sage Wieso "+conv_akt(what,0)+" Du mich?");
        if (align > 0)
            do_command("ich freut sich weil das so nett war.");
        if (msg_typ_wen & MT_FEEL)
            do_command("sage Bitte beruehre mich nicht. Das mag ich nicht.");
    }
}

Dann meldet sich das Monster immer nach einem Seele-Kommando ;-).
VERWEISE: conv_akt, message, wrap_say, allowed_seele, forbidden_seele,
          set_notify_soul_ob, query_notify_soul_ob_file,
          notify_comm, forbidden_comm, add_controller
GRUPPEN: seele
*/

/*
FUNKTION: allowed_seele
DEKLARATION: int allowed_seele()
BESCHREIBUNG:
Verlangt ein Seele-Befehl ein Lebewesen als Partner und es wird der
Seele-Befehl mit einem toten Objekt aufgerufen, so wuerde normal die
Meldung "Das geht nur mit Lebewesen." ausgegeben werden.
Vor der Ausgabe diese Meldung wird aber nochmals mit
totes_objekt->allowed("seele") gefragt, ob der Seele-Befehl doch
ausgefuehrt werden darf.

BEISPIEL:
In totem Objekt (z.B. Teddy, der auch gewuergt und geknuddelt
werden koennen soll):
void create()
{
   ...
   add_controller("allowed_seele", this_object());
}

int allowed_seele()
{
    return 1;
}

VERWEISE: notify_seele, forbidden_seele, notify_comm, forbidden_comm, add_controller
GRUPPEN: seele
*/

/*
FUNKTION: forbidden_seele
DEKLARATION: int forbidden_seele(object wer, mixed wen, string what, string adverb, int align, int flags, int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
BESCHREIBUNG:
Die Seele ruft jedesmal, BEVOR ein Seele-Befehl ausgefuehrt wird,
in allen Lebewesen im Raum und im Raum selbst forbidden("seele", ...) auf.
Ist ein Objekt bei einem dieser Lebewesen oder dem Raum als Controller
fuer "forbidden_seele" angemeldet
 (lebewesen_oder_raum->add_controller("forbidden_seele", object);),
so wird dann in diesem Objekt die Funktion
forbidden_seele(wer, wen, what, adverb, align, flags, msg_typ_wer,
msg_typ_wen, msg_typ_andere) aufgerufen.
Liefert  forbidden_seele  als Ergebnis 1, so wird die Seele-Aktion damit
verhindert. forbidden_seele  sollte sich dabei um die Augabe einer passenden
Meldung als Begrundung kuemmern.

Damit kann man z.B. bei einem Monster verhindern, dass es gestreichelt
wird oder man kann aus dem Streicheln ganz was anderes machen.
Wenn eine Reaktion nach der Aktion erwuenscht ist, nimmt man notify_seele()!
Achtung: Die Kommunikationsbefehle der Seele (antworte, frage, ...) verwenden
         statt forbidden_seele, forbidden_comm.

Bedeutung der Parameter: siehe notify_seele().

BEISPIEL:
In einem Monster (ein grosser Riese) deklariert:
void create()
{
   ...
   add_controller("forbidden_seele", this_object());
}

int forbidden_seele(object wer, mixed wen, string what, string adverb,
                     int align, int flags,
                     int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
{
    if (wer == this_object()  ||  wen != this_object())
        return 0;

    switch (what) {
        // Aktion abaendern
        case "tret":
            wer->send_message(MT_LOOK,MA_EMOTE,
		wrap(Der(wer) + " versucht gegen das Knie des Riesen zu "
                "treten, aber da er viel zu gross fuer " + ihn(wer) +
                " ist, tritt " + er(wer)+" ihm lieber auf die Zehen."),
                wrap("Du versuchst gegen das Knie des Riesen zu treten, "
                "aber da er viel zu gross fuer dich ist, trittst "
                "Du ihm lieber auf die Zehen."),wer);
	    send_message_to(this_object(),MT_LOOK,MA_EMOTE,
		    Der(wer) + " tritt dir auf die Zehen.\n");
            // Koennte man sich fuer Monster auch sparen ;-)
            return 1;

        // Aktion verbieten
        case "stups":
	    send_message_to(wer,MT_NOTIFY,MA_UNKNOWN,
		"Das geht nicht, da du nicht an den Riesen "
                "heranreichst!\n");
            return 1;
        // Alles andere erlauben
        default:
            return 0;
    }
}

VERWEISE: notify_seele, allowed_seele, add_controller
GRUPPEN: seele
*/

/*
FUNKTION: notify_comm
DEKLARATION: void notify_comm(object wer, mixed wen, string what, string adverb, int flags, int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
BESCHREIBUNG:
Die Seele ruft jedesmal, NACHDEM ein Seele-Kommunikationsbefehl ausgefuehrt
wurde, in allen Lebewesen im Raum und im Raum selbst notify("comm", ...) auf.
Ist ein Objekt bei einem dieser Lebewesen oder dem Raum als Controller
fuer "notify_comm" angemeldet, so wird dann in diesem Objekt die Funktion
notify_comm(wer, wen, what, adverb, flags, msg_typ_wer, msg_typ_wen,
msg_typ_andere) aufgerufen.

Parameter & Beispiel: siehe notify_seele

VERWEISE: notify_seele, forbidden_seele, forbidden_comm, add_controller
GRUPPEN: seele
*/

/*
FUNKTION: forbidden_comm
DEKLARATION: int forbidden_comm(object wer, mixed wen, string what, string adverb, int flags, int msg_typ_wer, int msg_typ_wen, int msg_typ_andere)
BESCHREIBUNG:
Die Seele ruft jedesmal, BEVOR ein Seele-Kommunikationsbefehl ausgefuehrt
wird, in allen Lebewesen im Raum und im Raum selbst forbidden("comm", ...)
auf. Ist ein Objekt bei einem dieser Lebewesen oder dem Raum als Controller
fuer "forbidden_comm" angemeldet, so wird dann in diesem Objekt die Funktion
forbidden_comm(wer, wen, what, adverb, flags, msg_typ_wer, msg_typ_wen,
msg_typ_andere) aufgerufen.
Liefert  forbidden_comm  als Ergebnis 1, so wird der Seele-Kommunikations-
befehl damit verhindert (d.h. der Seelende ist geknebelt ;-).
forbidden_seele  sollte sich dabei um die Augabe einer passenden
Meldung als Begrundung kuemmern.

Bedeutung der Parameter: siehe notify_seele().
Beispiel:                siehe forbidden_seele().

VERWEISE: notify_comm, notify_seele, forbidden_seele, add_controller
GRUPPEN: seele
*/

void set_owner(object ob)   { owner = ob;}
object query_owner()        { return owner;}

#if 0
string wrap_it(string str1, string str2)
{
    return wrap_say(str1, str2);
}

varargs void Say(string str, mixed obs)
{
    if (obs && !mappingp (obs))
        say(wrap(str), obs);
    else
        say(wrap(str));
}

void Write(string str) { write(wrap(str)); }

void Tell_object(object ob, string str)
{
    if (objectp (ob)) tell_object (ob, wrap(str));
}
#endif


void feel(object wer)
{
    if (!playerp (wer) || !interactive(wer) || query_idle(wer)) return;
    if (sizeof(filter(all_inventory(environment(wer)),#'living //'
       )) > 1)
        wer->add_sum_feel(1);
    // Skriptspeiler kriegen nix dazu.
}


/*****
  Alle  x<Grammatikfunktion>-Befehle dienen dazu bessere Ausgaben
  bei Seele-Befehlen zu erzeugen.
  Zu verwenden sind sie beim Aufruf der Funktion
  forbidden_msg_soul_action_notify().

  Probleme gibt's mit den normalen Grammatikbefehlen vorallem bei
  v-items an den Partnern der Seele Befehle -
  hier muss die Bezugsperson (hier ist das der owner)
  der die Meldung erhaelt beruecksichtigt werden.
  Und bei Meldungen an die Partner muss es abwechselnd
  "dich" - "dein v-item" heissen.
****/

varargs string xdein(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
            return ihr(who, adjektiv, o, 0, 0, ART_DEIN);
    else
        return dein(who, adjektiv, o);
}
varargs string xdeinen(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
        return ihren(who, adjektiv, o, 0, 0, ART_DEIN);
    else
        return deinen(who, adjektiv, o);
}
varargs string xdeinem(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
        return ihrem(who, adjektiv, o, 0, 0, ART_DEIN);
    else
        return deinem(who, adjektiv, o);
}
varargs string xsein(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
        return ihr(who, adjektiv, o, 0, 0, ART_SEIN);
    else
        return sein(who, adjektiv, o);
}
varargs string xseinen(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
        return ihren(who, adjektiv, o, 0, 0, ART_SEIN);
    else
        return seinen(who, adjektiv, o);
}
varargs string xseinem(mixed who, mixed adjektiv, object o)
{
    if (mappingp(who) && auto_owner_search(who) != owner)
        return ihrem(who, adjektiv, o, 0, 0, ART_SEIN);
    else
        return seinem(who, adjektiv, o);
}
string xdu(mixed who)
{
    if (mappingp(who))
        return dein(who);
    else
        return "du";
}
string xdich(mixed who)
{
    if (mappingp(who))
        return deinen(who);
    else
        return "dich";
}
string xdir(mixed who)
{
    if (mappingp(who))
        return deinem(who);
    else
        return "dir";
}
varargs string xquery_deklin(mixed who, int art, int fall, mixed adjektiv, mixed o)
{
    if ((art & ART_DU) && mappingp(who))
        return query_deklin(who, ART_DEIN, fall, adjektiv);
    else if ((art & (ART_DEIN | ART_SEIN)) &&
              mappingp(who) && auto_owner_search(who) != owner)
        return query_deklin_owner(who, 0, fall, adjektiv, o, art);
    else
        return query_deklin(who, art, fall, adjektiv, o);
}

varargs string xDein(mixed who, mixed adjektiv, object o)
{
    return capitalize(xdein(who, adjektiv, o));
}
varargs string xDeinen(mixed who, mixed adjektiv, object o)
{
    return capitalize(xdeinen(who, adjektiv, o));
}
varargs string xDeinem(mixed who, mixed adjektiv, object o)
{
    return capitalize(xdeinem(who, adjektiv, o));
}
varargs string xSein(mixed who, mixed adjektiv, object o)
{
    return capitalize(xsein(who, adjektiv, o));
}
varargs string xSeinen(mixed who, mixed adjektiv, object o)
{
    return capitalize(xseinen(who, adjektiv, o));
}
varargs string xSeinem(mixed who, mixed adjektiv, object o)
{
    return capitalize(xseinem(who, adjektiv, o));
}
string xDu(mixed who)
{
    return capitalize(xdu(who));
}
string xDich(mixed who)
{
    return capitalize(xdich(who));
}
string xDir(mixed who)
{
    return capitalize(xdir(who));
}


string Wrap_say(string str1, string str2)
{
    return wrap_say(str1, str2);
}

string wrap_emote(string str)
{
    if (!sizeof(str) || str[<1] == '\n')
        return str;
    else
        return wrap(str);

    // Es gibt jetzt ja Farben ... Parsec & Freaky
    //    if (str[<1] == '\n')
    //        return str;
    //  return sprintf("%s%=-67s\n",str[0..7],str[8..]);
}

void msg_notify(string str)
{
    if (str) {
        notify_fail (str);
        str = query_notify_fail ();
#ifdef MT_FAIL
        owner->send_message_to(owner, MT_FAIL|MT_NOTIFY, MA_EMOTE, wrap_emote(str));
#else
        owner->send_message_to(owner, MT_NOTIFY, MA_EMOTE, wrap_emote(str));
#endif
    }
}


void msg_write(int msg_type, int msg_action, string str)
{
    if (str)
        owner->send_message_to(
            owner, msg_type | MT_NOTIFY, msg_action, wrap_emote(str));
}


varargs void msg_say(int msg_type, int msg_action, string str, mixed excludes)
{
    if (excludes && !mappingp(excludes))
        owner->send_message(msg_type, msg_action, wrap_emote(str), 0, excludes);
    else
        owner->send_message(msg_type, msg_action, wrap_emote(str));
}


void msg_to(mixed who, int msg_type, int msg_action, string str)
{
    if (objectp(who) && str)
        owner->send_message_to(who, msg_type, msg_action, wrap_emote(str));
}


/* nicht fuer externen Gebrauch - wird nur in soul.c verwender */
varargs void msg_soul_action(
    int msg_action,
    string str_owner, string str_other,
    int msg_type_owner, int msg_type_other)
{
    if (!msg_type_other)
        msg_type_other = msg_type_owner;

    msg_write(msg_type_owner, msg_action, str_owner);
    msg_say(msg_type_other, msg_action, str_other);
}


/* nicht fuer externen Gebrauch */
private varargs void msg_soul_action_partner(
    mixed partner, int msg_action,
    string str_owner, string str_partner, string str_other,
    int msg_type_owner, int msg_type_partner, int msg_type_other)
{
    if (!msg_type_partner)
        msg_type_partner = msg_type_owner;
    if (!msg_type_other)
        msg_type_other = msg_type_partner;

    if (mappingp(partner))
        partner = auto_owner_search(partner);
    if (partner == owner)
        partner = 0;

    msg_write(msg_type_owner, msg_action, str_owner);
    msg_to(partner, msg_type_partner, msg_action, str_partner);
    msg_say(msg_type_other, msg_action, str_other, partner);
}

/*
FUNKTION: seele_AKTION
DEKLARATION: string seele_[a-z]+(_msg|_msg_me|_msg_other)
BESCHREIBUNG:
Fuer jede Seeleaktion gibt es drei Funktionen (Objekt) bzw. zwei Eintraeges
(vitem-Mapping), die am Partner aufgerufen werden.
msg meldet an den Ausfuehrenden, msg_me an den Partner, 
msg_other an die Umgebenden. Zusaetzlich gibt es noch:
    query_seele_default_msg bzw ["seele_default_msg"]
    query_seele_default_msg_me  
    query_seele_default_msg_other ["seele_default_msg_other"]
Die default Meldung wird nur dann abgefragt, wenn es die vorherige seele_AKTION
nicht vorhanden war (0 liefert).
Beispiele:
    "seele_knabber_msg" : "Du knabberst am Hibiscus.",
    "seele_knabber_msg_other" : "$Der(OBJ_TP) knabbert am Hibiscus.",
    partner->query_seele_knabber_msg_me() gibt 
            Der(TP)+" kaut an Deinem Ohr." zurueck.
    // msg_me macht als Meldung ans Vitem keinen Sinn.
VERWEISE: forbidden_msg_soul_action_partner_notify
GRUPPEN: seele
*/

/*
FUNKTION: forbidden_msg_soul_action_partner_notify
DEKLARATION: varargs int forbidden_msg_soul_action_partner_notify(mixed partner, string aktion, string adverb, int align, int flags, string str_owner, string str_partner, string str_other, int msg_action, int msg_type_owner, int msg_type_partner, int msg_type_other)
BESCHREIBUNG:
Funktion uebernimmt die Komplette Ausgabe aller Messages eines Seele-Befehls
MIT Partner, sowie den korrekten Aufruf von forbidden() und notify().
Rueckgabewert: 0 Erfolgreich, 1 Aktion wurde verboten.

Parameter: siehe notify_seele
           Ist einer der msg_*-Parameter gleich 0, so nimmt er automatisch den
           Wert des davorliegenden msg_*-Parameter an.
           msg_action:  MA_EMOTE oder MA_COMM (aus message.h)

Beachte: Fuer korrekte Meldungen bitte die x-Grammatikfunktionen aus soul_hlp.c
         verwenden und Ausgaben in /p/Doc/Lehre/Demo/room/soul-test-room
	 kontrollieren.

Beispiel aus soul_commands.c:
    if (forbidden_msg_soul_action_partner_notify(
        was, "frag", str, NEUTRAL, 0,
        Wrap_say("Du fragst "+xdeinen(was)+":", str),
        Wrap_say(Der(owner)+" fragt "+xdich(was)+":", str),
        Wrap_say(Der(owner)+" fragt "+xseinen(was)+":", str),
        MA_COMM, MT_NOISE))
            return 1;

VERWEISE: notify_seele, forbidden_seele, forbidden_comm,
          forbidden_msg_soul_action_notify
GRUPPEN: seele
*/
varargs int forbidden_msg_soul_action_partner_notify(
    mixed partner, string aktion, string adverb, int align, int flags,
    string str_owner, string str_partner, string str_other,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other)
{
    if (soul_forbidden(partner, aktion, adverb, align, flags,
                         msg_action,
                         msg_type_owner, msg_type_partner,
                         msg_type_other))
        return 1;

    if (mappingp(partner)||objectp(partner))
    {
        string tmp;
        string aktion2 = convert_umlaute(aktion);
        if (stringp(tmp=QUERY("seele_"+aktion2+"_msg",partner)))
            str_owner = tmp;
        else if (stringp(tmp=QUERY("seele_default_msg",partner)))
            str_owner = tmp;
        if (stringp(tmp=QUERY("seele_"+aktion2+"_msg_me",partner)))
            str_partner = tmp;
        else if (stringp(tmp=QUERY("seele_default_msg_me",partner)))
            str_partner = tmp;
        if (stringp(tmp=QUERY("seele_"+aktion2+"_msg_other",partner)))
            str_other = tmp;
        else if (stringp(tmp=QUERY("seele_default_msg_other",partner)))
            str_other = tmp;
    }

    msg_soul_action_partner(partner, msg_action,
                    str_owner, str_partner, str_other,
           msg_type_owner, msg_type_partner, msg_type_other);

    soul_notify(partner, aktion, adverb, align, flags,
                 msg_action,
                 msg_type_owner, msg_type_partner,
                 msg_type_other);

    return 0;
}


/*
FUNKTION: forbidden_msg_soul_action_notify
DEKLARATION: varargs int forbidden_msg_soul_action_notify(string aktion, string adverb, int align, int flags, string str_owner, string str_other, int msg_action, int msg_type_owner, int msg_type_other)
BESCHREIBUNG:
Funktion uebernimmt die Komplette Ausgabe aller Messages eines Seele-Befehls
OHNE Partner, sowie den korrekten Aufruf von forbidden() und notify().
Rueckgabewert: 0 Erfolgreich, 1 Aktion wurde verboten.

Parameter: siehe notify_seele
           Ist einer der msg_*-Parameter gleich 0, so nimmt er automatisch den
           Wert des davorliegenden msg_*-Parameter an.
           msg_action:  MA_EMOTE oder MA_COMM (aus message.h)

Beispiel aus soul_commands.c:
    if (forbidden_msg_soul_action_notify(
        "antwort", str, NEUTRAL, 0,
        Wrap_say("Du antwortest:", str),
        Wrap_say(Der(owner)+" antwortet:", str),
        MA_COMM, MT_NOISE))
            return 1;

VERWEISE: notify_seele, forbidden_seele, forbidden_comm,
          forbidden_msg_soul_action_partner_notify
GRUPPEN: seele
*/
varargs int forbidden_msg_soul_action_notify(
    string aktion, string adverb, int align, int flags,
    string str_owner, string str_other,
    int msg_action,
    int msg_type_owner, int msg_type_other)
{
    if (soul_forbidden(0, aktion, adverb, align, flags,
                         msg_action,
                         msg_type_owner, 0,
                         msg_type_other))
        return 1;

    msg_soul_action(msg_action,
           str_owner, str_other,
           msg_type_owner, msg_type_other);

    soul_notify(0, aktion, adverb, align, flags,
                 msg_action,
                 msg_type_owner, 0,
                 msg_type_other);

    return 0;
}


string add_space(mixed str)
{
        // Jetzt aber mit Brachialgewalt, das sollte der 'Space'-Misere ein
        // ruhmloses Ende bereiten!
        return (!str || str =="") ? "" : ((str[0..0]==" ") ? str : " "+str);
}


int in_mir(mixed was)
{
    object  ob;

    return
        was && (ob = auto_owner_search(was)) == owner  ||
        ob && auto_owner_search(ob) == owner;
}


int livingp(mixed m)
{
    return
      objectp(m) &&
           (living(m) || m->allowed("seele")) //   || m->material("pflanzlich"))
        || mappingp(m) && m["living"];
}


int pflanzlichp(mixed m)
{
    return objectp(m) && (m->material("pflanzlich"));
}


private string fill_check(string str, string fill, string satzzeichen)
{
    string tmp1, tmp2;

    tmp2 = space(fill || "");
    tmp1 = space(str);

    if (tmp1[<strlen(tmp2)+strlen(satzzeichen)..] == tmp2+satzzeichen)
        return add_space(str);
    else if (tmp1[<strlen(tmp2)..] == tmp2)
        return add_space(str)+satzzeichen;
    else
        return str+add_space(fill)+satzzeichen;
}


/*
FUNKTION: conv_akt
DEKLARATION: string conv_akt(string aktion, int flag)
BESCHREIBUNG:
Eine wahnsinnig kryptische Funktion, um den Verbrumpf mit der korrekten
Endung zu versehen. Wenn Flag gesetzt ist, wird das Verb fuer die anderen
berechnet, sonst das fuer TP. Die unregelmaessigen Bildungen werden beachtet.
Bitte nicht von den vielen ?'s und :'s verwirren lassen :)
VERWEISE: plus_part, adv_part
GRUPPEN: seele
*/
string conv_akt(string aktion, int flag)
{
        return flag ? (aktion[<2..] == "tt" ? aktion : aktion[<1] == 't' ? aktion + "et" : aktion+"t") :
                ((aktion[<1]=='s' || aktion[<1]=='z' || aktion[<1]=='x') ? aktion+"t" :
                 (aktion[<1]=='t' && aktion[<2..] != "tt") ? aktion+"est" : aktion+"st");
}

/*
FUNKTION: plus_adv
DEKLARATION: deprecated int plus_adv(string adv, string aktion [, mixed fill,string fail ,string default_adv, int reflexiv])
BESCHREIBUNG:

ACHTUNG: Bitte  soul_plus_adv  statt  plus_adv  verwenden!

Es werden die Ausgaben fuer ein Seelekommando mit moeglichem Adverb
erzeugt.
        adv     Das gewuenschte Adverb
        aktion  Verbrumpf der Aktion (huepf, aerger)
        fill    Phrase am Ende (in die Luft). Optional
                                Es kann ein Array angegeben werden, dann wird der 1.
                String an owner ausgegeben, der 2. an den Raum.
        fail    Wenn hier nicht 0 steht wird fail als Fehlermeldung
                ausgegeben, sollte adv=0 sein ("Huepfe wie?\n")
        default_adv Adverb, das genommen wird, wenn adv 0 ist.
        reflexiv    Die Aktion wird reflexiv konjugiert (kratzt sich/dich), wenn
                dieser Parameter == 1 ist, mit reflexiv == 2 kann man
                                sich/dir (Du streichelst dir den Bauch) erzeugen.

BEISPIEL:
        plus_adv("nervoes", "gruebel");
        ->  Monty gruebelt nervoes.
                Du gruebelst nervoes.
        plus_adv("veraergert", "beiss", "auf ein Beissholz");
        ->  Monty beisst veraergert auf ein Beissholz.
                Du beisst veraergert auf ein Beissholz.
        plus_adv(0, "stink", 0, "Stinke wie?\n");
        ->  Fehlermeldung: Stinke wie?
        plus_adv(0, "stink", "herum", 0, "entsetzlich");
        ->  Monty stinkt entsetzlich herum.
                Du stinkst entsetzlich herum.
        plus_adv("verwirrt", "kratz", ({"an deinem Kopf", "an "+
                seinem((["name":"kopf","gender":"maennlich"]),0 , owner), 0, 0, 1);
        ->  Monty kratzt sich verwirrt an seinem Kopf.
                Du kratzt dich verwirrt an deinem Kopf.
VERWEISE: adv_part
GRUPPEN: seele
*/
#if !ALTLASTEN_RAUS
deprecated varargs int plus_adv(string str, string aktion, mixed fill, string fail,
        string default_adv, int reflexiv)
{
#if 1
    return soul_plus_adv(str, aktion, NEUTRAL, 0,
                          MA_UNKNOWN, MT_UNKNOWN, MT_UNKNOWN,
                          fill, fail, default_adv, reflexiv);
#else
        object who;
        string fill_du, fill_er;
        GHOST(owner);

        // Wegen Fehlermeldungen :(
        if (!owner || !ENV(owner))
        return 0;

        // Was ist, wenn str leer ist ?
        if (!str)
                if (fail)  // klar: Fehlermeldung ausgeben.
                        return notify_fail(fail);
                else
                        // ok, dann eben das default-adverb oder ""
                        str = default_adv ? " "+default_adv : "";

        if (pointerp(fill))
        {
                fill_du = fill[0];
                fill_er = fill[1];
        }
        else
                fill_du = fill_er = fill;
        who = present(lower_case(str), ENV(owner));
        if (who)
                return notify_fail(CAP(query_verb())+" wie?\n");
        str = add_space(str);
        Write("Du "+conv_akt(aktion, DU)+
                // bei reflexiv 1: Dich, 2: Dir, sonst: nix
                (reflexiv==1?" dich":(reflexiv==2?" dir":""))+
                str+add_space(fill_du)+".");
        Say(Der(owner)+" "+conv_akt(aktion, ER)+(reflexiv?" sich":"")+
                str+add_space(fill_er)+".");
        FEEL(owner);
        return 1;
#endif
}
#endif // ALTLASTEN_RAUS


/*
FUNKTION: soul_plus_adv
DEKLARATION: int soul_plus_adv(string adv, string aktion, int align, int flags, int msg_action, int msg_type_owner, [ int msg_type_other, mixed fill, string fail ,string default_adv, int reflexiv])
BESCHREIBUNG:
Es werden die Ausgaben fuer ein Seelekommando mit moeglichem Adverb
erzeugt.
    adv     Das gewuenschte Adverb
    aktion  Verbrumpf der Aktion (huepf, aerger)
    align   Ist Seelekommando eher neutral (NEUTRAL == 0), mies (MIES == -1)
            oder nett (NETT == 1) gemeint (Konstanten aus soul.h).
    flags   Andere Eigenschaften der Seele-Aktion. Mit bitweisem-&
            zu testen. Momentan implementierte Eigenschaften:
            flags & PSEUDO_MOVE   Bewegt man sich bei der Aktion
                                  (z.B. waelzen, tanzen, huepfen, ...)
    msg_action      MA_EMOTE oder MA_COMM (aus message.h)
    msg_typ_owner   Welche Sinne des Seelenden werden durch die Aktion
                    angesprochen (z.B. knuddle: msg_typ_wer & MT_FEEL - der
                    Seelende spuert die Aktion). Konstanten aus message.h
    msg_typ_other   Welche Sinne der Beobachter werden angesprochen.
    fill    Phrase am Ende (in die Luft). Optional
            Es kann ein Array angegeben werden, dann wird der 1.
            String an owner ausgegeben, der 2. an den Raum.
    fail    Wenn hier nicht 0 steht wird fail als Fehlermeldung
            ausgegeben, sollte adv=0 sein ("Huepfe wie?\n")
    default_adv Adverb, das genommen wird, wenn adv 0 ist.
    reflexiv    Die Aktion wird reflexiv konjugiert (kratzt sich/dich),
                wenn dieser Parameter == 1 ist, mit reflexiv == 2 kann man
                sich/dir (Du streichelst dir den Bauch) erzeugen.

    Ist einer der msg_*-Parameter gleich 0, so nimmt er automatisch den
    Wert des davorliegenden msg_*-Parameter an.

BEISPIEL:
        soul_plus_adv("nervoes", "gruebel", NEUTRAL, 0, MA_EMOTE, MT_LOOK);
        ->  Monty gruebelt nervoes.
                Du gruebelst nervoes.
        soul_plus_adv("veraergert", "beiss", NEUTRAL, 0, MA_EMOTE,
                      MT_LOOK | MT_FEEL, MT_LOOK, "auf ein Beissholz");
        ->  Monty beisst veraergert auf ein Beissholz.
                Du beisst veraergert auf ein Beissholz.
        soul_plus_adv(0, "stink", MIES, 0, MA_EMOTE,
                       MT_SMELL, 0, 0, "Stinke wie?\n");
        ->  Fehlermeldung: Stinke wie?
        soul_plus_adv(0, "stink", "herum", MIES, 0, MA_EMOTE, MT_SMELL, 0,
                      0, "entsetzlich");
        ->  Monty stinkt entsetzlich herum.
                Du stinkst entsetzlich herum.
        plus_adv("verwirrt", "kratz",
                 NEUTRAL, 0, MA_EMOTE, MT_LOOK | MT_FEEL, MT_LOOK,
                 ({"an deinem Kopf", "an "+
                 seinem((["name":"kopf","gender":"maennlich"]),0 , owner),
                 0, 0, 1);
        ->  Monty kratzt sich verwirrt an seinem Kopf.
                Du kratzt dich verwirrt an deinem Kopf.
VERWEISE: soul_adv_part
GRUPPEN: seele
*/
varargs int soul_plus_adv(
    string str, string aktion, int align, int flags,
    int msg_action, int msg_type_owner, int msg_type_other,
    mixed fill, string fail,
    string default_adv, int reflexiv)
{
    object  who;
    string  fill_du, fill_er;

    GHOST(owner);

    // Wegen Fehlermeldungen :(
    if (!owner || !ENV(owner))
        return 0;

    if (!msg_type_other)
        msg_type_other = msg_type_owner;

    // Was ist, wenn str leer ist ?
    if (!str)
        if (fail)
            return notify_fail(wrap_emote(fail));
        else
            // ok, dann eben das default-adverb oder ""
            str = default_adv ? " "+default_adv : "";

    if (pointerp(fill))
    {
        fill_du = fill[0];
        fill_er = fill[1];
    }
    else
        fill_du = fill_er = fill;
    who = present(lower_case(str), ENV(owner));
    if (who)
        return notify_fail(CAP(query_verb())+" wie?\n");
    str = add_space(str);

    if (forbidden_msg_soul_action_notify(
        aktion, str, align, flags,
        "Du "+conv_akt(aktion, DU)+
        // bei reflexiv 1: Dich, 2: Dir, sonst: nix
        (reflexiv==1?" dich":(reflexiv==2?" dir":""))+
        fill_check(str, fill_du, "."),
        Wer(owner, ART_AAA)+" "+conv_akt(aktion, ER)+(reflexiv?" sich":"")+
        fill_check(str, fill_er, "."),
        msg_action, msg_type_owner, msg_type_other))
        return 1;

    FEEL(owner);

    return 1;
}

/*
FUNKTION: soul_parser
DEKLARATION: mixed *soul_parser(string str, [int flags, string preposition])
BESCHREIBUNG:
Der soul_parser liefert vor allem den passenden Partner zu einem
Seele-Kommando. Uebergeben wird der String str, also alles, was nach dem
Kommando kommt und die Flags.
Die Bedeutungen fuer die Flags sind in soul.h definiert:
        PARNTER : wird an adv_part weitergereicht: Fehlermeldung, wenn in str
                  kein/e ParerIn gefunden wurde, sonst plus_adv machen
        LIVING  : Fehlermeldung, wenn der gefundene Partner kein Living ist,
                  wobei hierzu auch lebende v-items zaehlen (siehe add_v_item).
                  Antwortet Partner auf  allowed("seele")  mit 1, so
                  akzepptiert soul_parser() ihn wie ein living
                  (siehe allowed_seele()).
        OBJECT  : Fehlermeldung, wenn der gefundene Partner kein Objekt ist.
                  (LIVING und OBJECT muessen nicht zusammen angegeben werden)
        IGNORE_FAR:  wenn's egal ist ob ein gefundenes V-Item weit weg ist.
                     d.h. Aktion geht auch mit "far"-V-Items.
        SOUL_PO : nur mit Partner und nur mit Objekten (PARTNER|OBJECT)
        SOUL_PL : nur mit Partner und nur mit Livings (PARTNER|LIVING)
	NOT_TO_ENV:  Geht nicht mit der Umgebung als Partner.

Ist  preposition  angegeben, so wird auch eine eventuell dem Partner
vorangestellte Praeposition akzeptiert.

soul_parser liefert:
        soul_parser()[0] ist der gefundene Partner,
        soul_parser()[1] ist der Rest von str (mit space vorndran) und
        soul_parser()[2] sind die Flags, wie sie uebergeben wurden.
        soul_parser()[3] eine Fehlermeldung wenn kein Partner gefunden wurde.
                         (erzeugt durch parse_com_error(). etwa so:
                          'Bla nicht gefunden.')
        ({})             wenn mehr als ein Objekt gefunden wurde, oder wenn
                         kein Objekt gefunden wurde, das mit den flags naeher
                         spezifiziert wurde.
BEISPIEL:
        soul_parser("den Stein inniglich", SOUL_PL)
          liefert ({}) und erzeugt die Fehlermeldung "Das geht nur mit
          Lebewesen!"
          (vorausgesetzt, es gibt einen Stein in der Umgebung...)
        soul_parser("den Stein inniglich", PARTNER)
          liefert ({<objektpointer des steins>, " inniglich", 1, 0})
        soul_parser("um den 3. Stein", SOUL_PO, "um")
          liefert ({<objectpointer des 3. steins>, "", 1, 0}).
VERWEISE: soul_adv_part
GRUPPEN: seele
*/
varargs mixed *soul_parser(string str, int flags, string preposition)
{
    mixed  *res, parsed, parsed2, was;
    string|int tmp;

    res = ({0,0,flags,0});


    parsed = parse_com(str, ({ ENV(owner), owner }));

    // Nicht gefunden -> Ist die Umgebung gemeint?
    if(str && !parsed[PARSE_NUM_OBS] && !(flags&NOT_TO_ENV) && ENV(owner) &&
	!ENV(owner)->query_room() && (tmp=ENV(owner)->me(str)))
	    parsed = ({1, ({ENV(owner)}), tmp, PARSE_OK, 
		ENV(owner)->query_name(), 0, 0});

    // wenn nix gefunden, dann schauen wir mal ob wir was mit der
    // preposition finden
    if (str  &&
         preposition  &&
         !parsed[PARSE_NUM_OBS]   &&
         str != (tmp = regreplace(str, "^ *("+preposition+"|"+convert_umlaute(preposition)+") *", "", 1)))
    {
        parsed2 = parse_com(tmp, ({ ENV(owner), owner }));
        if(parsed2[PARSE_NUM_OBS])
            parsed = parsed2;
        // wenn ja, dann nehmen wir die
	// ansonsten schauen, ob die Umgebung gemeint war.
        else if(!(flags&NOT_TO_ENV) && ENV(owner) && !ENV(owner)->query_room() &&
	    (tmp=ENV(owner)->me(tmp)))
		parsed = ({1, ({ENV(owner)}), tmp, PARSE_OK, 
		    ENV(owner)->query_name(), 0, 0});
    }

    if(parsed[PARSE_NUM_OBS])
    {
        if(sizeof(parsed[PARSE_OBS]) > 1)
        {
            msg_notify("Eins nach dem anderen, bitte!");
            return ({});
        }
        was = parsed[PARSE_OBS][0];

        if (mappingp(was))
        {
            if (stringp(tmp = funcall(was["soul"])))
            {
                msg_notify(tmp);
                return ({});
            }
            if((flags & LIVING) && !was["living"])
            {
                msg_notify("Das geht nur mit Lebewesen!");
                return ({});
            }
            if ((tmp=QUERY("far",was)) &&
                 !(flags & IGNORE_FAR)  // Aktion ist trotz "far" erlaubt
               )
            {
                msg_notify(stringp(tmp)?tmp:(Der(was)+
                            (QUERY("plural",was)?" sind":" ist")
                            +" viel zu weit weg."));
                return ({});
            }
        }
        else if ((flags & LIVING)                &&
                  !(objectp(was) &&
                    (living(was) || was->allowed("seele") || was->material("pflanzlich"))))
        {
            msg_notify("Das geht nur mit Lebewesen!");
            return ({});
        }

#if 0
        // Alte Version ...
        if(flags & LIVING)
        {
            if (mappingp(was))
            {
                if (stringp(tmp = funcall(was["soul"])))
                {
                    msg_notify(tmp);
                    return ({});
                }
                if (!was["living"])
                {
                    msg_notify("Das geht nur mit Lebewesen!");
                    return ({});
                }
                if (QUERY("far",was))
                {
                    msg_notify(Der(was)+
                                (QUERY("plural",was)?" sind":" ist")
                                +" viel zu weit weg.");
                    return ({});
                }
            }
            else if (!objectp(was) || !living(was))
            {
                msg_notify("Das geht nur mit Lebewesen!");
                return ({});
            }
        }
#endif
        if((flags & OBJECT) && !objectp(was))
        {
            msg_notify("Das geht nicht!");
            return ({});
        }

        res[0] = was;
        res[1] = add_space(parsed[PARSE_REST]);
    }
    else
    {
        // Schade keinen Partner gefunden
        // ABER: wenn ich eine Id von exec_command() in str
        //       finde, weiss ich, dass ein Partner gesucht wurde
        //       und ich kann eine Fehlermeldung ausgeben.
        if (sizeof(regexp(({ str }), EXEC_COMMAND_ID_REGEXP)))
        {
            if (flags & LIVING)
                msg_notify("So jemand ist nicht hier!");
            else
                msg_notify("So etwas ist nicht hier!");
            return ({});
        }
        // Da hat wohl der # versagt
        // ABER: ich weiss, ein Partner war gesucht -> Fehlermeldung
        else if (playerp(owner) &&
                  sizeof(regexp(({ str }), " *[a-zA-Z][a-zA-Z]*#\(\|$\)")))
        {
            msg_notify(capitalize(regreplace(
                str, " *\([a-zA-Z][a-zA-Z]*\)#.*", "\\1", 1))+
                        " ist nicht hier!");
            return ({});
        }

        res[1] = add_space(str);
        res[3] = parse_com_error_string(parsed);
        res[3] = strlen(res[3]) ? wrap_emote(res[3]) : "";
    }

    return res;
}


/*
FUNKTION: parse_text
DEKLARATION: varargs mixed *parse_text(string str, mixed where, int flag, string error_msg)
BESCHREIBUNG:
Alternative zu parse_com, wenn man am Anfang eines Textes ein Objekt finden
will und den Rest nach der Objekt-Id bekommen will. Weil parse_com, wenn
kein Objekt gefunden wird, sehr teuer bei langen Texten sein kann.

Parameter:
   str        Der zu analysierende Text
   where      Wo gesucht werden soll.
              Default ({ environment(this_player()), this_player() })
   flag       Darf PARSE_NO_V_ITEMS sein. Sieheparse_com().
   error_msg  Wie bei parse_com_error(). Wird mit notify_fail() gesetzt,
              wenn kein Objekt gefunden wird und kein bessere Fehler generiert
              werden kann.

Rueckgabe:
   0  falls kein Objekt gefunden wurde.
   ({ <GefundenesObjekt>, <RestText> }) wenn Objekt gefunden wurde.
   ({ 0, <RestText> }) wenn Objekt zwingend gesucht aber nicht gefunden wurde.
                       (ist der Fall wenn ueber exec_command() gearbeitet wurde)

Beispiel:
   parse_text("mein kuscheli Das suess ist schoen.")
   -> ({ OBJ(mein kuscheli), "Das ist schoen." })

   parse_text("Das hier ist mir egal!")
   -> 0

   Typische Anwendung in Befehlen wie:  'frage mein Kuscheli magst Du das so?'

BEISPIEL:
VERWEISE: soul_adv_part
GRUPPEN: seele
*/
varargs mixed *parse_text(string str, mixed where, int flag, string error_msg)
{
    string *text;
    string *rest;
    string toadd="";
    int i,j;
    mixed parsed;

    if((i=strstr(str,":"))>=0)
    {
	text = explode(strip(str[0..i-1])," ");
	parsed = parse_com(implode((text - ({""}))[0..5]," "), where, 0, flag);
	if(!strlen(parsed[PARSE_REST]))
	    toadd = strip(str[i+1..<1]);
	else
	{
	    while((i--)>0 && str[i]==' ');
	    toadd = str[i+1..<1];
	}
    }
    else
    {
	text = explode(strip(str)," ");
    // Nur in den ersten 6 Woerten nach Partnern suchen, das
    // parse_com sonst zu teuer ist!
	parsed =
    	    parse_com(implode((text - ({""}))[0..5]," "), where, 0, flag);
    }

    // wenn ich eine Id von exec_command() in str gefunden.
    // Id nicht anwesend -> Fehler.
    if(!parsed[PARSE_NUM_OBS] &&
        sizeof(regexp(({ str }), "^"+EXEC_COMMAND_ID_REGEXP)))
    {
        notify_fail("So etwas ist nicht hier!\n");
        return ({ 0, regreplace(str, "^"+EXEC_COMMAND_ID_REGEXP, "", 1) });
    }

    if (parse_com_error(parsed, error_msg, 1) ||
	!strlen(parsed[PARSE_ID])) // Passiert bei "alle"
        return 0;

    // So jetzt muessen wir, da wir einen Partner gefunden haben,
    // Den Anfang von  str  abschneiden, der den Partner repraesentiert.

    // Wir schauen mal, wo wir die ersten 6 Woerter genau abgeschnitten haben
    for(i=0,j=0; i<sizeof(text) && j<6; i++)
        if(text[i]!="") j++;
    
    i--;
    
    // Und gehen nun die von parse_com geliefert Restwoerter zurueck
    rest=explode(parsed[PARSE_REST]||""," ")-({""});
    
    for(j=sizeof(rest)-1;j>=0 && i>=0;i--)
        if(text[i]==rest[j]) j--;
        else if(text[i]!="") break;

    i++;
    
    return ({ parsed[PARSE_OBS][0], implode(text[i..<1]," ") + toadd });
}



/*
FUNKTION: adv_part
DEKLARATION: deprecated int adv_part(mixed res, string aktion, mixed fill1, string fill, int fall [, string fail, string default_adv, string dot ]])
BESCHREIBUNG:

ACHTUNG BITTE NICHT MEHR VERWENDEN - neu: soul_adv_part()

adv_part gibt alle Meldungen fuer ein Seelekommando mit Partner und Adverb
aus.
Parameter
        res     Ergebnis eines Aufrufes von soul_parser, siehe dort
        aktion  Verbrumpf der Seele-Aktion (kratz, beiss, kuschel...)
        fill1       Fuellwoerter vor Partner und Adverb
        fill    Fuellwoerter nach Partner und Adverb, hier kann auch ein
                Array angegeben werden, das 1. geht an owner, das 2.
                an die Umgebung.
        fall    Der grammatikalische Fall, in dem der Partner auftaucht.
                es werden Konstanten aus <deklin.h> verwendet
        fail    Fehlermeldung, die ausgegeben wird, wenn kein Partner
                gefunden wird, wenn sie fehlt, wird eine (maessige)
                erzeugt. Nur von Belang, wenn der soul_parser mit
                flag 1 aufgerufen wurde. Optional
        default_adv Adverb, das genommen wird, wenn der soul_parser keines
                findet. Optional.
        dot     Satzzeichen am Ende der Meldung. Optional. Default ist
                        natuerlich "."
BEISPIEL:
        adv_part(soul_parser("freaky anmutig", 0), "verneig",
                ({"dich vor","sich vor"}), 0, FALL_DAT);
        ->  Monty verneigt sich anmutig vor Freaky.
                Du verneigst dich anmutig vor Freaky.
                Monty verneigt sich anmutig vor dir.
        adv_part(({OBJ(ork), "gemein", 3}), "grins", 0, "an", FALL_AKK,
                0, "verschmitzt");
        ->  Monty grins den Ork gemein an.
                Du grinst den Ork gemein an.
                Monty grinst dich gemein an.
        adv_part(({0, "lustig", 0}), "grins", 0, "an", FALL_AKK, 0, 0,"!")
        ->  Monty grinst lustig!
                Du grinst lustig!
        adv_part(({0, "hinterlistig", 1}), "attacker", 0, 0, FALL_AKK,
                "Attackiere wen oder was <wie>?\n");
        ->  Fehlermeldung: Attackiere wen oder was <wie>?
VERWEISE: plus_adv, soul_parser, soul_adv_part
GRUPPEN: seele
*/
#if !ALTLASTEN_RAUS
deprecated varargs int adv_part(mixed res, string aktion, mixed fill1, string fill,
        int fall, string fail, string default_adv, string dot)
{
#if 1
    return soul_adv_part(res, aktion, NEUTRAL, 0, MA_UNKNOWN,
                          MT_UNKNOWN, MT_UNKNOWN, MT_UNKNOWN,
                          fill1, fill, fall, fail, default_adv, dot);
#else
        string pron;

        // Wegen Fehlermeldungen :(
        if (!owner || !ENV(owner))
                return 0;

        // Aussteigen, wenn res leer ist oder wenn der owner ein Geist ist
        if(!res)
                return 0;
        if (!sizeof(res))
                return 1;
        GHOST(owner);

        // die Fuellausdruecke bearbeiten: Wenn = 0, dann "" draus machen,
        // sonst wird ein Space davor eingefuegt.
        res[1] = (res[1]=="" && default_adv) ? " "+default_adv : res[1];
        fill1 = pointerp(fill1) ? fill1 : ({fill1?fill1:"",fill1?fill1:""});

        // Ein abschliessendes Satzzeichen:
        dot = dot ? dot : ".";

        if(res[0])
        {
            pron =
                (pron = query_deklin(res[0], ART_DU | ART_VIS, fall))?" "+
                CAP(pron):"";
                if(res[0] == owner)
                {
                        Write(CAP(query_verb())+
                                (fill1[DU]==""?"":(" "+fill1[DU]))+pron+
                                " selbst"+add_space(fill)+"?");
                        return 1;
                }
                else
                {
                        if(objectp(res[0]))
                        {
                            // erst abfragen, ob aktion verboten ist
                            if(res[0]->soul_command(owner, aktion, res[1]))
                                return 1;

                            owner->send_message_to(
                                res[0],MT_UNKNOWN,MA_EMOTE,
                                wrap_emote(Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+
                                           add_space(fill1[ER])+pron+res[1]+add_space(fill)+dot));
                        } else {
                                if (QUERY("far",res[0])) {
                                        msg_notify(Der(res[0])+
                                        (QUERY("plural",res[0])?" sind ":" ist ")+
                                        "viel zu weit weg.");
                                        return 1;
                                }
                                if (res[0]["living"] &&
                                     stringp(tmp = funcall(res[0]["soul"])))
                                {
                                    msg_notify( tmp);
                                    return 1;
                                }
                        }
                        Say(Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+add_space(fill1[ER])+
                                " "+query_deklin(res[0], ART_DER, fall)+res[1]+
                                add_space(fill)+dot,
                                objectp(res[0])?res[0]:this_object());
                        Write("Du "+conv_akt(aktion, DU)+add_space(fill1[DU])+" "+
                                query_deklin(res[0], ART_DER, fall)+res[1]+add_space(fill)+
                                dot);
                        if(objectp(res[0]))
                                res[0]->notify_soul(owner, aktion, res[1]);

                }
        }
        // Passende Fehlermeldung erzeugen, wenn Flag 1 gesetzt und kein
        // Partner gefunden wurde
        else if(res[2] & PARTNER)
        {
                if(fail)
                        msg_notify(fail);
                else
                        Write(CAP(query_verb())+" "+fill1[DU]+
                        ((fall == FALL_DAT)?"wem":"wen")+" oder was <wie>"+
                        add_space(fill)+"?");
                return 1;
        }
        // Kein Partner, kein Flag 1, also machen wir ein plus_adv
        else
        {
                Say(Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+res[1]+dot);
                Write("Du "+conv_akt(aktion, DU)+res[1]+dot);
        }
        FEEL(owner);
        return 1;
#endif
}
#endif// ALTLASTEN_RAUS


/*
FUNKTION: soul_adv_part
DEKLARATION: int soul_adv_part(mixed res, mixed aktion, int align, int flags, int msg_action, int msg_type_owner, int msg_type_partner, int msg_type_other, mixed fill1, string fill, int fall [, string fail, string default_adv, string dot ]])
BESCHREIBUNG:
soul_adv_part gibt alle Meldungen fuer ein Seelekommando mit Partner und
Adverb aus.
Parameter
    res     Ergebnis eines Aufrufes von soul_parser, siehe dort
    aktion  Verbrumpf der Aktion (huepf, aerger)
    adverb  Das ist das benutzte Adverb, oder eben 0, wenn keins verwendet
            wurde.
    align   Ist Seelekommando eher neutral (NEUTRAL == 0), mies (MIES == -1)
            oder nett (NETT == 1) gemeint (Konstanten aus soul.h).
    flags   Andere Eigenschaften der Seele-Aktion. Mit bitweisem-&
            zu testen. Momentan implementierte Eigenschaften:
            flags & PSEUDO_MOVE   Bewegt man sich bei der Aktion
                                  (z.B. waelzen, tanzen, huepfen, ...)
    msg_action       MA_EMOTE oder MA_COMM (aus message.h)
    msg_type_owner   Welche Sinne des Seelenden werden durch die Aktion
                     angesprochen (z.B. knuddle: msg_typ_wer & MT_FEEL - der
                     Seelende spuert die Aktion). Konstanten aus message.h
    msg_type_partner Welche Sinne des Geseelten werden angesprochen.
    msg_type_other   Welche Sinne der Beobachter werden angesprochen.
    fill1   Fuellwoerter vor Partner und Adverb
    fill    Fuellwoerter nach Partner und Adverb, hier kann auch ein
            Array angegeben werden, das 1. geht an owner, das 2.
            an die Umgebung.
    fall    Der grammatikalische Fall, in dem der Partner auftaucht.
            es werden Konstanten aus <deklin.h> verwendet
    fail    Fehlermeldung, die ausgegeben wird, wenn kein Partner
            gefunden wird, wenn sie fehlt, wird eine (maessige)
            erzeugt. Nur von Belang, wenn der soul_parser mit
            flag 1 aufgerufen wurde. Optional
    default_adv Adverb, das genommen wird, wenn der soul_parser keines
            findet. Optional.
    dot     Satzzeichen am Ende der Meldung. Optional. Default ist
            natuerlich "."

    Ist einer der msg_*-Parameter gleich 0, so nimmt er automatisch den
    Wert des davorliegenden msg_*-Parameter an.

BEISPIEL:
        soul_adv_part(soul_parser("freaky anmutig", 0), "verneig",
                      NETT, 0, MA_EMOTE, MT_LOOK, 0, 0,
                      ({"dich vor","sich vor"}), 0, FALL_DAT);
        ->  Monty verneigt sich anmutig vor Freaky.
                Du verneigst dich anmutig vor Freaky.
                Monty verneigt sich anmutig vor dir.
        soul_adv_part(({OBJ(ork), "gemein", 3}), "grins",
                      NETT, 0, MA_EMOTE, MT_LOOK, 0, 0,
                      0, "an", FALL_AKK, 0, "verschmitzt");
        ->  Monty grins den Ork gemein an.
                Du grinst den Ork gemein an.
                Monty grinst dich gemein an.
        soul_adv_part(({0, "lustig", 0}), "grins",
                      NETT, 0, MA_EMOTE, MT_LOOK, 0, 0,
                      0, "an", FALL_AKK, 0, 0,"!")
        ->  Monty grinst lustig!
                Du grinst lustig!
        soul_adv_part(({0, "hinterlistig", 1}), "attacker",
                      MIES, 0, MA_EMOTE, MT_LOOK | MT_FEEL, 0, MT_LOOK,
                      0, 0, FALL_AKK, "Attackiere wen oder was <wie>?\n");
        ->  Fehlermeldung: Attackiere wen oder was <wie>?
VERWEISE: soul_plus_adv, soul_parser
GRUPPEN: seele
*/
varargs int soul_adv_part(
    mixed res, string aktion,
    int align, int flags, int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other,
    mixed fill1, mixed fill,
    int fall, string fail, string default_adv, string dot)
{
    string pron;
    string|int tmp;

    // Wegen Fehlermeldungen :(
    if (!owner || !ENV(owner))
        return 0;

    // Aussteigen, wenn res leer ist oder wenn der owner ein Geist ist
    if(!res)
        return 0;
    if (!sizeof(res))
        return 1;
    GHOST(owner);

    if (!msg_type_partner)
        msg_type_partner = msg_type_owner;
    if (!msg_type_other)
        msg_type_other = msg_type_partner;

    // die Fuellausdruecke bearbeiten: Wenn = 0, dann "" draus machen,
    // sonst wird ein Space davor eingefuegt.
    res[1] = (res[1]=="" && default_adv) ? " "+default_adv : res[1];
    fill = pointerp(fill) ? fill : ({fill?fill:"",fill?fill:""});
    fill1 = pointerp(fill1) ? fill1 : ({fill1?fill1:"",fill1?fill1:""});

    // Ein abschliessendes Satzzeichen:
    dot = dot ? dot : ".";

    if(res[0])
    {
        pron =
            (pron = xquery_deklin(res[0], ART_DU | ART_VIS, fall))?" "+
            pron:"";
        if(res[0] == owner)
        {
            msg_notify(CAP(query_verb())+
                        (fill1[DU]==""?"":(" "+fill1[DU]))+pron+
                        " selbst"+add_space(fill[DU])+"?");
            return 1;
        }
        else
        {
            // falls  res  ueber soul_parser() erzeugt wurde sollte dies nicht
            // mehr noetig sein
            if(mappingp(res[0]))
            {
                if (res[0]["living"] &&
                     stringp(tmp = funcall(res[0]["soul"])))
                {
                    msg_notify(tmp);
                    return 1;
                }
                if ((tmp=QUERY("far",res[0])) &&
                     !(res[2] & IGNORE_FAR) && // Aktion ist trotz "far" erlaubt
                     (msg_type_partner & MT_FEEL)  // Partner muss beruehrt
                                                   // werden
                   )
                // if (res[0]["far"])
                {
                    msg_notify(stringp(tmp)?tmp:(Der(res[0])+
                                (QUERY("plural",res[0])?" sind ":" ist ")+
                                "viel zu weit weg."));
                    return 1;
                }
            }

            if (forbidden_msg_soul_action_partner_notify(
                res[0], aktion, res[1], align, flags,
                "Du "+conv_akt(aktion, DU)+add_space(fill1[DU])+" "+
                  xquery_deklin(res[0], ART_DEIN, fall)+
                  fill_check(res[1], fill[DU], dot),
                Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+
                  add_space(fill1[ER])+pron+
                  fill_check(res[1], fill[ER], dot),
                Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+
                  add_space(fill1[ER])+" "+
                  xquery_deklin(res[0], ART_SEIN, fall)+
                  fill_check(res[1], fill[ER], dot),
                msg_action, msg_type_owner, msg_type_partner, msg_type_other))
                return 1;
        }
    }
    // Passende Fehlermeldung erzeugen, wenn Flag 1 gesetzt und kein
    // Partner gefunden wurde
    else if(res[2] & PARTNER)
    {
        if(fail)
        {
            res[3] += wrap_emote(fail);
        }

        else
        {
            res[3] += CAP(query_verb())+" "+fill1[DU]+
                ((fall == FALL_DAT)?"wem":"wen")+" oder was <wie>"+
                add_space(fill[DU])+"?\n";
        }

        notify_fail(res[3]);
        return 0;
    }

    // Kein Partner, kein Flag 1, also machen wir ein plus_adv
    else
    {
        if (forbidden_msg_soul_action_notify(
            aktion, res[1], align, flags,
            "Du "+conv_akt(aktion, DU)+res[1]+dot,
            Wer(owner,ART_AAA)+" "+conv_akt(aktion, ER)+res[1]+dot,
            msg_action, msg_type_owner, msg_type_other))
            return 1;
    }

    FEEL(owner);

    return 1;
}



private int soul_forbidden(
    mixed partner, string aktion, string adverb, int align, int flags,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other)
{
    int     i;
    mixed   partner_ob;
    object  *obs;


    if (msg_action == MA_COMM  &&
         owner->query_stat(STAT_INT) < MIN_INT_COMM)
    {
	msg_notify("Dazu bist du nicht intelligent genug.");
	return 1;
    }

    if (!msg_type_partner)
        msg_type_partner = msg_type_owner;
    if (!msg_type_other)
        msg_type_other = msg_type_partner;
    if (!partner)
        msg_type_partner = 0;

    aktion = convert_umlaute(aktion);
    partner_ob = partner;
    while (mappingp(partner_ob))
        partner_ob = partner_ob["environment"];

    // Owner & Partner sicher zuerst fragen
    obs = ({ owner });
    obs += ({ partner_ob }) - obs;
    if (owner && environment(owner))
    {
        obs +=
            filter(
                all_inventory(environment(owner)), #'living //'
               ) - obs;
        obs += ({ environment(owner) }) - obs;
    }
    obs -= ({ 0 });

    if (msg_action == MA_COMM)
    {
        for (i = 0; i < sizeof(obs); i++)
            if (obs[i]->forbidden("comm", owner, partner, aktion,
                                    adverb, flags,
                                    msg_type_owner, msg_type_partner,
                                    msg_type_other))
                return 1;
    }
    else
    {
        for (i = 0; i < sizeof(obs); i++)
            if (obs[i]->forbidden("seele", owner, partner, aktion,
                                       adverb, align, flags,
                                       msg_type_owner, msg_type_partner,
                                       msg_type_other))
                return 1;

        // Altes soul_command auch unterstuetzen
        if (partner && objectp(partner) &&
             partner != owner &&
             partner->soul_command(owner, aktion, adverb))
            return 1;
    }

    return 0;
}


private void soul_notify(
    mixed partner, string aktion, string adverb, int align, int flags,
    int msg_action,
    int msg_type_owner, int msg_type_partner, int msg_type_other)
{
    mixed   partner_ob;
    object  *obs;


    if (!msg_type_partner)
        msg_type_partner = msg_type_owner;
    if (!msg_type_other)
        msg_type_other = msg_type_partner;
    if (!partner)
        msg_type_partner = 0;

    aktion = convert_umlaute(aktion);
    partner_ob = partner;
    while (mappingp(partner_ob))
        partner_ob = partner_ob["environment"];

    // Owner & Partner sicher zuerst benachrichtigen
    obs = ({ owner });
    obs += ({ partner_ob }) - obs;
    if (owner && environment(owner))
    {
        obs +=
            filter(
                all_inventory(environment(owner)), #'living //'
               ) - obs;
        obs += ({ environment(owner) }) - obs;
    }
    obs -= ({ 0 });

    if (msg_action == MA_COMM)
        map_objects(obs,
                     "notify", "comm",
                     owner, partner, aktion, adverb, flags,
                     msg_type_owner, msg_type_partner, msg_type_other);
    else
    {
        map_objects(obs,
                     "notify", "seele",
                     owner, partner, aktion, adverb, align, flags,
                     msg_type_owner, msg_type_partner, msg_type_other);

        // Altes notify_soul auch unterstuetzen
        if (partner && objectp(partner) && partner != owner)
            partner->notify_soul(owner, aktion, adverb);
    }
}



// Altlasten:
/*
FUNKTION: notify_soul
DEKLARATION: deprecated void notify_soul(object who, string what, string adverb)
BESCHREIBUNG:

ACHTUNG: statt  notify_soul  sollte  notify_seele  verwendet werden!

Die Seele ruft jedesmal, wenn eine Aktion mit Partner ausgefuehrt wurde, diese
Funktion im Parnter auf und zwar NACH der Seele-Aktion. Damit kann z.B. ein
Monster erkennen, dass gerade ein Spieler mit der Seele auf ihn eingewirkt
hat. In diesem Monster muss man nur noch die Funktion notify_soul
implementieren und kann entsprechend reagieren.
Wenn man eine Seele-Aktion vorher behandeln oder gar verhindern will, verwendet
man soul_command()!

Die Parameter bedeuten:
    who		Das ist der, der "geseelt" hat. (oder geknuddelt oder so)
    what	Das ist (in etwa) der Seele-Befehl, der benutzt wurde. Wenn
		auf spezielle Befehle reagiert werden soll, muss man eben
		ausprobieren, was die returnen.
    adverb	Das ist das benutzte Adverb, oder eben 0, wenn keins verwendet
		wurde.
BEISPIEL:
In einem Monster deklariert:

void notify_soul(object who, string what, string adverb) {
    send_message(MT_NOISE,MA_COMM,wrap_say(Der()+" sagt", "Wieso "+conv_akt(what,0)+" Du mich?"));
}

Dann meldet sich das Monster immer nach einem Seele-Kommando :).
VERWEISE: conv_akt, message, wrap_say, notify_seele, forbidden_seele,
          set_notify_soul_ob, query_notify_soul_ob_file
GRUPPEN: seele
*/

/*
FUNKTION: soul_command
DEKLARATION: deprecated int soul_command(object who, string what, string adverb)
BESCHREIBUNG:

ACHTUNG: statt  soul_command  sollte  forbidden_seele  verwendet werden!

Die Seele eruft jedesmal, BEVOR eine Aktion mit Partner ausgefuehrt
wird, diese Funktion im Partner auf. Wenn das Ergebnis 1 ist, dann
kuemmert sich der Partner um die Ausgabe saemtlicher Texte.
Damit kann man zB bei einem Monster verhindern, dass es gestreichelt
wird oder man kann aus dem Streicheln ganz was anderes machen.
In diesem Monster muss man nur noch die Funktion soul_command
implementieren und schonm kann es entsprechend reagieren.

Wenn eine Reaktion nach der Aktion erwuenscht ist, nimmt man notify_soul()!

Bedeutung der Parameter: siehe notify_soul().

BEISPIEL:
In einem Monster (ein grosser Riese) deklariert:

int soul_command(object who, string what, string adverb) {
    switch (what) {
    // Aktion abaendern
	case "tret":
	    send_message(MT_LOOK,MA_EMOTE,
	        wrap(Der(who)+" versucht gegen das Knie des Riesen zu "
		"treten, aber da er viel zu gross fuer "+ihn(who)+
		" ist, tritt "+ er(who)+" ihm lieber auf die Zehen."),
		wrap("Du versuchst gegen das Knie des Riesen zu treten, "
	        "aber da er viel zu gross fuer Dich ist, trittst "
		"Du ihm lieber auf die Zehen."), who);
	    send_message_to(this_object(),MT_FEEL,MA_EMOTE,
	        Der(who)+" tritt Dir auf die Zehen.\n");
	    // Koennte man sich fuer Monster auch sparen ;-)
	    return 1;

	// Aktion verbieten
	case "stups":
	    send_message_to(who,MT_NOTIFY,MA_EMOTE,
		"Das geht nicht, da Du nicht an den Riesen "
		"heranreichst!\n");
	    return 1;
	// Alles andere erlauben
	default:
	    return 0;
    }
}

VERWEISE: notify_seele, forbidden_seele
GRUPPEN: seele
*/
