// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:        /room/rathaus/obj/gesellenvertrag.c
// Description: Der Gesellenvertrag
// Author:

inherit "/i/item";
inherit "/i/move";
inherit "/i/tools/security";

#include <config.h>
#include <apps.h>
#include <more.h>
#include <level.h>

#define CAP(x) capitalize(x)
#define FAIL(x) { notify_fail(x); return 0; }
#define CHECK_LEO if (!previous_object() || object_name(previous_object())!= \
	"/room/rathaus/div/leo") return;

string geselle, vogt1, vogt2;
int signed;

void create() {
    set_name("gesellenvertrag");
    set_short("Der Gesellenvertrag");
    set_gender("maennlich");
    set_id(({"vertrag", "vertrach","gesellenvertrag"}));
    set_smell("Er riecht nach Arbeit. Viel Arbeit. Ganz viel Arbeit.\n");
    set_weight(0);
    init_security_for_actions();
}

int query_prevent_shadow(object shadow) { return 1; }

void init() {
    add_action("unterschreibe","unterschreibe",-12);
    add_action("unterschreibe","unterzeichne");
    add_action("zerreisse","zerreiße",-7);
}

int query_signed() { return signed; }

string query_long(object who)
{
    string tmp;

    if (!signed)
	tmp="Bisher hat noch niemand unterschrieben.";
    else if (signed==7)
	tmp="Alle haben unterschrieben.";
    else {
	tmp="";
	if (signed & 1)
	    tmp+=CAP(geselle)+" und ";
	if (signed & 2)
	    tmp+=CAP(vogt1)+" und ";
	if (signed & 4)
	    tmp+=CAP(vogt2)+" und ";
	tmp=tmp[0..<5];
	if (signed==1 || signed==2 || signed==4)
	    tmp+="hat schon unterschrieben.";
	else
	    tmp+="haben schon unterschrieben.";
	}
   return wrap("Der Gesellenvertrag ist eine Pergamentrolle, an der ein "
	"großes rotes Siegel hängt. Mit diesem Gesellenvertrag bestätigt "
	"jeder Lehrling, sich an den Götterkodex zu halten.\n"+
	(geselle ?
	"Dieser Vertrag besagt, dass "+CAP(geselle)+" Geselle bei "+
	CAP(vogt1)+" und "+(vogt2?CAP(vogt2):"einem weiteren Vogt")+
	" werden möchte.\n"+tmp:" "));
}

varargs string query_read(string rest, string str)
{
    string text;

    text = read_file("/static/goetter/vertrag.geselle");
    if(geselle && vogt1)
       text = implode(explode(implode(explode(implode(explode(text,
		"$G"), CAP(geselle)),
		"$V1"), CAP(vogt1)),
		"$V2"), vogt2?CAP(vogt2):"Vogt");
    text += "   "+implode(explode(vtimestr(vtime())," ")[0..<2]," ")+"\n\n";
    text += "   Lehrling "+(query_signed() & 1 ? CAP(geselle) : "..........")+
	    "\t"+"Lehrmeister "+(query_signed()&2? CAP(vogt1) : "..........")+
	    "\t"+"Vogt "+(query_signed()&4? CAP(vogt2)        : "..........");
    text += "\n";
    this_player()->more(explode(text,"\n"),"   --Mehr--",0,M_AUTO_END);
    return "";
}

int unterschreibe(string str)
{
    string name;

    if (!check_security())
	return 0;

    if (!me(str))
	FAIL("Was willst du unterschreiben ?\n");

    if (!playerp(this_interactive()))
	FAIL(Den()+" können nur Spieler unterschreiben.\n");

    name=this_interactive()->query_real_name();
    if (!name)
	FAIL("Huch!!! Du hast ja gar keinen Namen.\n");

    if (!geselle)
	FAIL(Dieser()+" ist noch nicht zum Unterschreiben vorbereitet.\n");

    if (name != geselle && name != vogt1 &&
    	    !GOETTER_REGISTER->query_lehrerlaubnis(name) &&
	    !adminp(this_interactive()))
	FAIL("Du musst Dir erst die Lehrerlaubnis von einem "
		"Domainlord holen.\n");

    if (name == geselle)
    {
	if (signed & 1)
	    FAIL("Du hast "+diesen()+" schon unterzeichnet.\n");
	signed |= 1;
    }
    else if (name == vogt1)
    {
	if (signed & 2)
	    FAIL("Du hast "+diesen()+" schon unterzeichnet.\n");
	signed |= 2;
    }
    else
    {
	if (!this_interactive() || this_player()!=this_interactive() ||
		!wizp(this_player()))
	    FAIL("So nicht!\n");
	vogt2 = this_player()->query_real_name();
	if (signed & 4)
	    FAIL(Dieser()+" ist schon unterzeichnet.\n");
	signed |= 4;
    }
    say(wrap(Der(this_player())+
    	" unterzeichnet mit schnörkeligen Buchstaben "+den()+"."));
    write("Du unterschreibst "+den()+".\n");
    return 1;
}

int zerreisse(string str)
{
   if(!me(str))
      FAIL("Zerreiße was?\n");
   write("Du zerreist "+deinen()+".\n");
   say(Der(this_player())+" zerreißt "+seinen()+".\n");
   remove();
   return 1;
}

void set_geselle(string str) {
    string tmp;

    if (!stringp(str) || str=="")
	return;

    CHECK_LEO;
    tmp=GOETTER_REGISTER->query_lehrmeister(str);
    if (!tmp || GOETTER_REGISTER->query_lehrmeister(str,1))
	return;
    geselle=str;
    vogt1=tmp;
    if(vogt1 == "leo")
       signed|=2;
}

string query_vogt1() { return vogt1; }
string query_vogt2() { return vogt2; }


string query_geselle() { return geselle; }

string query_look_msg()
{
   object ges;
   return geselle && (ges = present(geselle, environment(this_player()))) ?
      Der(this_player()) +" schaut "+
      (ges == this_player() ? seinen() : ihren(0,0,ges)) + " an."
     : 0;
}

string query_read_msg()
{
   object ges;
   return geselle && (ges = present(geselle, environment(this_player()))) ?
      Der(this_player())+" liest "+
      (ges == this_player() ? seinen() : ihren(0,0,ges)) + "."
     : 0;
}

mixed *query_auto_load()
{
   if(learnerp(environment()) && geselle == environment()->query_real_name())
   {
      if(playerp(previous_object()))
         call_out("do_remove", 0);
      return ({ vogt1, vogt2, signed });
   }
   return 0;
}

void do_remove()
{
   if(geselle && !find_player(geselle))
      remove();
}

void init_arg(mixed *strs)
{
   if(learnerp(previous_object()) && sizeof(strs) >= 3)
   {
      geselle = previous_object()->query_real_name();
      vogt1 = strs[0];
      vogt2 = strs[1];
      signed = strs[2];
   }
}

