// This file is part of UNItopia Mudlib.
// -----------------------------------------------------------------------
//  Datei:  /room/rathaus/obj/uiko.c
//  Autor:  Myonara 28.Sep.2014 
// -----------------------------------------------------------------------
// Beschreibung: Der AutoLoader/Kobold fuer Umfragen und Ideen
// -----------------------------------------------------------------------

inherit "/i/tools/security";
inherit "%monster/monster";
inherit "/i/tools/dynamic_browser";

#include <commands.h>
#include <database.h>
#include <dynamic_browser.h>
#include <editor.h>
#include <files.h>
#include <input_to.h>
#include <level.h>
#include <message.h>
#include <misc.h>
#include <move.h>
#include <properties.h>
#include <umfragen.h>

#define IST_RAUM    1
#define IST_SPIELER 2
#define IST_ENGEL   3
#define IST_GOTT    4
#define IST_ADMIN   5
#define IDEE_TMP_FILE ("/w/"+geteuid()\
    	              +((file_size("/w/"+geteuid()+"/priv")==FSIZE_DIR)?"/priv":"")\
	              +"/IDEE")
#define PANTHEON2TEXT(x) ("Publikum: "+((x)?"Pantheon":"Spieler und Pantheon"))
#define PDBG(x) send_message_to( ({find_player("myonara") }), \
                    MT_DEBUG, MA_UNKNOWN, wrap(x))
                  
private int umgebung, *selected_status = ({ }), *selected = ({});
private mapping pl_tmp = ([ ]);
private object master, *rclones = ({});
private int db_version;

private int uiko_lerne();
void msg_to_clones(string pre, string msg);

nomask int query_no_move() 
{
    return 1;
}

private object get_master()
{
    if (objectp(master))
        return master;
    master = find_object(UMFRAGE_MASTER) || touch(UMFRAGE_MASTER);
    return master;
}

private varargs int debuglog(string msg, int severity, string user, 
    string category)
{
    return get_master()->debuglog(msg,severity||DB_DBGLVL_DEBUG,
        user||TP_RN,category||"uiko");
}

private int query_commander(object pl)
{
    return adminp(pl);
}

private int check_ob(object env)
{
    if (!env || guestp(env)) 
        return 0;
    if (env->is_room())
        return 0; // IST_RAUM; deaktiviert, da nicht getestet.
    else if (adminp(env))
        return IST_ADMIN;
    else if (wizp(env))
        return IST_GOTT;
    else if (hlpp(env))
        return IST_ENGEL;
    else if (playerp(env))
        return IST_SPIELER;
    return 0;
}

private void check_env(object pl)
{
    umgebung = check_ob(ENV_TO);
    if (clonep() && umgebung == 0 )
    {
        call_out("remove",0);
        send_message_to( ({ pl }),MT_LOOK, MA_LOOK, wrap(
            "Der Kobold verschwindet so schnell wie er erschien."));
    }
}

string query_long(object viewer)
{
    string l = "Ein winziger Umfrage- und Ideenkobold (kurz UIKO), ";
    if (umgebung == IST_RAUM)
    {
        l += "der in einer Ecke hockt. ";
    }
    else if (viewer == ENV_TO)
    {
        l += "der Dir helfend auf der Schulter hockt. ";
    }
    else
    {
        l += "der "+ihm(ENV_TO)+" helfend auf der Schulter hockt.";
        return wrap(l);
    }
    l += "Die Befehlsliste/Hilfe ist mit idee:? bzw. i:? einsehbar.";
    return wrap(l);
    
}

private void check_version()
{
    int db_neu = get_master()->get_current_version();
    if (db_version == db_neu)
    {
        return;
    }
    db_version = db_neu;
    // TODO Nachricht ueber alle Aenderungen??
}

mixed query_auto_load()
{
    return ([
           "DB_VERSION" : db_version,
           ]);
}

void init_arg(mixed data)
{
    if (!mappingp(data))
    	return;
    if (member(data,"DB_VERSION"))
    {
        db_version = data["DB_VERSION"];
        call_out(#'check_version, 4);
    }
}

private void set_pl_tmp(object pl, string key, mixed value)
{
    if (!playerp(pl))
        return;
    if (!stringp(key))
    {
        m_delete(pl_tmp, pl);
        return;
    }
    if (value == 0)
    {
        if (!member(pl_tmp,pl))
        {
            return; //nothing to delete
        }
        else if (!member(pl_tmp[pl],key))
        {
            return; // key not existent, nothing to delete
        }
        else
        {
            m_delete(pl_tmp[pl],key);
            if (!sizeof(pl_tmp[pl]))
            {
                m_delete(pl_tmp, pl);
            }
        }
    }
    else
    {
        if (!member(pl_tmp,pl))
        {
            pl_tmp[pl] = ([ key : value ]);
        }
        else
        {
            pl_tmp[pl][key] = value;
        }
    }
}

private mixed query_pl_tmp(object pl, string key)
{
    if (member(pl_tmp,pl) && member(pl_tmp[pl],key))
        return pl_tmp[pl][key];
    return 0;
}

//---------------------------------------------------------------------------
#define L_DATA_UMFRAGE 0
#define L_DATA_THEMA   1
#define L_DATA_VARIANT 2
#define L_DATA_FRAGE   3
#define L_DATA_ANTWORT 4

#define L_OPTIONS       "ui:local:options"
#define L_DATA          "ui:local:data"
#define L_SONSTIGE      "ui:local:sonstige"
#define L_MULTI         "ui:local:multi"
#define L_ANTWORTEN     "ui:local:antworten"

private void loop_x_umfrage(mapping l_options,mapping *l_data);


//---------------------------------------------------------------------------

static mixed auswahl_x_antworten_init(mapping old)
{
    old[B_START_LINE] = 0;
    if (old[L_SONSTIGE])
        old[B_NUM_LINES] -= 2;
    else
        old[B_NUM_LINES] -= 1;
}

static int auswahl_x_antworten_total(mapping menue)
{
    return get_master()->get_xantworten(
        menue+([UI_OPT_COUNTALL:1]));
}

static string * auswahl_x_antworten_display(mapping menue)
{
    string * lines = ({});
    mapping cache = ([]);
    int i;
    mapping * antworten = get_master()->get_xantworten(menue
            +([UI_OPT_LIMIT:  menue[B_NUM_LINES],
            UI_OPT_OFFSET: menue[B_CURRENT_LINE],
            UI_OPT_MAPPING : 1]) );
    mapping frage = master->get_xfrage(menue[UI_OPT_FRAGE]);
    if (!mappingp(frage)) 
    {
        return ({"Frage vergessen."});
    }
    lines += frage[UI_OPT_FRAGE];
    if (antworten == 0 || sizeof(antworten) <= 0)
    {
        if (menue[L_SONSTIGE])
        {
            return lines + ({"Keine Antwort vorhanden",
                "s <text> um eine Antwort zu ergänzen."});
        }
        else
        {
            return lines+({"Keine Antwort vorhanden"});
        }
    }
    for (i=0;i<sizeof(antworten);i++)
    {
        int ix = i+menue[B_CURRENT_LINE]+1;
        cache[ix] = antworten[i][UI_OPT_XA_ID];
        lines += ({ sprintf("%02d %s",
            ix,antworten[i][UI_OPT_ANTWORT]||"") });
    }
    if (menue[L_SONSTIGE] && i>=menue[B_END_LINE])
    {
        lines += ({"s <text> um eine Antwort zu ergänzen."});
    }
    menue[UI_OPT_CACHE] = cache;
    return lines;
}

static string auswahl_x_antworten_prompt(mapping menue)
{
    string str,sonst="";
    if (member(menue,L_SONSTIGE))
    {
        sonst = ",s";
    }
    if (member(menue,L_MULTI))
    {
        if (menue[L_MULTI]>1)
        {
            str = sprintf("%d aus %d Antworten auswählen [<nr>,f%s]",
                menue[L_MULTI],menue[B_END_LINE],sonst);
        }
        else
        {
            str = sprintf("Beliebige Antworten auswählen (%d,%d) [<nr>,f%s]",
                menue[B_CURRENT_LINE]+1,menue[B_END_LINE],sonst);
        }
    }
    else
    {
        str = sprintf("Eine Antwort auswählen (%d,%d) [<nr>%s]",
                menue[B_CURRENT_LINE]+1,menue[B_END_LINE],sonst);
    }
    return str;
}

static mixed auswahl_x_antworten_action(string str, mapping * menues)
{
    mapping menue = menues[<1] , antworten = menue[L_ANTWORTEN];
    mapping cache = menue[UI_OPT_CACHE];
    int ix = to_int(space(str));
    if (member(cache,ix))
    {
        if (!mappingp(antworten) || !member(menue,L_MULTI))
        {
            antworten = ([ cache[ix] : 1]);
            browse_write_line("Antwort ausgewählt.");
        }
        else if (member(antworten,cache[ix]))
        {
            m_delete(antworten,cache[ix]);
            browse_write_line("Antwort gelöscht.");
        }
        else
        {
            antworten[cache[ix]] = 1;
            browse_write_line("Antwort gewählt.");
        }
        if (!member(menue,L_MULTI))
        {
            // mapping m_antwort = get_master()->get_x_antwort(cache[ix]);
            
        }
        else
        {
            menue[L_ANTWORTEN] = antworten;
            return B_REBUILT;
        }
    }
    else if (ix != 0)
    {
        browse_write_line("Zahl außerhalb des angezeigten Zahlenbereichs.");
        return B_DONE;
    }
}

//---------------------------------------------------------------------------

private void inp_gilde(string str,mapping l_options,mapping *l_data)
{
    if (str == "~q" || str == "q")
    {
        browse_write_line("Umfrage/Eingabe abgebrochen.");
        return;
    }
    mixed* l = get_master()->get_x_gilden(
        l_data[L_DATA_UMFRAGE][UI_OPT_UNIFIED_ID],
        l_options[UI_OPT_USER_NAME]);
    int ix = to_int(space(str))-1;
    if (ix < 0 || ix >= sizeof(l))
    {
        browse_write_line("Ungültige Zahl.");
        input_to(#'inp_gilde,INPUT_PROMPT,"Gilde wählen: ",l_options,l_data);
        return;
    }
    if (l[ix][0]!=0)
    {
        browse_write_line("Gilde nicht nochmal auswählbar.");
        input_to(#'inp_gilde,INPUT_PROMPT,"Gilde wählen: ",l_options,l_data);
        return;
    }
    string gilde = l[ix][1];
    l_options[UI_OPT_GILDE] = gilde;
    // TODO replacement of gilde in l_data[L_DATA_THEMA]
    l_data += ({([ UI_OPT_GILDE : gilde,UI_OPT_UNIFIED_ID : gilde,
            UI_OPT_TITLE: gilde ])});
    loop_x_umfrage(l_options,l_data);
}
private int inp2_gilde(mapping l_options,mapping *l_data)
{
    int cnt_done = 0;
    mixed* l = get_master()->get_x_gilden(
        l_data[L_DATA_UMFRAGE][UI_OPT_UNIFIED_ID],
        l_options[UI_OPT_USER_NAME]);
    if (!sizeof(l))
    {
        debuglog("Datenfehler l",DB_DBGLVL_ERROR,TP_RN,
                "uiko:inp2_gilde");
        return 0;
    }
    string line;
    for (int i=0;i<sizeof(l);i++)
    {
        if (l[i][0]) 
        {
            cnt_done++;
            line = sprintf("%02d [X] %s",i+1,l[i][1]);
        }
        else
        {
            line = sprintf("%02d [.] %s",i+1,l[i][1]);
        }
        browse_write_line(line);      
    }
    if (cnt_done >= sizeof(l))
    {
        return -2; // end of guild data
    }
    input_to(#'inp_gilde,INPUT_PROMPT,"Gilde wählen: ",l_options,l_data);
    return -1; // InputData.
}

private string update_unified_path(mapping l_options,mapping *l_data)
{
    string *uids = ({});
    for (int i = 0;i<sizeof(l_data);i++)
    {
        uids += ({ l_data[i][UI_OPT_UNIFIED_ID] });
    }
    l_options[UI_OPT_UNIFIED_PATH] = uids;
    return l_options[UI_OPT_UNIFIED_ID] = implode(uids,"|");;
}

private int get_next_x_thema(mapping l_options,mapping *l_data)
{
    mapping l_thema;
    switch (sizeof(l_data))
    {
        case L_DATA_UMFRAGE:
            debuglog("Parameterfehler l_data",DB_DBGLVL_ERROR,TP_RN,
                "uiko:get_next_x_thema");
            return 0;
        case L_DATA_THEMA:
            l_thema = master->get_next_x_thema(
                    l_data[L_DATA_UMFRAGE][UI_OPT_XU_ID],0);
            break;
        default: 
            l_thema = l_data[L_DATA_UMFRAGE];
            if (strstr(l_thema[UI_OPT_UNIFIED_ID],"<gilde>")!=-1)
            {
                break;
            }
            else
            {
                l_thema = master->get_next_x_thema(
                    l_data[L_DATA_UMFRAGE][UI_OPT_XU_ID],
                    l_data[L_DATA_THEMA][UI_OPT_SEQUENCE]);
                break;
            }
    }
    if (!mappingp(l_thema))
    {
        return -3; // kein weiteres Thema mehr...
    }
    l_data = l_data[0..0] + ({ l_thema });
    if (strstr(l_thema[UI_OPT_UNIFIED_ID],"<gilde>")!=-1)
    {
        switch (inp2_gilde(l_options,l_data))
        {
            case -1: // input_to
                return -1;
            case 0: // Fehler
                return 0;
            case -2: // gilde fertig
                l_thema = master->get_next_x_thema(
                    l_data[L_DATA_UMFRAGE][UI_OPT_XU_ID],
                    l_data[L_DATA_THEMA][UI_OPT_SEQUENCE]);
                if (!mappingp(l_thema))
                {
                    return -3; // kein weiteres Thema mehr...
                }
                // <gilde> soll nur einmal vorkommen!
                l_data = l_data[0..0] + ({ l_thema });
                
        }
        // return -2: end of gilden, -1 input_to aktiv, 0 fehler.
    }
    l_data += ({([
            UI_OPT_UNIFIED_ID:"NONV",
            UI_OPT_TITLE:l_thema[UI_OPT_TITLE] 
            ])});
    return 1;
}

private int get_next_x_frage(mapping l_options,mapping *l_data)
{
    mapping l_frage;
    switch (sizeof(l_data))
    {
        case L_DATA_UMFRAGE: 
        case L_DATA_THEMA: 
            debuglog("Parameterfehler l_data",DB_DBGLVL_ERROR,TP_RN,
                "uiko:get_next_x_frage");
            return 0;
        case L_DATA_VARIANT:
            l_frage = master->get_next_x_frage(
                    l_data[L_DATA_THEMA][UI_OPT_XT_ID],0);
            break;
        default: 
            l_frage = master->get_next_x_thema(
                    l_data[L_DATA_THEMA][UI_OPT_XT_ID],
                    l_data[L_DATA_FRAGE][UI_OPT_SEQUENCE]);
            break;
    }
    if (!mappingp(l_frage))
    {
        return -3; // keine frage mehr
    }
    l_data = l_data[0..2] + ({ l_frage });
    return 1;
}

private void list_zeit3_3_10(string val)
{
    browse_write_line("(1) weniger als drei Monate");
    browse_write_line("(2) weniger als drei Jahre");
    browse_write_line("(3) weniger als 10 Jahre");
    browse_write_line("(4) mehr als 10 Jahre");
    if (val)
        browse_write_line("Vorherige Eingabe: "+val);
}

private void inp_frage_zeit3_3_10(string str,string uidp,string val,
    mapping l_options,mapping *l_data)
{
    if (str == "q" || str == "~q")
    {
        browse_write_line("Eingabe abgebrochen.");
        return;
    }
    int ausw = to_int(space(str));
    if (ausw < 1 || ausw > 4)
    {
        list_zeit3_3_10(val);
        input_to(#'inp_frage_zeit3_3_10,INPUT_PROMPT,
            "Eine Antwort 1 bis 4 wählen: ",uidp,val,l_options,l_data);
        return;
    }
    get_master()->set_x_auswertung(
        uidp,
        l_options[UI_OPT_USER_NAME],
        to_string(ausw),
        l_options[UI_OPT_USER_TYPE],
        l_options[UI_OPT_GILDE]
    );
    loop_x_umfrage(l_options,l_data);
}

private void list_schlecht5gut(string val)
{
    browse_write_line("(1) sehr schlecht");
    browse_write_line("(2) eher schlecht");
    browse_write_line("(3) neutral");
    browse_write_line("(4) eher gut");
    browse_write_line("(5) sehr gut");
    if (val)
        browse_write_line("Vorherige Eingabe: "+val);
}

private void inp_frage_schlecht5gut(string str,string uidp,string val,
    mapping l_options,mapping *l_data)
{
    if (str == "q" || str == "~q")
    {
        browse_write_line("Eingabe abgebrochen.");
        return;
    }
    int ausw = to_int(space(str));
    if (ausw < 1 || ausw > 5)
    {
        list_schlecht5gut(val);
        input_to(#'inp_frage_schlecht5gut,INPUT_PROMPT,
            "Eine Antwort 1 bis 5 wählen: ",uidp,val,l_options,l_data);
        return;
    }
    get_master()->set_x_auswertung(
        uidp,
        l_options[UI_OPT_USER_NAME],
        to_string(ausw),
        l_options[UI_OPT_USER_TYPE],
        l_options[UI_OPT_GILDE]
    );
    loop_x_umfrage(l_options,l_data);
}

private void list_antworten(string uidp,string val,
    mapping l_options,mapping *l_data)
{
    
}

private void inp_frage_frei_einzel(string str,string uidp,string val,
    mapping l_options,mapping *l_data)
{
    
}

private void inp_frage_frei_multi(string str,string uidp,string val,
    mapping l_options,mapping *l_data)
{
    
}

private int inp2_frage(string uidp,string val,
    mapping l_options,mapping *l_data)
{
    mapping l_frage = l_data[L_DATA_FRAGE];
    switch (l_frage[UI_OPT_FRAGEN_TYP])
    {
        case UI_FRAGEN_TYP_FREI:
            list_antworten(uidp,val,l_options,l_data);
            switch (l_frage[UI_OPT_ANZ_ANTWORTEN])
            {
                case 1:
                    input_to(#'inp_frage_frei_einzel,INPUT_PROMPT,
        "Eine Antwort über Nummer auswählen: ",
                        uidp,val,l_options,l_data);
                    return -1;
                case 0:
                    input_to(#'inp_frage_frei_multi,INPUT_PROMPT,
        "Beliebige Antworten über Nummer auswählen, f wenn fertig: ",
                        uidp,val,l_options,l_data);
                    return -1;
                default:
                    input_to(#'inp_frage_frei_multi,INPUT_PROMPT,
        sprintf("%d Antworten über Nummer auswählen, f wenn fertig: ",
                        l_frage[UI_OPT_ANZ_ANTWORTEN]),
                        uidp,val,l_options,l_data);
                    return -1;
            }
        case UI_FRAGEN_TYP_FREI_ERWEIT:
            list_antworten(uidp,val,l_options,l_data);
            switch (l_frage[UI_OPT_ANZ_ANTWORTEN])
            {
                case 1:
                    input_to(#'inp_frage_frei_einzel,INPUT_PROMPT,
  "Eine Antwort über Nummer auswählen, s für sonstige Antwort ergänzen: ",
                        uidp,val,l_options,l_data);
                    return -1;
                case 0:
                    input_to(#'inp_frage_frei_multi,INPUT_PROMPT,
  "Beliebige Antworten über Nummer auswählen, s für sonstige Antworten\n"
  "f für fertig: ",
                        uidp,val,l_options,l_data);
                    return -1;
                default:
                    input_to(#'inp_frage_frei_multi,INPUT_PROMPT,
  sprintf("%d Antworten über Nummer auswählen, s für sonstige Antworten\n"
  "f für fertig: ", l_frage[UI_OPT_ANZ_ANTWORTEN]),
                        uidp,val,l_options,l_data);
                    return -1;
            }
        case UI_FRAGEN_TYP_ZEIT3_3_10:
            list_zeit3_3_10(val);
            input_to(#'inp_frage_zeit3_3_10,INPUT_PROMPT,
                "Eine Antwort 1 bis 4 wählen: ",uidp,val,l_options,l_data);
            return -1;
        case UI_FRAGEN_TYP_SCHLECHT5GUT:
            list_schlecht5gut(val);
            input_to(#'inp_frage_schlecht5gut,INPUT_PROMPT,
                "Eine Antwort 1 bis 5 wählen: ",uidp,val,l_options,l_data);
            return -1;
        case UI_FRAGEN_TYP_KOMMENTAR:
            // TODO editor!
        default:
        case UI_FRAGEN_TYP_GILDE:
            browse_write_line("Unbekannter Fragentyp");
            return 0;
    }
    return 0;
}

private void loop_x_umfrage(mapping l_options,mapping *l_data)
{
    mapping m_data;
    if (!mappingp(l_options))
    {
        debuglog("Parameterfehler l_options",DB_DBGLVL_ERROR,TP_RN,
            "uiko:loop_x_umfrage");
        return;
    }
    string usr = l_options[UI_OPT_USER_NAME];
    if (!stringp(usr)||!player_exists(usr))
    {
        debuglog("Parameterfehler usr",DB_DBGLVL_ERROR,TP_RN,
            "uiko:loop_x_umfrage");
        return;
    }
    if (!sizeof(l_data))
    {
        m_data = get_master()->get_x_umfrage(l_options[UI_OPT_XU_ID]);
        if (!mappingp(m_data))
        {
            debuglog("Parameterfehler xu_id",DB_DBGLVL_ERROR,usr,
                "uiko:loop_x_umfrage");
            return;
        }
        l_data = ({m_data});
    }
    while (42)
    {
        string uidp = update_unified_path(l_options,l_data);
        string val = master->get_x_auswertung(uidp,usr);
        int update_level = -1;
        switch (sizeof(l_data))
        {
            case 1: // umfrage
                if (!val || 
                    ((l_options[UI_OPT_LOOP_MODE] & UI_LOOP_RESTART)>0) )
                {
                    browse_write_line("Start Umfrage: "+
                        l_data[L_DATA_UMFRAGE][UI_OPT_TITLE]);
                }
                update_level = L_DATA_THEMA;
                break;
            case 2: // umfrage|thema
                debuglog("Level 2 thema ohne variant?",DB_DBGLVL_ERROR,usr,
                    "uiko:loop_x_umfrage");
                return;
            case 3: // umfrage|thema|variant/gilde
                update_level = L_DATA_FRAGE;
                break;
            case 4: // umfrage|thema|variant/gilde|frage
                switch (inp2_frage(uidp,val,l_options,l_data))
                {
                    case 0: // Fehler
                        return; 
                    case -1:
                        return; // input to aktiv...
                }
        }
        if (update_level == L_DATA_FRAGE)
        {
            switch(get_next_x_frage(l_options,l_data))
            {
                case -3:
                    update_level = L_DATA_THEMA;
                    break;
                case 0:
                    return;
                case 1:
                    continue;
            }
        }
        if (update_level == L_DATA_THEMA)
        {
            uidp = implode(l_options[UI_OPT_UNIFIED_PATH][0..2],"|");
            get_master()->set_x_auswertung(
                uidp,
                l_options[UI_OPT_USER_NAME],
                "START",
                l_options[UI_OPT_USER_TYPE],
                l_options[UI_OPT_GILDE]
            );
            switch(get_next_x_thema(l_options,l_data))
            {
                case -3:
                    browse_write_line("Ende der Umfrage erreicht.");
                    // TODO Endemarkierung?
                    return;
                case -2: // keine weiteren gilden, wurde abgefangen
                    continue;
                case -1: // input_to ist aktiv
                    return;
                case 0: // fehler
                    return;
                case 1: // no variant
                    continue;
            }
        }
    }
}

//---------------------------------------------------------------------------

private void idee_neu_pantheon(string str)
{
    int flag;
    switch (lower_case(space(str)))
    {
    case "~q":
    case ".":
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Idee abgebrochen."));
        set_pl_tmp(TP, 0, 0);
        return;
    case "s":
        flag = 0;
        break;
    case "p":
        flag = 1;
        break;
    default:
        input_to(#'idee_neu_pantheon, INPUT_PROMPT, 
            "Sollen die Idee (S)pieler und Pantheon \n"
            "oder nur das (P)antheon sehen? (S/P/~q):");
        return;
    }
    int i_id = get_master()->add_idee(
                    query_pl_tmp(TP,"Titel"),
                    query_pl_tmp(TP,"Inhalt"),
                    TP_RN, UMFRAGE_S_NEU, flag);
    set_pl_tmp(TP, 0, 0);
    if (i_id <= 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Fehler beim Anlegen der Idee."));
        return;
    }
    int u_id = master->add_umfrage(
                    "Wie gefällt Dir die Idee?",
                    TP_RN, UMFRAGE_S_NEU, i_id, flag);
    if (u_id <= 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Fehler beim Anlegen der Umfrage zur Idee."));
        return;
    }
    if (  master->add_antwort(u_id, "Mag ich") <= 0
        || master->add_antwort(u_id, "Neutral") <= 0
        || master->add_antwort(u_id, "Mag ich nicht") <= 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Fehler beim Anlegen der Antworten zur Idee."));
        return;
    }
    if ( master->aktiviere_idee_und_umfrage(i_id, u_id) <= 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Fehler beim Aktivieren der Idee."));
        return;
    }
    send_message_to(TP, MT_LOOK, MA_READ, 
        wrap("Idee erfolgreich bereitgestellt, warten auf Adminfreigabe."));
    PDBG("Neue Idee von "+TP_RCN);
}

private void idee_neu_rumpf(string *puffer)
{
    if (puffer == 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Idee nicht gespeichert."));
        set_pl_tmp(TP, 0, 0);
        return;
    }
    // TODO Inhalt pruefen
    set_pl_tmp(TP, "Inhalt", implode(puffer,"\n"));
    input_to(#'idee_neu_pantheon, INPUT_PROMPT, 
        "Sollen es (S)pieler und Pantheon \n"
        "oder nur das (P)antheon sehen (S/P/~q):");
}

private void idee_neu_titel(string titel)
{
    titel = space(titel);
    if (titel == "" || lower_case(titel) == "~q" || titel == ".")
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Idee abgebrochen."));
        return;
    }
    if (strlen(titel)>70) 
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Titel zu lang (70)."));
        return;
    }
    set_pl_tmp(TP, 0, 0);
    set_pl_tmp(TP, "Titel", titel);
    TP->mini_ed(#'idee_neu_rumpf, 0,
      TP->uses_gmcp_edit() ? 0 : IDEE_TMP_FILE, 
      ([
         MINI_ED_START_TEXT:
            "Gib nun die Ideenzusammenfassung an\n"+
            "Mit '**' oder '.' beenden, mit '~q' abbrechen.\n",
         MINI_ED_TITLE:
            titel,
      ]),
      ([
         MINI_ED_FORCE_WRAP: 1
      ]), 0);    
}

private int idee_neu()
{
    string intro = 
"Bitte gib einen kurzen und prägnanten Titel ein gefolgt von einer "
"Zusammenfassung Deiner Idee. Die Erfahrung zeigt, dass zu lange Texte "
"von anderen nicht gelesen werden. Die Ideen werden den anderen Spielern "
"anonym zur Wahl gestellt, aber die Admins können den Ersteller einsehen. "
"Mit ~q am Zeilen Anfang kannst Du ohne zu speichern abbrechen.";
    
    send_message_to(TP, MT_LOOK, MA_READ, wrap(intro));
    
    input_to(#'idee_neu_titel, INPUT_PROMPT, "Titel:");
    return 1;
}

private void umfrage_neue_antwort(string str, mixed menues, int u_id)
{
    str = space(str);
    get_master();
    switch (str)
    {
    case ".":
    case "~q":
        browse_write_line("Eingabe neue Antwort abgebrochen");
        break;
    case "":
        input_to(#'umfrage_neue_antwort, INPUT_PROMPT, 
                         "Neue Antwort:",menues,u_id);
        return;
    default:
        if (master->add_antwort(u_id, str))
        {
            browse_write_line("Antwort eingefügt.");
            break;
        }
        else
        {
            browse_write_line("Antwort nicht eingefügt.");
            break;
        }
    }
    dynamic_browse(menues);
}

private void umfrage_neu_pantheon(string str, mixed menues, int i_id, 
        string frage)
{
    int flag, u_id;
    switch (lower_case(space(str)))
    {
    case "~q":
    case ".":
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Umfrage abgebrochen."));
        return;
    case "s":
        flag = 0;
        break;
    case "p":
        flag = 1;
        break;
    default:
        input_to(#'umfrage_neu_pantheon, INPUT_PROMPT, 
            "Soll die Umfrage von (S)pieler und Pantheon \n"
            "oder nur vom (P)antheon gesehen werden? (S/P/~q):",
            menues,i_id, frage);
        return;
    }
    u_id = get_master()->add_umfrage(frage,TP_RN,UMFRAGE_S_NEU,i_id,flag);
    if (u_id == 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap(
            "Umfrage aufgrund eines internen Fehler nicht angelegt."));
        return;
    }
    if (pointerp(menues))
    {
        menues[<1][B_FLAGS] |= BF_DIRTY;
        dynamic_browse(menues + ({([
                        B_TYPE : "umfrage_wahl",
                        UI_OPT_U_ID : u_id,
                        B_START_LINE : 0
                    ]) }) );
    }
    else
    {
        dynamic_browse( ([
                        B_TYPE : "umfrage_wahl",
                        UI_OPT_U_ID : u_id,
                        B_START_LINE : 0
                    ]) );
    }
    browse_write_line("Umfrage angelegt, Antworten einfügen.");
}

private void umfrage_neu_frage(string frage, mixed menues, int i_id)
{
    frage = space(frage);
    if (frage=="~q" || frage==".")
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Umfrage abgebrochen."));
        return;
    }
    if (frage == "" || strlen(frage)> 70)
    {
        input_to(#'umfrage_neu_frage, INPUT_PROMPT, "Frage:", menues, i_id);
        return;
    }
    input_to(#'umfrage_neu_pantheon, INPUT_PROMPT, 
            "Soll die Umfrage von (S)pieler und Pantheon \n"
            "oder nur vom (P)antheon gesehen werden? (S/P/~q):",
            menues,i_id, frage);
}

varargs private int umfrage_neu(mixed menues, int i_id)
{
    string intro =
"Bitte gib die Frage der Umfrage ein, die neu zu erstellen ist. "
"Abbruch jederzeit mit ~q oder einem einzelnen Punkt möglich.";
    send_message_to(TP, MT_LOOK, MA_READ, wrap(intro));
    
    input_to(#'umfrage_neu_frage, INPUT_PROMPT, "Frage:", menues, i_id);
    return 1;
}

private void diskussion_neu_rumpf(string *puffer)
{
    if (puffer == 0)
    {
        send_message_to(TP, MT_LOOK, MA_READ, 
            wrap("Diskussionsbeitrag nicht gespeichert."));
        mixed menues = query_pl_tmp(TP, "Menüs");
        set_pl_tmp(TP, 0, 0);
        dynamic_browse(menues);
        return;
    }
    // TODO Inhalt pruefen
    set_pl_tmp(TP, "Inhalt", implode(puffer,"\n"));
    switch(get_master()->add_diskussionsbeitrag(
            query_pl_tmp(TP, "d_id"),TP_RN,
            query_pl_tmp(TP, "Titel"),
            query_pl_tmp(TP, "Inhalt")))
    {
    case 0:
    case -2:
        browse_write_line("Interner Fehler bei der Beitragserstellung.");
        break;
    case -1: 
        browse_write_line("Diskussion nimmt keine neuen Beiträge an.");
        break;
    case 1:
        browse_write_line("Beitrag angelegt.");
        break;
    break;
    }
    mixed menues = query_pl_tmp(TP, "Menüs");
    menues[<1][B_FLAGS] |= BF_DIRTY;
    set_pl_tmp(TP, 0, 0);
    dynamic_browse(menues);
}

private void diskussion_neu_titel(string titel, mixed menues, int d_id)
{
    titel = space(titel);
    if (titel == "" || lower_case(titel) == "~q" || titel == ".")
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Beitrag abgebrochen."));
        dynamic_browse(menues);
        return;
    }
    if (strlen(titel)>50) 
    {
        send_message_to(TP, MT_LOOK, MA_READ, wrap("Titel zu lang (50)."));
        dynamic_browse(menues);
        return;
    }
    set_pl_tmp(TP, 0, 0);
    set_pl_tmp(TP, "Titel", titel);
    set_pl_tmp(TP, "Menüs", menues);
    set_pl_tmp(TP, "d_id", d_id);
    
    TP->mini_ed(#'diskussion_neu_rumpf, 0,
      TP->uses_gmcp_edit() ? 0 : IDEE_TMP_FILE, 
      ([
         MINI_ED_START_TEXT:
            "Gib nun den Diskussionsbeitrag an\n"+
            "Mit '**' oder '.' beenden, mit '~q' abbrechen.\n",
         MINI_ED_TITLE:
            titel,
      ]),
      ([
         MINI_ED_FORCE_WRAP: 1
      ]), 0);    
}

private int diskussionsbeitrag_neu(mixed menues, int d_id)
{
    string intro =
"Bitte gib den Diskussionsbeitrag mit Titel und Inhalt an. "
"Abbruch jederzeit mit ~q oder einem einzelnen Punkt möglich.";
    send_message_to(TP, MT_LOOK, MA_READ, wrap(intro));
    
    input_to(#'diskussion_neu_titel, INPUT_PROMPT, "Titel:", menues, d_id);
    return 1;
}


#define HILFE_TEXT \
"Im folgenden wird eine alternative kurzform in Klammern angegeben:\n"\
"idee:neu(i:n) legt eine neue Idee mit Standardumfrage an.\n" \
"idee:wahl(i:w) zeigt die aktiv zur Wahl stehenden Ideen in einem Menu an.\n" \
"idee:offeneStimmen (i:o) zeigt nur die Ideen an, deren Wahl aussteht.\n" \
"idee:meine(i:m) zeigt die eigenen Ideen an, unabhaengig vom Status.\n" \
"idee:aktuell(i:a) zeigt die aktuell zur Diskussion stehende Idee an.\n" \
"idee:diskussion(i:d) zeigt die zur Diskussion stehenden Ideen an.\n" \
"idee:ignoriert zeigt alle ignorierten Ideen an.\n" \
"uiko:lerne   Falls noetig, verreist Uiko kurz, um was neues zu lernen.\n" \
"uiko:verjage  verjagt den Kobold.\n"

#define HILFE_TEXT_ADMIN \
"Befehle nur fuer Admins:\n" \
"idee:bereit(i:b) zeigt die auf Freigabe wartenden Ideen in einem Menu an.\n" \
"idee:alle zeigt Ideen unabhaengig vom Status an.\n" \
"idee:aktuellaus deaktiviert die aktuelle Idee wieder.\n" \
"idee:auswertung-s zeigt die Auswertung aller Spielerideen an.\n" \
"idee:auswertung-p zeigt die Auswertung aller Pantheonsideen an.\n" \
"idee:auswertung-sp zeigt die Auswertung aller Ideen(Spieler und Pantheon)an.\n" \
"umf:neu (u:n) legt eine neue Umfrage unabhaengig von einer Idee an.\n"\
"uiko:sql <query> setzt ein sql-Kommando an die Datenbank ab.\n" \
"uiko:tablelist <table> zeigt alle oder eine bestimmte Tabelle an.\n"\
"uiko:dbglog zeigt das debuglog der Datenbank an.\n"

int idee_cmd(string str)
{
    if (!check_security())
        return 0;
    str = lower_case(space(str)); // behandelt zuviele spaces und 0=>""
    switch (str)
    {
    default:
    case "":
    case "hilfe":
    case "?":
        send_message_to(TP, MT_LOOK, MA_READ, 
            query_commander(TP) ? HILFE_TEXT+HILFE_TEXT_ADMIN : HILFE_TEXT
            );
        return 1;
    case "n":
    case "neu":
        return idee_neu();
    case "a":
    case "aktuell":
        dynamic_browse( ([ B_TYPE:"idee_aktuell" ]) );
        return 1;
    case "aktuellaus":
        if (query_commander(TP))
        {
            get_master()->set_aktuelle_idee(0);
            browse_write_line("Aktuelle Idee zurückgesetzt.");
        }
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "alle":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                               UI_OPT_MENUTYPE : UI_MENUTYPE_ALLE,
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "w":
    case "wahl":
        dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                           UI_OPT_MENUTYPE : UI_MENUTYPE_WAHL,
                           UI_OPT_STATUS : ({UMFRAGE_S_AKTIV,
                                             UMFRAGE_S_OFFEN})
                         ]) );
        return 1;
    case "o":
    case "offen":
    case "offenestimmen":
        dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                           UI_OPT_MENUTYPE : UI_MENUTYPE_OFFEN,
                           UI_OPT_UMFRAGE_OFFEN : 1,
                           UI_OPT_STATUS : ({UMFRAGE_S_AKTIV,
                                             UMFRAGE_S_OFFEN})
                         ]) );
        return 1;
    case "m":
    case "meine":
        dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                           UI_OPT_MENUTYPE : UI_MENUTYPE_OWN,
                           UI_OPT_ERSTELLER : TP_RN]) );
        return 1;
    case "b":
    case "bereit":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                               UI_OPT_MENUTYPE : UI_MENUTYPE_BEREIT,
                               UI_OPT_STATUS : ({UMFRAGE_S_BEREIT})
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "d":
    case "diskussion":
            dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                               UI_OPT_MENUTYPE : UI_MENUTYPE_DISKU,
                               UI_OPT_DISKUSSION : 1
                           ]) );
            return 1;
    case "keinediskussion":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                               UI_OPT_MENUTYPE : UI_MENUTYPE_NDISKU,
                               UI_OPT_OHNE_DISKUSSION : 1
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "ignoriert":
            dynamic_browse( ([ B_TYPE:"idee_uebersicht",
                               UI_OPT_MENUTYPE : UI_MENUTYPE_IGNORIERT,
                               UI_OPT_IGNORELIST : 2
                           ]) );
        return 1;
    case "auswertung-s":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"ideen_auswertung",
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "auswertung-p":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"ideen_auswertung",
                               UI_OPT_PANTHEON : -1,
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    case "auswertung-sp":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"ideen_auswertung",
                               UI_OPT_PANTHEON : 1,
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    }
}

int umfrage_cmd(string str)
{
    if (!check_security())
        return 0;
    str = lower_case(space(str)); // behandelt zuviele spaces und 0=>""
    switch (str)
    {
    default:
    case "":
    case "hilfe":
    case "?":
        send_message_to(TP, MT_LOOK, MA_READ, 
            query_commander(TP) ? HILFE_TEXT+HILFE_TEXT_ADMIN : HILFE_TEXT
            );
        return 1;
    case "n":
    case "neu":
        if (!query_commander(TP))
        {
            FAIL("Kommando ist reserviert.\n");
        }
        return umfrage_neu();
    case "w":
    case "wahl":
        if (query_commander(TP))
            dynamic_browse( ([ B_TYPE:"x_umfragen_wahl",
                           ]) );
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return 1;
    }
   return 0;
}

int uiko_cmd(string str)
{
    if (!check_security())
        return 0;
    string * parts = explode(space(str), " ");
    string param = implode(parts[1..]," ");
    switch (lower_case(parts[0]))
    {
    default:
    case "":
    case "hilfe":
    case "?":
        send_message_to(TP, MT_LOOK, MA_READ, 
            query_commander(TP) ? HILFE_TEXT+HILFE_TEXT_ADMIN : HILFE_TEXT
            );
        return 1;
    case "lern":
    case "lerne":
    case "lernen":
        return uiko_lerne();
    case "verjage":
        send_message_to(TP, MT_NOISE, MA_COMM, wrap(
            "Der Kobold schaut dich traurig an bevor er verschwindet "
            "und flüstert dir zu: Du weißt ja, "
            "wo du mich wiederfinden kannst."));
        call_out("remove",0);
        return 1;
    case "sql":
        if (!query_commander(TP)) 
            FAIL("Kein Zugriff");
        return get_master()->sql(param);
    case "tablelist":
        if (!query_commander(TP)) 
            FAIL("Kein Zugriff");
        return get_master()->tablelist(param);
    case "dbglog":
        if (!query_commander(TP)) 
            FAIL("Kein Zugriff");
        TP->more(get_master()->dbglog());
        return 1;
    }
   return 1;
}

void init()
{
    switch (check_ob(TP))
    {
    case IST_ADMIN: 
    case IST_GOTT:
    case IST_ENGEL:
    case IST_SPIELER:
        add_action("umfrage_cmd","umf:",AA_NOSPACE);
        add_action("umfrage_cmd","u:",AA_NOSPACE);
        add_action("idee_cmd", "idee:",AA_NOSPACE);
        add_action("idee_cmd", "i:",AA_NOSPACE);
        add_action("uiko_cmd", "uiko:",AA_NOSPACE);
        PDBG("uiko:init: "+TP_RCN);
    }
}

private string get_menue_path(mixed menues)
{
    if (mappingp(menues))
    {
        return menues[B_TYPE];
    }
    if (pointerp(menues)&&sizeof(menues))
    {
        string path = menues[<1][B_TYPE];
        for (int i=sizeof(menues)-2;i>=0 ; i--)
        {
            path += "/" + menues[i][B_TYPE];
        }
        return path;
    }
    return "";
}

private string get_status(int st)
{
    switch (st)
    {
    case UMFRAGE_S_GELOESCHT:  return "Gelöscht";
    case UMFRAGE_S_NEU:        return "Neu";
    case UMFRAGE_S_STANDARD:   return "Standard";
    case UMFRAGE_S_BEREIT:     return "Bereit";
    case UMFRAGE_S_AKTIV:      return "Aktiv";
    case UMFRAGE_S_OFFEN:      return "Offen";
    case UMFRAGE_S_BEENDET:    return "Beendet";
    case UMFRAGE_S_ABGELEHNT:  return "Abgelehnt";
    case UMFRAGE_S_ARCHIVIERT: return "Archiviert";
    default:                   return "";
    }
}

private int get_status_id(string st)
{
    switch (st)
    {
    case "Gelöscht":  return UMFRAGE_S_GELOESCHT;
    case "Neu":        return UMFRAGE_S_NEU;
    case "Standard":   return UMFRAGE_S_STANDARD;
    case "Bereit":     return UMFRAGE_S_BEREIT;
    case "Aktiv":      return UMFRAGE_S_AKTIV;
    case "Offen":      return UMFRAGE_S_OFFEN;
    case "Beendet":    return UMFRAGE_S_BEENDET;
    case "Abgelehnt":  return UMFRAGE_S_ABGELEHNT;
    case "Archiviert": return UMFRAGE_S_ARCHIVIERT;
    default:                   return -1;
    }
}

private mixed neue_diskussion(string was, int id, int flags)
{
    get_master();
    int i = 0;
    if (flags & 1)
    {
        switch(master->add_diskussion(was,id,0,UMFRAGE_S_BEREIT))
        {
        case 1: 
            i++; 
        case -1:
            break;
        case -2:
        case 0:
            browse_write_line("Interner Fehler beim Diskussion anlegen");
            return B_DONE;
           
        }
    }
    if (flags & 2)
    {
        switch(master->add_diskussion(was,id,1,UMFRAGE_S_BEREIT))
        {
        case 1: 
            i++; 
        case -1:
            break;
        case -2:
        case 0:
            browse_write_line("Interner Fehler beim Diskussion anlegen");
            return B_DONE;
           
        }
    }
    switch (i)
    {
    default:
    case 0: 
        browse_write_line("Keine Diskussion angelegt, schon vorhanden.");
        return B_DONE;
    case 1:
        browse_write_line("Eine Diskussion angelegt.");
        return B_DONE;
    case 2:
        browse_write_line("Zwei Diskussionen angelegt.");
        return B_DONE;
    }
}

/*
//---------------------------------------------------------------------------

static mixed <type>_init(mapping old)
{
}

static int <type>_total(mapping menue)
{
}

static string * <type>_display(mapping menue)
{
}

static string <type>_prompt(mapping menue)
{
}

static mixed <type>_action(string str, mapping * menues)
{
}

*/

#define HELP_standard ({ \
"Im folgenden werden die Standardbefehle erlaeutert wobei die Angabe ohne ''",\
"anzugeben sind: 'd' blaettert eine Seite runter 'u' eine Seite nach oben.",\
"'<' blaettert an den Anfang zurueck, '>' blaettert ans Ende. ",\
"'q' Beendet das Menu, 'z' geht eine Menuebene hoeher, falls verfuegbar.",\
"'?' zeigt diese Hilfe.", \
})

#define HELP_idee_uebersicht_wahl ({ \
"In dieser Ansicht kann man die zur Wahl aktiven Ideen sehen. Diese sind ", \
"durchnummeriert und koennen ueber Angabe der Nummer angewaehlt werden. ",  \
"Mit f <begriff> kann in den Ideen nach einem Begriff gefiltert werden. ", \
"Mit N sind alle Ideen mit ungelesenen Diskussionen markiert. ", \
"mit gelesen/ungelesen kann man alle Ideen entsprechend markieren. ", \
})

#define HELP_idee_uebersicht_meine ({ \
"In dieser Ansicht kann man die eigenen Ideen sehen. Diese sind ", \
"durchnummeriert und koennen ueber Angabe der Nummer angewaehlt werden. ",\
"Mit f <begriff> kann in den Ideen nach einem Begriff gefiltert werden. ", \
"Mittels dem Befehl 'status' kann man nach Status filtern.",\
})


static mixed idee_uebersicht_init(mapping old)
{
    old[B_START_LINE] = 1;
    switch (old[UI_OPT_MENUTYPE])
    {
    case UI_MENUTYPE_WAHL:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:wahl-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_wahl+HELP_standard ]);
        old[B_PROMPT] = "idee:wahl";
        break;
    case UI_MENUTYPE_OWN:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:meine-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_meine+HELP_standard ]);
        old[B_PROMPT] = "idee:meine";
        break;
    case UI_MENUTYPE_BEREIT:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:bereit-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_meine+HELP_standard ]);
        old[B_PROMPT] = "idee:bereit";
        break;
    case UI_MENUTYPE_ALLE:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:alle-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_meine+HELP_standard ]);
        old[B_PROMPT] = "idee:alle";
        break;
    case UI_MENUTYPE_OFFEN:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:offeneStimmen-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_wahl+HELP_standard ]);
        old[B_PROMPT] = "idee:offeneStimmen";
        break;
    case UI_MENUTYPE_DISKU:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:diskussion-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_wahl+HELP_standard ]);
        old[B_PROMPT] = "idee:diskussion";
        old[UI_OPT_DISKUSSION] = 1;
        break;
    case UI_MENUTYPE_NDISKU:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:keinediskussion-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_wahl+HELP_standard ]);
        old[UI_OPT_OHNE_DISKUSSION] = 1;
        old[B_PROMPT] = "idee:keinediskussion";
        break;
    case UI_MENUTYPE_IGNORIERT:
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu idee:ignoriert-Übersicht Zeile",
                         B_DATA : HELP_idee_uebersicht_wahl+HELP_standard ]);
        old[B_PROMPT] = "idee:ignoriert";
        break;
    }
    return old;
}

private mapping idee_uebersicht_suchmuster(mapping menue)
{
    mapping opt = ([ UI_OPT_PANTHEON : wizp(TP) ]);
    mixed data;
    if (member(menue, UI_OPT_SUCHFILTER))
    {
        opt[UI_OPT_SUCHFILTER] = menue[UI_OPT_SUCHFILTER];
    }
    if (member(menue, UI_OPT_ERSTELLER))
    {
        opt[UI_OPT_ERSTELLER] = menue[UI_OPT_ERSTELLER];
    }
    if (member(menue, UI_OPT_UMFRAGE_OFFEN))
    {
        opt[UI_OPT_UMFRAGE_OFFEN] = menue[UI_OPT_UMFRAGE_OFFEN];
    }
    if (member(menue, UI_OPT_STATUS))
    {
        data = menue[UI_OPT_STATUS];
        if (intp(data))
        {
            selected_status = opt[UI_OPT_STATUS] = ({ menue[UI_OPT_STATUS] });
        }
        else if (pointerp(data))
        {
            selected_status = opt[UI_OPT_STATUS] = menue[UI_OPT_STATUS];
        }
        else if (sizeof(selected_status))
        {
            opt[UI_OPT_STATUS] = selected_status;
        }
    }
    if (member(menue, UI_OPT_DISKUSSION))
    {
        opt[UI_OPT_DISKUSSION] = menue[UI_OPT_DISKUSSION];
    }
    if (member(menue, UI_OPT_OHNE_DISKUSSION))
    {
        opt[UI_OPT_OHNE_DISKUSSION] = menue[UI_OPT_OHNE_DISKUSSION];
    }
    if (member(menue, UI_OPT_IGNORELIST))
    {
        opt[UI_OPT_IGNORELIST] = menue[UI_OPT_IGNORELIST];
    }
    return opt;
}

static mixed idee_uebersicht_reload(mixed menues)
{
    menues[<1] = idee_uebersicht_init(menues[<1]);
    menues[<1][B_FLAGS] &= ~BF_DIRTY;
    return menues;
}

static string * idee_uebersicht_display(mapping menue)
{
    int i;
    string c;
    mapping opt = idee_uebersicht_suchmuster(menue);
    mixed data;
    int total = get_master()->get_ideen_wahl_uebersicht(
                    opt+([UI_OPT_COUNTALL:1]) );
    int start = max(menue[B_START_LINE],
                min(menue[B_CURRENT_LINE]-1,total-menue[B_NUM_LINES]+1));
    opt[UI_OPT_OFFSET] = start-1;
    opt[UI_OPT_LIMIT] = menue[B_NUM_LINES]+1;
    data = get_master()->get_ideen_wahl_uebersicht(opt);
    string * content = ({});
    int end = min(total, start+menue[B_NUM_LINES]);
    //start = max(min(start, total - menue[B_NUM_LINES]),menue[B_START_LINE]);
    //PDBG(sprintf("idee_uebersicht_display %d/%d\n%Q\n%Q\n",
    //                                    start,end,opt,data));
    //if (start > sizeof(data))
    //    return 0;
    if (start < 1)
        start = 1;
    if (end < start) 
        return 0;
    for (i = start; i <= end; i++)
    {
        c = master->ist_idee_ungelesen(opt
                +([UI_OPT_I_ID : data[i-start][0]]), TP_RN) ? "N" : " ";
        content += ({ sprintf("%3d %s %s", i, c, data[i-start][1]) });
    }
    return content;
}

static int idee_uebersicht_total(mapping menue)
{
    mapping opt = idee_uebersicht_suchmuster(menue);
    opt[UI_OPT_COUNTALL] = 1;
    return get_master()->get_ideen_wahl_uebersicht(opt);
}

static string idee_uebersicht_prompt(mapping menue)
{
    string r = "Ideenuebersicht ";
    int total = idee_uebersicht_total(menue);
    int line = min(menue[B_CURRENT_LINE], total); // TODO workaround loesen
    if (member(menue, UI_OPT_SUCHFILTER))
    {
        r+= "<"+menue[UI_OPT_SUCHFILTER]+"> ";
    }
    if (member(menue, UI_OPT_ERSTELLER))
    {
        r+= "<"+menue[UI_OPT_ERSTELLER]+"> ";
    }
    r+= sprintf("%s Zeile %d/%d [q,z,?,<nr>]",menue[B_PROMPT], line, total);
    return r;
}

static mixed idee_uebersicht_action(string str, mapping * menues)
{
    mapping menue = menues[<1];
    int end = idee_uebersicht_total(menue);
    // int start = menue[B_CURRENT_LINE];
    int item;
    mapping opt = idee_uebersicht_suchmuster(menue);
    mixed r;
    string * parts = explode(space(str), " ");
    string param = implode(parts[1..]," ");
    switch (lower_case(parts[0]))
    {
    case "gelesen":
        get_master()->update_gelesen(opt, TP_RN);
        return B_REBUILT;
    case "ungelesen":
        get_master()->update_ungelesen(opt, TP_RN);
        return B_REBUILT;
    case "f":
    case "finde":
        if (stringp(param) && param != "")
        {
            menues[<1][UI_OPT_SUCHFILTER] = param;
        }
        else
        {
            m_delete(menues[<1],UI_OPT_SUCHFILTER);
        }
        menues[<1][B_FLAGS] |= ~BF_DIRTY;
        return menues;
    case "status":
        if (query_commander(TP) || menue[UI_OPT_MENUTYPE] == UI_MENUTYPE_OWN)
        {
            return menues+({ ([
                            B_TYPE : "status_wahl",
                            UI_OPT_MC : 1,
                          ]) });;
        }
        else
        {
            browse_write_line("Befehl Status nicht verfügbar.");
            return B_DONE;
        }
    case "bereit":
    case "aktiv":
    case "offen":
    case "beendet":
    case "abgelehnt":
        if (query_commander(TP) || menue[UI_OPT_MENUTYPE] == UI_MENUTYPE_OWN)
        {
            menues[<1][UI_OPT_STATUS] = selected_status = ({
                    get_status_id(capitalize(lower_case(parts[0]))) });
            return menues;
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
            return B_DONE;
        }
    case "alle":
        if (query_commander(TP) || menue[UI_OPT_MENUTYPE] == UI_MENUTYPE_OWN)
        {
            m_delete(menues[<1],UI_OPT_STATUS);
            selected_status = ({ });
            return menues;
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
            return B_DONE;
        }       
    default:
        if (!str2int(space(str),&item) && item >= 1 && item <= end)
        {
            opt[UI_OPT_OFFSET] = item-1;
            opt[UI_OPT_LIMIT] = 1;
            r = master->get_ideen_wahl_uebersicht(opt);
            //PDBG(sprintf("idee_uebersicht_action %Q %Q",opt,r));
            if (sizeof(r) == 1)
            {
                int i_id = r[0][0];
                menues[<1][B_CURRENT_LINE] = item;
                menues += ({ ([ B_TYPE : "idee_umfragen",
                                UI_OPT_I_ID : i_id,
                                UI_OPT_ITEM : item,
                                B_START_LINE : 0 ]) });
                return menues;
            }
        }
        break;
    }
    return B_NOTHING;
}

#define HELP_idee_umfragen ({ \
"In dieser Ansicht kann man eine Idee (i) mit all ihren Umfragen (U) sehen. ",\
"Diese sind durchnummeriert und koennen ueber Angabe der Nummer ",\
"angewaehlt werden. Mit '+' bzw. '-' kann die naechste bzw. vorherige Idee ",\
"angezeigt werden. Mit 'ignoriere' kann man das Ignorieren einzelner Ideen ", \
"an- und abschalten. Mit gelessen/ungelesen kann man die Diskussionen ", \
"entsprechend markieren.",  \
})
#define HELP_idee_umfragen_admin ({ \
"Mit Status kann der Status gesetzt werden, die Befehle bereit, aktiv, ",\
"offen, beendet, abgelehnt, archiviert machen das direkt.",  \
"Mittels uneu kann eine neue Umfrage hinzugefuegt werden.", \
"Mit aktuell kann man eine aktuelle Idee fuer alle setzen.", \
})
static mixed idee_umfragen_init(mapping old)
{
    int i;
    string * result = ({});
    mixed data = get_master()->get_ideen_wahl(([
            UI_OPT_I_ID : old[UI_OPT_I_ID],
            UI_OPT_PANTHEON : wizp(TP),
            ]));
    //PDBG(sprintf("idee_umfragen_init data %Q",data));
    if (!pointerp(data))
    {
        return B_QUIT;
    }
    for (i = 0; i< sizeof(data); i++)
    {
        result += ({ sprintf("%3d %s: %s",
                     i+1,
                     data[i][IDEEN_W_ZEICHEN],
                     data[i][IDEEN_W_TEXT] ) });
    }
    //PDBG(sprintf("idee_umfragen_init result %Q",result));
    old[UI_OPT_DATA] = data;
    old[B_DATA] = result;
    old[B_START_LINE] = 0;
    old[B_CURRENT_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Idee x mit Umfragen Zeile",
                     B_DATA : HELP_idee_umfragen
                            +(query_commander(TP)?HELP_idee_umfragen_admin:({}))
                            +HELP_standard ]);
    return old;
}

static mixed idee_umfragen_reload(mixed menues)
{
    string str = menues[<1][UI_OPT_GOTO]||""; 
    int end, item, i_id;
    menues[<1][B_FLAGS] &= ~BF_DIRTY;
    switch (str)
    {
    case "+":
        item = 1;
        break;
    case "-":
        item = -1;
        break;
    default:
        return menues[..<2] + ({idee_umfragen_init(menues[<1])});
    }
    mapping menue = menues[<2];
    mapping opt = idee_uebersicht_suchmuster(menue);
    mixed r;
    end = idee_uebersicht_total(menue);
    item += menue[B_CURRENT_LINE];
    if (item < 1 || item > end)
    {
        browse_write_line("Anfang bzw. Ende der Daten erreicht.");
        return menues;
    }
    opt[UI_OPT_OFFSET] = item-1;
    opt[UI_OPT_LIMIT] = 1;
    r = master->get_ideen_wahl_uebersicht(opt);
    if (sizeof(r) == 1)
    {
        i_id = r[0][0];
        menue[B_CURRENT_LINE] = item;
        menues = menues[..<3] + ({menue}) +
                  ({ idee_umfragen_init( 
                     ([ B_TYPE : "idee_umfragen",
                        UI_OPT_I_ID : i_id,
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ])) });
        return menues;
    }
    return menues;
}

static string * idee_umfragen_display(mapping menue)
{
    // string * result = ({});
    if (!mappingp(menue) )
        return 0;
    int start = menue[B_CURRENT_LINE];
    mixed * content = menue[B_DATA];
    int end = min(sizeof(content)-1, start+menue[B_NUM_LINES]);
    //start = min(start, end - menue[B_NUM_LINES]);
    if (start > sizeof(content))
        return 0;
    if (start < 0)
        start = 0;
    if (end < start) 
        return 0;
    return content[start..end];
}

static int idee_umfragen_total(mapping menue)
{
    return sizeof(menue[B_DATA]);
}

static string idee_umfragen_prompt(mapping menue)
{
    return sprintf("Idee Nr.%d mit Umfragen Zeile %d/%d [+,-,<nr>,z,q,?]",
        menue[UI_OPT_ITEM], 
        menue[B_CURRENT_LINE]+1,
        idee_umfragen_total(menue));
}

static mixed idee_umfragen_action(string str, mapping * menues)
{
    mapping menue = menues[<1];
    int i, item, end = sizeof(menue[B_DATA]);
    string * parts = explode(space(str), " ");
    // string param = implode(parts[1..]," ");
    get_master();
    
    switch(lower_case(parts[0]))
    {
    case "gelesen":
        master->update_gelesen(menue, TP_RN);
        return B_REBUILT;
    case "ungelesen":
        master->update_ungelesen(menue, TP_RN);
        return B_REBUILT;
    case "+":
    case "-":
        menues[<1][UI_OPT_GOTO] = space(str);
        menues[<1][B_FLAGS] |= BF_DIRTY;
        return menues;
    case "uneu":
        if (query_commander(TP))
        {
            umfrage_neu(menues, menues[<1][UI_OPT_I_ID]);
            return B_QUIT;
        }
        else
        {
            browse_write_line("Befehl uneu nicht verfügbar.");
            return B_DONE;
        }
    case "ignore":
    case "ignoriere":
        switch(get_master()->toggle_ignore_idee(menues[<1][UI_OPT_I_ID],TP_RN))
        {
        case 0:
            browse_write_line("Fehler beim Ignorieren.");
            break;
        case 1:
            browse_write_line("Idee wird nun ignoriert.");
            break;
        case -1:
            browse_write_line("Idee wird nicht mehr ignoriert.");
            break;
        }
        return B_DONE;
    case "aktuell":
        if (query_commander(TP))
        {
            get_master()->set_aktuelle_idee(menues[<1][UI_OPT_I_ID]);
            __FILE__->msg_to_clones("Uiko flüstert Dir zu:",
                "Es wurde eine neue, aktuelle Idee gesetzt. "
                "Aufruf mit idee:aktuell oder kurz i:a .");
            browse_write_line("Aktuelle Idee gesetzt.");
        }
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return B_DONE;
    case "bereit":
    case "aktiv":
    case "offen":
    case "beendet":
    case "abgelehnt":
    case "archiviert":
        if (query_commander(TP))
        {
            for (i=0;i<sizeof(menue[UI_OPT_DATA]);i++)
            {
                mixed * data = menue[UI_OPT_DATA][i];
                switch(data[IDEEN_W_ZEICHEN])
                {
                case "Idee":
                    master->aktualisiere_ideen_status(
                            data[IDEEN_W_I_ID],
                            get_status_id(capitalize(lower_case(parts[0]))));
                case "Umfrage":
                    master->aktualisiere_umfragen_status(
                            data[IDEEN_W_U_ID],
                            get_status_id(capitalize(lower_case(parts[0]))));
                case "Diskussion":
                    master->aktualisiere_diskussion_status(
                            data[IDEEN_W_U_ID], // ist das Feld fuer d_id hier.
                            get_status_id(capitalize(lower_case(parts[0]))));
                default:
                }
            }
            browse_write_line("Status aktualisiert");
            return B_DONE;
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
            return B_DONE;
        }
   case "diskussion-s":
        if (query_commander(TP))
            return neue_diskussion("ideen", menues[<1][UI_OPT_I_ID], 1);
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return B_DONE;
   case "diskussion-p":
        if (query_commander(TP))
            return neue_diskussion("ideen", menues[<1][UI_OPT_I_ID], 2);
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return B_DONE;
   case "diskussion-sp":
        if (query_commander(TP))
            return neue_diskussion("ideen", menues[<1][UI_OPT_I_ID], 3);
        else
            browse_write_line("Befehl steht nicht zur Verfügung.");
        return B_DONE;
   default:
        if (!str2int(space(str),&item) && item >= 1 && item <= end)
        {
            mixed * data = menue[UI_OPT_DATA][item-1];
            switch (data[IDEEN_W_ZEICHEN])
            {
            case "Idee":
                menues += ({ ([ B_TYPE : "idee_details",
                        UI_OPT_I_ID : data[IDEEN_W_I_ID],
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ]) });
                return menues;
            case "Umfrage":
                menues += ({ ([ B_TYPE : "umfrage_wahl",
                        UI_OPT_U_ID : data[IDEEN_W_U_ID],
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ]) });
                return menues;
            case "Diskussion":
                menues += ({ ([ B_TYPE : "diskussion",
                        UI_OPT_D_ID : data[IDEEN_W_U_ID],
                        UI_OPT_ITEM : item,
                        B_PROMPT: data[IDEEN_W_TEXT],
                        B_START_LINE : 0 ]) });
                return menues;
            default:
                break;
            }
        }
        break;
    }
    return B_NOTHING;
}

#define HELP_idee_aktuell ({ \
"In dieser Ansicht kann man die aktuelle Idee (i) mit all ihren Umfragen (U)",\
"sehen. Diese sind durchnummeriert und koennen ueber Angabe der Nummer ",\
"angewaehlt werden. Mit 'ignoriere' kann man das Ignorieren der Idee an- ",\
"und abschalten.",  \
})
#define HELP_idee_aktuell_admin ({ \
"Mit Status kann der Status gesetzt werden, die Befehle bereit, aktiv, ",\
"offen, beendet, abgelehnt, archiviert machen das direkt.",  \
"Mittels uneu kann eine neue Umfrage hinzugefuegt werden.", \
})
static mixed idee_aktuell_init(mapping old)
{
    int i;
    string * result = ({});
    mixed data = get_master()->get_aktuelle_idee(([
            UI_OPT_PANTHEON : wizp(TP),
            ]));
    //PDBG(sprintf("idee_aktuell_init data %Q",data));
    if (!pointerp(data) || !sizeof(data))
    {
        browse_write_line("Zur Zeit ist keine aktuelle Idee gesetzt.");
        return B_QUIT;
    }
    for (i = 0; i< sizeof(data); i++)
    {
        result += ({ sprintf("%3d %s: %s",
                     i+1,
                     data[i][IDEEN_W_ZEICHEN],
                     data[i][IDEEN_W_TEXT] ) });
    }
    //PDBG(sprintf("idee_umfragen_init result %Q",result));
    old[UI_OPT_DATA] = data;
    old[B_DATA] = result;
    old[B_START_LINE] = 0;
    old[B_CURRENT_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zur aktuellen Idee x Zeile",
                     B_DATA : HELP_idee_aktuell
                            +(query_commander(TP)?HELP_idee_aktuell_admin:({}))
                            +HELP_standard ]);
    return old;
}

static string * idee_aktuell_display(mapping menue)
{
    // string * result = ({});
    if (!mappingp(menue) )
        return 0;
    int start = menue[B_CURRENT_LINE];
    mixed * content = menue[B_DATA];
    int end = min(sizeof(content)-1, start+menue[B_NUM_LINES]);
    //start = min(start, end - menue[B_NUM_LINES]);
    if (start > sizeof(content))
        return 0;
    if (start < 0)
        start = 0;
    if (end < start) 
        return 0;
    return content[start..end];
}

static int idee_aktuell_total(mapping menue)
{
    return sizeof(menue[B_DATA]);
}

static string idee_aktuell_prompt(mapping menue)
{
    return sprintf("Aktuelle Idee mit Umfragen Zeile %d/%d [<nr>,z,q,?]",
        menue[B_CURRENT_LINE]+1,
        idee_umfragen_total(menue));
}

static mixed idee_aktuell_action(string str, mapping * menues)
{
    mapping menue = menues[<1];
    int i, item, end = sizeof(menue[B_DATA]);
    string * parts = explode(space(str), " ");
    // string param = implode(parts[1..]," ");
    get_master();
    
    switch(lower_case(parts[0]))
    {
    case "uneu":
        if (query_commander(TP))
        {
            umfrage_neu(menues, menues[<1][UI_OPT_I_ID]);
            return B_QUIT;
        }
        else
        {
            browse_write_line("Befehl uneu nicht verfügbar.");
            return B_DONE;
        }
    case "ignore":
    case "ignoriere":
        switch(get_master()->toggle_ignore_idee(menues[<1][UI_OPT_I_ID],TP_RN))
        {
        case 0:
            browse_write_line("Fehler beim Ignorieren.");
            break;
        case 1:
            browse_write_line("Idee wird nun ignoriert.");
            break;
        case -1:
            browse_write_line("Idee wird nicht mehr ignoriert.");
            break;
        }
        return B_DONE;
    case "bereit":
    case "aktiv":
    case "offen":
    case "beendet":
    case "abgelehnt":
    case "archiviert":
        if (query_commander(TP))
        {
            for (i=0;i<sizeof(menue[UI_OPT_DATA]);i++)
            {
                mixed * data = menue[UI_OPT_DATA][i];
                switch(data[IDEEN_W_ZEICHEN])
                {
                case "Idee":
                    master->aktualisiere_ideen_status(
                            data[IDEEN_W_I_ID],
                            get_status_id(capitalize(lower_case(parts[0]))));
                case "Umfrage":
                    master->aktualisiere_umfragen_status(
                            data[IDEEN_W_U_ID],
                            get_status_id(capitalize(lower_case(parts[0]))));
                default:
                }
            }
            browse_write_line("Status aktualisiert");
            return B_DONE;
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
            return B_DONE;
        }
   default:
        if (!str2int(space(str),&item) && item >= 1 && item <= end)
        {
            mixed * data = menue[UI_OPT_DATA][item-1];
            switch (data[IDEEN_W_ZEICHEN])
            {
            case "Idee":
                menues += ({ ([ B_TYPE : "idee_details",
                        UI_OPT_I_ID : data[IDEEN_W_I_ID],
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ]) });
                return menues;
            case "Umfrage":
                menues += ({ ([ B_TYPE : "umfrage_wahl",
                        UI_OPT_U_ID : data[IDEEN_W_U_ID],
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ]) });
                return menues;
            default:
                break;
            }
        }
        break;
    }
    return B_NOTHING;
}



#define HELP_idee_details ({ \
"In dieser Ansicht kann man eine Idee mit Titel umd Zusammenfassung sehen.",  \
"Mit 'ignoriere' kann man das Ignorieren der Idee an- und abschalten.", \
})
#define HELP_idee_details_admin ({ \
"Mit Status kann der Status gesetzt werden, die Befehle bereit, aktiv, ",\
"offen, beendet, abgelehnt, archiviert machen das direkt.",  \
})
static mixed idee_details_init(mapping old)
{
    int i_id = old[UI_OPT_I_ID], i_st, i_pa;
    string * r2,*r1 = get_master()->get_idee(i_id);
    if (sizeof(r1) != 2)
    {
        browse_write_line("Idee nicht vorhanden.");
        return B_QUIT;
    }
    r2 = ({ "Titel: "+r1[0] });
    i_st = master->get_ideen_status(i_id);
    i_pa = master->get_ideen_pantheon(i_id);
    r2 += ({ "Status: "+get_status(i_st)+" "+PANTHEON2TEXT(i_pa) });
    if (query_commander(TP))
    {
        r2+= ({ "I_ID:"+i_id+" Ersteller: "+master->get_ideen_ersteller(i_id)});
    }
    r2 += explode(r1[1], "\n");
    old[B_DATA] = r2;
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Ideendetailansicht Zeile",
                     B_DATA : HELP_idee_details
                            +(query_commander(TP)?HELP_idee_details_admin:({}))
                            +HELP_standard ]);
    return old;
}

static mixed idee_details_reload(mixed menues)
{
    menues[<1] = idee_details_init(menues[<1]);
    menues[<1][B_FLAGS] &= ~BF_DIRTY;
    return menues;
}

static string * idee_details_display(mapping menue)
{
    return staticmore_display(menue);
}

static int idee_details_total(mapping menue)
{
    return sizeof(menue[B_DATA]);
}

static string idee_details_prompt(mapping menue)
{
    return sprintf("Ideendetailansicht Zeile %d/%d) [q,z,?]",
        menue[B_CURRENT_LINE]+1, sizeof(menue[B_DATA]) );
}

static mixed idee_details_action(string str, mapping * menues)
{
    string * parts = explode(space(str), " ");
    // string param = implode(parts[1..]," ");
    get_master();
    
    switch(lower_case(parts[0]))
    {
    case "ignore":
    case "ignoriere":
        switch(get_master()->toggle_ignore_idee(menues[<1][UI_OPT_I_ID],TP_RN))
        {
        case 0:
            browse_write_line("Fehler beim Ignorieren.");
            break;
        case 1:
            browse_write_line("Idee wird nun ignoriert.");
            break;
        case -1:
            browse_write_line("Idee wird nicht mehr ignoriert.");
            break;
        }
        return B_DONE;
    case "status":
        if (query_commander(TP))
        {
            return menues+({ ([
                            B_TYPE : "status_wahl",
                            UI_OPT_STATUS : 
                get_master()->get_ideen_status(menues[<1][UI_OPT_I_ID])
                          ]) });;
        }
        else
        {
            browse_write_line("Befehl Status nicht verfügbar.");
            return B_DONE;
        }
    case "bereit":
    case "aktiv":
    case "offen":
    case "beendet":
    case "abgelehnt":
    case "archiviert":
        if (query_commander(TP))
        {
            master->aktualisiere_ideen_status(
                    menues[<1][UI_OPT_I_ID],
                    get_status_id(capitalize(lower_case(parts[0]))));
            browse_write_line("Status aktualisiert");
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
        }
        return B_DONE;
    }
    return B_NOTHING;
}

#define HELP_umfrage_wahl ({ \
"In dieser Ansicht kann man eine Umfrage optional mit der zugehoerigen Idee ",\
"zu sehen. Alle moeglichen Antworten sind dargestellt, die mit der Nummer ", \
"auszuwaehlen sind. Ein * kennzeichnet die Wahl, die man getaetigt hat.",  \
})
#define HELP_umfrage_wahl_admin ({ \
"Mit Status kann der Status gesetzt werden, die Befehle bereit, aktiv, ",\
"offen, beendet, abgelehnt, archiviert machen das direkt.",  \
"Mittels aneu kann im Status neu und bereit eine neue Antwort ",\
"hinzugefuegt werden",\
})
static mixed umfrage_wahl_init(mapping old)
{
    int u_id = old[UI_OPT_U_ID],u_st,u_pa;
    mixed r1 = get_master()->get_ideen_wahl(([
            UI_OPT_U_ID:u_id,
            UI_OPT_PANTHEON : wizp(TP)?1:0,
            ]));
    string sel,*lines = ({});
    int i,a_id;
    if (sizeof(r1)==2)
    {
        for (i = 0; i< 2; i++)
        {
            lines  += ({ sprintf("%s: %s",
                         r1[i][IDEEN_W_ZEICHEN],
                         r1[i][IDEEN_W_TEXT] ) });
        }
        u_st = master->get_umfragen_status(u_id);
        u_pa = master->get_umfragen_pantheon(u_id);
        lines += ({ "Status: "+get_status(u_st)+" "+PANTHEON2TEXT(u_pa) });
        if (query_commander(TP))
        {
            lines+= ({ "U_ID:"+u_id+" Ersteller: "
                    +master->get_umfragen_ersteller(u_id) });
        }
    }
    else
    {
        r1 = master->get_umfrage(old[UI_OPT_U_ID]);
        if (stringp(r1))
        {
            lines  += ({ sprintf("0 Umfrage: %s",r1) }) ;
        }
        else
        {
            browse_write_line("Keine Umfrage gefunden.");
            return B_QUIT;
        }
        u_st = master->get_umfragen_status(u_id);
        u_pa = master->get_umfragen_pantheon(u_id);
        lines += ({ "Status: "+get_status(u_st)+" "+PANTHEON2TEXT(u_pa) });
        if (query_commander(TP))
        {
            lines+= ({ "U_ID:"+u_id+" Ersteller: "
                    +master->get_umfragen_ersteller(u_id) });
        }
    }
    lines += ({ "Antworten auf die aktuelle Umfrage:" });
    r1 = master->get_antworten(old[UI_OPT_U_ID]);
    a_id = master->hat_gewaehlt(old[UI_OPT_U_ID],TP_RN);
    for (i = 0; i<sizeof(r1); i++)
    {
        sel = (a_id == r1[i][0])? "*" :" ";
        lines += ({ sprintf("%s%2d %s",sel,i+1,r1[i][1]) });
    }
    old[B_DATA] = lines;
    old[UI_OPT_DATA] = r1;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Umfragenabstimmung Zeile",
                     B_DATA : HELP_umfrage_wahl
                            +(query_commander(TP)?HELP_umfrage_wahl_admin:({}))
                            +HELP_standard ]);
    return old;
}

static mixed umfrage_wahl_reload(mixed menues)
{
    if (strstr(get_menue_path(menues),
            "umfrage_wahl/idee_umfragen/idee_uebersicht")==0)
    {
        ;
    }
    mixed m = umfrage_wahl_init(menues[<1]);
    if (mappingp(m))
    {
        m[B_FLAGS] &= ~BF_DIRTY;
        return menues[..<2] + ({ m });
    }
    return m;
}

static string * umfrage_wahl_display(mapping menue)
{
    return staticmore_display(menue);
}

static int umfrage_wahl_total(mapping menue)
{
    return sizeof(menue[B_DATA]);
}

static string umfrage_wahl_prompt(mapping menue)
{
    return sprintf("Umfragenabstimmung Zeile:%d/%d [q,z,?,<nr>]",
        menue[B_CURRENT_LINE],sizeof(menue[B_DATA]));
}

static mixed umfrage_wahl_action(string str, mapping * menues)
{
    int item,end,a_id, u_id = menues[<1][UI_OPT_U_ID];
    mapping menue = menues[<1];
    mixed data = menue[UI_OPT_DATA];
    end = sizeof(data);
    string * parts = explode(space(str), " ");
    string param = implode(parts[1..]," ");
    get_master();
    
    switch(lower_case(parts[0]))
    {
    case "aneu":
        if (query_commander(TP))
        {
            if (!param || param == "")
            {
                input_to(#'umfrage_neue_antwort, INPUT_PROMPT, 
                         "Neue Antwort:",menues,u_id);
            }
            else
            {
                umfrage_neue_antwort(param, menues, u_id);
            }
            return B_QUIT;
        }
        else
        {
            browse_write_line("Befehl aneu nicht verfügbar.");
            return B_DONE;
        }
    case "status":
        if (query_commander(TP))
        {
            return menues+({ ([
                            B_TYPE : "status_wahl",
                            UI_OPT_STATUS : 
                master->get_umfragen_status(u_id)
                          ]) });;
        }
        else
        {
            browse_write_line("Befehl Status nicht verfügbar.");
            return B_DONE;
        }
    case "bereit":
    case "aktiv":
    case "offen":
    case "beendet":
    case "abgelehnt":
    case "archiviert":
        if (query_commander(TP))
        {
            master->aktualisiere_umfragen_status(
                    menues[<1][UI_OPT_U_ID],
                    get_status_id(capitalize(lower_case(parts[0]))));
            browse_write_line("Status aktualisiert");
        }
        else
        {
            browse_write_line("Befehl "+parts[0]+" nicht verfügbar.");
        }
        return B_DONE;
    default:
        if (!str2int(space(str),&item) && item >= 1 && item <= end)
        {
            a_id = data[item-1][0];
            if (master->add_erhebung(a_id, TP_RN,wizp(TP)!=0))
            {
                browse_write_line("Wahl erfolgt.");
                menue[B_FLAGS] |= BF_DIRTY;
                menues[<1] = menue;
                return menues;
            }
            else
            {
                browse_write_line("Wahl nicht erfolgt.");
                return B_DONE;
            }
        }
        break;
    }
    return B_NOTHING;
}

#define HELP_status_wahl_mc ({ \
"In dieser Ansicht kann man einen oder mehrere Status ueber <nr> auswaehlen.",\
})
#define HELP_status_wahl ({ \
"In dieser Ansicht kann man genau einen Status ueber <nr> auswaehlen.",\
})
#define HELP_status_wahl_liste ({ \
"Die Bedeutung der einzelnen Status ist wie folgt:",\
"Geloescht: ist ein interner Status, der nicht angewaehlt werden kann.",\
"Neu: ist ein Zustand beim Anlegen der Idee oder Umfrage.",\
"Standard: sind Ideen und Umfragen, die als Vorlage dienen sollen.",\
"Bereit: sind alle Ideen und Umfragen, die auf Freigabe warten.",\
"Aktiv: sind alle Ideen und Umfragen, die zur Wahl stehen.",\
"Offen: sind die Umfragen, wo man Antworten frei ergaenzen kann.",\
"Beendet: sind Ideen und Umfragen, welche so oder so aehnlich umgesetzt wurden.",\
"Abgelehnt: sind die abgelehnten Ideen.",\
"Archiviert: sind veraltete Ideen und Umfragen",\
})

static mixed status_wahl_init(mapping old)
{
    selected = selected_status;
    if (member(old,UI_OPT_STATUS))
    {
        if (intp(old[UI_OPT_STATUS]))
        {
            selected = ({ old[UI_OPT_STATUS] });
        }
        else if (pointerp(old[UI_OPT_STATUS]))
        {
            selected = old[UI_OPT_STATUS];
        }
    }
   if (member(old, UI_OPT_MC))
   {
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu Mehrfach-Statusauswahl Zeile", 
                         B_DATA : HELP_status_wahl_mc+HELP_standard
                                 +HELP_status_wahl_liste]);
   }
   else
   {
        old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                         B_PROMPT : "Hilfe zu Statusauswahl Zeile", 
                         B_DATA : HELP_status_wahl+HELP_standard
                                 +HELP_status_wahl_liste ]);
   }
   old[B_START_LINE] = 1;
   return old;
}

static string * status_wahl_display(mapping menue)
{
    int i;
    string sel,* result = ({});
    for (i=UMFRAGE_S_GELOESCHT; i <= UMFRAGE_S_ARCHIVIERT; i++)
    {
        sel = (member(selected,i)!=-1) ? "*" : " ";
        result += ({ sprintf("%s %d %s", sel, i+1, get_status(i)) });
    }
    return result;
}

static int status_wahl_total(mapping menue)
{
    return UMFRAGE_S_ARCHIVIERT+1;
}

static string status_wahl_prompt(mapping menue)
{
    return member(menue,UI_OPT_MC) ? 
            "Mehrfach-Statusauswahl [<nr>,z,q,?]" :
            "Statusauswahl [<nr>,z,q,?]";
}

static mixed status_wahl_action(string str, mapping * menues)
{
    int item, i;
    string path = get_menue_path(menues);
    get_master();
    // PDBG("status_wahl_action: "+path);
    if (space(str)=="z")
    {
        if (strstr(path,
            "status_wahl/idee_uebersicht")==0)
        {
            selected_status = selected;
            if (sizeof(selected_status))
            {
                menues[<2][UI_OPT_STATUS] = selected_status;
            }
            else
            {
                menues[<2][UI_OPT_STATUS] = UMFRAGE_S_AKTIV;
            }
        }
        else if (strstr(get_menue_path(menues),
            "status_wahl/umfrage_wahl")==0)
        {
            item = menues[<2][UI_OPT_U_ID];
            if (sizeof(selected))
            {
                master->aktualisiere_umfragen_status(item,selected[0]);
            }
        }
        else if (strstr(get_menue_path(menues),
            "status_wahl/idee_details")==0)
        {
            item = menues[<2][UI_OPT_I_ID];
            if (sizeof(selected))
            {
                master->aktualisiere_ideen_status(item,selected[0]);
            }
        }
        else if (strstr(get_menue_path(menues),
            "status_wahl/x_umfrage_admin")==0)
        {
            if (sizeof(selected))
            {
                mapping opt = master->get_x_umfrage(TP,
                    menues[<2][UI_OPT_XU_ID]);
                opt[UI_OPT_STATUS] = selected[0];
                master->set_x_umfrage(TP,opt);
            }
        }
        else
        {
            browse_write_line("Unbekannter Pfad");
            return B_DONE;
        }
        menues[<2][B_FLAGS] |= BF_DIRTY;
        return B_NOTHING;
    }
    if (!str2int(space(str),&item) 
            && item > UMFRAGE_S_NEU && item <= UMFRAGE_S_ARCHIVIERT+1)
    {
        item--;
        if (member(menues[<1], UI_OPT_MC))
        {
            if ((i=member(selected,item)) != -1)
            {
                selected = arr_delete(selected,i);
            }
            else
            {
                selected += ({ item });
            }
        }
        else 
        {
            if ((i=member(selected,item)) != -1)
            {
                selected = arr_delete(selected,i);
            }
            else
            {
                selected = ({ item });
            }
        }
        return B_REBUILT;
    }
    return B_NOTHING;
}

#define HELP_ideen_auswertung ({ \
"In dieser Ansicht sind die Ideen inkl. Auswertung angezeigt und",\
"absteigend sortiert. Per <nr> kann man eine Idee auswaehlen.",\
})
static mixed ideen_auswertung_init(mapping old)
{
    mixed * data = get_master()->get_erhebung_alle_ideen(old);
    string * list = ({ });
    int i;
    if (!query_commander(TP) || TI!=TP) return B_QUIT;
    for (i=0; i < sizeof(data);i ++)
    {
        list += ({  sprintf("%3d %11s %60s",i+1,
            data[i][UI_ERHEBUNG1I_SCHLUESSEL],
            data[i][UI_ERHEBUNG1I_TEXT] )
            });
    }
   old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                    B_PROMPT : "Hilfe zur Auswertung Zeile", 
                    B_DATA : HELP_ideen_auswertung+HELP_standard ]);
    old[B_DATA] = list;
    old[B_START_LINE] = 0;
    old[UI_OPT_DATA] = data;
    return old;
}

static string * ideen_auswertung_display(mapping menue)
{
    return staticmore_display(menue);
}

static int ideen_auswertung_total(mapping menue)
{
    return sizeof(menue[UI_OPT_DATA]);
}

static string ideen_auswertung_prompt(mapping menue)
{
    int ix = menue[B_CURRENT_LINE];
    int ex = ideen_auswertung_total(menue);
    return sprintf("Ideen Auswertung $d/%d [u,d,<nr>,?,q]",ix,ex);
}

static mixed ideen_auswertung_action(string str, mapping * menues)
{
    int item, i_id;
    mapping menue = menues[<1];
    if (!str2int(space(str),&item) 
            && item > 0 && item <= sizeof(menue[UI_OPT_DATA]))
    {
        item--;
        i_id = menue[UI_OPT_DATA][item][UI_ERHEBUNG1I_I_ID];
        
        menues[<1][B_CURRENT_LINE] = item;
        menues += ({ ([ B_TYPE : "idee_umfragen",
                        UI_OPT_I_ID : i_id,
                        UI_OPT_ITEM : item,
                        B_START_LINE : 0 ]) });
        return menues;
    }
    return B_NOTHING;
}

#define HELP_diskussion ({ \
"In dieser Ansicht ist die Uebersicht ueber die Beitraege.",\
"Mit gelesen/ungelesen kann man alle Beitraege entsprechend markieren.",\
})
static mixed diskussion_init(mapping old)
{
   old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                    B_PROMPT : "Hilfe zur Auswertung Zeile", 
                    B_DATA : HELP_diskussion+HELP_standard ]);
   return old;
}

static int diskussion_total(mapping menue)
{
    return get_master()->get_diskussionsbeitraege(
                     menue+([UI_OPT_COUNTALL:1,
                     UI_OPT_PANTHEON:wizp(TP)]) );
}

static string * diskussion_display(mapping menue)
{
    string * result = ({}), c;
    if (!mappingp(menue) )
        return 0;
    mixed data = get_master()->get_diskussionsbeitraege(menue
            +([UI_OPT_PANTHEON:wizp(TP)]) );
    int start = menue[B_CURRENT_LINE];
    int end = min(diskussion_total(menue), start+menue[B_NUM_LINES]);
    start = min(start, end - menue[B_NUM_LINES]);
    int i;
    if (start < 0)
        start = 0;
    if (end < start) 
        return 0;
    for(i=start;i<end;i++)
    {
        c = master->ist_beitrag_ungelesen(
                data[i-start][UI_BEITRAG_B_ID], TP_RN) ? "N" : " ";
        result += ({ sprintf("%3d %s %s %10s %s",i,c, 
                shorttimestr(data[i-start][UI_BEITRAG_WANN]),
                data[i-start][UI_BEITRAG_WER],
                data[i-start][UI_BEITRAG_TITEL] ) });
    }
    return result;

}

static string diskussion_prompt(mapping menue)
{
    int ix = menue[B_CURRENT_LINE];
    int ex = diskussion_total(menue);
    return sprintf("Diskussionsbeiträge %s %d/%d [u,d,neu,<nr>,?,q]",
            menue[B_PROMPT]||"",ix,ex);
}

static mixed diskussion_action(string str, mapping * menues)
{
    int item, b_id;
    mapping menue = menues[<1];
    switch (lower_case(space(str)))
    {
    case "gelesen":
        get_master()->update_gelesen(menue, TP_RN);
        return B_REBUILT;
    case "ungelesen":
        get_master()->update_ungelesen(menue, TP_RN);
        return B_REBUILT;
    case "neu":
        diskussionsbeitrag_neu(menues, menues[<1][UI_OPT_D_ID]);
        return B_QUIT;
    case "z":
        menues[<2][B_FLAGS] |= BF_DIRTY;
        return B_NOTHING;
    default:
        if (!str2int(space(str),&item) && item >= 0)
        {
            mixed data = get_master()->get_diskussionsbeitraege(menue
                +([UI_OPT_PANTHEON:wizp(TP), 
                    UI_OPT_OFFSET : item, UI_OPT_LIMIT : 1 ]) );
            if (data == 0 || sizeof(data)!=1)
            {
                break;
            }
            b_id = data[0][UI_BEITRAG_B_ID];
            
            menues[<1][B_CURRENT_LINE] = item;
            menues += ({ ([ B_TYPE : "beitrag",
                            UI_OPT_B_ID : b_id,
                            UI_OPT_ITEM : item,
                            B_START_LINE : 0 ]) });
            return menues;
        }
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_beitrag ({ \
"In dieser Ansicht ist ein Beitrag zu sehen.",\
})

static mixed beitrag_init(mapping old)
{
    old[B_DATA] = get_master()->get_beitrag(old[UI_OPT_B_ID])
        +({"------------ Ende Beitrag ----------------"});
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Beitragsansicht Zeile",
                     B_DATA : HELP_beitrag+HELP_standard ]);
    master->update_gelesen(old, TP_RN);
    return old;
}

static string * beitrag_display(mapping menue)
{
    return staticmore_display(menue);
}

static int beitrag_total(mapping menue)
{
    return sizeof(menue[B_DATA]);
}

static string beitrag_prompt(mapping menue)
{
    return sprintf("Diskussionsbeitrag Zeile %d/%d [z,q,?]",
        menue[B_CURRENT_LINE],sizeof(menue[B_DATA]) );
}

static mixed beitrag_action(string str, mapping * menues)
{
    mapping menue = menues[<1];
    switch (lower_case(space(str)))
    {
    case "gelesen":
        get_master()->update_gelesen(menue, TP_RN);
        return B_REBUILT;
    case "ungelesen":
        get_master()->update_ungelesen(menue, TP_RN);
        return B_REBUILT;
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_umfragen_wahl ({ \
"In dieser Ansicht sind Umfragen zur Wahl zu sehen.",\
})
static mixed x_umfragen_wahl_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Umfragen Wahl Zeile",
                     B_DATA : HELP_x_umfragen_wahl+HELP_standard ]);
    return old;
}

static int x_umfragen_wahl_total(mapping menue)
{
    return get_master()->get_xumfragen(TP,menue
        +([UI_OPT_COUNTALL:1]) );
}

static string * x_umfragen_wahl_display(mapping menue)
{
    mixed * xlines = get_master()->get_xumfragen(TP,menue
        +([UI_OPT_LIMIT:  menue[B_NUM_LINES],
           UI_OPT_OFFSET: menue[B_CURRENT_LINE] ]) );
    if (xlines == 0 || sizeof(xlines) <= 0) 
        return ({"Keine Umfragen zur Auswahl!"});
    string * lines = ({});
    mapping cache = ([]);
    int i;
    for (i=0;i<sizeof(xlines);i++)
    {
        int ix = i+menue[B_CURRENT_LINE]+1;
        cache[ix] = xlines[i][0]; // id
        lines += ({ sprintf("%02d %s",
            ix,xlines[i][2]||"") }); // title
    }
    menue[UI_OPT_CACHE] = cache;
    return lines;
}

static string x_umfragen_wahl_prompt(mapping menue)
{
    return sprintf("Offene Umfragen %d,%d [<nr>,q,z,?]",
        menue[B_CURRENT_LINE]+1,menue[B_END_LINE]);
}

static mixed x_umfragen_wahl_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    mapping options = menues[<1];
    mapping cache = options[UI_OPT_CACHE]||([]);
    int ix = to_int(split[<1]);
    if (sizeof(split)==1 && ix > 0)
    {
        if (member(cache,ix))
        {
            browse_write_line("Wahlmodus derzeit nicht unterstützt.");
            return B_DONE;
        }
        else
        {
            browse_write_line("Nummer passt nicht zur Anzeige.");
            return B_DONE;
        }
    }
    switch(split[0])
    {
        case "n": // neu
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zur Neuanlage.");
                return B_DONE;
            }
            ix = get_master()->create_x_umfrage();
            if (ix <= 0)
            {
                browse_write_line("Fehler beim Erstellen.");
                return B_DONE;
            }
            return menues + ({([
                B_TYPE : "x_umfrage_admin",
                UI_OPT_XU_ID : ix,
            ])});
        case "e": // editiere <nr>
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zum Editieren.");
                return B_DONE;
            }
            if (ix <= 0)
            {
                browse_write_line("Syntax: e <nr>");;
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Nummer passt nicht zur Anzeige.");
            }
            return menues + ({([
                B_TYPE : "x_umfrage_admin",
                UI_OPT_XU_ID : cache[ix],
            ])});
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_umfrage_admin ({ \
"In dieser Ansicht wird eine Umfrage editiert.",\
})
private string* get_x_umfrage(mapping options)
{
    string * lines = ({});
    mapping m_umf = get_master()->get_x_umfrage(TP,options[UI_OPT_XU_ID]);
    if (!mappingp(m_umf))
    {
        return ({"Keine Anzeige."});
    }
    lines+=({"Umfragen-ID: "+m_umf[UI_OPT_XU_ID]
             +" (i)Unified-ID: "+m_umf[UI_OPT_UNIFIED_ID]});
    lines+=({"(t) Titel: "+m_umf[UI_OPT_TITLE],"(b) Beschreibung:"});
    lines+=explode(wrap(m_umf[UI_OPT_DESCRIPTION]||""),"\n")[..<2];
    lines+=explode(wrap_say("(ut) User-Type: ",
        get_master()->resolve_user_status(m_umf[UI_OPT_USER_TYPE])),
        "\n")[..<2];
    lines+=({"(s) Status: "+get_status(m_umf[UI_OPT_STATUS])});
    lines+=({"(l) Liste der Themen"});
    return lines;
}
static mixed x_umfrage_admin_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zum Editieren der Umfrage Zeile",
                     B_DATA : HELP_x_umfrage_admin+HELP_standard ]);
    return old;
}

static int x_umfrage_admin_total(mapping menue)
{
    return sizeof(get_x_umfrage(menue));
}

static string * x_umfrage_admin_display(mapping menue)
{
    menue[B_DATA] = get_x_umfrage(menue);
    return staticmore_display(menue);
}

static string x_umfrage_admin_prompt(mapping menue)
{
    return sprintf("Umfrage %d %d/%d [i,t,b,ut,s,z,q]",
        menue[UI_OPT_XU_ID],menue[B_CURRENT_LINE],menue[B_END_LINE]);
}

static mixed x_umfrage_admin_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    string param = implode(split[1..]," ");
    mapping menue = menues[<1];
    mapping opt = ([UI_OPT_XU_ID:menue[UI_OPT_XU_ID] ]);
    if (!query_commander(TP) || TI!=TP) 
    {
        browse_write_line("Kein Zugriff!");
        return B_QUIT;
    }
    switch (split[0])
    {
        case "l":
            return menues + ({([
                B_TYPE : "x_themen_liste",
                UI_OPT_XU_ID : menue[UI_OPT_XU_ID],
            ])});
        case "i": // i <unified-id>
            if (sizeof(split)==2 || param=="")
            {
                opt[UI_OPT_UNIFIED_ID] = param;
                if (get_master()->set_x_umfrage(TP,opt))
                {
                    browse_write_line("unified-id gesetzt.");
                    return B_REBUILT;
                }
                else
                {
                    browse_write_line("Fehler beim Speichern.");
                    return B_DONE;
                }
            }
            else
            {
                browse_write_line("Syntax: i <unified-id>");
                browse_write_line(
                    "Wobei die ID keine Leerzeichen enthalten darf.");
                return B_DONE;
            }
        case "t": // t <titel>
            opt[UI_OPT_TITLE] = param;
            if (get_master()->set_x_umfrage(TP,opt))
            {
                browse_write_line("Titel gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "b": // b <beschreibung> bzw b -> editor?
            opt[UI_OPT_DESCRIPTION] = param;
            if (get_master()->set_x_umfrage(TP,opt))
            {
                browse_write_line("Beschreibung gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "ut": // ut+/ut- <utype>
            browse_write_line(
                "Entweder ut+ oder ut- mit einem der folgenden Parameter:");
            browse_write_line(get_master()->resolve_user_status(
                UI_USER_TYPE_ALL));
            return B_DONE;
        case "ut+":
            if (param == "")
            {
                browse_write_line(
                    "ut+ <parameter> mit einem der folgenden Parameter:");
                browse_write_line(get_master()->resolve_user_status(
                    UI_USER_TYPE_ALL));
                return B_DONE;
            }
            int new_ut = get_master()->user_status2value(param);
            if (new_ut == 0)
            {
                browse_write_line(
                    "ut+ <parameter> mit einem der folgenden Parameter:");
                browse_write_line(get_master()->resolve_user_status(
                    UI_USER_TYPE_ALL));
                return B_DONE;
            }
            opt = get_master()->get_x_umfrage(TP,menue[UI_OPT_XU_ID]);
            if ( (opt[UI_OPT_USER_TYPE]&new_ut)==new_ut)
            {
                browse_write_line("User Type(s) schon gesetzt.");
                return B_DONE;
            }
            opt[UI_OPT_USER_TYPE]|=new_ut;
            if (get_master()->set_x_umfrage(TP,opt))
            {
                browse_write_line("User Type(s) gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "ut-":
            if (param == "")
            {
                browse_write_line(
                    "ut- <parameter> mit einem der folgenden Parameter:");
                browse_write_line(get_master()->resolve_user_status(
                    UI_USER_TYPE_ALL));
                return B_DONE;
            }
            int new_ut = get_master()->user_status2value(param);
            if (new_ut == 0)
            {
                browse_write_line(
                    "ut- <parameter> mit einem der folgenden Parameter:");
                browse_write_line(get_master()->resolve_user_status(
                    UI_USER_TYPE_ALL));
                return B_DONE;
            }
            opt = get_master()->get_x_umfrage(TP,menue[UI_OPT_XU_ID]);
            new_ut = opt[UI_OPT_USER_TYPE] & (UI_USER_TYPE_ALL ^ new_ut);
            if (new_ut == opt[UI_OPT_USER_TYPE])
            {
                browse_write_line("User Type(s) schon gelöscht.");
                return B_DONE;
            }
            opt[UI_OPT_USER_TYPE] = new_ut;
            if (get_master()->set_x_umfrage(TP,opt))
            {
                browse_write_line("User Type(s) gelöscht.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "s": // s <status>
            opt = get_master()->get_x_umfrage(TP,menue[UI_OPT_XU_ID]);
            return menues + ({([
                B_TYPE : "status_wahl",
                UI_OPT_STATUS: opt[UI_OPT_STATUS],
                UI_OPT_XU_ID: menue[UI_OPT_XU_ID]
            ])});
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_themen_liste ({ \
"In dieser Ansicht werden die Themen einer Umfrage gelistet.",\
})

static mixed x_themen_liste_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu den Themen einer Umfrage Zeile",
                     B_DATA : HELP_x_themen_liste+HELP_standard ]);
    old[B_NUM_LINES] -= sizeof(old[UI_OPT_DESCRIPTION]||({}));
    return old;
}

static int x_themen_liste_total(mapping menue)
{
    return get_master()->get_xthemen(menue+([UI_OPT_COUNTALL:1]) );
}

static string * x_themen_liste_display(mapping menue)
{
    string * lines = ({});
    mixed *xlines = get_master()->get_xthemen(menue
        +([UI_OPT_LIMIT:  menue[B_NUM_LINES],
           UI_OPT_OFFSET: menue[B_CURRENT_LINE] ]) );
    if (xlines == 0 || sizeof(xlines) <= 0) 
        return ({"Keine Themen zur Auswahl!"});
    mapping cache = ([]);
    int i;
    for (i=0;i<sizeof(xlines);i++)
    {
        int ix = i+menue[B_CURRENT_LINE]+1;
        cache[ix] = xlines[i][0]; // id
        lines += ({ sprintf("%02d %s",
            ix,xlines[i][3]||"") }); // title
    }
    menue[UI_OPT_CACHE] = cache;
    return lines;
}

static string x_themen_liste_prompt(mapping menue)
{
    return sprintf("Themen (%d,%d) [<nr>,q,z,?]",
        menue[B_CURRENT_LINE]+1,menue[B_END_LINE]);
}

static mixed x_themen_liste_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    mapping menue = menues[<1];
    mapping cache = menue[UI_OPT_CACHE]||([]);
    int ix = to_int(split[<1]);
    if (sizeof(split)==1 && ix > 0)
    {
        if (member(cache,ix))
        {
            return menues + ({([
                B_TYPE : "x_fragen_liste",
                UI_OPT_XT_ID : cache[ix],
            ])});
        }
        else
        {
            browse_write_line("Nummer passt nicht zur Anzeige.");
            return B_DONE;
        }
    }
    switch(split[0])
    {
        case "n": // neu
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zur Neuanlage.");
                return B_DONE;
            }
            ix = get_master()->create_x_thema(TP,menue[UI_OPT_XU_ID]);
            if (ix <= 0)
            {
                browse_write_line("Fehler beim Erstellen.");
                return B_DONE;
            }
            return menues + ({([
                B_TYPE : "x_thema_admin",
                UI_OPT_XT_ID : ix,
            ])});
        case "e": // editiere <nr>
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zum Editieren.");
                return B_DONE;
            }
            if (ix <= 0)
            {
                browse_write_line("Syntax: e <nr>");;
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Nummer passt nicht zur Anzeige.");
            }
            return menues + ({([
                B_TYPE : "x_thema_admin",
                UI_OPT_XT_ID : cache[ix],
            ])});
        case "up":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei up <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xthema_seq(cache[ix],-1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Anfang erreicht.");
                    return B_DONE;
            }
        case "down":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei down <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xthema_seq(cache[ix],1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Ende erreicht.");
                    return B_DONE;
            }
    }
}

//---------------------------------------------------------------------------
#define HELP_x_thema_admin ({ \
"In dieser Ansicht wird ein Thema editiert.",\
})

private string* get_x_thema(mapping options)
{
    string* lines = ({});
    mapping m_thema = get_master()->get_x_thema(options[UI_OPT_XT_ID]);
    if (!mappingp(m_thema))
    {
        return ({ "Keine Anzeige."});
    }
    lines+=({"Themen-ID: "+m_thema[UI_OPT_XT_ID]
             +" (i)Unified-ID: "+m_thema[UI_OPT_UNIFIED_ID]});
    lines+=({"Sequenz: "+m_thema[UI_OPT_SEQUENCE]});
    lines+=({"Umfragen-ID: "+m_thema[UI_OPT_XU_ID] });
    lines+=({"(t) Titel: "+m_thema[UI_OPT_TITLE],"(b) Beschreibung:"});
    lines+=explode(wrap(m_thema[UI_OPT_DESCRIPTION]||""),"\n")[..<2];
    lines+=({"(l) Liste der Fragen"});
    return lines;
}

static mixed x_thema_admin_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zum Editieren des Themas Zeile",
                     B_DATA : HELP_x_thema_admin+HELP_standard ]);
    return old;
}

static int x_thema_admin_total(mapping menue)
{
    return sizeof(get_x_thema(menue));
}

static string * x_thema_admin_display(mapping menue)
{
    menue[B_DATA] = get_x_thema(menue);
    return staticmore_display(menue);
}

static string x_thema_admin_prompt(mapping menue)
{
    return sprintf("Thema %d %d/%d [i,t,b,z,q]",
        menue[UI_OPT_XT_ID],menue[B_CURRENT_LINE],menue[B_END_LINE]);
}

static mixed x_thema_admin_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    string param = implode(split[1..]," ");
    mapping menue = menues[<1];
    mapping opt = ([UI_OPT_XT_ID:menue[UI_OPT_XT_ID] ]);
    if (!query_commander(TP) || TI!=TP) 
    {
        browse_write_line("Kein Zugriff!");
        return B_QUIT;
    }
    switch (split[0])
    {
        case "i": // i <unified-id>
            if (sizeof(split)==2 || param=="")
            {
                opt[UI_OPT_UNIFIED_ID] = param;
                if (get_master()->set_x_thema(opt))
                {
                    browse_write_line("unified-id gesetzt.");
                    return B_REBUILT;
                }
                else
                {
                    browse_write_line("Fehler beim Speichern.");
                    return B_DONE;
                }
            }
            else
            {
                browse_write_line("Syntax: i <unified-id>");
                browse_write_line(
                    "Wobei die ID keine Leerzeichen enthalten darf.");
                return B_DONE;
            }
        case "t": // t <titel>
            opt[UI_OPT_TITLE] = param;
            if (get_master()->set_x_thema(opt))
            {
                browse_write_line("Titel gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "b": // b <beschreibung> bzw b -> editor?
            opt[UI_OPT_DESCRIPTION] = param;
            if (get_master()->set_x_thema(opt))
            {
                browse_write_line("Beschreibung gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "l":
            return menues + ({([
                B_TYPE : "x_fragen_liste",
                UI_OPT_XT_ID: menue[UI_OPT_XT_ID],
            ])});
    }
    return B_NOTHING;    
}

//---------------------------------------------------------------------------
#define HELP_x_fragen_liste ({ \
"In dieser Ansicht sind Fragen zur Wahl zu sehen.",\
})
static mixed x_fragen_liste_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Fragen Liste Zeile",
                     B_DATA : HELP_x_fragen_liste+HELP_standard ]);
    return old;
}

static int x_fragen_liste_total(mapping menue)
{
    return get_master()->get_xfragen(menue
        +([UI_OPT_COUNTALL:1]) );
}

static string * x_fragen_liste_display(mapping menue)
{
    mixed * xlines = get_master()->get_xfragen(menue
        +([UI_OPT_LIMIT:  menue[B_NUM_LINES],
           UI_OPT_OFFSET: menue[B_CURRENT_LINE] ]) );
    if (xlines == 0 || sizeof(xlines) <= 0) 
        return ({"Keine Fragen zur Auswahl!"});
    string * lines = ({});
    mapping cache = ([]);
    int i;
    for (i=0;i<sizeof(xlines);i++)
    {
        int ix = i+menue[B_CURRENT_LINE]+1;
        cache[ix] = xlines[i][0]; // id
        lines += ({ sprintf("%02d %s",
            ix,xlines[i][3]||"") }); // frage
    }
    menue[UI_OPT_CACHE] = cache;
    return lines;
}

static string x_fragen_liste_prompt(mapping menue)
{
    return sprintf("Fragen %d,%d [<nr>,q,z,?]",
        menue[B_CURRENT_LINE]+1,menue[B_END_LINE]);
}

static mixed x_fragen_liste_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    mapping menue = menues[<1];
    mapping cache = menue[UI_OPT_CACHE]||([]);
    int ix = to_int(split[<1]);
    if (sizeof(split)==1 && ix > 0)
    {
        if (member(cache,ix))
        {
            return menues + ({([
                B_TYPE : "x_frage_admin",
                UI_OPT_XF_ID : cache[ix],
            ])});
        }
        else
        {
            browse_write_line("Nummer passt nicht zur Anzeige.");
            return B_DONE;
        }
    }
    switch(split[0])
    {
        case "n": // neu
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zur Neuanlage.");
                return B_DONE;
            }
            ix = get_master()->create_x_frage(menue[UI_OPT_XT_ID]);
            if (ix <= 0)
            {
                browse_write_line("Fehler beim Erstellen.");
                return B_DONE;
            }
            return menues + ({([
                B_TYPE : "x_frage_admin",
                UI_OPT_XF_ID : ix,
            ])});   
        case "up":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei up <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xfrage_seq(cache[ix],-1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Anfang erreicht.");
                    return B_DONE;
            }
        case "down":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei down <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xfrage_seq(cache[ix],1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Ende erreicht.");
                    return B_DONE;
            }
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_frage_admin ({ \
"In dieser Ansicht wird eine Frage editiert.",\
})

private string* get_x_frage(mapping options)
{
    string * lines = ({});
    mapping m_frage = get_master()->get_x_frage(options[UI_OPT_XF_ID]);
    if (!mappingp(m_frage))
    {
        return ({"Keine Anzeige."});
    }
    lines+=({"Fragen-ID: "+m_frage[UI_OPT_XF_ID]
             +" (i)Unified-ID: "+m_frage[UI_OPT_UNIFIED_ID]});
    lines+=({"Themen-ID: "+m_frage[UI_OPT_XT_ID]
             +" Sequenz: "+m_frage[UI_OPT_SEQUENCE]});
    lines+=({"(f) Frage: "+m_frage[UI_OPT_FRAGE],"(b) Beschreibung:"});
    lines+=explode(wrap(m_frage[UI_OPT_DESCRIPTION]||""),"\n")[..<2];
    lines+=({"(ft) Fragentyp: "
        +UI_FRAGEN_TYP_LISTE[m_frage[UI_OPT_FRAGEN_TYP]]});
    lines+=({"(aa) Anzahl Antworten: "+m_frage[UI_OPT_ANZ_ANTWORTEN]});
    lines+=({"(a) Antwortliste"});
    return lines;
}

static mixed x_frage_admin_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zum Editieren der Frage Zeile",
                     B_DATA : HELP_x_frage_admin+HELP_standard ]);
    return old;
}

static int x_frage_admin_total(mapping menue)
{
    return sizeof(get_x_frage(menue));
}

static string * x_frage_admin_display(mapping menue)
{
    menue[B_DATA] = get_x_frage(menue);
    return staticmore_display(menue);
}

static string x_frage_admin_prompt(mapping menue)
{
    return sprintf("Frage %d %d/%d [i,t,b,ut,s,z,q]",
        menue[UI_OPT_XF_ID],menue[B_CURRENT_LINE],menue[B_END_LINE]);
}

static mixed x_frage_admin_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    string param = implode(split[1..]," ");
    mapping menue = menues[<1];
    mapping opt = ([UI_OPT_XF_ID:menue[UI_OPT_XF_ID] ]);
    int ix;
    if (!query_commander(TP) || TI!=TP) 
    {
        browse_write_line("Kein Zugriff!");
        return B_QUIT;
    }
    switch (split[0])
    {
        case "i": // i <unified-id>
            if (sizeof(split)==2 || param=="")
            {
                opt[UI_OPT_UNIFIED_ID] = param;
                if (get_master()->set_x_frage(opt))
                {
                    browse_write_line("unified-id gesetzt.");
                    return B_REBUILT;
                }
                else
                {
                    browse_write_line("Fehler beim Speichern.");
                    return B_DONE;
                }
            }
            else
            {
                browse_write_line("Syntax: i <unified-id>");
                browse_write_line(
                    "Wobei die ID keine Leerzeichen enthalten darf.");
                return B_DONE;
            }
        case "f": // f <frage>
            opt[UI_OPT_FRAGE] = param;
            if (get_master()->set_x_frage(opt))
            {
                browse_write_line("Frage gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "b": // b <beschreibung> bzw b -> editor?
            opt[UI_OPT_DESCRIPTION] = param;
            if (get_master()->set_x_frage(opt))
            {
                browse_write_line("Beschreibung gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "ft":
            // TODO pruefen ob schon Auswertungen
            // TODO pruefen ob frei oder frei/erw. und Antworten vorhanden
            return menues + ({([
                B_TYPE : "x_fragen_typ_wahl",
                UI_OPT_XF_ID: menue[UI_OPT_XF_ID],
            ])});
        case "aa":
            ix = to_int(param);
            if (ix < -1)
            {
                browse_write_line("Ungültiger Wert.");
                return B_DONE;
            }
            opt[UI_OPT_ANZ_ANTWORTEN] = ix;
            if (get_master()->set_x_frage(opt))
            {
                browse_write_line("Anzahl Antworten gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
        case "a":
            return menues + ({([
                B_TYPE : "x_antworten_liste",
                UI_OPT_XF_ID: menue[UI_OPT_XF_ID],
            ])});
    }
    return B_NOTHING;    
}

//---------------------------------------------------------------------------
#define HELP_x_fragen_typ_wahl ({ \
"In dieser Ansicht wird der Fragentyp ausgewaehlt.",\
})

static mixed x_fragen_typ_wahl_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zur Auswahl des Fragentyps Zeile",
                     B_DATA : HELP_x_fragen_typ_wahl+HELP_standard ]);
    return old;
}

static int x_fragen_typ_wahl_total(mapping menue)
{
    return sizeof(UI_FRAGEN_TYP_LISTE);
}

static string * x_fragen_typ_wahl_display(mapping menue)
{
    string * liste = ({});
    int i;
    for (i=0;i<sizeof(UI_FRAGEN_TYP_LISTE);i++)
    {
        liste += ({sprintf("%02d %s",
            i+1,UI_FRAGEN_TYP_LISTE[i])});
    }
    menue[B_DATA] = liste;
    return staticmore_display(menue);
}

static string x_fragen_typ_wahl_prompt(mapping menue)
{
    return sprintf("Auswahl Fragentyp %d/%d [<nr>,z,q]",
        menue[B_CURRENT_LINE]+1,menue[B_END_LINE]);
}

static mixed x_fragen_typ_wahl_action(string str, mapping * menues)
{
    int ix = to_int(space(str));
    if (ix != 0)
    {
        if (ix <= 0 || ix >= sizeof(UI_FRAGEN_TYP_LISTE))
        {
            browse_write_line("Ungültiger Zahlenbereich");
            return B_DONE;
        }
        mapping opt = ([UI_OPT_XF_ID:menues[<2][UI_OPT_XF_ID] ]);
        opt[UI_OPT_FRAGEN_TYP] = ix - 1;
        if (get_master()->set_x_frage(opt))
        {
            browse_write_line("Fragentyp gesetzt.");
            menues[<2][B_FLAGS] |= BF_DIRTY;
            return menues[0..<2];
        }
        else
        {
            browse_write_line("Fehler beim Speichern.");
            return B_DONE;
        }
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_antworten_liste ({ \
"In dieser Ansicht sind Fragen zur Wahl zu sehen.",\
})
static mixed x_antworten_liste_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zu Antworten Liste Zeile",
                     B_DATA : HELP_x_antworten_liste+HELP_standard ]);
    return old;
}

static int x_antworten_liste_total(mapping menue)
{
    return get_master()->get_xantworten(menue
        +([UI_OPT_COUNTALL:1]) );
}

static string * x_antworten_liste_display(mapping menue)
{
    mixed * xlines = get_master()->get_xantworten(menue
        +([UI_OPT_LIMIT:  menue[B_NUM_LINES],
           UI_OPT_OFFSET: menue[B_CURRENT_LINE] ]) );
    if (xlines == 0 || sizeof(xlines) <= 0) 
        return ({"Keine Antworten zur Auswahl!"});
    string * lines = ({});
    mapping cache = ([]);
    int i;
    for (i=0;i<sizeof(xlines);i++)
    {
        int ix = i+menue[B_CURRENT_LINE]+1;
        cache[ix] = xlines[i][0]; // id
        lines += ({ sprintf("%02d %s",
            ix,xlines[i][3]||"") }); // antwort
    }
    menue[UI_OPT_CACHE] = cache;
    return lines;
}

static string x_antworten_liste_prompt(mapping menue)
{
    return sprintf("Antworten %d,%d [<nr>,q,z,?]",
        menue[B_CURRENT_LINE]+1,menue[B_END_LINE]);
}

static mixed x_antworten_liste_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    mapping menue = menues[<1];
    mapping cache = menue[UI_OPT_CACHE]||([]);
    int ix = to_int(split[<1]);
    if (sizeof(split)==1 && ix > 0)
    {
        if (member(cache,ix))
        {
            return menues + ({([
                B_TYPE : "x_antwort_admin",
                UI_OPT_XA_ID : cache[ix],
            ])});
        }
        else
        {
            browse_write_line("Nummer passt nicht zur Anzeige.");
            return B_DONE;
        }
    }
    switch(split[0])
    {
        case "n": // neu
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung zur Neuanlage.");
                return B_DONE;
            }
            ix = get_master()->create_x_antwort(menue[UI_OPT_XF_ID]);
            if (ix <= 0)
            {
                browse_write_line("Fehler beim Erstellen.");
                return B_DONE;
            }
            return menues + ({([
                B_TYPE : "x_antwort_admin",
                UI_OPT_XA_ID : ix,
            ])});   
        case "up":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei up <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xantwort_seq(cache[ix],-1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Anfang erreicht.");
                    return B_DONE;
            }
        case "down":
            if (!query_commander(TP))
            {
                browse_write_line("Keine Berechtigung für die Sequenz.");
                return B_DONE;
            }
            if (!member(cache,ix))
            {
                browse_write_line("Ungültiges Element bei down <nr>");
                return B_DONE;
            }
            switch (get_master()->change_xantwort_seq(cache[ix],1))
            {
                default:
                case 0:
                    browse_write_line("Interner Fehler Sequenzänderung.");
                    return B_DONE;
                case 1:
                    browse_write_line("Sequenz geändert.");
                    return B_REBUILT;
                case 2:
                    browse_write_line("Ende erreicht.");
                    return B_DONE;
            }
    }
    return B_NOTHING;
}

//---------------------------------------------------------------------------
#define HELP_x_antwort_admin ({ \
"In dieser Ansicht wird eine Antwort editiert.",\
})

private string* get_x_antwort(mapping options)
{
    string * lines = ({});
    mapping m_antwort = get_master()->get_x_antwort(options[UI_OPT_XA_ID]);
    if (!mappingp(m_antwort))
    {
        return ({"Keine Anzeige."});
    }
    lines+=({"Auswertungs-ID: "+m_antwort[UI_OPT_XA_ID]
             +" (i)Unified-ID: "+m_antwort[UI_OPT_UNIFIED_ID]});
    lines+=({"Fragen-ID: "+m_antwort[UI_OPT_XF_ID]
             +" Sequenz: "+m_antwort[UI_OPT_SEQUENCE]});
    lines+=({"(a) Antwort: "+m_antwort[UI_OPT_ANTWORT]});
    return lines;
}

static mixed x_antwort_admin_init(mapping old)
{
    old[B_START_LINE] = 0;
    old[B_HELP] = ([ B_TYPE : B_STATICMORE,
                     B_PROMPT : "Hilfe zum Editieren der Antwort Zeile",
                     B_DATA : HELP_x_antwort_admin+HELP_standard ]);
    return old;
}

static int x_antwort_admin_total(mapping menue)
{
    return sizeof(get_x_antwort(menue));
}

static string * x_antwort_admin_display(mapping menue)
{
    menue[B_DATA] = get_x_antwort(menue);
    return staticmore_display(menue);
}

static string x_antwort_admin_prompt(mapping menue)
{
    return sprintf("Antwort %d %d/%d [i,a,z,q]",
        menue[UI_OPT_XA_ID],menue[B_CURRENT_LINE],menue[B_END_LINE]);
}

static mixed x_antwort_admin_action(string str, mapping * menues)
{
    string * split = explode(space(str)," ");
    string param = implode(split[1..]," ");
    mapping menue = menues[<1];
    mapping opt = ([UI_OPT_XA_ID:menue[UI_OPT_XA_ID] ]);
    if (!query_commander(TP) || TI!=TP) 
    {
        browse_write_line("Kein Zugriff!");
        return B_QUIT;
    }
    switch (split[0])
    {
        case "i": // i <unified-id>
            if (sizeof(split)==2 || param=="")
            {
                opt[UI_OPT_UNIFIED_ID] = param;
                if (get_master()->set_x_antwort(opt))
                {
                    browse_write_line("unified-id gesetzt.");
                    return B_REBUILT;
                }
                else
                {
                    browse_write_line("Fehler beim Speichern.");
                    return B_DONE;
                }
            }
            else
            {
                browse_write_line("Syntax: i <unified-id>");
                browse_write_line(
                    "Wobei die ID keine Leerzeichen enthalten darf.");
                return B_DONE;
            }
        case "a": // a <antwort>
            opt[UI_OPT_ANTWORT] = param;
            if (get_master()->set_x_antwort(opt))
            {
                browse_write_line("Antwort gesetzt.");
                return B_REBUILT;
            }
            else
            {
                browse_write_line("Fehler beim Speichern.");
                return B_DONE;
            }
    }
    return B_NOTHING;    
}

//---------------------------------------------------------------------------

private void uiko_will_lernen()
{
    send_message_to( ({ENV_TO}), MT_NOISE, MA_COMM, wrap_say(
        "Uiko jammert dir leise ins Ohr:","Lass mich bitte mit uiko:lerne "
        "etwas dazulernen."));
}

int check_current()
{
    if (clonep())
    {
        if (__FILE__->check_current() > object_time())
        {
            call_out(#'uiko_will_lernen, 3+random(7));
            return 0;
        }
        return object_time();
    }
    else
    {
        return program_time();
    }
}

private void abbruch_lehrstunde(object pl, object old, object new)
{
    if (pl && pl->query_ghost())
    {
        pl->close_con();
    }
    send_message_to(({pl}), MT_LOOK, MA_LOOK, wrap(
            "Uiko ist auf seiner Reise wohl mit einem Kuscheli "
            "zusammengestoßen und kommt nicht wieder. Du wirst Dir bei Leo "
            "einen neuen Uiko holen müssen."));
    if (old)
    {
        destruct(old);
    }
    if (new)
    {
        destruct(new);
    }
}

void lehrstunde(object pl, object old)
{
    if (clonep() || !playerp(pl) || !objectp(old) || !clonep(old) 
        || ENV(old)!=pl || program_name(old) != __FILE__ 
        || object_time(old) >= program_time())
    {
        send_message_to(pl, MT_LOOK, MA_LOOK, wrap(
            "Uiko braucht momentan nichts lernen, es jammert schon, "
            "sollte es nötig sein."));
        return;
    }
    send_message_to(pl, MT_LOOK, MA_LOOK, wrap(
        "Uiko macht sich reisefertig und verschwindet mit einem Plopp."));
    closure cl = #'abbruch_lehrstunde;
    call_out(cl ,0, pl, old, 0);
    mixed data = old->query_auto_load();
    old->remove();
    if (old)
        destruct(old);
    object new = clone_object(__FILE__);
    new->init_arg(data);
    remove_call_out(cl);
    call_out(cl ,0, pl, old, new);
    if (pl->query_ghost())
    {
        pl->open_con();
    }
    int mr = new->move(pl, ([MOVE_FLAGS:MOVE_ERR_REMOVE]));
    if (pl->query_ghost())
    {
        pl->close_con();
    }
    switch (mr)
    {
    case MOVE_OK:
        send_message_to(pl, MT_LOOK, MA_LOOK, wrap(
            "Uiko erscheint wohlgelaunt wieder."));
        break;
    default:
        send_message_to(pl, MT_LOOK, MA_LOOK, wrap(
            "Uiko bleibt in den weitläufigen Strukturen des "
            "Raum-Zeit-Kontinuums verloren. Du wirst Dir bei Leo einen "
            "neuen Uiko holen müssen."));
        break;
    }
    remove_call_out(cl);
}

private int uiko_lerne()
{
    __FILE__->lehrstunde(TP,TO);
    return 1;
}


void test_clones()
{
    if (!clonep())
    {
        rclones -= ({ 0 });
        rclones->check_current();
    }
}

void msg_to_clones(string pre, string msg)
{
    if (!clonep())
    {
        rclones -= ({ 0 });
        send_message_to( map(rclones, (: ENV($1) :)), MT_NOISE, MA_COMM, 
                wrap_say(pre,msg));
    }
}


void register_clone(object ob)
{
    if (!clonep() && objectp(ob) && program_name(ob) == __FILE__)
    {
        rclones -= ({ 0, ob });
        rclones += ({ ob });
        //PDBG(sprintf("uiko %Q registriert",ob));
    }
}

void reset()
{
    // Momentan muss reset noch nicht im create aufgerufen werden
    check_current();
}

void create()
{
    "*"::create();
    initialize("kobold",100);
    set_weight(0);
    set_adjektiv("klein");
    set_name("uiko");
    set_short("Uiko, ein Umfrage- und Ideenkobold");
    set_gender("maennlich");
    set_id(({"uiko","umfragekobold","ideenkobold","kobold"}));
    set_long( "Ein winziger Umfrage- und Ideenkobold kurz Uiko, "
            "der helfend auf Deiner Schulter hockt.");
    set_rollen_dirs(0); //Keine Rollen
    set(P_LOOK_MSG, ""); // Kein Anzeichen, dass der Eigentuemer es anschaut.
    
    init_security_trust_mudlib();
    init_security_for_actions();
    add_security_condition((: return adminp($1); :));
    
    clone_object("/obj/soul")->move(this_object());
    close_con();
    call_out(#'check_env,0, TI); // nach dem Move...
    if (clonep())
        __FILE__->register_clone(this_object());
    else
        call_out("test_clones",0);
}

public void transfer_clones(object * obs)
{
    string * on1 = explode(object_name(TO),"/");
    string * on2 = explode(object_name(PO),"/");
    if (implode(on1[..<2],"/") != implode(on2[..<2],"/"))
        return;
    if ( on2[<1] != ("$"+on1[<1]+"$"))
        return;
    if (!clonep() && pointerp(obs))
    {
        rclones -= obs;
        rclones += obs;
        rclones -= ({ 0 });
    }
}

void abort_renewal() 
{
}
void finish_renewal(object neu) 
{
    if (!clonep())
    {
        neu->transfer_clones(rclones);
    }
}
void prepare_renewal() 
{
}

// EOF
