// This file is part of UNItopia Mudlib.
// ----------------------------------------------------------------
// File:	/obj/newsreader.c
// Description: Newsreader
// Author:	Freaky (16.01.93)
// Modified by: Garthan	(13.12.94) "BEACHTE:" Meldungen
//		Monty (12.05.95)  query_hash() eingefuegt
//		Garthan (28.05.95)  y Modus, R Befehl
//		Garthan	(13.09.95)  unsub Gruppen/Bretter anzeigen mit R,Y
//		Freaky (21.12.1998) News-Structure vorbelegen fuer +Neues+
//              Gnomi (??.06.2001)  Neu geschrieben
//                                  Dazu Teile von Crofts Newstool verwendet

/*
Kleinere Probleme:
  - Wenn Leute auf der Ignore-Liste Artikel schreiben, so wird das Brett
    und die Gruppe erstmal als Neu gekennzeichnet, da es zu aufwendig
    wird, wenn der Newsd jedesmal den Baum einlesen muesste, um die
    Autoren der Artikel festzustellen.
  - Im Optionsmenue im CHARMODE wird nach einem 'q' ein '!befehl' ignoriert
    (sprich nicht als Befehl, sondern als Eingabe fuer input_to aufgefasst),
    passiert beim Mushcient (www.mushclient.com, 3.03)
*/

/************************************************************************
 *                                                                      *
 *   Inhaltsverzeichnis:                                                *
 *    - Inherits, Includes, Defines und globale Variablen               *
 *    - Interface zum Newsreader                                        *
 *    - Diverse Hilfsfunktionen                                         *
 *    - internes Interface zu den Daten im Spieler                      *
 *    - Funktionen, um mit den Newsdaten zu arbeiten                    *
 *    - Funktionen, um mit den Menuedaten zu arbeiten                   *
 *    - Menue - Implementation                                          *
 *    - Funktionen zum Schreiben von Artikeln                           *
 *    - Funktionen zum Parsing der Eingabe                              *
 *    - Diverse Befehle des Newssreaders                                *
 *    - Zusaetzliche add_actions                                        *
 *    - Applied Lfuns                                                   *
 *                                                                      *
 ************************************************************************/

/************************************************************************
 *   Inherits, Includes, Defines und globale Variablen                  *
 ************************************************************************/

#pragma strong_types
#pragma pedantic
#pragma no_inherit

inherit "/i/install";
inherit "/i/item";
inherit "/i/tools/browser";

#include <apps.h>
#include <browser.h>
#include <config.h>
#include <editor.h>
#include <files.h>
#include <hlp.h>
#include <invis.h>
#include <input_to.h>
#include <level.h>
#include <mail.h>
#include <message.h>
#include <more.h>
#include <news.h>
#include <rtlimits.h>
#include <stats.h>

// Falls das Optionsmenue im Charmode betrieben werden soll
#undef OPTIONS_IN_CHARMODE
#ifdef UNItopia
//#define EVAL_DEBUGGER "gnomi"
#endif

#define IS_AL (wizp(owner) || GABE(owner,"mn") ||  ((flags["Flags"]&32) && testplayerp(owner)))  // Ist Autoloader?

//Bretter, bei denen keine Anzeige des Titels und der Gruppe erfolgt.
#define GEHEIMBRETTER (["Domains","Goetter","Interdomain","Programmierung", \
                        "Ratsmitglieder", "Gilden"])

// Defines fuer die ganzen More-Parameter
#define BRETT_STAT_LINE ({ \
    (sizeof(searchinfo)?("Suche: \""+searchinfo[<1][SRCH_TEXT]+"\" "):"")+ \
    "[q,<nr>,n,r,?] ", \
    "----------- Folgende Bretter stehen zur Auswahl: ------------------------------", \
    "-------------------------------------------------------------------------------", \
    "...--------------------------------------------------------------------- (MORE)", \
    "...--------------------------------------------------------------------- (MORE)" })
#define BRETT_MORE_FLAG (M_DO_NOT_END|M_FRAME|(flags["Scroll"]&1?M_SCROLL:0))
#define BRETT_LEER_STAT_LINE(grund) ({ \
    (sizeof(searchinfo)?("Suche: \""+searchinfo[<1][SRCH_TEXT]+"\" "):"")+ \
    "[q,r,Y,?] ", \
    left("----------- " + (grund) + " ",79,"-"), \
    "-------------------------------------------------------------------------------", \
    "...--------------------------------------------------------------------- (MORE)", \
    "...--------------------------------------------------------------------- (MORE)" })
#define BRETT_LEER_MORE_FLAG (M_DO_NOT_END|M_FRAME|M_FORCE|(flags["Scroll"]&1?M_SCROLL:0))

#define GRUPPE_STAT_LINE(brett) ({ \
    (sizeof(searchinfo)?("Suche: \""+searchinfo[<1][SRCH_TEXT]+"\" "):("Brett '"+(brett)+"' "))+\
    "[q,<nr>,n,z,r,?] ", \
    left("----------- Brett '"+(brett)+"' ",79,"-"),\
    "-------------------------------------------------------------------------------", \
    "...--------------------------------------------------------------------- (MORE)", \
    "...--------------------------------------------------------------------- (MORE)" })

#define GRUPPE_MORE_FLAG(brett) (M_DO_NOT_END|M_FRAME|(flags["Scroll"]&2?M_SCROLL:0))
#define GRUPPE_LEER_STAT_LINE(brett,grund) ({ \
    (sizeof(searchinfo)?("Suche: \""+searchinfo[<1][SRCH_TEXT]+"\" "):("Brett '"+(brett)+"' "))+\
    "[q,<nr>,n,z,r,?] ", \
    left("----------- Brett '"+(brett)+"': "+(grund)+" ",79,"-"),\
    "-------------------------------------------------------------------------------", \
    "...--------------------------------------------------------------------- (MORE)", \
    "...--------------------------------------------------------------------- (MORE)" })
#define GRUPPE_LEER_MORE_FLAG(brett) (M_DO_NOT_END|M_FRAME|M_FORCE|(flags["Scroll"]&2?M_SCROLL:0))

#define ARTIKEL_STAT_LINE(brett,gruppe) ({ \
    (sizeof(searchinfo)?("Suche: \""+searchinfo[<1][SRCH_TEXT]+"\" ")\
    :("Brett '"+(no_back?"":((brett)+"' Gruppe '"))+(gruppe)+"' "))+"[q,<nr>,n,"+(no_back?"":"z,")+"l,b,s,r,?] ", \
    left("----------- Brett '"+(no_back?"":((brett)+"' Gruppe '"))+(gruppe)+"' ",79,"-"),\
    "-------------------------------------------------------------------------------", \
    "...--------------------------------------------------------------------- (MORE)", \
    "...--------------------------------------------------------------------- (MORE)" })
#define ARTIKEL_MORE_FLAG(brett,gruppe) (M_DO_NOT_END|M_FRAME|(flags["Scroll"]&4?M_SCROLL:0))

#define TEXT_STAT_LINE(brett,gruppe,nr) \
    "*** Artikel "+nr+": %i/%i [q,<,>,+,-,n,z,r,?] "
#define TEXT_MORE_FLAG(brett,gruppe,nr) (BF_NO_MENUE|(flags["Scroll"]&8?M_SCROLL:0))

#define NEWS_TMP_FILE ("/w/"+({string})owner->query_real_name()\
		       +((file_size("/w/"+({string})owner->query_real_name()+"/priv")==FSIZE_DIR)?"/priv":"")\
		       +"/NEWS_ARTICLE")

// Einfach, um die Menuestruktur m zu erneuern und dabei aber die Meldung
// x ausgeben.
#define RENEW(x,m) ((m[0][BR_STATUS_BYTE]&BF_NO_MENUE)\
                    ?((m[0][BR_USER][DISP_MENUE]=\
                        renew_menue_more_info(m[0][BR_USER][DISP_MENUE],1)),x)\
                    :(write(x), renew_menue_more_info(m,1)))
// Mit speziellem Renew-Flag
#define RENEW2(x,m,f) ((m[0][BR_STATUS_BYTE]&BF_NO_MENUE)\
                     ?((m[0][BR_USER][DISP_MENUE]=\
                         renew_menue_more_info(m[0][BR_USER][DISP_MENUE],f)),x)\
                     :(write(x), renew_menue_more_info(m,f)))

#define GET_PATH(m) get_path(map(m[1..<1],#'[,BR_PATH),m)

#define BEGIN_CACHE NEWSD->add_to_cache(owner)
#define END_CACHE   NEWSD->delete_from_cache(owner)

/*
 * Elemente von flags:
 *   "Editor":
 *      0: Mini-Ed
 *      1: ED (Goetter: Driver-ED, Spieler: Player-ED)
 *      2: Player_ED
 *      3: XED (Wizards only ;-)
 *   "Scroll": (Bitflag)
 *      1: Brett
 *      2: Gruppe
 *      4: Artikeluebersicht
 *      8: Artikel (und Zusammenfassungen)
 *     16: Sonstiges (Hilfe, Threadanzeige, Optionen)
 *   "SeeArticle":
 *     -1: Nur neue Nachrichten sehen
 *      0: Nur ungelesene Nachrichten
 *      1: Auch gelesene Nachrichten sehen
 *   "SeeGroup":
 *     -1: Nur Bretter/Gruppen mit ungelesenen Artikeln anzeigen
 *      0: Nur angemeldete Bretter/Gruppen sehen
 *      1: Auch nichtangemeldete Bretter/Gruppen sehen
 *   "Flags": (Bitflag)
 *      1: Threads immer zusammengeklappt zeigen
 *      2: Gruppenuebergreifend suchen
 *      4: 'z' springt vom Artikel zur Gruppenuebersicht
 *      8: Meldung beim Lesen/Schreiben eines Artikels ausgeben (Wiz only)
 *     16: Sicherheitsabfrage beim Schreiben von Artikeln ausgeschaltet
 *   "Ignore": (["name",...])
 *   "Last":
 *      0 oder ({}):      Er war zuletzt im Hauptmenue
 *      ({Brett}):        Er war zuletzt in diesem Brett
 *      ({Brett,Gruppe}): Er war zuletzt in dieser Gruppe
 */
mapping flags=([]);

// Eine Fehlermeldung, siehe set_brett_name
mixed fail_msg;

/*
 * Die Struktur des Spielers nach query_read_artikel():
 *   Brettname: Datum; Gruppen
 * (Datum -1 heisst nicht abonniert, ansonsten dient es zum groben Testen,
 * ob in dieser Newsgroup was neues gibt. Ausschlaggebend sind aber die
 * Daten der Gruppen.)
 *
 * Gruppen:
 *   Gruppenname: Datum, Mapping aller ungelesenen Artikel,
 *                       Mapping aller gelesenen Artikel seit Datum,
 *                       Mapping aller zwingend aufgeklappten Artikel,
 *                       Mapping aller zwingend zugeklappten Artikel
 */
mapping plinfo;

// Die Indizes fuer plinfo:
#define PLB_DATE      0
#define PLB_GROUPS    1

#define PLG_DATE      0
#define PLG_UNREAD    1
#define PLG_READ      2
#define PLG_OPEN      3
#define PLG_COLLAPSED 4

/*
 * Enthaelt die Suchergebnisse fuer jede Ebene der Suche
 * Pro Ebene, in welcher eine Suche durchgefuehrt wurde, ein Mapping
 * mit folgenden Eintraegen:
 *   SRCH_TEXT: Der Text, nachdem gesucht wurde
 *   SRCH_FILES: Array mit den Dateinamen ("Brett/Gruppe/Artikel")
 *   SRCH_ARTIKEL: Mapping ([ Brett: ([Gruppe: ([Artikel]) ]) ])
 */
mixed *searchinfo=({}); //TODO: Das ins BR_USER integrieren

#define SRCH_TEXT	"Text"
#define SRCH_FILES	"Dateien"
#define SRCH_ARTIKEL	"Artikel"


#define NEUES_BRETT_AN(name,zeit) plinfo+=([ name:zeit;m_allocate(1,5) ])
#define NEUES_BRETT_AB(name)      plinfo+=([ name:-1;0 ])
#define NEUE_GRUPPE_AN(brett,name,zeit) plinfo[brett,1]+=([name:zeit;([]);([]);([]);([]) ])
#define NEUE_GRUPPE_AB(brett,name)      plinfo[brett,1]+=([name:-1;0;0;0;0 ])

#define BRETT_AN(name,zeit) plinfo+=([ name:zeit;m_allocate(1,5) ])
#define BRETT_AB(name) plinfo+=([ name:-1;0 ])
#define GRUPPE_AN(brett,name,zeit) plinfo[brett,1]+=([name:zeit;([]);([]);([]);([]) ])
#define GRUPPE_AB(brett,name) plinfo[brett,1]+=([name:-1;0;0;0;0 ])

/*
 * Benutzerdefinierte Daten im Browser:
 * Level 1: ({ ({brettname, datum, nr}), ({brettname, datum, nr}),...})
 * Level 2: ({ ({gruppennname, datum, nr}), ({gruppenname, datum, nr}),...})
 * Level 3: ([ ART_INFO: ([ Nummer: Datum;
 *                                  Weg zur Wurzel (Array aus Pointern
 *                                  zu den Arrays in der Baumstruktur);
 *                                  Teilliste aller Artikelnummern in ihrer
 *                                  Reihenfolge der Teilbaeume (ohne diesen
 *                                  Artikel),
 *                          Nummer: ... 
 *                       ]),
 *             ART_LIST: ({ Nummer, Nummer, ... }),
 *             ART_THREADS: ({ Threadanfangsartikelnummer, ...})
 *             ART_DATE: aktuelles Datum
 *          ])
 * Level 4: 0
 *
 * In den benutzerdefinierten Daten stehen alle (auch ausgeblendete)
 * Bretter/Gruppen/Artikel drin.
 */

#define ART_INFO    "Info"
#define ART_LIST    "Liste"
#define ART_THREADS "Threads"
#define ART_DATE    "Date"

#define AI_DATE    0
#define AI_ROOTWAY 1
#define AI_SUBLIST 2
/*
 * Besondere Anzeigen (Optionen, Hilfe, Thread-Liste, Zusammenfassungen usw.):
 *
 *  - Menue mit Tiefe 1
 *  - BR_STATUS_BYTE enthaelt BF_NO_MENUE
 *  - BR_PATH: Art der Anzeige (siehe folgende Defines)
 *  - BR_USER: Mapping mit:
 *       DISP_MENUE: Original Menue des Newsreaders (dahin kommt man mit 'z')
 *       DISP_PATH: Enthaelt bei der Zusammenfassung/Threadanzeige den Pfad
 *                  ({Brett,Gruppe,Artikel})
 */
#define DISP_OPTIONS 1
#define DISP_HELP    2
#define DISP_THREAD  3
#define DISP_SUMMARY 4
#define DISP_MODS    5

#define DISP_MENUE "Menue"
#define DISP_PATH "Path"

// Darf man zurueckgehen?
#define NO_BACK1 (!(sizeof(bretter) || IS_AL))
#define NO_BACK2 no_back

object owner; // Der Eigentuemer
int no_back;  // Man kommt aus der Gruppe nicht mehr heraus.

mapping bretter; // Enthaelt die Namen der Bretter, die JEDER
                 // mit diesem Brett lesen kann. Standard: 0
		 // Bei 0 oder ([]) kommt man nicht mehr ins Hauptmenue,
		 // es sei denn man ist Gott oder Engel mit "mn".
		 // Das Mapping enthaelt entweder Eintraege der Form:
		 // ([ "brettname": 0 ]) oder
		 // ([ "brettname": ([ "gruppe"," gruppe" ]) ])
int zustand; // Wenn 1, dann im browse_end zerstoeren.
int reading; // Wenn 1, dann wird gerade gelesen.

/************************************************************************
 *   Interface zum Newsreader                                           *
 ************************************************************************/

/*
FUNKTION: query_bretter
DEKLARATION: string *query_bretter()
BESCHREIBUNG:
Liefert die Bretter, welche mit diesem Newsreader gelesen werden koennen.
VERWEISE: set_bretter, set_brett_name, query_brett_name, query_gruppen_name
GRUPPEN: news
*/
string *query_bretter()
{
    string *result = ({});
    
    if(!bretter)
	return 0;
    
    foreach(string brett, mapping gruppen: bretter)
    {
	if(!sizeof(gruppen))
	    result += ({ brett });
	else
	    foreach(string gruppe: gruppen)
		result += ({ brett+":"+gruppe });
    }
    return result;
}

/*
FUNKTION: set_bretter
DEKLARATION: string *set_bretter(string *strs)
BESCHREIBUNG:
Damit setzt man, welche Bretter mit diesem Newsreader gelesen werden koennen.
Bei 0 oder ({}) kann man nicht mehr ins Hauptmenue gelangen bzw. gar nix
lesen. Auf Engel mit der Gabe "mn" oder fuer Goetter hat diese Einstellung
keinen Einfluss. Moechte man nur bestimmte Gruppen an einem Brett erlauben,
so kann man "Brett:Gruppe" als String angeben.
VERWEISE: query_bretter, set_brett_name, query_brett_name, query_gruppen_name
GRUPPEN: news
*/
string *set_bretter(string *strs)
{
    if(!strs)
	return bretter = 0;
    
    bretter = ([]);
    
    foreach(string brett: strs)
    {
	string* teile = explode(brett, ":");
	if(sizeof(teile)==1)
	    m_add(bretter, brett, 0);
	else
	{
	    if(!member(bretter, teile[0]))
		m_add(bretter, teile[0], ([:0]));
	    if(bretter[teile[0]])
		m_add(bretter[teile[0]], teile[1]);
	}
    }
    return query_bretter();
}

/*
FUNKTION: set_brett_name
DEKLARATION: varargs void set_brett_name(string str, string grp, mixed fail)
BESCHREIBUNG:
Damit wird gesetzt, wo man mit diesem Newsreader zu lesen beginnt.
Wurde eine Gruppe angegeben, so kann man nur diese Gruppe lesen
(ein Rueckschritt zur Gruppenauswahl ist dann unmoeglich), es sei
denn, mit set_bretter wurden zuvor weitere lesbare Bretter angegeben.

In fail kann man eine Fehlermeldung (als String) oder eine Closure, welche
einen String mit der Fehlermeldung zurueckliefert, angeben. Die Fehlermeldung
wird ausgegeben, wenn derjenige dieses Brett nicht lesen darf und ein
Ruecksprung zur Gruppen- oder Brettauswahl nicht moeglich ist.
Der angegebene bzw. zurueckgelieferte String muss bereits Zeilenumbrueche
beinhalten. Die Closure erhaelt den Spieler (object) als 1. Parameter,
das Brett (string) als 2. und die Gruppe (string) als 3. Parameter.
Es kann passieren, dass diese Closure aufgerufen wird, auch wenn die
zurueckgelieferte Meldung im Endeffekt gar nicht genutzt wird.

Ruft man diese Funktion direkt im Newsreader auf, so sollte dies erst
geschehen, nachdem der Newsreader in den Spieler bewegt wurde.

VERWEISE: set_bretter, query_bretter, query_brett_name, query_gruppen_name
GRUPPEN: news
*/
varargs void set_brett_name(string str, string grp, mixed fail)
{
    flags["Last"]=(!str)?0:(!grp)?({str}):({str,grp});
    // Es wurde eine Gruppe angegeben und es gibt keine Auswahl an Brettern
    if (grp && !(IS_AL || sizeof(bretter)))
        no_back=1;	// Er darf die Gruppe nicht verlassen.
    else
        no_back=0;
    fail_msg = fail;
}

/*
FUNKTION: query_brett_name
DEKLARATION: string query_brett_name()
BESCHREIBUNG:
Liefert das Brett, an welchem der Newsreader zu lesen beginnt.
VERWEISE: set_bretter, query_bretter, set_brett_name, query_gruppen_name
GRUPPEN: news
*/
string query_brett_name() { return sizeof(flags["Last"])?flags["Last"][0]:0; }

/*
FUNKTION: query_gruppen_name
DEKLARATION: string query_gruppen_name()
BESCHREIBUNG:
Liefert die Gruppe, an welcher der Newsreader zu lesen beginnt.
VERWEISE: set_bretter, query_bretter, set_brett_name, query_brett_name
GRUPPEN: news
*/
string query_gruppen_name() { return (sizeof(flags["Last"])>1)?flags["Last"][1]:0; }

/*
FUNKTION: query_newsreader_active
DEKLARATION: int query_newsreader_active()
BESCHREIBUNG:
Liefert einen Wert !=0, wenn gerade mit diesem Newsreader gelesen wird.
VERWEISE: is_reading
GRUPPEN: news
*/
int query_newsreader_active() {return reading;}

/*
FUNKTION: is_reading
DEKLARATION: object is_reading()
BESCHREIBUNG:
Liefert den Leser dieses Newsreaders, falls mit ihm gerade gelesen wird.
VERWEISE: query_newsreader_active
GRUPPEN: news
*/
object is_reading() {return query_newsreader_active()?owner:0;}

/************************************************************************
 *   Diverse Hilfsfunktionen                                            *
 ************************************************************************/

/*
FUNKTION: zahl2str
DEKLARATION: private string zahl2str(int i)
BESCHREIBUNG:
Liefert diese Zahl als String. Zahlen von 0 bis 12 werden als 
Woerter geliefert.
VERWEISE: to_string
GRUPPEN: Root:Tools:Newsreader
*/
private string zahl2str(int i)
{
   if(i >= 0 && i <= 12)
      return ({"kein","ein", "zwei", "drei", "vier", "fünf", "sechs",
	       "sieben", "acht", "neun", "zehn", "elf", "zwölf"})[i];
   return to_string(i);
}

/************************************************************************
 *   internes Interface zu den Daten im Spieler                         *
 ************************************************************************/

/*
 * FUNKTION: get_read_artikel
 * DEKLARATION: mapping get_read_artikel()
 * BESCHREIBUNG:
 * Liefert die Artikelstruktur zurueck. (Das Original, keine Kopie)
 * Dies darf nur vom Newsreader selbst aufgerufen werden und dient dazu,
 * dem Newsreader der Wizard-Shell diese Daten zur Verfuegung zu stellen.
 * VERWEISE: query_read_artikel
 * GRUPPEN: news
 */
mapping get_read_artikel() //Fuer Wiz-Shell
{
    if(load_name(previous_object())==load_name(this_object())
    && previous_object()!=this_object())
        return plinfo;
}

mapping update_bretter(mapping struc)
{
    if(mappingp(struc) && struc["Projekte",PLB_DATE]>0 && member(({string*})NEWSD->query_bretter(),"Staedte")>=0)
    {
	string* snames = ({string*}) NEWSD->query_gruppen("Staedte");
	mapping staedte = (({mapping})struc["Projekte", PLB_GROUPS]) & snames;
	
	if(sizeof(filter(staedte, (: $2[PLG_DATE] != -1 :))) > 0)
	{
	    if(!member(struc, "Staedte") || struc["Staedte", PLB_DATE]<=0)
		m_add(struc, "Staedte", struc["Projekte", PLB_DATE], ([:5]));
		
	    struc["Staedte", PLB_GROUPS] += staedte;
	    struc["Projekte", PLB_GROUPS] -= staedte;
	}
    }
    
    return struc;
}

/*
 * FUNKTION: query_read_artikel
 * DEKLARATION: mapping query_read_artikel()
 * BESCHREIBUNG:
 * Fragt die Artikelstruktur aus dem Spieler ab.
 * Zur Zeit werden einige Konvertierungen veranstaltet.
 * VERWEISE: set_read_artikel
 * GRUPPEN: news
 */
mapping query_read_artikel() // In Datenstruktur konvertieren
{
    mapping struc;
// Dieser Teil ist fuer die Wiz-Shell und sollte sich eruebrigen,
// wenn keine Konversion zur alten Struktur stattfindet.
    object nr = find_player(owner->query_real_name());
    if(nr) nr = present_clone(load_name(this_object()),nr);
    if(nr) struc = ({mapping})nr->get_read_artikel();
    if(struc) return struc;
// Ende des Wiz-Shell-Teil
    struc = ({mapping}) NEWSD->query_read_artikel(owner);
    return update_bretter(struc);
}

/*
 * FUNKTION: set_read_artikel
 * DEKLARATION: void set_read_artikel(mapping m)
 * BESCHREIBUNG:
 * Speichert die Artikelstruktur im Spieler ab.
 * Es werden Konvertierungen in die alte Struktur vorgenommen.
 * VERWEISE: query_read_artikel
 * GRUPPEN: news
 */
void set_read_artikel(mapping m) // In alte Datenstruktur konvertieren
{
#if 0
    mixed struc = unmkmapping(m);
    struc[2] =  map(struc[2],
	(: if(!$1) return 0;
	   $1 = unmkmapping($1);
	   $1[2] = map($1[2],(:$1 && unmkmapping($1)[0]:));
	   return $1[0..2];
        :));
#endif
    NEWSD->set_read_artikel(m,owner);
}

/************************************************************************
 *   Funktionen, um mit den Newsdaten zu arbeiten                       *
 ************************************************************************/

/*
 * FUNKTION: check_existance
 * DEKLARATION: private varargs void check_existance(mixed *wo, int recursive)
 * BESCHREIBUNG:
 * Testet, ob die in der Artikelstruktur eingetragenen Bretter, Gruppen bzw.
 * Artikel noch vorhanden sind. Es wird normalerweise nur in wo (entweder
 * ({}) fuer alle Bretter, ({brett}) fuer alle Gruppen an diesem Brett oder
 * ({brett,gruppe}) fuer alle Artikel) getestet. Wenn recursive!=0, werden
 * auch die Gruppen der Bretter bzw. die Artikel der Gruppen getestet,
 * wenn eigentlich nur die Bretter bzw. Gruppen ueberprueft haetten werden
 * sollen.
 * VERWEISE: query_read_artikel, update_plinfo
 * GRUPPEN: news
 */
private varargs void check_existance(mixed *wo, int recursive)
{
    if(!plinfo) return;
    if(!sizeof(wo))
    {
	string *alle_bretter = ({string*}) NEWSD->query_bretter();
	// Alle Bretter aus plinfo, die nicht in alle_bretter drin sind,
	// rausschmeissen.
	plinfo-=plinfo-mkmapping(alle_bretter);
	if(recursive)
	    foreach(string brett,int datum:plinfo)
		if(datum>=0) check_existance(({brett}),recursive);
    }
    else if(sizeof(wo)==1)
    {
	string *alle_gruppen = ({string*}) NEWSD->query_gruppen(wo[0]);
        
	if(!member(plinfo,wo[0]) || plinfo[wo[0],PLB_DATE]<0)
            return;

	// Alle Gruppen rausschmeissen, die nicht in alle_gruppen drin sind.
	plinfo[wo[0],PLB_GROUPS] -= plinfo[wo[0],PLB_GROUPS]-mkmapping(alle_gruppen);
	if(recursive)
	    foreach(string gruppe, int datum:plinfo[wo[0],PLB_GROUPS])
		if(datum>=0) check_existance(wo+({gruppe}),recursive);
    }
    else if(sizeof(wo)==2)
    {
	mapping alle_nummern = mkmapping(({int*})NEWSD->query_artikel_numbers(wo[0],wo[1]));

	if(!member(plinfo,wo[0]) || plinfo[wo[0],PLB_DATE]<0 ||
           !member(plinfo[wo[0],PLB_GROUPS],wo[1]) ||
           plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]<0)
            return;

        for(int i=1;i<5;i++)
	    plinfo[wo[0],PLB_GROUPS][wo[1],i] -= plinfo[wo[0],PLB_GROUPS][wo[1],i]-alle_nummern;
    }
}

/*
 * FUNKTION: update_plinfo
 * DEKLARATION: private void update_plinfo(mixed wo, mixed m, int all)
 * BESCHREIBUNG:
 * Dies sollte aufgerufen werden, wenn ein Menue verlassen wird.
 * Die Artikelstruktur wird dann so aktualisiert, dass diese Gruppe bzw. Brett
 * als 'nicht mehr neu' gekennzeichnet wird.
 * Falls der Newsreader verlassen wird, sollte all!=0 sein.
 * VERWEISE: query_read_artikel, check_existance
 * GRUPPEN: news
 */
private void update_plinfo(mixed wo, mixed m, int all)
{
    switch(sizeof(wo))
    {
        case 3:
	case 2:
	    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]+=
		m_reallocate(filter(m[2][BR_USER][ART_INFO],
                (:$2[AI_DATE]>=$3:),plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]),0)
	        - plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ];
	    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]=
                filter(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ],
                (: $3[$1,AI_DATE]>=$4 :),
                m[2][BR_USER][ART_INFO],
                m[2][BR_USER][ART_DATE]);
	    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]=m[2][BR_USER][ART_DATE];
	    if(!all)
                break;
	case 1:
	    plinfo[wo[0],PLB_DATE]=time();
    }
}

/*
 * FUNKTION: mark_all_as_read
 * DEKLARATION: private varargs void mark_all_as_read(<string|int|int*>* wo)
 * BESCHREIBUNG:
 * Alle Artikel von wo=({ [brett [,gruppe [,({artikel})]]] }) werden als
 * gelesen markiert. Dazu sollte vorher check_existance aufgerufen werden,
 * damit alle Bretter/Gruppen/Artikel noch existieren.
 * VERWEISE: mark_all_as_unread, query_read_artikel, check_existance
 * GRUPPEN: news
 */
private varargs void mark_all_as_read(<string|int|int*>* wo)
{
    mapping nrs;

    if(!sizeof(wo))
    {
        foreach(string brett, int datum:plinfo)
	    if(datum>=0) mark_all_as_read(({brett}));
	return;
    }

    if(!member(plinfo,wo[0]))
        NEUES_BRETT_AN(wo[0],time());

    if(sizeof(wo)==1)
    {
	foreach(string gruppe, int datum:plinfo[wo[0],PLB_GROUPS])
	    if(datum>=0) mark_all_as_read(wo+({gruppe}));
	return;
    }

    if(sizeof(wo)==2)
    {
        NEUE_GRUPPE_AN(wo[0],wo[1],time());
        return;
    }
    
    if(!member(plinfo[wo[0],PLB_GROUPS],wo[1]))
        NEUE_GRUPPE_AN(wo[0],wo[1],time());

    if(intp(wo[2]))
        nrs = ([wo[2]]);
    else
        nrs = mkmapping(wo[2]);

    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD] -= nrs;
    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ] += nrs;
}

/*
 * FUNKTION: mark_all_as_unread
 * DEKLARATION: private varargs void mark_all_as_read(<string|int|int*>* wo)
 * BESCHREIBUNG:
 * Alle Artikel von wo=({ [brett [,gruppe [,({artikel})]]] }) werden als
 * ungelesen markiert. Dazu sollte vorher check_existance aufgerufen werden,
 * damit alle Bretter/Gruppen/Artikel noch existieren.
 * VERWEISE: mark_all_as_read, query_read_artikel, check_existance
 * GRUPPEN: news
 */
private varargs void mark_all_as_unread(<string|int|int*>* wo)
{
    mapping nrs;

    if(!sizeof(wo))
    {
        foreach(string brett, int datum:plinfo)
	    if(datum>=0) mark_all_as_unread(({brett}));
	return;
    }

    if(!member(plinfo,wo[0]))
    {
        NEUES_BRETT_AN(wo[0],0);
        return;
    }

    if(sizeof(wo)==1)
    {
	foreach(string gruppe, int datum:plinfo[wo[0],PLB_GROUPS])
	    if(datum>=0) mark_all_as_unread(wo+({gruppe}));
	return;
    }

    if(!member(plinfo[wo[0],PLB_GROUPS],wo[1]))
    {
        NEUE_GRUPPE_AN(wo[0],wo[1],0);
        return;
    }

    if(sizeof(wo)==2)
    {
        plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD] = mkmapping(
            ({int*})NEWSD->query_artikel_numbers(wo[0],wo[1]) || ({}));
        plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ] = ([]);
        return;
    }

    if(intp(wo[2]))
        nrs = ([wo[2]]);
    else
        nrs = mkmapping(wo[2]);

    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD] += nrs;
    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ] -= nrs;
}

/*
 * FUNKTION: tree2info
 * DEKLARATION: private void tree2info(mixed *tree, mapping artikel_daten, mapping info, int nr)
 * BESCHREIBUNG:
 * Der Artikelbaum und die aktuellen Zeiten werden in eine eigene Datenstruktur
 * umgewandelt, welche in info gespeichert wird.
 * Parameter:
 *   tree:          Das ist der Artikelbaum vom Newsd
 *   artikel_daten: Ein Mapping ([ artikelnummer: uhrzeit ])
 *                  (Die Uhrzeiten in tree sind moeglicherweise nicht aktuell.)
 *   info:          Ein Mapping:
 *                    ([ ART_LIST: ({ Liste der Artikelnummern in der korrekten
 *                                    Reihenfolge }),
 *                       ART_THREADS: ({ Liste aller Artikelnummern der
 *                                       Threadanfaenge }),
 *                       ART_INFO: ([ Artikelnummer:
 *                         AI_DATE:       Datum der letzten Aenderung;
 *                         AI_ROOTWAY:    Weg zur Wurzel (Array aus den Arrays
 *                                            der Baumstruktur);
 *                         AI_SUBLIST:    Teilliste aller Artikelnummern in
 *                                            ihrer Reihenfolge ohne diesen
 *                                            Artikel
 *                                 ])
 *                    ])
 *                  Dieses Mapping muss mit diesen 3 Werten initialisiert sein!
 *   nr:            Die Artikelnummer, zu dessen Thread dieser Artikelbaum
 *                  gehoert (0 fuer keinen Artikel.)
 * VERWEISE: 
 * GRUPPEN: news
 */
private void tree2info(mixed *starttree, mapping artikel_daten, mapping info, int startnr)
{
#ifdef EVAL_DEBUGGER
    int eval = get_eval_cost();
#endif

    mapping art_info = info[ART_INFO];
    mixed *stack = ({ ({starttree, startnr, ({})}) });
    while(sizeof(stack))
    {
	int nr = stack[0][1];
	mixed *addstack = ({ });

	foreach(mixed thread:stack[0][0])
	{
    	    int tnr=thread[N_NUMBER];
    	    art_info+=([tnr:max(artikel_daten[tnr],thread[N_DATE]);
                                  ({thread})+(art_info[nr,AI_ROOTWAY]||({}));
                                  ({})
                    	    ]);
	    addstack+=({ ({thread[N_SUBTREE], tnr, ({ nr }) }) });
	}
	
	if(sizeof(addstack))
	    addstack[<1][2]+=stack[0][2];
	else
	{
	    int mynr = nr;
	    foreach(int parentnr:stack[0][2])
	    {
		if(parentnr)
        	    art_info[parentnr,AI_SUBLIST]+=({mynr})
                                    	     + art_info[mynr,AI_SUBLIST];
    		else
		{
        	    info[ART_THREADS]+=({mynr});
		    info[ART_LIST]+=({mynr})+art_info[mynr,AI_SUBLIST];
		}
		mynr = parentnr;
	    }
	}
	stack = addstack+stack[1..<1];
    }

#ifdef EVAL_DEBUGGER
    if(({string})this_player()->query_real_name()==EVAL_DEBUGGER)
	printf("T2I Eval: %d\n", eval - get_eval_cost());
#endif
}


/*
 * FUNKTION: ask_subscribe
 * DEKLARATION: static void ask_subscribe(string str, string *unsubscribed, string brett, closure callback, varargs mixed *pars)
 * BESCHREIBUNG:
 * Mit Hilfe dieser Funktion fragt man den Spieler, ob ein bestimmtes Brett
 * oder eine bestimmte Gruppe angemeldet werden soll.
 * Die Liste der Bretter/Gruppen wird als zweiter Parameter (unsubscribed)
 * uebergeben. Ist brett=0, so sind dies Bretter, ansonsten Gruppen an dem
 * Brett brett. Ist die Fragerei fertig, so wird callback(pars) aufgerufen.
 * VERWEISE:
 * GRUPPEN: news
 */
static void ask_subscribe(string str, mixed *unsubscribed, string brett, closure callback, varargs mixed *pars)
{
    if (!strlen(str))
    {
	apply(#'input_to,"ask_subscribe", INPUT_PROMPT,
	    sprintf("%-20s Anmelden (j/n) ",unsubscribed[0][0]),
	    unsubscribed,brett,callback,pars);
	return;
    }
    str=lower_case(trim(str));
    if (str=="j" || str=="ja")
    {
	if(!brett)
	{
            NEUES_BRETT_AN(unsubscribed[0][0],0);
	    if(unsubscribed[0][0] == "InterMUD")
		cat("/static/adm/USENET_INFO");
	}
	else
            NEUE_GRUPPE_AN(brett,unsubscribed[0][0],0);
    }
    else
    {
	if(!brett)
            NEUES_BRETT_AB(unsubscribed[0][0]);
	else
            NEUE_GRUPPE_AB(brett,unsubscribed[0][0]);
    }

    if(sizeof(unsubscribed)>1)
	apply(#'ask_subscribe,0,unsubscribed[1..],brett,callback,pars);
    else
	apply(callback,pars);
}

/*
 * FUNKTION: ist_neue_gruppe
 * DEKLARATION: private int ist_neue_gruppe(mixed *gruppe, string brett)
 * BESCHREIBUNG:
 * Liefert 5, wenn die Gruppe neue Artikel hat (N),
 *         4, wenn sie ungelesene Artikel (A)
 * hat und 0, wenn sie nix Neues zu bieten hat.
 * (3 weniger, wenn sich die Ergebnisse nur auf durch die Suche
 * ausgeblendete Artikel beziehen)
 * gruppe hat die Form: ({brettname, datum, nummer})
 * VERWEISE:
 * GRUPPEN: news
 */
private int ist_neue_gruppe(mixed *gruppe, string brett)
{
    mapping searchnrs = sizeof(searchinfo) && searchinfo[<1][SRCH_ARTIKEL][brett][gruppe[0]];
    int t=plinfo[brett,PLB_GROUPS][gruppe[0],PLG_DATE];
    if(t<0) return 0;
    if(t<=gruppe[1])
    {
        mixed *artikel_daten;
	artikel_daten = ({mixed*}) NEWSD->query_times(brett,gruppe[0]);
	// Wir haben nun ein Format: ({artikel, datum, artikel, datum,...})
	for(int i=0;i<sizeof(artikel_daten);i+=2)
        {
	    int nr;
	    if(artikel_daten[i]!="artikel.o" && artikel_daten[i+1]>=t &&
                !member(plinfo[brett,PLB_GROUPS][gruppe[0],PLG_READ],
                    (nr = to_int(artikel_daten[i]))))
		return (!searchnrs || member(searchnrs,nr))?5:2;
	}
    }
    if(sizeof(plinfo[brett,PLB_GROUPS][gruppe[0],PLG_UNREAD]))
	return (!searchnrs ||
	     sizeof(filter(plinfo[brett,PLB_GROUPS][gruppe[0],PLG_UNREAD],
	        (:member($3,$1):),searchnrs)))
	    ?4:1;
    //Alte Artikel
    return 0;
}

/*
 * FUNKTION: ist_neues_brett
 * DEKLARATION: private int ist_neues_brett(mixed *brett)
 * BESCHREIBUNG:
 * Liefert 5, wenn das Brett neue Artikel hat (N),
 *         4, wenn es ungelesene Artikel (A)
 * hat und 0, wenn es nix Neues zu bieten hat.
 * (3 weniger, wenn sich die Ergebnisse nur auf durch die Suche ausgeblendete
 * Artikel beziehen)
 * brett hat die Form: ({brettname, datum, nummer})
 * VERWEISE:
 * GRUPPEN: news
 */
private int ist_neues_brett(mixed *brett)
{
    mixed *gruppen_daten;
    int tmp, alt;

    gruppen_daten = ({mixed*}) NEWSD->query_dates(brett[0]);
    // Wir haben nun ein Format: ({brettname, datum, brettname, datum,...})
    for(int i=0,int n=1;i<sizeof(gruppen_daten);n++)
    {
	//Immer 2 Elemente zu einem zusammenbasteln.
	gruppen_daten[i..i+1]=({ gruppen_daten[i..i+1]+({n}) });
	if(!member(plinfo[brett[0],PLB_GROUPS],gruppen_daten[i][0]))
	    return sizeof(searchinfo)?2:5;	// Neue Gruppe -> Brett ist neu
	tmp=ist_neue_gruppe(gruppen_daten[i],brett[0]);
	if(tmp>alt)
	{
	    alt = tmp;
	    if(alt==5) break;
	}
	i++;
    }
    // Neue Artikel, aber altes Brett (d.h. wir haben die neuen
    // Gruppen schon gesehen)-> "A"
    if((alt==2 || alt==5) && plinfo[brett[0],PLB_DATE]>brett[1]) return alt-1;
    return alt;
}

/*
 * FUNKTION: get_new_articles
 * DEKLARATION: private mapping get_new_articles(mixed *wo, int was)
 * BESCHREIBUNG:
 * Liefert alle ungelesenen/neuen Artikel in einem Mapping aus Zahlen zurueck.
 * was ist ein Bitflag:
 *   Bit 0: Es werden ungelesene Artikel gesucht
 *   Bit 1: Die neuen Artikel sind interessant
 * VERWEISE:
 * GRUPPEN: news
 */
private mapping get_new_articles(mixed *wo, int was)
{
    int t=plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE];
    mapping nrs = ([]);

    if(was&1) nrs=plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]; // Die alten Artikel
    if(was&2)
    {
        mixed *artikel_daten;
        artikel_daten=({mixed*})NEWSD->query_times(wo[0],wo[1]);
        // Wir haben nun ein Format: ({artikel, datum, artikel, datum,...})
        for(int i=0;i<sizeof(artikel_daten);i+=2)
        {
            if(artikel_daten[i]!="artikel.o" && artikel_daten[i+1]>=t &&
                !member(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ],to_int(artikel_daten[i])))
                    nrs+=([to_int(artikel_daten[i])]);
        }
    }
    return nrs;
}

/*
 * FUNKTION: check_group_for_new_articles
 * DEKLARATION: private int check_group_for_new_articles(mixed *gruppe, string brett, int was, int oberflaechlich)
 * BESCHREIBUNG:
 * Liefert 1, wenn diese Gruppe ungelesene Artikel beeinhaltet.
 * gruppe hat das Format ({gruppenname, datum, nummer}).
 * was ist ein Bitflag:
 *   Bit 0: Es werden ungelesene Artikel gesucht (check_existance notwendig)
 *   Bit 1: Die neuen Artikel sind interessant
 * Falls oberflaechlich!=0 ist, so werden die Ignore-Listen nicht beachtet.
 * VERWEISE:
 * GRUPPEN: news
 */
private int check_group_for_new_articles(mixed *gruppe, string brett, int was, int oberflaechlich)
{
    int t=plinfo[brett,PLB_GROUPS][gruppe[0],PLG_DATE];
    mixed artikel_baum, artikel_info, artikel_daten, tmp;
    mapping unread, searchnrs;
    if(t<0) return 0;  // Nicht angemeldet

    // Daten vom NEWSD holen und erstmal aufbereiten.
    artikel_daten = ({mixed*}) NEWSD->query_times(brett,gruppe[0]);
    if(sizeof(searchinfo))
	searchnrs = searchinfo[<1][SRCH_ARTIKEL][brett][gruppe[0]];
    unread = plinfo[brett,PLB_GROUPS][gruppe[0],PLG_UNREAD];
    if(searchnrs)
	unread = filter(unread, (:member($3,$1):), searchnrs);

    if(oberflaechlich)
    {
        if((was&1) && sizeof(unread))
            return 1;
    }
    else
    {
        artikel_baum = ({mixed*}) NEWSD->query_artikel_baum(brett,gruppe[0]);

        tmp = ([]);	
        for(int i=0;i<sizeof(artikel_daten);i+=2)
            if(artikel_daten[i]!="artikel.o")
                tmp[to_int(artikel_daten[i])]=artikel_daten[i+1];

        artikel_info=([ART_LIST: ({}), ART_THREADS: ({}),
                       ART_INFO: m_allocate(1,3) ]);
        tree2info(artikel_baum, tmp, &artikel_info, 0);
        artikel_info[ART_DATE] = time();

        if(was&1)
            foreach(int nr: unread)
            {
                if(member(flags["Ignore"]||([]),lower_case(
                    artikel_info[ART_INFO][nr,AI_ROOTWAY][0][N_AUTHOR])))
                {
                    plinfo[brett,PLB_GROUPS][gruppe[0],PLG_UNREAD]-=([nr]);
                    plinfo[brett,PLB_GROUPS][gruppe[0],PLG_READ]+=([nr]);
                }
                else
                    return 1;
            }
    }

    if((was&2) && t<=gruppe[1])
    {
        // Wir haben nun ein Format: ({artikel, datum, artikel, datum,...})
        for(int i=0;i<sizeof(artikel_daten);i+=2)
        {
	    int nr = to_int(artikel_daten[i]);
	    if(searchnrs && !member(searchnrs, nr))
		continue;
            if(artikel_daten[i]!="artikel.o" && artikel_daten[i+1]>=t &&
                !member(plinfo[brett,PLB_GROUPS][gruppe[0],PLG_READ],nr))
            {
                if(oberflaechlich)
                    return 1;
                if(member(flags["Ignore"]||([]),
                    lower_case(artikel_info[ART_INFO][to_int(artikel_daten[i]),AI_ROOTWAY][0][N_AUTHOR])))
                {
                    plinfo[brett,PLB_GROUPS][gruppe[0],PLG_UNREAD]-=([to_int(artikel_daten[i])]);
                    plinfo[brett,PLB_GROUPS][gruppe[0],PLG_READ]+=([to_int(artikel_daten[i])]);
                }
                else
                    return 1;
            }
        }
    }
    return 0;
}

/*
 * FUNKTION: check_board_for_new_articles
 * DEKLARATION: private int check_board_for_new_articles(mixed *brett, int was)
 * BESCHREIBUNG:
 * Liefert 1, wenn dieses Brett ungelesene Artikel beeinhaltet.
 * brett hat das Format ({brettname, datum, nummer}).
 * was ist ein Bitflag:
 *   Bit 0: Es werden ungelesene Artikel gesucht
 *   Bit 1: Die neuen Artikel sind interessant
 * VERWEISE:
 * GRUPPEN: news
 */
private int check_board_for_new_articles(mixed *brett, int was)
{
    mixed *gruppen_daten;

    if(!member(plinfo,brett[0]) || plinfo[brett[0],PLB_DATE]<0)  // Nicht angemeldet
        return 0;

    gruppen_daten = ({mixed*}) NEWSD->query_dates(brett[0]);
    // Wir haben nun ein Format: ({brettname, datum, brettname, datum,...})
    for(int i=0;i<sizeof(gruppen_daten);i++)
    {
	if(!IS_AL && sizeof(bretter) && bretter[brett[0]] && 
	    !member(bretter[brett[0]],gruppen_daten[i]))
    	{
	    //Dieses Brett rausnehmen
	    gruppen_daten[i..i+1] = ({});
            i--;
	}
	else
	{
	    //Immer 2 Elemente zu einem zusammenbasteln.
	    gruppen_daten[i..i+1]=({ gruppen_daten[i..i+1]+({i+1}) });
	    if((was&2) && !member(plinfo[brett[0],PLB_GROUPS],gruppen_daten[i][0]))
		return 1;	// Neue Gruppe -> Brett ist neu
	    check_existance(({brett[0], gruppen_daten[i][0]}));
    	    if(check_group_for_new_articles(gruppen_daten[i],brett[0],was,1))
        	return 1;
	}
    }
    return 0;
}

/*
 * FUNKTION: brett_zeile
 * DEKLARATION: private mixed brett_zeile(mixed *brett)
 * BESCHREIBUNG:
 * Liefert die Zeile in der Form ({string Zeile, int Nummer}) fuer ein Brett
 * in der Hauptuebersicht zurueck.
 * brett hat die Form: ({brettname, datum, nummer})
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed brett_zeile(mixed *brett)
{
    int neu, anz;
    if(sizeof(searchinfo))
    {
	if(!member(searchinfo[<1][SRCH_ARTIKEL], brett[0]))
	    return 0;
	foreach(string gruppe, mapping artnrs: searchinfo[<1][SRCH_ARTIKEL][brett[0]])
	    anz+=sizeof(artnrs);
    }

    // Sollte bei nicht angezeigten Brettern 0 liefern.
    if(!member(plinfo,brett[0]) || plinfo[brett[0],PLB_DATE]<0)
    {
        if(flags["SeeGroup"]==1)
            neu = 3;
        else
	    return 0;
    }
    else
        neu = ist_neues_brett(brett);

    // Wenn derjenige dieses Menue gar nicht sieht, Brett trotzdem anzeigen.
    if(flags["SeeGroup"]==-1 && neu==0 &&
        !((NO_BACK1 || NO_BACK2) && sizeof(flags["Last"])))
        return 0;
    return ({ sprintf("%s %5i  %s",
		({" ","a","n","u","A","N","U"})[neu],brett[2],brett[0]+(anz?(" ("+anz+")"):"")),
	      brett[2]
	   });
}

/*
 * FUNKTION: gruppe_zeile
 * DEKLARATION: private mixed gruppe_zeile(mixed *gruppe, string brett)
 * BESCHREIBUNG:
 * Liefert die Zeile in der Form ({string Zeile, int Nummer}) fuer eine Gruppe
 * in der Uebersicht des Brettes brett zurueck.
 * gruppe hat die Form: ({gruppenname, datum, nummer})
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed gruppe_zeile(mixed *gruppe, string brett)
{
    int neu, anz;

    if(sizeof(searchinfo))
    {
	if(!member(searchinfo[<1][SRCH_ARTIKEL], brett) ||
	   !member(searchinfo[<1][SRCH_ARTIKEL][brett], gruppe[0]))
	    return 0;
	anz = sizeof(searchinfo[<1][SRCH_ARTIKEL][brett][gruppe[0]]);
    }
    
    // Sollte bei nicht angezeigten Brettern 0 liefern.
    if(!member(plinfo[brett,PLB_GROUPS],gruppe[0]) ||
        plinfo[brett,PLB_GROUPS][gruppe[0],PLG_DATE]<0)
    {
        if(flags["SeeGroup"]==1)
            neu = 3;
        else
	    return 0;
    }
    else
        neu = ist_neue_gruppe(gruppe,brett);
    if(flags["SeeGroup"]==-1 && neu==0 && !NO_BACK2)
        return 0;
    return ({ sprintf("%s %5i  %s",
		({" ","a","n","u","A","N","U"})[neu],
		gruppe[2],gruppe[0]+(anz?(" ("+anz+")"):"")),
	      gruppe[2]
	   });
}

/*
 * FUNKTION: tree2str
 * DEKLARATION: private mixed *tree2str(mixed *tree, mapping artikel_daten, string brett, string gruppe, int force)
 * BESCHREIBUNG:
 * Diese Funktion liefert die Zeilen der Artikeluebersicht zurueck.
 * Jeder Zeile hat dabei das Format ({ Text, Nr }).
 * Parameter:
 *   tree:          Der Artikelbaum
 *   artikel_daten: Die Datenstruktur von tree2info
 *   brett, gruppe: Das Brett und die Gruppe
 *   force:         Bit 0: Immer alles aufklappen
 *                  Bit 1: Ungelesene Artikel anzeigen
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed *tree2str(mixed *starttree, mapping artikel_daten, string brett, string gruppe, int force)
{
#ifdef EVAL_DEBUGGER
    int eval = get_eval_cost();
#endif
    mapping ungelesen=  plinfo[brett,PLB_GROUPS][gruppe,PLG_UNREAD],
            gelesen=    plinfo[brett,PLB_GROUPS][gruppe,PLG_READ],
            aufgeklappt=plinfo[brett,PLB_GROUPS][gruppe,PLG_OPEN],
            zugeklappt= plinfo[brett,PLB_GROUPS][gruppe,PLG_COLLAPSED];
    mixed text = ({});
    mixed sresult;
    int pldate = plinfo[brett,PLB_GROUPS][gruppe,PLG_DATE]; // Zuletzt das Brett gelesen
    mapping ign = flags["Ignore"]||([]);
    int force_show = (force&2) || flags["SeeArticle"]==1;
    
    // Stackeintrag: ({ subtrees,
    //			Tiefe,
    //			uebergeordneter Text,
    //			soviel soll vom "---->" weggeschnitten werden,
    //			})
    mixed *stack = ({ ({starttree, 0, ({}), 0 }) });

    if(sizeof(searchinfo))
    {
	if(!member(searchinfo[<1][SRCH_ARTIKEL], brett) ||
	   !member(searchinfo[<1][SRCH_ARTIKEL][brett], gruppe))
    	       return ({});
	sresult = searchinfo[<1][SRCH_ARTIKEL][brett][gruppe];
    }

    while(sizeof(stack))
    {
	mixed *addstack = ({ });
	
	if(get_eval_cost()<20000)
	{
	    text+=({ ({ "-"*79, -1 }),
	             ({ "Es gibt zuviele anzuzeigende Artikel.", -1}),
		  });
	    return text;
	}
	
	if(stack[0][0])
        {	
	    foreach(mixed thread:stack[0][0])
    	    {
		int neu, plus=' ';
    		int nr=thread[N_NUMBER];
		int temp;

    		// Wir ermitteln, wie neu der Artikel ist.
		if(member(gelesen,nr))
        	    neu = ' ';
		else if(member(ungelesen,nr))
        	    neu = 'A';
		else if(artikel_daten[nr,AI_DATE]>=pldate)
        	    neu = 'N';
		else
        	    neu = ' ';
    
		// Subtree-Betrachtungen
		if((!sresult || member(sresult, nr)) && // Falls es ein Suchergebnis ist, dann normal behandeln
		   !(force&1) &&
                   (member(zugeklappt,nr) ||
                   (flags["Flags"]&1 && !member(aufgeklappt,nr))) &&
                   !member(flags["Ignore"]||([]),lower_case(thread[N_AUTHOR])))
		{
        	    // Okay, dieser Thread ist zugeklappt,
        	    // wir schauen aber trotzdem mal,
        	    // ob er neue Artikel hat.
        	    int *subliste=artikel_daten[nr,AI_SUBLIST];
        	    if(sizeof(subliste))
        	    {
        		plus='+';
        		subliste-=m_indices(gelesen);
	        	if(sizeof(filter(subliste,ungelesen)))
        		{
                	    if(neu!='N' && (neu!='A' || flags["SeeArticle"]==-1))
                		neu = 'a';
                	    subliste -= m_indices(ungelesen);
        		}
        		if(neu!='N' && (neu!='A' || flags["SeeArticle"]==-1) &&
                	    sizeof(filter(subliste, (: $2[$1,AI_DATE]>=$3 :),
                	    artikel_daten, pldate)))
                		neu = 'n';
        	    }
		    addstack += ({ ({0, stack[0][1]+1, 0, stack[0][3]}) });
		}
    		else
		    addstack += ({ ({thread[N_SUBTREE], stack[0][1]+1, ({}), stack[0][3] }) });

		if(sresult? // Suchergebnis
		    (temp=member(sresult, nr)):
		   
		    ((temp=(force_show ||
        	     (neu != ' '  && flags["SeeArticle"]>=0) ||
        	     (member(" aA",neu)<0))) || addstack[<1][0]))
		{
		    string author = thread[N_AUTHOR];
        	    if(member(ign,lower_case(author)))
			addstack[<1][2] = ({});
		    else
		    {
			if(temp)
			{
			    if(addstack[<1][0])
				addstack[<1..<2] = ({ ({ 0, stack[0][1]+1, 0, stack[0][3] }) });
			    else
				temp = 0;
			}
			addstack[<(temp?2:1)][2] = ({ ({ sprintf("%c %5d %c%:-37s %5s %:-10s %s", neu,
                	    nr, plus,
			    thread[N_TITLE][2*stack[0][3]..<1],
                	    "("+thread[N_LINES]+")", 
			    sizeof(author)>10?author[0..8]+">":author,
                	    shorttimestr(thread[N_DATE])[0..<4]),
                	    nr }) });
		    }
		}
		else if(addstack[<1][0])
		{
		    addstack[<1][3] += 1;
		}
		else
		    addstack = addstack[0..<2];
	    }

	    stack[1..0] = addstack;
	
	    if(sizeof(stack)>1)
		stack[1][2] = stack[0][2][0..stack[1][1]-2] + stack[1][2];
	}
	else
    	    text += stack[0][2];
	stack = stack[1..<1];
    }
#ifdef EVAL_DEBUGGER
    if(({string})this_player()->query_real_name()==EVAL_DEBUGGER)
	printf("T2S Eval: %d\n", eval - get_eval_cost());
#endif
    return text;
}

/************************************************************************
 *   Funktionen um mit den Menuedaten zu arbeiten                       *
 ************************************************************************/

/*
 * FUNKTION: brett_subscribed
 * DEKLARATION: private void brett_subscribed(mixed *bretter_daten)
 * BESCHREIBUNG:
 * Diese Funktion wird nach den Abfragen zu neuen Brettern aufgerufen
 * und soll die Brettuebersicht anzeigen.
 * bretter_daten ist ein Array aus ({ brettname, datum, nummer }).
 * VERWEISE:
 * GRUPPEN: news
 */
private void brett_subscribed(mixed *bretter_daten)
{
    mixed tmp=map(bretter_daten,#'brett_zeile) - ({0});
    if(!sizeof(tmp))
        browse( ({
	           ({ ({}), BRETT_LEER_STAT_LINE(
                       flags["SeeGroup"]<0
                       ?"Keine ungelesenen angemeldeten Bretter"
                       :"Du hast keine Bretter angemeldet!"),
		      0, BRETT_LEER_MORE_FLAG, 0, bretter_daten, ({}) })
	      }) );
    else
    {
	tmp = transpose_array(tmp); // Zeilen und Nummern trennen
	browse( ({
	          ({tmp[0], BRETT_STAT_LINE, 0, BRETT_MORE_FLAG, 0,
		    bretter_daten, tmp[1] })
	      }) );
    }
}

/*
 * FUNKTION: gruppe_subscribed
 * DEKLARATION: private void gruppe_subscribed(mixed *gruppen_daten, mixed menue, string brett, int index)
 * BESCHREIBUNG:
 * Diese Funktion wird nach den Abfragen zu neuen Gruppen aufgerufen
 * und soll die Gruppenuebersicht anzeigen.
 * gruppen_daten ist ein Array aus ({ brettname, datum, nummer }).
 * menue ist das bisherige Menue (mit der Hauptuebersicht). index stellt den
 * darin angewaehlten Menuepunkt dar. (BR_PATH in der neuen Menuestruktur.)
 * VERWEISE:
 * GRUPPEN: news
 */
private void gruppe_subscribed(mixed *gruppen_daten, mixed menue, string brett, int index)
{
    mixed tmp=map(gruppen_daten,#'gruppe_zeile,brett) - ({0});
    if(!sizeof(tmp))
        browse( menue + ({
	           ({ ({}), GRUPPE_LEER_STAT_LINE(brett,
                       flags["SeeGroup"]<0
                       ?"Keine ungelesenen angemeldeten Gruppen"
                       :"Du hast keine Gruppen angemeldet."),
		      0, GRUPPE_LEER_MORE_FLAG(brett), index, gruppen_daten, ({}) })
	      }) );
    else
    {
	tmp = transpose_array(tmp); // Zeilen und Nummern trennen
	browse( menue + ({
		  ({tmp[0], GRUPPE_STAT_LINE(brett), 0, GRUPPE_MORE_FLAG(brett),
                    index, gruppen_daten, tmp[1] })
	      }) );
    }
}

/*
 * FUNKTION: get_one_menue
 * DEKLARATION: private mixed get_one_menue(mixed wo, mixed *menue, int dont_ask)
 * BESCHREIBUNG:
 * Damit konstruieren wir ein Menue fuer eine bestimmte Stufe.
 * Es sollte vorher aber check_existance aufgerufen werden.
 * Parameter:
 *   wo: ({brett,gruppe,artikel}), je nachdem, wo das Menue konstruiert wird.
 *   menue: Die bisherige Menuestruktur
 *   dont_ask: Nicht nach neuen Brettern fragen.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed get_one_menue(mixed wo, mixed *menue, mixed dont_ask)
{
    mixed tmp;
    if(!sizeof(wo))  // Brettuebersicht
    {
	mixed *bretter_daten = ({mixed*}) NEWSD->query_dates(0);
	mixed *unsubscribed=({});

	if(!sizeof(bretter_daten))
            return ({ ({}), BRETT_LEER_STAT_LINE("Es gibt noch keine Bretter!"),
		      0, BRETT_LEER_MORE_FLAG, 0, bretter_daten, ({}) });

	plinfo = update_bretter(plinfo);

	// Wir haben nun ein Format: ({brettname, datum, brettname, datum,...})
	for(int i=0,int n=1;i<sizeof(bretter_daten);n++)
	{
	    if(!IS_AL && sizeof(bretter) && !member(bretter,bretter_daten[i]))
            {
		//Dieses Brett rausnehmen
		bretter_daten[i..i+1] = ({});
                n--;
            }
	    else
	    {
		//Immer 2 Elemente zu einem zusammenbasteln.
		bretter_daten[i..i+1]=({ bretter_daten[i..i+1]+({n}) });

		if(!member(plinfo,bretter_daten[i][0])) // Neues Brett
		    unsubscribed+=({bretter_daten[i]});
		i++;
	    }
	}
	// Wir haben nun ein Format: ({ ({brettname, datum, nr}), ({brettname, datum, nr}),...})
	if(sizeof(unsubscribed))
	{
            if(dont_ask)
                dont_ask = ({"Es gibt neue Bretter:\n",
                    0,unsubscribed,0,#'brett_subscribed,bretter_daten});
            else
            {
	        write("Es gibt neue Bretter:\n");
	        ask_subscribe(0,unsubscribed,0,#'brett_subscribed,bretter_daten);
	        return END_BROWSE;
            }
	}

	if(!sizeof(bretter_daten))
            return ({ ({}), BRETT_LEER_STAT_LINE("Es stehen keine Bretter zur Auswahl!"),
		      0, BRETT_LEER_MORE_FLAG, 0, bretter_daten, ({}) });

	tmp=map(bretter_daten,#'brett_zeile) - ({0});
	if(!sizeof(tmp))
            return ({ ({}), BRETT_LEER_STAT_LINE(
                       flags["SeeGroup"]<0
                       ?"Keine ungelesenen angemeldeten Bretter"
                       :"Du hast keine Bretter angemeldet!"),
	          0, BRETT_LEER_MORE_FLAG, 0, bretter_daten, ({}) });

	tmp = transpose_array(tmp); // Zeilen und Nummern trennen
	return ({tmp[0], BRETT_STAT_LINE, 0, BRETT_MORE_FLAG, 0,
		 bretter_daten, tmp[1] });
    }
    else if(sizeof(wo)==1) // Gruppenuebersicht
    {
	mixed *gruppen_daten= ({mixed*}) NEWSD->query_dates(wo[0]);
	mixed *unsubscribed=({});
	int index;

	// Den Index herausbekommen, fuer BR_PATH
	for(int j=0;j<sizeof(menue[0][BR_USER]);j++)
	    if(menue[0][BR_USER][j][0]==wo[0])
	    {
		index=member(menue[0][BR_NUMBERS],j+1)+1;
		break;
	    }

        if(!index)
            return NOTHING;

	if(!member(plinfo,wo[0]))
            NEUES_BRETT_AN(wo[0],0);

	if(!sizeof(gruppen_daten))
	{
	    plinfo[wo[0],PLB_DATE]=time();	// Zeit aktualisieren.
            return ({ ({}), GRUPPE_LEER_STAT_LINE(wo[0],"Es gibt noch keine Gruppen."),
		      0, GRUPPE_LEER_MORE_FLAG(wo[0]), index, gruppen_daten, ({}) });
	}

	if(plinfo[wo[0],PLB_DATE]<0)
            BRETT_AN(wo[0],0);

	// Wir haben nun ein Format: ({gruppenname, datum, gruppenname, datum,...})
	for(int i=0,int n=1;i<sizeof(gruppen_daten);n++)
	{
	    if(!IS_AL && sizeof(bretter) && bretter[wo[0]] && 
		!member(bretter[wo[0]],gruppen_daten[i]))
            {
		//Dieses Brett rausnehmen
		gruppen_daten[i..i+1] = ({});
                n--;
            }
	    else
	    {
		//Immer 2 Elemente zu einem zusammenbasteln + Nummer hinzufuegen.
		gruppen_daten[i..i+1]=({ gruppen_daten[i..i+1]+({n}) });

		if(!member(plinfo[wo[0],PLB_GROUPS],gruppen_daten[i][0]))
	    	    unsubscribed+=({gruppen_daten[i]}); // Neue Gruppe
		i++;
	    }
	}

	// Wir haben nun ein Format: ({ ({gruppennname, datum, nr}), ({gruppenname, datum, nr}),...})
	if(sizeof(unsubscribed))
	{
            if(dont_ask)
                dont_ask = ({ "Es gibt neue Gruppen:\n", 0,unsubscribed,wo[0],
		#'gruppe_subscribed,gruppen_daten,menue,wo[0],index});
            else
            {
	        write("Es gibt neue Gruppen:\n");
	        ask_subscribe(0,unsubscribed,wo[0],
		    #'gruppe_subscribed,gruppen_daten,menue,wo[0],index);
	        return END_BROWSE;
            }
	}

	tmp=map(gruppen_daten,#'gruppe_zeile,wo[0]) - ({0});
	if(!sizeof(tmp))
            return ({ ({}), GRUPPE_LEER_STAT_LINE(wo[0],
		       sizeof(searchinfo)
		       ?"Es gibt keine Suchergebnisse an diesem Brett."
		       :(flags["SeeGroup"]<0)
                       ?"Keine ungelesenen angemeldeten Gruppen"
                       :"Du hast keine Gruppen angemeldet."),
		      0, GRUPPE_LEER_MORE_FLAG(wo[0]), index, gruppen_daten, ({}) });

	tmp = transpose_array(tmp); // Zeilen und Nummern trennen
	return ({tmp[0], GRUPPE_STAT_LINE(wo[0]), 0, GRUPPE_MORE_FLAG(wo[0]),
	         index, gruppen_daten, tmp[1] });
    }
    else if(sizeof(wo)==2) // Artikeluebersicht
    {
	// Format: Array aus ({int nummer, int datum, string autor, string titel, int zeilen, mixed subtree})
	mixed *artikel_baum=({mixed*})NEWSD->query_artikel_baum(wo[0],wo[1]);
	mixed *orig_artikel_daten=({mixed*})NEWSD->query_times(wo[0],wo[1]);
	mapping artikel_info;
	int index;

	// Den Index herausbekommen, fuer BR_PATH
	for(int j=0;j<sizeof(menue[1][BR_USER]);j++)
	    if(menue[1][BR_USER][j][0]==wo[1])
	    {
		index=member(menue[1][BR_NUMBERS],j+1)+1;
		break;
	    }

        if(!index)
            return NOTHING;

	if(!member(plinfo,wo[0]))
            NEUES_BRETT_AN(wo[0],0);
	else if(plinfo[wo[0],PLB_DATE]<0)
            BRETT_AN(wo[0],0);             // Wieder abbonieren

	if(!member(plinfo[wo[0],PLB_GROUPS],wo[1]))
	    NEUE_GRUPPE_AN(wo[0],wo[1],0);
	else if(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]<0)
            GRUPPE_AN(wo[0],wo[1],0);       // Wieder abbonieren

	if(!sizeof(artikel_baum))
	{
	    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]=time();	// Zeit aktualisieren.
            return ({ ({"Es gibt noch keine Artikel in dieser Gruppe."}),
	              ARTIKEL_STAT_LINE(wo[0],wo[1]), 0,
		      ARTIKEL_MORE_FLAG(wo[0],wo[1]), index, 
                      ([ART_INFO:([]), ART_LIST: ({}),
                        ART_THREADS: ({}), ART_DATE: time()]),
                      ({}) });
	}
        tmp = ([]);
	for(int i=0;i<sizeof(orig_artikel_daten);i+=2)
	    if(orig_artikel_daten[i]!="artikel.o")
		tmp[to_int(orig_artikel_daten[i])]=orig_artikel_daten[i+1];
        artikel_info=([ART_LIST: ({}), ART_THREADS: ({}),
                       ART_INFO: m_allocate(1,3) ]);
        tree2info(artikel_baum, tmp, &artikel_info, 0);
        artikel_info[ART_DATE] = time();
	tmp = tree2str(artikel_baum, artikel_info[ART_INFO], wo[0], wo[1], 0);
	if(sizeof(tmp))
	{
	    tmp = transpose_array(tmp); // Zeilen und Nummern trennen
	    return ({tmp[0], ARTIKEL_STAT_LINE(wo[0],wo[1]), 0,
		     ARTIKEL_MORE_FLAG(wo[0],gruppr), index,
                     artikel_info, tmp[1] });
	}
	
	return ({ ({ sizeof(searchinfo)
		     ?"Es gibt keine Suchergebnisse in dieser Gruppe."
		     :(capitalize(zahl2str(sizeof(artikel_info[ART_INFO])))+" alte" +
                     (sizeof(artikel_info[ART_INFO])>1?"":"r")+" Artikel. "
                     "Es gibt keine "+
                     (flags["SeeArticle"]<0?"NEUEN":"UNGELESENEN")+
                     " Artikel in dieser Gruppe.")
                  }), ARTIKEL_STAT_LINE(wo[0],wo[1]), 0,
                  ARTIKEL_MORE_FLAG(wo[0],wo[1]), index,
                  artikel_info, ({-1}) });
    }
    else if(sizeof(wo)==3) // Einen Artikel anzeigen
    {
	plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([wo[2]]); //Von ungelesenen abziehen
	plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([wo[2]]); //Und zu den gelesenen hinzuzaehlen
#if 0
        tmp = ({string}) NEWSD->query_titel(wo[0],wo[1],wo[2]);
        if (!IS_HIDDEN(owner) && (!(flags["Flags"]&8) || !wizp(owner)))
            owner->send_message(MT_LOOK,MA_READ,wrap(Der(owner) + " liest " +
                (((wizp(owner) || spielerratp(owner)) && member(GEHEIMBRETTER,wo[0]))
		?"einen Artikel."
		:("den Artikel '" + tmp + "' in der Gruppe '" + wo[1] + "'."))));
#endif
	return ({ NEWSD->query_file_name(wo[0],wo[1],wo[2]),
		  TEXT_STAT_LINE(wo[0],wo[1],wo[2]), 0,
		  TEXT_MORE_FLAG(wo[0],wo[1],wo[2]),
		  wo[2], 0, 0 });
    }
}

/*
 * FUNKTION: get_the_menue
 * DEKLARATION: private mixed get_the_menue(mixed *wo)
 * BESCHREIBUNG:
 * Das Menue fuer wo=({brett,gruppe,artikel}) wird berechnet und
 * zurueckgeliefert.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed get_the_menue(mixed *wo)
{
    mixed *menue=({}), subscr=({});

    if(sizeof(wo))
    {
	if(!member(plinfo,wo[0]))
            NEUES_BRETT_AN(wo[0],0);
	else if(plinfo[wo[0],PLB_DATE]<0)
            BRETT_AN(wo[0],0);
        
        if(sizeof(wo)>1)
        {
	    if(!member(plinfo[wo[0],PLB_GROUPS],wo[1]))
	    	NEUE_GRUPPE_AN(wo[0],wo[1],0);
	    else if(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_DATE]<0)
            	GRUPPE_AN(wo[0],wo[1],0);       // Wieder abbonieren
        }
    }
        
    for(int i=-1;i<sizeof(wo)-1;i++)
    {
	string err;
        subscr+=({1});
	switch(i)
	{
	    case 0: if(!member(plinfo,wo[0]))
			err = stringp(fail_msg)?fail_msg
			    : closurep(fail_msg)?funcall(fail_msg,this_player(),wo[0],0)
			    : ("Du darfst das Brett "+wo[0]+" nicht lesen.\n");
		    break;
	    case 1:  if(!member(plinfo[wo[0],PLB_GROUPS],wo[1]))
			err = stringp(fail_msg)?fail_msg
			    : closurep(fail_msg)?funcall(fail_msg,this_player(),wo[0],wo[1])
			    : ("Du darfst "+(NO_BACK2?"das Brett ":"die Gruppe ")+wo[1]+" nicht lesen.\n");
		    break;
	}
	if(err)
	    menue += ({err});
	else
	{
    	    check_existance(wo[0..i],0);
	    menue+=({ get_one_menue(wo[0..i], menue, &(subscr[<1])) });
	}
	if(!pointerp(menue[<1])) // Ein Fehler
	{
	    if(sizeof(menue)==1
            ||(sizeof(menue)==2 && NO_BACK1)
            ||(sizeof(menue)==3 && NO_BACK2))
		return menue[<1];
	    else 
            {
                if(pointerp(subscr[<2]))
                {
                    write(subscr[<2][0]);
                    apply(#'ask_subscribe,subscr[<2][1..<1]);
                    return END_BROWSE;
                }
                else
		    return menue[0..<2];
            }
	}
    }
    if(sizeof(wo)==1 && !member(plinfo,wo[0]))
    	menue += ({ stringp(fail_msg)?fail_msg
		:   closurep(fail_msg)?funcall(fail_msg,this_player(),wo[0],0)
		:   ("Du darfst das Brett "+wo[0]+" nicht lesen.\n") });
    else if(sizeof(wo)>1 && !member(plinfo[wo[0],PLB_GROUPS],wo[1]))
        menue += ({ stringp(fail_msg)?fail_msg
		:   closurep(fail_msg)?funcall(fail_msg,this_player(),wo[0],wo[1])
		:   ("Du darfst "+(NO_BACK2?"das Brett ":"die Gruppe ")+wo[1]+" nicht lesen.\n") });
    else
    {
	check_existance(wo,0);
	menue += ({ get_one_menue(wo, menue, 0) });
    }
    if(menue[<1]==END_BROWSE)		// Anmeldungsdialog
	return menue[<1];
    else if(pointerp(menue[<1])) // Alles klar, ein Menue
	return menue;
    else if(sizeof(menue)==1     // Fehler
         ||(sizeof(menue)==2 && NO_BACK1)
         ||(sizeof(menue)==3 && NO_BACK2))
	return menue[<1];
    else if(pointerp(subscr[<1]))
    {
        write(subscr[<1][0]);
        apply(#'ask_subscribe,subscr[<1][1..<1]);
        return END_BROWSE;
    }
    else
	return menue[0..<2];	// Ausgeben, was wir bekommen konnten.
}

/*
 * FUNKTION: get_path
 * DEKLARATION: static mixed* get_path(int *indizes, mixed m)
 * BESCHREIBUNG:
 * Aus den Indizes (Menuepunkte) und dem Menue wird der Brettname,
 * Gruppenname und Artikelnummer ausgelesen.
 * VERWEISE:
 * GRUPPEN: news
 */
static mixed* get_path(int *indizes, mixed m)
{
    mixed *tmp=({});
    // Vom voherigen Menue die Brettinfos (mit den Namen) holen
    // und dort unter dem gewaehlten Menuepunkt den Namen nachschlagen.
    if(sizeof(indizes)>0)
    {
	// m[0]: Das aeusserste Menue
        //     BR_USER: ein Array mit ({Name, Datum, Nummer})
	//     BR_NUMBERS: Enthaelt zu jedem Menuepunkt den Index in BR_USER+1
	//     indizes: der Angewaehlte Menuepunkt (Index in BR_NUMBERS + 1)
	tmp+=({ m[0][BR_USER][ m[0][BR_NUMBERS][ indizes[0] -1] -1][0] });
	if(sizeof(indizes)>1)
	{
	    tmp+=({ m[1][BR_USER][ m[1][BR_NUMBERS][ indizes[1] -1] -1][0] });
	    if(sizeof(indizes)>2)
		// BR_NUMBERS enthaelt zu jedem Menuepunkt die Artikelnummer
		// indizes: Der angewaehlte Menuepunkt (Index in BR_NUMBERS+1)
		tmp+= ({ indizes[2] });
	}
    }
    return tmp;
}


/*
 * FUNKTION: renew_menue_more_info
 * DEKLARATION: static mixed renew_menue_more_info(mixed m, int flag)
 * BESCHREIBUNG:
 * Das Menue (zumindest die letzte Ebene davon) wird neu berechnet.
 * flag:
 *   Bit 0: Leises Update (keine Ausgabe verursachen)
 *   Bit 1: Einen Bildschirm zurueckgehen
 *   Bit 2: An Anfang zurueckgehen (ignoriert Bit 1)
 * VERWEISE:
 * GRUPPEN: news
 */
static mixed renew_menue_more_info(mixed m, int flag)
{
    mixed tmp=GET_PATH(m);
    check_existance(tmp,0);
    tmp = get_one_menue(tmp, m[0..<2], 0);
    if(pointerp(tmp))
    {
        if((flag&4) ||
           ((flag&2) && (m[<1][BR_STATUS_BYTE]&M_SCROLL)))
        {
            tmp[BR_BEGIN_LINE] = 1;
        }
        else
        {
	    tmp[BR_BEGIN_LINE] = m[<1][BR_BEGIN_LINE]
			       - ((flag&2)?({int})owner->query_more_chunk()-2:0);
            if(sizeof(m)<4) // Nicht im Artikellesen
            {
                // Hier mal etwas Aufwand, um nach der Erneuerung des Menues
                // an der Stelle weiterzulesen, wo wir aufgehoert haben.
                // Dazu schauen wir nach, an welcher Stelle die Nummer
                // (->BR_NUMBERS), bei der wir aufgehoert haben (nr), nun im
                // neuen Menue (tmp) drin ist.
                int nr;

                nr=tmp[BR_BEGIN_LINE];
                if(nr>sizeof(m[<1][BR_NUMBERS]) ||
                   !sizeof(m[<1][BR_NUMBERS])) // Am Ende oder darueberhinaus?
                    tmp[BR_BEGIN_LINE] = sizeof(tmp[BR_NUMBERS])+1;
                else
                {
                    nr--;
                    if(nr<0) nr=0;
                    nr=member(tmp[BR_NUMBERS],m[<1][BR_NUMBERS][nr])+1;
                    if(nr)
                        tmp[BR_BEGIN_LINE]=nr;
                }
            }
        }
	if(flag&1) tmp[BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
	return m[0..<2]+ ({tmp});
    }
    else
	return tmp;
}

private void wait_callback(string str, int start, int timeout, string waitmsg, mixed m)
{
    if(start+timeout<time())
    {
	write("Ein Fehler ist aufgetreten. Breche ab.\n");
    	m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
	browse(m);
    }
    else
    {
	write(waitmsg);
	input_to(#'wait_callback,0, start, timeout, waitmsg, m);
    }
}

void task_callback(mixed ret, mixed oldm, mixed m)
{
    if(ret[1])
    {
	remove_input_to(owner, this_object());
	if(pointerp(m))
	{
	    browse(m);
	    return;
	}
	if(stringp(m))
	    write(m);
	else if(m==END_BROWSE)
	    return;
    	oldm[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
	browse(oldm);
    }
    else
    {
	ret[0] = m;
	ret[1] = 1;
    }
}

mixed wait_for_task(mixed m, string wait_msg, int timeout, closure task, varargs mixed param)
{
    mixed ret = ({0,0});
    closure callback = lambda(({'m}),({#'task_callback, quote(ret), quote(m), 'm}));

    apply(task, callback, param);

    if(ret[1]) // Bereits fertig
	return ret[0];
    
    ret[1]=1;
    write(wait_msg);
    input_to(#'wait_callback, 0, time(), timeout, wait_msg, m);
    return END_BROWSE;
}

/************************************************************************
 *   Menue - Implementation                                             *
 ************************************************************************
 *   Die Doku zu diesen Funktionen gibt es in der Enzy.                 *
 ************************************************************************/
 
// Hier ein paar Sicherheitsueberlagerungen,
// damit andere Goetter nicht geheime Artikel mitlesen koennen
mixed *query_browse() {return 0;}

public void more_end(string str, int more_line, int max_line, mixed more_id)
{
    if(this_player()!=owner) return;
    ::more_end(str,more_line,max_line,more_id);
}

static mixed get_menue_more_info(mixed m, int nr)
{
    mixed tmp=get_path(map(m[1..<1],#'[,BR_PATH)+({nr}),m);

    if(!member(plinfo,tmp[0]) || plinfo[tmp[0],PLB_DATE]<0)
        return "Das Brett '" + tmp[0] + "' hast Du nicht angemeldet.\n"
               "Du kannst es mit 'an "+
               (sizeof(m)>1?m[0][BR_NUMBERS][m[1][BR_PATH]-1]:nr)+
               "' anmelden.\n";

    if(sizeof(tmp)>1 &&
      (!member(plinfo[tmp[0],PLB_GROUPS],tmp[1]) ||
        plinfo[tmp[0],PLB_GROUPS][tmp[1],PLG_DATE]<0))
        return "Die Gruppe '" + tmp[1] + "' hast Du nicht angemeldet.\n"
               "Du kannst sie mit 'an "+
               (sizeof(m)>2?m[1][BR_NUMBERS][m[2][BR_PATH]-1]:nr)+
               "' anmelden.\n";

    check_existance(tmp,0);
    tmp = get_one_menue(tmp, m, 0);
    if(pointerp(tmp)) return m+ ({tmp});
    else return tmp;
}

static mixed get_initial_more_info(mixed m)
{
    reading = 1;
    return m;
}

static mixed get_back_more_info(mixed m)
{
    int n, beginline;
    mixed wo,tmp;

    if(m[0][BR_STATUS_BYTE]&BF_NO_MENUE) //Hilfe oder Optionen
    {
        mixed m2;
        if(m[0][BR_PATH]==DISP_HELP)
        {
            m[0][BR_USER][DISP_MENUE][<1][BR_STATUS_BYTE]|=M_NO_FIRST_SCREEN;
            return m[0][BR_USER][DISP_MENUE];
        }
        m2 = m[0][BR_USER][DISP_MENUE];
        for(int i=sizeof(m2)-1;!pointerp(tmp) && i>=0;i--)
        {
            if((i==0 && NO_BACK1) || (i==1 && NO_BACK2))
                break;
            tmp = renew_menue_more_info(m2[0..i],
                (m[0][BR_PATH]==DISP_MODS)?1:4);
        }
        return tmp;
    }

    if(sizeof(searchinfo)==sizeof(m))
    {
	searchinfo=searchinfo[0..<2];
	while(sizeof(searchinfo) && !searchinfo[<1])
	    searchinfo=searchinfo[0..<2];
	return renew_menue_more_info(m, 4);
    }

    if(sizeof(m)<2 ||
       (sizeof(m)==2 && NO_BACK1) ||
       (sizeof(m)==3 && NO_BACK2))
           return 0;

    // Datum aktualisieren
    wo = GET_PATH(m);
    update_plinfo(wo,m,0);
    check_existance(wo,0);

    // Wir gehen soweit zurueck, bis kein Fehler mehr auftaucht.
    for(int i=sizeof(m)-3;!pointerp(tmp) && i>=-1;i--)
        tmp = get_one_menue(wo[0..i],m[0..i], 0); // Menue neu berechnen

    if(!pointerp(tmp))
        return tmp;

    // Die groesste sinnvolle Zeile, um mit der Ausgabe zu beginnen.
    n = sizeof(tmp[BR_MENUE])-({int})owner->query_more_chunk()+2;
    if(n<0) n = 0;
    beginline=m[<1][BR_PATH];
    if(sizeof(m)==4)		// Bei Artikeln steht die richtige Nummer drin
	beginline=member(m[<2][BR_NUMBERS],beginline)+1;
    if(beginline>n)		// Der Menuepunkt ist spaeter
	tmp[BR_BEGIN_LINE]=n+1; // -> wir nehmen die groesste sinnvolle Zeile
    else
	tmp[BR_BEGIN_LINE]=beginline; // Ansonsten beim Menuepunkt beginnen
    return m[0..<3] + ({tmp});
}

static mixed get_next_more_info(mixed m)
{
    int nr;

    if(m[0][BR_STATUS_BYTE]&BF_NO_MENUE) //Hilfe oder Optionen
        return 0;
    
    if(sizeof(m)<2 ||
        (NO_BACK1 && sizeof(m)<3) ||
        (NO_BACK2 && sizeof(m)<4))
        return 0;
    if(sizeof(m)!=4)
        return ::get_next_more_info(m);

    // naechsten Artikel
    if(sizeof(searchinfo))
    {
	nr = member(m[<2][BR_NUMBERS],m[<1][BR_PATH]) + 1;
	if(!nr)
    	    return "Es gibt keinen weiteren Artikel.\n";
	if(nr>=sizeof(m[<2][BR_NUMBERS]))
    	    return "Es gibt keinen weiteren Artikel.\n";
	return get_menue_more_info(m[0..<2],m[<2][BR_NUMBERS][nr]);
    }
    else
    {
	nr = member(m[<2][BR_USER][ART_LIST],m[<1][BR_PATH]) + 1;
	if(!nr)
    	    return "Es gibt keinen weiteren Artikel.\n";

	// Naechsten Artikel suchen...
	while(nr<sizeof(m[<2][BR_USER][ART_LIST]) &&
    	    member(flags["Ignore"]||([]),lower_case(
    	    m[<2][BR_USER][ART_INFO][m[<2][BR_USER][ART_LIST][nr],AI_ROOTWAY][0][N_AUTHOR])))
        	nr++;
	if(nr>=sizeof(m[<2][BR_USER][ART_LIST]))
    	    return "Es gibt keinen weiteren Artikel.\n";
	return get_menue_more_info(m[0..<2],m[<2][BR_USER][ART_LIST][nr]);
    }

}

static mixed get_prev_more_info(mixed m)
{
    int nr;

    if(m[0][BR_STATUS_BYTE]&BF_NO_MENUE) //Hilfe oder Optionen
        return 0;
    if(sizeof(m)<2 ||
        (NO_BACK1 && sizeof(m)<3) ||
        (NO_BACK2 && sizeof(m)<4))
        return 0;
    if(sizeof(m)!=4)
        return ::get_prev_more_info(m);
    // vorherigen Artikel
    if(sizeof(searchinfo))
    {
	nr = member(m[<2][BR_NUMBERS],m[<1][BR_PATH]) - 1;
	if(nr<0)
    	    return "Es gibt keinen vorherigen Artikel.\n";
	return get_menue_more_info(m[0..<2],m[<2][BR_NUMBERS][nr]);
    }
    else
    {
	nr = member(m[<2][BR_USER][ART_LIST],m[<1][BR_PATH]) - 1;

	while(nr>=0 &&
    	    member(flags["Ignore"]||([]),lower_case(
    	    m[<2][BR_USER][ART_INFO][m[<2][BR_USER][ART_LIST][nr],AI_ROOTWAY][0][N_AUTHOR])))
        	nr--;
	if(nr<0)
    	    return "Es gibt keinen vorherigen Artikel.\n";
	return get_menue_more_info(m[0..<2],m[<2][BR_USER][ART_LIST][nr]);
    }
}

static void browse_end(mixed m)
{
    mixed wo;

    reading = 0;

    // Datum aktualisieren
    if(m[0][BR_STATUS_BYTE]&BF_NO_MENUE) //Hilfe oder Optionen
        m=m[0][BR_USER][DISP_MENUE];
    wo = GET_PATH(m);
    flags["Last"] = wo[0..1];
    update_plinfo(wo,m,1);
    set_read_artikel(plinfo);
    this_player()->set_newsreader_settings(flags);
    if(zustand&1)
        remove();
}

/************************************************************************
 *   Funktionen zum Schreiben von Artikeln                              *
 ************************************************************************/

/*
 * FUNKTION: cant_write
 * DEKLARATION: private string cant_write()
 * BESCHREIBUNG:
 * Testet, ob owner schreiben darf. Testies und Goetter duerfen das ungefragt.
 * Ansonsten gelten folgende Bedingungen:
 *   Geschicklichkeit >=35
 *   Staerke >=35
 *   Ausdauer >=35
 *   Intelligenz >=35
 *   Alter >1 Tag
 *   Erfahrung > 5000
 * Wird die Schreiberlaubnis gewaehrt, liefert diese Funktion 0 zurueck,
 * ansonsten einen String mit der Begruendung.
 * VERWEISE:
 * GRUPPEN: news
 */
private string cant_write()
{
    if(testplayerp(owner) || wizardshellp(owner) || wizp(owner) ||
	SECOND->is_writer(owner->query_real_name()) ||
	SECOND->is_special(owner->query_real_name()))
        return 0;
    if(owner->query_real_stat(STAT_DEX)<35)
        return wrap("Diese dumme Schreibfeder rutscht Dir dauernd aus Deinen"
                    " ungeschickten Händen.");
    if(owner->query_real_stat(STAT_STR)<35)
        return wrap("Du brauchtest etwas mehr Kraft, um den Artikel dann ans"
                    " Brett zu nageln.");
    if(owner->query_real_stat(STAT_CON)<35)
        return wrap("Du hast noch nicht die Ausdauer, um einen Artikel zu Ende"
                    " zu schreiben.");
    if(owner->query_real_stat(STAT_INT)<35)
        return wrap("Du hast Probleme, mehrere Buchstaben zu einem Text"
                    " zusammenzufügen.");
    if(owner->query_age() < 86400)
        return wrap("Du bist noch zu jung, um die vielen Buchstaben"
                    " auseinanderhalten zu können.");
    if(owner->compute_one_skill(({"skill"}))<5000)
        return wrap("Dir fehlt etwas an Erfahrung, um einen Artikel zu"
                    " schreiben.");
    if(guestp(owner))
        return wrap("Als Gast besitzt Du keine Schreibfeder, um einen Artikel"
                    " zu schreiben.");
    if(PLAYER_ANNOYER->query("Schreibverbot", owner->query_real_name()))
        return wrap("Du hast Schreibverbot.");
    return 0;
}

/*
 * FUNKTION: post_it
 * DEKLARATION: static void post_it(string str, mixed m, mixed wo, string titel, string *txt)
 * BESCHREIBUNG:
 * Diese Funktion wird entweder direkt (mit str=0) oder ueber input_to
 * aufgerufen und haengt den Artikel txt mit dem Titel titel an das
 * Brett wo = ({brett, gruppe[, reply-artikelnummer]}). Anschliessend wird
 * das Menue m leise (d.h. nur der Prompt) ausgegeben.
 * VERWEISE:
 * GRUPPEN: news
 */
static void post_it(string str, mixed m, mixed wo, string titel, string *txt)
{
    if(member((["","ja","j","y","yes"]), str=lower_case(trim(str||""))))
    {
        string ret;
        if(ret=({string})NEWSD->write_artikel(wo[0], wo[1], titel,
            sizeof(wo)>2?wo[2]:0, implode(txt,"\n")+"\n"))
                write(ret);
        else
        {
            browse(renew_menue_more_info(m[0..2],2));
            return;
        }
    }
    else if(!member((["nein", "noe", "nö"]), str))
    {
        input_to("post_it",INPUT_PROMPT,
	    "Willst Du den Artikel abschicken (Ja/Nein)? Ja\b\b",
	    m,wo,titel,txt);
	return;
    }

    m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
    browse(m);
}

/*
 * FUNKTION: ed_ends
 * DEKLARATION: static void ed_ends(string *text, mixed m, mixed wo, string titel)
 * BESCHREIBUNG:
 * Diese Funktion wird nach Beendigung des Editors aufgerufen.
 * Parameter:
 *   text:  Der Text. (Bei 0 wurde abgebrochen.)
 *   m:     Das anschliessend auszugebende Menue.
 *   wo:    Dort soll dieser Artikel hingehaengt werden.
 *          = ({ brett, gruppe[, beantworteter Artikel]})
 *   titel: Der Titel des Textes
 * VERWEISE:
 * GRUPPEN: news
 */
static void ed_ends(string *text, mixed m, mixed wo, string titel)
{
    string *ntext = ({});
    int maxlen = query_limits()[LIMIT_ARRAY];
    
    if(!text)
    {
        if(!IS_HIDDEN(owner) && (!(flags["Flags"]&8) || !wizp(owner)))
            owner->send_message(MT_LOOK, MA_CRAFT, wrap(Der(owner) +
                " zerreißt den Artikel wieder."));
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    // Text umbrechen
    foreach(string str:text)
    {
	string *toadd = explode(wrap(str,79)[0..<2],"\n");
	if(sizeof(ntext)+sizeof(toadd)>maxlen)
	{
    	    if(!IS_HIDDEN(owner) && (!(flags["Flags"]&8) || !wizp(owner)))
        	owner->send_message(MT_LOOK, MA_CRAFT, wrap(Dem(owner) +
            	    " geht das Papier aus."));
	    write("Der Artikel ist zu lang!");
    	    m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
    	    browse(m);
    	    return;
	}
	ntext += toadd;
    }
    
    if(!(flags["Flags"]&16))
    {
        input_to("post_it", INPUT_PROMPT,
    	    "Willst Du den " + (!sizeof(ntext)?"kurzen ":"") +
            "Artikel wirklich abschicken (Ja/Nein)? Ja\b\b",
	    m,wo,titel,ntext);
    }
    else
        post_it(0,m,wo,titel,ntext);
}

/*
 * FUNKTION: input_titel
 * DEKLARATION: static void input_titel(string str, mixed m, mixed wo, string standard)
 * BESCHREIBUNG:
 * Diese Funktion wird durch die Eingabe des Titels von input_to aufgerufen.
 * Parameter:
 *   str:      Der eingegebene Titel
 *   m:        Das anschliessend auszugebende Menue.
 *   wo:       Dort soll dieser Artikel hingehaengt werden.
 *             = ({ brett, gruppe[, beantworteter Artikel]})
 *   standard: Der vorgeschlagene Titel
 * VERWEISE:
 * GRUPPEN: news
 */
static void input_titel(string str, mixed m, mixed wo, string standard)
{
    string tmp;

    str=trim(str||"");
    if(str=="")
    {
        if(standard)
            str=standard;
        else
        {
            m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
            browse(m);
            return;
        }
    }
    else if(str=="~q")
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    else if(tmp = ({string}) NEWSD->invalid_titel(str))
    {
        write(tmp);
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    if(!IS_HIDDEN(owner) && (!(flags["Flags"]&8) || !wizp(owner)))
        owner->send_message(MT_LOOK, MA_CRAFT, wrap(Der(owner) +
            " beginnt gerade einen "+ ((sizeof(wo)==2)?
            "neuen Artikel zu schreiben.":"Artikel zu beantworten.")));
    if(wizp(owner) && NEWSD->is_ingame_brett(wo[0]))
       write("\nBEACHTE: Diese Gruppe können SPIELER lesen!\n"
             "BEDENKE: Als Gott sollst Du Dich aus allen "
             "Spielerangelegenheiten so gut\n"
             "         heraushalten wie nur irgend möglich!\n\n");
    else if(wo[0]=="InterMUD")
	write("\nBEDENKE: Artikel in dieser Gruppe werden weltweit veröffentlicht\n"
	        "         und können nicht mehr gelöscht werden.\n\n");
    write("BEACHTE: Jede Zeile darf nur 80 Zeichen haben!\n");
    if(!wizp(owner) && flags["Editor"]>1)
        flags["Editor"]=1;
    if(!owner->mini_ed(
        lambda(({'text}),({#'ed_ends,'text,quote(m),quote(wo),str})),
        flags["Editor"], owner->uses_gmcp_edit() ? 0 : NEWS_TMP_FILE,
        ([MINI_ED_START_TEXT: "Gib nun den Artikel ein. Mit '**' oder '.' "
                              "beenden, mit '~q' abbrechen.\n",
          MINI_ED_TITLE: wo[0]+", "+wo[1],
        ])))
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
    }
}

/*
 * FUNKTION: reply_to
 * DEKLARATION: private int reply_to(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Diese Funktion fordert zur Eingabe des Titels beim Beantworten eines
 * Artikels wo = ({brett,gruppe,nummer}) auf. Sie liefert 1, wenn es diesen
 * Artikel nicht mehr gibt.
 * VERWEISE:
 * GRUPPEN: news
 */
private int reply_to(mixed m, mixed wo)
{
    string titel = ({string}) NEWSD->query_re_titel(wo[0],wo[1],wo[2]);
    if (!titel || titel=="")
        return 1;
    write(wrap("Für 'Titel: "+titel+"' nur Return eingeben. Abbruch mit '~q'"));
    input_to("input_titel", INPUT_PROMPT, "Titel: ", m,wo,titel);
    return 0;
}

/*
 * FUNKTION: ask_reply_number
 * DEKLARATION: static void ask_reply_number(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Diese Funktion wird von input_to nach der Eingabe der Nummer des zu
 * beantwortenden Artikels. Dieser Artikel solle ich in wo = ({brett, gruppe})
 * befinden. m ist das Menue, was nach Ende der Eingabe ausgegeben werden
 * sollte.
 * VERWEISE:
 * GRUPPEN: news
 */
static void ask_reply_number(string str, mixed m, mixed wo)
{
    int nr;
    if(!str || str=="" || str=="~q")
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    if(!sscanf(str,"%d",nr))
    {
        write("Du musst die Nummer angeben.\n");
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    if(reply_to(m,wo+({nr})))
    {
        write("Einen Artikel mit dieser Nummer gibt es nicht.\n");
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
    }
}

/************************************************************************
 *   Funktionen zum Parsing der Eingabe                                 *
 ************************************************************************
 *                                                                      *
 *  Die parse-Funktionen erhalten alle folgende Parameter:              *
 *    string str: Der zu parsende String                                *
 *    mixed m   : Das aktuelle Menue                                    *
 *    mixed wo  : = ({brett, gruppe, artikel}) bzw. nur ein Teil davon. *
 *                                                                      *
 *  Die Funktionen parsen den String und liefern entweder eine Fehler-  *
 *  meldung als String zurueck oder das Ergebnis als einziges Element   *
 *  im zurueckgelieferten Array.                                        *
 *                                                                      *
 ************************************************************************/

/*
 * FUNKTION: parse_article
 * DEKLARATION: private mixed parse_article(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Parst den String als Zahl. Eine Ueberpruefung, ob dieser Artikel existiert,
 * findet nicht statt.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_article(string str, mixed m, mixed wo)
{
    int nr;
    if(!sscanf(str,"%d",nr))
        return "Du musst die Nummer angeben.\n";
    return ({nr});

}

/*
 * FUNKTION: parse_articles
 * DEKLARATION: private mixed parse_articles(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Parst den String als eine Anzahl von Artikeln. Dies ist eine durch
 * Komma, Semikolon oder Leerzeichen getrennte Aufzaehlung von Nummern oder
 * Intervallen in der Form <von>-<bis>. Eine Untersuchung, ob die Artikel
 * auch existieren findet nur Teilweise (bei den Intervallen) statt.
 * Zurueckgeliefert wird eine Liste von Artikelnummern.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_articles(string str, mixed m, mixed wo)
{
    int *nrs=({});
    string *teile;
    teile=regexplode(str," |,|;|/|:|\\.")-({""," ",",",";","/",":","."});
    foreach(string s:teile)
    {
        int von,bis,a,b;
        if(!(a=sscanf(s,"%d-%d",von,bis)))
            return "Du musst eine Nummer angeben.\n";
        if(a==1)
            nrs+=({von});
        else
        {
            a=member(m[2][BR_USER][ART_LIST],von);
            if(a<0)
                return "Artikel "+von+" gibt es nicht.\n";
            b=member(m[2][BR_USER][ART_LIST],bis);
            if(b<0)
                return "Artikel "+bis+" gibt es nicht.\n";
            nrs+=m[2][BR_USER][ART_LIST][a..b];
        }
    }
    if(!sizeof(nrs))
        return "Du musst eine Nummer angeben.\n";
    return ({nrs});
}

/*
 * FUNKTION: parse_group
 * DEKLARATION: private mixed parse_group(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Der String wird als Brett- bzw. Gruppennummer oder Brett- bzw. Gruppenname
 * interpretiert. Zurueckgeliefert wird die Nummer des Brettes bzw. der
 * Gruppe (beginnend bei 1, so wie sie in der Anzeige steht).
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_group(string str, mixed m, mixed wo)
{
    int nr;
    if(!sscanf(str,"%d",nr))
    {
        nr = member(map(m[<1][BR_USER],(:lower_case($1[0]):)),lower_case(strip(str)))+1;
        if(!nr)
            return ((sizeof(m)==1)?"Ein Brett":"Eine Gruppe") +
	        " mit dieser Nummer gibt es nicht.\n";
    }
    else if(nr<1 || nr>sizeof(m[<1][BR_USER]))
        return ((sizeof(m)==1)?"Ein Brett":"Eine Gruppe") +
            " mit dieser Nummer gibt es nicht.\n";
    return ({nr});
}

/*
 * FUNKTION: parse_subscribed_group
 * DEKLARATION: private mixed parse_subscribed_group(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Verhaelt sich wie parse_group mit dem Unterschied, dass auch getestet wird,
 * ob das Brett angemeldet ist.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_subscribed_group(string str, mixed m, mixed wo)
{
    mixed back=parse_group(str,m,wo);
    if(!pointerp(back))
        return back;

    if(!sizeof(wo))
    {
        string brett=m[0][BR_USER][back[0]-1][0];
        if(!member(plinfo,brett) || plinfo[brett,PLB_DATE]<0)
            return "Das Brett '"+brett+"' hast Du nicht angemeldet.\n";
        return back;
    }
    else
    {
        string gruppe=m[1][BR_USER][back[0]-1][0];
        if(!member(plinfo[wo[0],PLB_GROUPS],gruppe) ||
            plinfo[wo[0],PLB_GROUPS][gruppe,PLG_DATE]<0)
                return "Die Gruppe '"+gruppe+"' hast Du nicht angemeldet.\n";
        return back;
    }
}

/*
 * FUNKTION: parse_yesno
 * DEKLARATION: private mixed parse_yesno(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Liefert nix, wenn "Ja" oder nix (-> Standard ist ja) eingegeben wurde,
 * ansonsten einen leeren String als Fehler.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_yesno(string str, mixed m, mixed wo)
{
    if(member((["","ja","j","y","yes"]), lower_case(trim(str||""))))
        return ({});
    else
        return "";
}

/*
 * FUNKTION: parse_noyes
 * DEKLARATION: private mixed parse_noyes(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Liefert nix, wenn "Ja" (Standard ist nein) eingegeben wurde, ansonsten
 * einen leeren String als Fehler.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed parse_noyes(string str, mixed m, mixed wo)
{
    if(member((["ja","j","y","yes"]), lower_case(trim(str||""))))
        return ({});
    else
        return "";
}
    
/*
 * FUNKTION: ask_for_menu
 * DEKLARATION: static void ask_for_menu(string str, mixed m, mixed wo, closure parser, closure task, int accept_empty)
 * BESCHREIBUNG:
 * Diese Funktion wird von input_to aufgerufen.
 * Bei einem leeren String (wenn accept_empty==0 ist) oder "~q" wird
 * abgebrochen, ansonsten wird der angegebene Parser gefragt und das Ergebnis
 * an wo angehaengt. Anschliessend wird task mit m und wo als Parameter
 * aufgerufen, welches dann ein Menue zurueckliefern sollte.
 * Im Fehlerfalle wird zu alten Menue m zurueckgegangen.
 * VERWEISE:
 * GRUPPEN: news
 */
static void ask_for_menu(string str, mixed m, mixed wo, closure parser, closure task, int accept_empty)
{
    mixed tmp;
    if((!accept_empty && (!str || str=="")) || str=="~q")
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    tmp=funcall(parser,str,m,wo);
    if(stringp(tmp))
    {
        write(tmp);
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    tmp=funcall(task,m,wo+tmp);
    if(pointerp(tmp))
    {
         browse(tmp);
        return;
    }
    if(stringp(tmp)) write(tmp);
    else if(tmp==END_BROWSE)
        return;
    m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
    browse(m);
}

/************************************************************************
 *   Diverse Befehle des Newssreaders                                   *
 ************************************************************************/

/*
 * FUNKTION: search_new_article
 * DEKLARATION: private mixed search_new_article(mixed *m, int was, int wieweit)
 * BESCHREIBUNG:
 * Sucht nach einem neuen Artikel. (Befehl 'n','N' oder 'A' im Newsreader.)
 * Parameter:
 *   m: Das aktuelle Menue
 *   was: Wonach soll gesucht werden?
 *      1: nur alte Artikel
 *      2: nur neue Artikel
 *      3: beides
 *   wieweit: Wie weit darf man zurueckgehen?
 *      0: nur in der aktuellen Uebersicht
 *      1: Gruppenuebergreifend (Es sei denn, wir sind in der Brettuebersicht)
 *      2: Brettuebergreifend
 * Zurueckgeliefert wird ein neues Menue.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed search_new_article(mixed *m, int was, int wieweit)
{
    mixed *wo=GET_PATH(m);
    int i;
    mixed nrs, nnrs;
    string *brtlist;

    if(sizeof(m)==4) // Beim Artikellesen
    {
	int *artlist = m[<2][BR_USER][ART_LIST];
	if(sizeof(searchinfo))
	    artlist &= m_indices(searchinfo[<1][SRCH_ARTIKEL][wo[0]][wo[1]]);

        nnrs = get_new_articles(wo,was);
        i = member(artlist, wo[2]) + 1; // Ab da suchen wir.
        nrs = filter(artlist[i..<1], nnrs);
        while(sizeof(nrs))
            if(member(flags["Ignore"]||([]),
                lower_case(m[<2][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
            {
                // Als gelesen markieren.
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                nrs=nrs[1..<1];
            }
            else
                return get_menue_more_info(m[0..<2], nrs[0]);
        if(!wieweit)
        {
            nrs = filter(artlist[0..i-2], nnrs);
            while(sizeof(nrs))
                if(member(flags["Ignore"]||([]),
                    lower_case(m[<2][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
                {
                    // Als gelesen markieren.
                    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                    nrs=nrs[1..<1];
                }
                else
                    return get_menue_more_info(m[0..<2], nrs[0]);
            return "Es gibt keine "+(was>1?"neuen":"ungelesenen")+
                   " Artikel in dieser Gruppe.\n";
        }
    }
    else if(sizeof(m)==3) // In der Artikeluebersicht
    {
	int *artlist = m[<1][BR_USER][ART_LIST];
	if(sizeof(searchinfo))
	    artlist &= m_indices(searchinfo[<1][SRCH_ARTIKEL][wo[0]][wo[1]]);

        nrs = get_new_articles(wo,was);
        nrs = filter(artlist, nrs);
        while(sizeof(nrs))
            if(member(flags["Ignore"]||([]),
                lower_case(m[<1][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
            {
                // Als gelesen markieren.
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                nrs=nrs[1..<1];
            }
            else
                return get_menue_more_info(m[0..<1], nrs[0]);
        if(!wieweit)
            return "Es gibt keine "+(was>1?"neuen":"ungelesenen")+
                   " Artikel in dieser Gruppe.\n";
    }

    if(sizeof(m)>2)
        update_plinfo(wo[0..1],m,0);

    if(sizeof(m)>=2)
    {
	mixed *grplist = m[1][BR_USER];
	if(sizeof(searchinfo))
	    grplist = filter(grplist, (: member($2, $1[0]) :),
			searchinfo[<1][SRCH_ARTIKEL][wo[0]]);

        // So, wir suchen nun die passende Gruppe.
        if(sizeof(m)>2)
            i = m[2][BR_PATH];    // Ab da suchen wir (implizites +1, da BR_PATH ab 1 zaehlt)
        else
            i = 0;
        foreach(mixed gruppe: grplist[i..<1]
	    + ((wieweit<2 && i)?grplist[0..i-1]:({})))
	{
	    check_existance(({wo[0],gruppe[0]}));
            if(check_group_for_new_articles(gruppe, wo[0], was, 0))
                return get_menue_more_info(m[0..1],
                    member(m[1][BR_NUMBERS],member(m[1][BR_USER],gruppe)+1)+1);
	}
        if(wieweit<2)
            return "Es gibt keine "+(was>1?"neuen":"ungelesenen")+
                   " Artikel an diesem Brett.\n";
    }


    // Wir suchen nun ein passendes Brett
    if(sizeof(m)>1)
    {
        i = m[1][BR_PATH];
        update_plinfo(wo[0..0],m,0);
    }
    else
        i = 0;

    brtlist = m[0][BR_USER];
    if(sizeof(searchinfo))
    {
	brtlist = filter(brtlist, (: member($2, $1[0]) :),
		    searchinfo[<1][SRCH_ARTIKEL]);
    }


    foreach(mixed brett: brtlist[i..<1])
        if(check_board_for_new_articles(brett, was))
            return get_menue_more_info(m[0..0],
                member(m[0][BR_NUMBERS],member(m[0][BR_USER],brett)+1)+1);
    if(i)
        foreach(mixed brett: brtlist[0..i-1])
            if(check_board_for_new_articles(brett, was))
                return get_menue_more_info(m[0..0],
                    member(m[0][BR_NUMBERS],member(m[0][BR_USER],brett)+1)+1);
    return "Es gibt keine "+(was>1?"neuen":"ungelesenen")+" Artikel.\n";
}

private void search_new_article_board2(closure callback, mixed *m, int was, mixed *brtlist)
{
    while(get_eval_cost()>80000 && sizeof(brtlist))
    {
	mixed brett = brtlist[0];
        if(check_board_for_new_articles(brett, was))
	{
	    funcall(callback, get_menue_more_info(m[0..0],
                member(m[0][BR_NUMBERS],member(m[0][BR_USER],brett)+1)+1));
	    return;
	}
	brtlist = brtlist[1..<1];
    }
    if(sizeof(brtlist))
	call_out(#'search_new_article_board2, 2, callback, m, was, brtlist);
    else
	funcall(callback,
	    "Es gibt keine "+(was>1?"neuen":"ungelesenen")+" Artikel.\n");
}

private void search_new_article_board(closure callback, mixed *m, mixed *wo, int was)
{
    mixed *brtlist;
    int i;

    // Wir suchen nun ein passendes Brett
    if(sizeof(m)>1)
    {
        i = m[1][BR_PATH];
        update_plinfo(wo[0..0],m,0);
    }
    else
        i = 0;

    brtlist = m[0][BR_USER];
    if(sizeof(searchinfo))
    {
	brtlist = filter(brtlist, (: member($2, $1[0]) :),
		    searchinfo[<1][SRCH_ARTIKEL]);
    }

    search_new_article_board2(callback, m, was, brtlist[i..<1]+brtlist[0..i-1]);
}

private void search_new_article_group2(closure callback, mixed *m, mixed *wo, int was, int wieweit, mixed *grplist)
{
    while(get_eval_cost()>80000 && sizeof(grplist))
    {
	mixed gruppe = grplist[0];
	check_existance(({wo[0],gruppe[0]}));
        if(check_group_for_new_articles(gruppe, wo[0], was, 0))
	{
	    funcall(callback, get_menue_more_info(m[0..1],
                member(m[1][BR_NUMBERS],member(m[1][BR_USER],gruppe)+1)+1));
	    return;
	}
	grplist = grplist[1..<1];
    }

    if(sizeof(grplist))
	call_out(#'search_new_article_group2, 2, callback, m, wo, was, wieweit, grplist);
    else if(wieweit<2)
    {
	funcall(callback,
	    "Es gibt keine "+(was>1?"neuen":"ungelesenen")+
            " Artikel an diesem Brett.\n");
	return;
    }
    else
	search_new_article_board(callback, m, wo, was);
}

private void search_new_article_group(closure callback, mixed *m, mixed *wo, int was, int wieweit)
{
    if(sizeof(m)>2)
        update_plinfo(wo[0..1],m,0);

    if(sizeof(m)>=2)
    {
	int i;
	mixed *grplist = m[1][BR_USER];

	if(sizeof(searchinfo))
	    grplist = filter(grplist, (: member($2, $1[0]) :),
			searchinfo[<1][SRCH_ARTIKEL][wo[0]]);

        // So, wir suchen nun die passende Gruppe.
        if(sizeof(m)>2)
            i = m[2][BR_PATH];    // Ab da suchen wir (implizites +1, da BR_PATH ab 1 zaehlt)
        else
            i = 0;
	    
	search_new_article_group2(callback, m, wo, was, wieweit, 
	    grplist[i..<1] + ((wieweit<2 && i)?grplist[0..i-1]:({})));
    }
    else
	search_new_article_board(callback, m, wo, was);
}

private void search_new_article2(closure callback, mixed *m, int was, int wieweit)
{
    mixed *wo=GET_PATH(m);
    int i;
    mixed nrs, nnrs;

    if(sizeof(m)==4) // Beim Artikellesen
    {
	int *artlist = m[<2][BR_USER][ART_LIST];
	if(sizeof(searchinfo))
	    artlist &= m_indices(searchinfo[<1][SRCH_ARTIKEL][wo[0]][wo[1]]);

        nnrs = get_new_articles(wo,was);
        i = member(artlist, wo[2]) + 1; // Ab da suchen wir.
        nrs = filter(artlist[i..<1], nnrs);
        while(sizeof(nrs))
            if(member(flags["Ignore"]||([]),
                lower_case(m[<2][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
            {
                // Als gelesen markieren.
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                nrs=nrs[1..<1];
            }
            else
            {
		funcall(callback, get_menue_more_info(m[0..<2], nrs[0]));
		return;
	    }
        if(!wieweit)
        {
            nrs = filter(artlist[0..i-2], nnrs);
            while(sizeof(nrs))
                if(member(flags["Ignore"]||([]),
                    lower_case(m[<2][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
                {
                    // Als gelesen markieren.
                    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                    plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                    nrs=nrs[1..<1];
                }
                else
		{
                    funcall(callback, get_menue_more_info(m[0..<2], nrs[0]));
		    return;
		}
            funcall(callback,
		"Es gibt keine "+(was>1?"neuen":"ungelesenen")+
                " Artikel in dieser Gruppe.\n");
	    return;
        }
    }
    else if(sizeof(m)==3) // In der Artikeluebersicht
    {
	int *artlist = m[<1][BR_USER][ART_LIST];
	if(sizeof(searchinfo))
	    artlist &= m_indices(searchinfo[<1][SRCH_ARTIKEL][wo[0]][wo[1]]);

        nrs = get_new_articles(wo,was);
        nrs = filter(artlist, nrs);
        while(sizeof(nrs))
            if(member(flags["Ignore"]||([]),
                lower_case(m[<1][BR_USER][ART_INFO][nrs[0],AI_ROOTWAY][0][N_AUTHOR])))
            {
                // Als gelesen markieren.
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_UNREAD]-=([nrs[0]]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_READ]+=([nrs[0]]);
                nrs=nrs[1..<1];
            }
            else
	    {
                funcall(callback, get_menue_more_info(m[0..<1], nrs[0]));
		return;
	    }
        if(!wieweit)
	{
	    funcall(callback,
		"Es gibt keine "+(was>1?"neuen":"ungelesenen")+
                " Artikel in dieser Gruppe.\n");
	    return;
	}
    }

    search_new_article_group(callback, m, wo, was, wieweit);
}


/*
 * FUNKTION: delete_article
 * DEKLARATION: private mixed delete_article(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Loescht die Artikel wo = ({brett,gruppe,artikelliste}) und liefert
 * das neue Menue zurueck.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed delete_article(mixed m, mixed wo)
{
    return NEWSD->loesche_artikel(wo[0],wo[1],wo[2]) ||
        RENEW2(wrap("Artikel "+liste(map(wo[2],#'to_string))+" gelöscht."),m[0..2],3);
}

/*
 * FUNKTION: delete_thread
 * DEKLARATION: private mixed delete_thread(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Loescht den Thread wo = ({brett,gruppe,artikelliste}) und liefert das
 * neue Menue zurueck.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed delete_thread(mixed m, mixed wo)
{
    int *nrs;
    if(!member(m[2][BR_USER][ART_INFO],wo[2]))
        return "Den Artikel "+wo[2]+" gibt es nicht.\n";
    nrs=({wo[2]})+m[2][BR_USER][ART_INFO][wo[2],AI_SUBLIST];
    return NEWSD->loesche_artikel(wo[0],wo[1],nrs) ||
        RENEW2(wrap("Artikel "+liste(map(nrs,#'to_string))+" gelöscht."),m[0..2],3);
}

/*
 * FUNKTION: summarize_articles
 * DEKLARATION: private mixed summarize_articles(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Es wird eine Zusammenfassung der Artikel wo = ({brett, gruppe,
 * artikelliste}) ausgegeben.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed summarize_articles(mixed m, mixed wo)
{
    string *text=({}), *tmp;
    string artikel,datei;
    int max_arr=query_limits()[LIMIT_ARRAY];
    if(!sizeof(wo[2]))
        return "Keine Artikel angegeben.\n";
    foreach(int nr:wo[2])
    {
        datei = ({string}) NEWSD->query_file_name(wo[0],wo[1],nr);
        if(!(artikel=read_file(datei)))
            return "Konnte Artikel Nr. "+nr+" nicht lesen.\n";
        tmp = explode(artikel,"\n");

        if(sizeof(text))
        {
            if(max_arr && sizeof(text)+sizeof(tmp)>max_arr)
                return "Es sind zuviele Artikel!\n";
            text+=({"",tmp[0]});
        }
        else if(max_arr && sizeof(tmp)>max_arr)
            return "Artikel Nr. "+nr+" ist zu lang.\n";
        text+=({"",tmp[1],tmp[3],""})+tmp[5..<2];
    }
    return ({ ({text,"Zusammenfassung, Zeile %d von %d "
        "[z,<,>,u,b,/<such>"+(wizp(this_player())?",w":"")+"] ",
        0, M_FRAME|BF_NO_MENUE|(flags["Scroll"]&8?M_SCROLL:0), DISP_SUMMARY,
        ([DISP_MENUE: m, DISP_PATH: wo ]) }) });
}

/*
 * FUNKTION: summarize_thread
 * DEKLARATION: private mixed summarize_thread(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Es wird eine Zusammenfassung des Threads wo = ({brett, gruppe,
 * artikelnummer}) ausgegeben.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed summarize_thread(mixed m, mixed wo)
{
    int *nrs;
    if(!member(m[2][BR_USER][ART_INFO],wo[2]))
        return "Den Artikel "+wo[2]+" gibt es nicht.\n";
    nrs=({wo[2]})+m[2][BR_USER][ART_INFO][wo[2],AI_SUBLIST];
    return summarize_articles(m, wo[0..1]+({nrs}));
}

/*
 * FUNKTION: input_sum_titel
 * DEKLARATION: static void input_sum_titel(string str, mixed m, mixed wo, string standard)
 * BESCHREIBUNG:
 * Diese Funktion wird von input_to nach Eingabe eines Titels zur Speicherung
 * der Zusammenfassung (welche im Menue m steht) aufgerufen.
 * Anschliessend wird wieder ins Menue zurueckgekehrt.
 * VERWEISE:
 * GRUPPEN: news
 */
static void input_sum_titel(string str, mixed m, mixed wo, string standard)
{
    string tmp;

    str=trim(str||"");
    if(str=="")
        str=standard;
    else if(str=="~q")
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    else if(tmp = ({string}) NEWSD->invalid_titel(str))
    {
        write(tmp);
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }

    if(tmp = ({string}) NEWSD->write_artikel(wo[0],wo[1],str,0,
        implode(m[0][BR_MENUE],"\n")+"\n"))
            write(tmp);
    else
    {
        m[0][BR_USER][DISP_MENUE]=renew_menue_more_info(
            m[0][BR_USER][DISP_MENUE],2);
        write("Zusammenfassung wurde ans Brett geschrieben.\n");
    }
    m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
    browse(m);
}

/*
 * FUNKTION: ignore
 * DEKLARATION: private mixed ignore(mixed m, mixed wo)
 * BESCHREIBUNG:
 * Der Name wo[<1] wird (nicht mehr) ignoriert.
 * Zurueckgeliefert wird ein aktualisiertes Menue.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed ignore(mixed m, mixed wo)
{
    if(member(flags["Ignore"]||([]),lower_case(wo[<1])))
    {
        flags["Ignore"]-=([lower_case(wo[<1])]);
        return RENEW(wrap("Du ignorierst "+capitalize(wo[<1])+" nicht mehr."),m);
    }
    else
    {
        if(!flags["Ignore"])
            flags["Ignore"]=([]);
        flags["Ignore"]+=([lower_case(wo[<1])]);
        return RENEW(wrap("*plonk* Du ignorierst nun "+capitalize(wo[<1])+"."),m);
    }
}

/*
 * FUNKTION: ask_for_new_group
 * DEKLARATION: static void ask_for_new_group(string str, mixed m, mixed wo)
 * BESCHREIBUNG:
 * Diese Funktion wird von input_to nach Eingabe eines Namens fuer eine
 * anzulegende Gruppe am Brett wo = ({brett}) aufgerufen.
 * Anschliessend wird das aktualisierte Menue m aufgerufen.
 * VERWEISE:
 * GRUPPEN: news
 */
static void ask_for_new_group(string str, mixed m, mixed wo)
{
    mixed tmp;
    if(!strlen(str) || str=="~q")
    {
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
        return;
    }
    str=strip(str);
    if (tmp = ({string}) NEWSD->eroeffne_gruppe(wo[0],str))
    {
        write(tmp);
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
    }
    else
    {
        write("Ok.\n");
        tmp=renew_menue_more_info(m,1);
        if(pointerp(tmp))
        {
            browse(tmp);
            return;
        }
        if(stringp(tmp)) write(tmp);
        else if(tmp==END_BROWSE)
            return;
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
        browse(m);
    }
}

/*
 * FUNKTION: get_mod_news_list
 * DEKLARATION: private mixed get_mod_news_list(string mod, mixed wo)
 * BESCHREIBUNG:
 * Listet alle Gruppen und deren Artikelanzahl eines Moderators auf und
 * liefert dies als Array aus den einzelnen Zeilen zurueck. Im Fehlerfalle
 * wird ein String mit einer Fehlermeldung zurueckgeliefert.
 * Wenn mod == 0 ist, werden alle Gruppen ausgegeben. Bei "" wird this_player()
 * genommen, ansonsten enthaelt mod den real_namen des Moderators.
 * Wenn wo = ({}) werden alle Gruppen betrachet, ansonsten nur die Gruppen
 * unter dem Brett wo = ({brett}).
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed get_mod_news_list(string mod, mixed wo)
{
    string *txt = ({});
    mixed tmp;
    mixed *anzeigen;

    int gruppenzahl, artikelzahl;
    wo = wo[0..1];

    if(mod) // Nach Moderator suchen
    {
	string name;
	mixed ob;
	mapping owners = ({mapping}) NEWSD->query_owners();
	
	if(player_exists(lower_case(mod)))
	    mod = lower_case(mod);
	    
        name = mod;
        if(!strlen(name))
	{
            name = ({string}) this_player()->query_real_name();
	    ob = this_player();
	}
	else
	    ob = name;

	if(NEWSD->is_owner(ob))
            txt+=strlen(mod)?({capitalize(name)+" ist Newsadmin."})
                            :({"Du bist Newsadmin."});

        anzeigen=map(sort_array(m_indices(owners)-({"/"}),#'>),
                (: $2[$1] && (!strstr($1,$4) || !strstr($4,$1+"/")) &&
		   NEWSD->is_owner($3,
		       strlen(($6 = explode($1,"/")+({0,0}))[1])?$6[1]:0,
		       strlen($6[2])?$6[2]:0, 1) &&
		   $6[1..<3]
                :),owners, ob, "/"+implode(wo,"/")+(sizeof(wo)?"/":""))-({0});

        if(!sizeof(anzeigen))
            return wrap((strlen(mod)?capitalize(name)+" ist":"Du bist") +
                (!sizeof(wo)  ?" für keine Gruppe":
                 sizeof(wo)==1?" für keine Gruppe am Brett "+wo[0]:
                               " nicht für die Gruppe "+implode(wo,"/")
                )+" zuständig.");
    }
    else // Alle anzeigen
    {
        if(sizeof(wo))
            anzeigen=({wo});
        else
        {
            anzeigen = transpose_array(({NEWSD->query_bretter()}));
            if(!sizeof(anzeigen))
                return "Es gibt keine Bretter.\n";
        }
    }
    // So wir haben nun alle auszugebenden Gruppen in anzeigen
    // im Format ({brett,gruppe}) oder ({brett}).

    foreach(mixed was:anzeigen)
    {
        string prefix;
        string *gruppen;
        if(sizeof(was)==1)
        {
            gruppen = ({string*}) NEWSD->query_gruppen(was[0]);
            txt += ({"Brett  "+was[0]});
            prefix = "";
        }
        else
        {
            gruppen=was[1..1];
            prefix = "Gruppe";
        }
        foreach(string gruppe:gruppen)
        {
            int anz=sizeof(tmp=({int*})NEWSD->query_artikel_numbers(
                        was[0],gruppe));
            if(!tmp)
                continue;
            gruppenzahl++;
            artikelzahl+=anz;
            if(wizp(this_player()) || NEWSD->is_owner(this_player(), was[0], gruppe))
                txt += ({sprintf("%-6s %-35s %4d%s", prefix,
                    was[0]+" "+gruppe, anz, (anz>25? " <"+("-"*(anz/25)):""))});
            else
                txt += ({sprintf("%-6s %-35s %4d", prefix,
                    was[0]+" "+gruppe, anz)});
        }
    }

    txt += ({sprintf("%d Artikel in %d Gruppen (Schnitt: %3.1f)",
        artikelzahl, gruppenzahl, artikelzahl / to_float(gruppenzahl||1))});

    return txt;
}

/*
 * FUNKTION: get_option_menu
 * DEKLARATION: private mixed get_option_menu(mixed *m)
 * BESCHREIBUNG:
 * Liefert das Optionsmenue zurueck. m ist das Menue, von welchem aus dieses
 * Menue aufgerufen wurde.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed get_option_menu(mixed *m)
{
    return ({ ({
        ({
" Anzeige:                                             Editor:",
"  ("+(flags["SeeArticle"]==-1?"*":" ")+") Nur (n)eue Artikel                               ("+(flags["Editor"]==0?"*":" ")+") (S)tandard",
"  ("+(flags["SeeArticle"]== 0?"*":" ")+") Nur (u)ngelesene Artikel                         ("+(flags["Editor"]==1?"*":" ")+") (E)d",
"  ("+(flags["SeeArticle"]== 1?"*":" ")+") Auch (g)elesene Artikel"+(wizp(this_player())?"                          ("+(flags["Editor"]==2?"*":" ")+") Pla(y)er-ED":""),
wizp(this_player())?"                                                       ("+(flags["Editor"]==3?"*":" ")+") (X)ED":"",
"  ("+(flags["SeeGroup"]==-1?"*":" ")+") Nur Bretter/Gruppen mit unge(l)esenen Artikeln",
"  ("+(flags["SeeGroup"]== 0?"*":" ")+") Nur ange(m)eldete Bretter/Gruppen               Scrollen:",
"  ("+(flags["SeeGroup"]== 1?"*":" ")+") Auch nicht angemel(d)ete Bretter/Gruppen         ["+(flags["Scroll"]&1?"*":" ")+"] (B)rettübersicht",
"                                                       ["+(flags["Scroll"]&2?"*":" ")+"] Gru(p)penübersicht",
" Sonstiges:                                            ["+(flags["Scroll"]&4?"*":" ")+"] A(r)tikelübersicht",
"  ["+(flags["Flags"]&1 ?"*":" ")+"] Neue (T)hreads immer zusammengeklappt anzeigen   ["+(flags["Scroll"]&8?"*":" ")+"] (A)rtikelanzeige",
"  ["+(flags["Flags"]&4 ?"*":" ")+"] 'z' spr(i)ngt vom Artikel zur Gruppenübersicht   ["+(flags["Scroll"]&16?"*":" ")+"] S(o)nstiges",
"  ["+(flags["Flags"]&2 ?"*":" ")+"] Gruppenübergreifend nach neuen Artikeln su(c)hen",
"  ["+(flags["Flags"]&16?" ":"*")+"] Sicherheitsab(f)rage vor dem Abschicken eines Artikels",
wizp(this_player())?(
"  ["+(flags["Flags"]&8 ?"*":" ")+"] (K)eine Meldung an Umstehende"
):0,
testplayerp(this_player())?(
"  ["+(flags["Flags"]&32?"*":" ")+"] (J)a, ich will's als Autoloader"
):0,
        })-({0}),
        ({
    "Option: ",
    "----------- Einstellungen: ----------------------------------------------------",
    "-------------------------------------------------------------------------------",
    "...----------------------------------------------------------------------------",
    "...----------------------------------------------------------------------------"
        }),
        0,
#ifdef OPTIONS_IN_CHARMODE
        BF_NO_MENUE|M_DO_NOT_END|M_FRAME|M_CHARMODE|
            (flags["Scroll"]&16?M_SCROLL:0),
#else
        BF_NO_MENUE|M_DO_NOT_END|M_FRAME|(flags["Scroll"]&16?M_SCROLL:0),
#endif
        DISP_OPTIONS,
        ([ DISP_MENUE: m ])
    }) });
}

/*
 * FUNKTION: option_action
 * DEKLARATION: private mixed option_action(string str, mixed *m)
 * BESCHREIBUNG:
 * Diese Funktion behandelt die Eingaben im Optionsmenue m und liefert
 * das neue Menue zurueck.
 * VERWEISE:
 * GRUPPEN: news
 */
private mixed option_action(string str, mixed *m)
{
    // Freie Buchstaben: v, w
    switch(lower_case(str)[0])
    {
        case 'n': flags["SeeArticle"]=-1; break;
        case 'u': flags["SeeArticle"]= 0; break;
        case 'g': flags["SeeArticle"]= 1; break;

        case 'l': flags["SeeGroup"]=-1; break;
        case 'm': flags["SeeGroup"]= 0; break;
        case 'd': flags["SeeGroup"]= 1; break;

        case 's': flags["Editor"]=0; break;
        case 'e': flags["Editor"]=1; break;
        case 'y': if(wizp(this_player())) flags["Editor"]=2; break;
        case 'x': if(wizp(this_player())) flags["Editor"]=3; break;

        case 'b': flags["Scroll"]^= 1; break;
        case 'p': flags["Scroll"]^= 2; break;
        case 'r': flags["Scroll"]^= 4; break;
        case 'a': flags["Scroll"]^= 8; break;
        case 'o': flags["Scroll"]^=16; break;

        case 't': flags["Flags"]^=1; break;
        case 'c': flags["Flags"]^=2; break;
        case 'i': flags["Flags"]^=4; break;
        case 'k': if(!wizp(this_player())) return NOTHING;
                  flags["Flags"]^=8; break;
        case 'f': flags["Flags"]^=16; break;
	case 'j': if(!testplayerp(this_player())) return NOTHING;
		  flags["Flags"]^=32;
		  set_invis((flags["Flags"]&32)?V_NOLIST:V_INVIS);
		  if(no_back && (flags["Flags"]&32))
		    no_back=0;
		  break;
        default:
            return NOTHING;
    }
    return get_option_menu(m && m[0][BR_USER][DISP_MENUE]);
}

#ifdef OPTIONS_IN_CHARMODE
/*
 * FUNKTION: ignore_char
 * DEKLARATION: static void ignore_char(string str, int mode, mixed m)
 * BESCHREIBUNG:
 * Sobald im Charmode ein Escape-Char (ASCII-Code 27) ankam, wird die
 * weitere Eingabe zu dieser Funktion umgeleitet. Nach Ende der Codesequenz
 * wird wieder das Menue m ausgegeben.
 * VERWEISE:
 * GRUPPEN: news
 */
static void ignore_char(string str, int mode, mixed m)
{
    // Sequenz: ESC + [ oder O oder Code 155 oder Code 143
    //        + Ziffern oder ; + Buchstabe

    // Nach dem zweiten Zeichen wird mode auf 1 gesetzt und
    // diese Funktion bleibt in der input_to-Schleife, bis keine Ziffer oder
    // Semikolon empfangen wurde.
    if(mode)
    {
        if((str[0]>='0' && str[0]<='9') || str[0]==';')
            input_to("ignore_char",INPUT_CHARMODE|INPUT_NOECHO,1,m);
        else
            browse(m);
    }
    else if(member(([91,155,79,143]),str[0]))
        input_to("ignore_char",INPUT_CHARMODE|INPUT_NOECHO,1,m);
    else
        browse(m);
}
#endif

void search_callback(mixed dateien, string text, mixed m, mixed wo)
{
    mapping result = ([]);
    efun::set_this_player(owner);
    remove_input_to(owner, this_object());
    
    if(!pointerp(dateien) || !sizeof(dateien))
    {
	if(pointerp(dateien))
	    write("Keine passenden Artikel gefunden.\n");
	else if(stringp(dateien))
	    write(wrap_say("Folgender Fehler ist aufgetreten:",
		dateien));
	else
	    write("Ein Fehler ist aufgetreten.\n");
        m[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
	browse(m);
	return;
    }

    if(sizeof(m)>3)
	m = m[0..2];
	
    if(sizeof(searchinfo))
	dateien &= searchinfo[<1][SRCH_FILES];

    foreach(string str:dateien)
    {
	string *a = explode(str,"/");
	if(!member(result,a[0]))
	    result[a[0]]=([]);
	if(!member(result[a[0]],a[1]))
	    result[a[0]][a[1]]=([to_int(a[2])]);
	else
	    result[a[0]][a[1]]+=([to_int(a[2])]);
    }
    
    searchinfo+=({0})*(sizeof(m)-sizeof(searchinfo));
    
    searchinfo[<1] = ([
	SRCH_TEXT: text,
	SRCH_FILES: dateien,
	SRCH_ARTIKEL: result
    ]);
    
    browse(renew_menue_more_info(m, 4));
}

/*
 * FUNKTION: browse_action
 * DEKLARATION: static mixed browse_action(string str, mixed *m)
 * BESCHREIBUNG:
 * Diese Funktion erhaelt die Eingabe waehrend des Menues m.
 * (Doku siehe Enzy.)
 * VERWEISE:
 * GRUPPEN: news
 */
static mixed browse_action(string str, mixed *m)
{
    int nr;
    string brett, gruppe;
    mixed wo, tmp, tmp2, txt;

    if(str)
        str=strip(str);

    if(m[0][BR_STATUS_BYTE]&BF_NO_MENUE) //Hilfe oder Optionen
    {
#ifdef OPTIONS_IN_CHARMODE
        // VT-Tastencodes ignorieren
        if(m[0][BR_STATUS_BYTE]&M_CHARMODE)
        {
            if(str!="\n")
                write("\n");
            if(str[0]==27)
            {
                m[<1][BR_STATUS_BYTE]|=M_NO_FIRST_SCREEN;
                input_to("ignore_char",INPUT_CHARMODE|INPUT_NOECHO,0,m);
                return END_BROWSE;
            }
        }
#endif
        if(m[0][BR_PATH]==DISP_OPTIONS && sizeof(str) &&
	    (str[0]=='h' || str[0]=='H'))
            str[0]='?';

        switch(sizeof(str) && str[0])
        {
            case 'Q':
                return END_BROWSE;
            case 'q':
            case 'z':
                if(m[0][BR_PATH]==DISP_OPTIONS)
                    this_player()->set_newsreader_settings(flags);
                return get_back_more_info(m);
            case 0:
            case ' ':
            case '\n':
            case '\r':
                return CONTINUE;
            case '?':
                tmp = 0;
                switch(m[0][BR_PATH])
                {
                    case DISP_OPTIONS:
                        tmp = ({
    "Weitere Befehle:",
    "",
    "  z / q       Einstellungsmenü verlassen",
    "  Q           Lesen beenden",
    "  ?           Diese Hilfe anzeigen",
    "  !<befehl>   Befehl als Spieler ausführen",
                        });
                        break;

                    case DISP_HELP:
                        if((m[0][BR_USER][DISP_MENUE][0][BR_STATUS_BYTE]&BF_NO_MENUE)
                            && m[0][BR_USER][DISP_MENUE][0][BR_PATH]==DISP_HELP)
                                return "Du befindest Dich bereits in der Hilfe zur Hilfe.\n";
                        tmp = ({
    "Mögliche Befehle:",
    "",
    "  z / q       Hilfe beenden",
    "  Q           Lesen beenden",
    "  ?           Diese Hilfe anzeigen",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  U / B       Eine halbe Seite nach oben blättern",
    "  D / d       Eine halbe Seite nach unten blättern",
    "  u / b       Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang des Textes gehen",
    "  >           Ans Ende des Textes gehen",
    "  c           Gesamten restlichen Text auf einmal anzeigen",
    "  <nr>        Zu dieser Zeile springen",
    "  /<text>     Nach diesem Text suchen",
    "  /           Suche wiederholen",
                        });
                        break;

                    case DISP_THREAD:
                        wo = m[0][BR_USER][DISP_PATH];
                        tmp = ({
    "Mögliche Befehle:",
    "",
    "  z / q       Threadanzeige verlassen",
    "  Q           Lesen der News beenden",
    "  o           Menü mit den Einstellungen aufrufen",
    "  E           ED als Editor verwenden ("+(flags["Editor"]?"ein":"aus")+")",
    "  ?           Diese Hilfe anzeigen",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  <nr>        Artikel mit dieser Nummer lesen",
    "  n           Den ersten ungelesenen Artikel lesen",
    "  N           Den ersten neuen Artikel lesen",
    "  A           Den ersten alten, aber ungelesenen Artikel lesen",
    "  x           Zum letzten Artikel springen",
    "  X           Zum ersten Artikel springen",
    "",
    "  M           Alle Artikel als gelesen markieren",
    "  M <nr>      Diesen Artikel als gelesen markieren",
    "  m           Alle Artikel als ungelesen markieren",
    "  m <nr>      Diesen Artikel als ungelesen markieren",
    "",
    "  s           Einen neuen Artikel schreiben",
    "  b <nr>      Diesen Artikel beantworten",
    "  P <nr>      Diesen Artikel als Brief schicken lassen",
    "  l <nr>      Diesen Artikel löschen",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  L <nr>      Diesen Thread löschen":0,
    "  Z <nummern> Eine Zusammenfassung dieser Artikel anzeigen",
    "              <nummern>: <nr>-<nr> oder <nummern>,<nummern>",
    "  T <nr>      Eine Zusammenfassung dieses Threads anzeigen",
    "  t <nr>      Den Baum dieses Threads anzeigen",
    "",
    "  i <name>    Artikel dieses Autors nicht anzeigen",
    "  i <nr>      Artikel des Autors dieses Artikels nicht mehr anzeigen",
    "  I           Alle Autoren anzeigen, deren Artikel nicht mehr angezeigt werden",
    "",
    "  O           Moderatoren dieser Gruppe anzeigen",
                        })-({0}) + (flags["Scroll"]&16?({}):({
    "",
    "  r           Beschreibung neu ausgeben",
    "  U / B       Eine halbe Seite nach oben blättern",
    "  D / d       Eine halbe Seite nach unten blättern",
    "  u / b       Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang des Textes gehen",
    "  >           Ans Ende des Textes gehen",
    "  c           Gesamten restlichen Text auf einmal anzeigen",
    "  <nr>        Zu dieser Zeile springen",
    "  /<text>     Nach diesem Text suchen",
    "  /           Suche wiederholen",
                        }));
                        break;

                    case DISP_SUMMARY:
                        wo = m[0][BR_USER][DISP_PATH];
                        tmp = ({
    "Mögliche Befehle:",
    "",
    "  z / q       Anzeige verlassen",
    "  Q           Lesen der News beenden",
    "  ?           Diese Hilfe anzeigen",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  M           Alle zugehörigen Artikel als gelesen markieren",
    "  m           Alle zugehörigen Artikel als ungelesen markieren",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  L           Alle zugehörigen Artikel löschen":0,
    "  s           Text als Artikel ans Brett schreiben",
    "  P           Text per Post schicken lassen",
    wizp(this_player())?
    "  w <datei>   Als Datei abspeichern":0,
                        })-({0}) + (flags["Scroll"]&16?({}):({
    "",
    "  U / B       Eine halbe Seite nach oben blättern",
    "  D / d       Eine halbe Seite nach unten blättern",
    "  u / b       Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang des Textes gehen",
    "  >           Ans Ende des Textes gehen",
    "  c           Gesamten restlichen Text auf einmal anzeigen",
    "  <nr>        Zu dieser Zeile springen",
    "  /<text>     Nach diesem Text suchen",
    "  /           Suche wiederholen",
                        }));
                        break;

                    case DISP_MODS:
                        tmp = ({
    "Mögliche Befehle:",
    "",
    "  z / q       Anzeige verlassen",
    "  Q           Lesen der News beenden",
    "  ?           Diese Hilfe anzeigen",
    "  !<befehl>   Befehl als Spieler ausführen",
                        }) + (flags["Scroll"]&16?({}):({
    "",
    "  U / B       Eine halbe Seite nach oben blättern",
    "  D / d       Eine halbe Seite nach unten blättern",
    "  u / b       Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang des Textes gehen",
    "  >           Ans Ende des Textes gehen",
    "  c           Gesamten restlichen Text auf einmal anzeigen",
    "  <nr>        Zu dieser Zeile springen",
    "  /<text>     Nach diesem Text suchen",
    "  /           Suche wiederholen",
                        }));
                        break;
                }

                if(!tmp)
                    return "Es gibt noch keine Hilfe.\n";
                return ({ ({tmp, "Hilfe - Zeile %d von %d [z,u,b,/<such>] ",
                    0, M_FRAME|M_AUTO_END|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                    DISP_HELP, ([DISP_MENUE: m]) }) });

            default:
                switch(m[0][BR_PATH])
                {
                    case DISP_OPTIONS:
                        return option_action(str,m);
                    case DISP_THREAD:
                        if(member((['0','1','2','3','4','5','6','7','8','9',
                            'A','b','E','i','I','j','J','l','L','m','M','n','N',
                            'o','O','P','s','t','x','X','Z']),str[0]))
                        {
                            m = m[0][BR_USER][DISP_MENUE][0..2];
                            break;
                        }
                        switch(str[0])
                        {

                            case 'T':  // Thread zusammenfassen
                                wo = m[0][BR_USER][DISP_PATH];

                                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
                                    wo=wo[0..1]+tmp;

                                return summarize_thread(m[0][BR_USER][DISP_MENUE],wo);

                            case 'v':
                                wo = m[0][BR_USER][DISP_PATH];
                                m = m[0][BR_USER][DISP_MENUE];
                                tmp = m[2][BR_USER][ART_INFO][wo[2],AI_ROOTWAY]; // Pfad zur Wurzel
                                if(sizeof(tmp)>1)
                                {
                                    nr = tmp[1][N_NUMBER];
                                    if(member(m[2][BR_USER][ART_LIST],nr)>=0)
                                    {
                                        tmp = tree2str(m[2][BR_USER][ART_INFO][nr,AI_ROOTWAY][0..0],
                                            m[2][BR_USER][ART_INFO], wo[0], wo[1], 3);
                                        if(sizeof(tmp))
                                        {
                                            tmp = transpose_array(tmp); // Zeilen und Nummern trennen
                                            return ({ ({tmp[0], "Threadanzeige, Zeile %d von %d "
                                                "[z,<,>,u,b,/<such>"+(wizp(this_player())?",w":"")+"] ",
                                                0, M_FRAME|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                                                DISP_THREAD, ([DISP_MENUE: m, DISP_PATH: wo[0..1]+({nr})]) }) });
                                        }
                                        return NOTHING;
                                    }
                                }
                                return "Es gibt keinen Bezugsartikel.\n";
                            default:
                                return browser::browse_action(str, m);
                        }
                    case DISP_SUMMARY:
                        switch(str[0])
                        {
                            case 'L': // Zugehoerige Artikel loeschen
                                return delete_article(m,m[0][BR_USER][DISP_PATH]);
                                
                            case 'm': // Als ungelesen markieren
                                mark_all_as_unread(m[0][BR_USER][DISP_PATH]);
                                return RENEW("Ok.\n",m);
                            
                            case 'M': // Als gelesen markieren
                                mark_all_as_read(m[0][BR_USER][DISP_PATH]);
                                return RENEW("Ok.\n",m);

                            case 's': // Ans Brett schreiben
                                if(tmp=cant_write())
                                    return tmp;
                                // FALLTHROUGH
                            case 'P':  // Per Post schicken
                                wo = m[0][BR_USER][DISP_PATH];
                                if(str[0]=='s' && !(NEWSD->valid_write(owner, wo[0], wo[1])))
                                    return "An diese"+(NO_BACK2?"s Brett":" Gruppe")+
                                        " darfst Du nicht schreiben.\n";

                                // m[0][BR_USER][DISP_MENUE]: Original Menuestruktur
                                // -> [2]:                    Artikeluebersicht
                                // -> [BR_USER][ART_INFO]:    Artikelinfos
                                // -> [wo[2][0],AI_ROOTWAY]:  Vom ersten Artikel in der
                                //                            Zusammenfassung (Weg zur Wurzel)
                                // -> [0]:                    Der Artikel selber (in dem Weg)
                                // -> [N_TITLE]:              Der Titel
                                tmp = m[0][BR_USER][DISP_MENUE][2][BR_USER][ART_INFO][wo[2][0],AI_ROOTWAY][0][N_TITLE];
                                if(lower_case(tmp[0..15])=="zusammenfassung:")
                                    tmp = "ZUS: "+strip(tmp[16..<1]);
                                else if(lower_case(tmp[0..4])=="zus:" ||
                                        lower_case(tmp[0..4])=="zsf:")
                                    tmp = "ZUS: "+strip(tmp[5..<1]);
                                else
                                    tmp = "ZUS: "+strip(tmp);

                                tmp = tmp[0..(MAX_TITLE_LENGTH-1)];

                                if(str[0]=='s')
                                {
                                    write(wrap("Für 'Titel: " + tmp +
                                        "' nur Return eingeben. Abbruch mit '~q'"));
                                    input_to("input_sum_titel", INPUT_PROMPT, "Titel: ",m,wo,tmp);
                                    return END_BROWSE;
                                }
                                else // Mail
                                {
                                    MAILD->send_news(
                                        this_player()->query_real_name(),
                                        time(), tmp,
                                        implode(m[0][BR_MENUE],"\n")+"\n");
                                    return "Brief wurde abgeschickt.\n";
                                }
                            default:
                                return browser::browse_action(str, m);
                        }
                    default:
                        return browser::browse_action(str, m);
                }
        }
    }

    if(str && sscanf(str,"%d",nr)) // Zahlen teilweise besonders behandeln
    {
        if(sizeof(m)<3) //Brett-/Gruppenauswahl
        {
            if(nr<1 || nr>sizeof(m[<1][BR_USER]))
                return ((sizeof(m)==1)?"Ein Brett":"Eine Gruppe")
                     + " mit dieser Nummer gibt es nicht.\n";
            else if(member(m[<1][BR_NUMBERS],nr)<0)
            {
                int i;
                wo = GET_PATH(m);
                // Angemeldet?
                if((sizeof(m)==1 &&
                        (!member(plinfo,m[<1][BR_USER][nr-1][0]) ||
                        plinfo[m[<1][BR_USER][nr-1][0],PLB_DATE]<0)) ||
                   (sizeof(m)==2 &&
                        (!member(plinfo[wo[0],PLB_GROUPS],m[<1][BR_USER][nr-1][0]) ||
                        plinfo[wo[0],PLB_GROUPS][m[<1][BR_USER][nr-1][0],PLG_DATE]<0)))
                {
                    return ((sizeof(m)==1)?"Das Brett":"Die Gruppe")
                         + " '" + m[<1][BR_USER][nr-1][0]
                         + "' hast Du nicht angemeldet.\n"
                           "Du kannst "+((sizeof(m)==1)?"es":"sie")
                         + " mit 'an "+nr+"' anmelden.\n";
                }
                // Okay, Gruppe ist angemeldet, wird aber nicht dargestellt.
                // Das heisst, wir schummeln sie einfach rein.
                while(i<sizeof(m[<1][BR_NUMBERS]) && m[<1][BR_NUMBERS][i]<nr)
                    i++;
                m[<1][BR_NUMBERS][i..i-1]=({nr});
                m[<1][BR_MENUE][i..i-1]=({sprintf("  %5d  %s",nr,
                    m[<1][BR_USER][nr-1][0])});
                return get_menue_more_info(m,i+1);
            }
        }
        else if(sizeof(m)==3) //Artikelauswahl
        {
            if(member(m[<1][BR_USER][ART_LIST],nr)>=0)
                return get_menue_more_info(m,nr);
            else
                return "Einen Artikel mit dieser Nummer gibt es nicht.\n";
        }
    }

    if(!sizeof(str) || (!member((['a','n','A','N']),str[0]) && ((str[1..1]-" ")!="")))
       return browser::browse_action(str, m);

    switch(str[0])
    {
        case 'A':	// Neuen Artikel suchen
        case 'n':
        case 'N':
//	    if(wizp(owner)) // Erstmal testweise
#if 1
	    return wait_for_task(m, "Suche neue Artikel...\n", 10,
		#'search_new_article2, m, member(({'A','N','n'}),str[0])+1,
                (lower_case(str[1..1])==lower_case(str[0..0]))
                ?(flags["Flags"]&2?0:NO_BACK2?0:NO_BACK1?1:2)
                :(flags["Flags"]&2?(NO_BACK2?0:NO_BACK1?1:2):0));
#else
            return search_new_article(m, member(({'A','N','n'}),str[0])+1,
                (lower_case(str[1..1])==lower_case(str[0..0]))
                ?(flags["Flags"]&2?0:NO_BACK2?0:NO_BACK1?1:2)
                :(flags["Flags"]&2?(NO_BACK2?0:NO_BACK1?1:2):0));
#endif
        case 'a':	// Brett/Gruppe an-/abmelden
            if(sizeof(m)>2)	// Weder in Brett- noch Gruppenuebersicht
                break;
            if(strlen(str)<2 || (str[1]!=' ' &&
               (!member((['n','N','b','B']),str[1]) ||
	        (strlen(str)>2 && str[2]!=' '))))
                return "'an <nr>' oder 'ab <nr>' um ein" +
                        ((sizeof(m)==1)?" Brett":"e Gruppe") +
                        " an- oder abzumelden.\n";

            tmp = parse_group(str[2..<1],m,0);
            if(stringp(tmp))
                return tmp;
            else
                nr=tmp[0];
	    gruppe = m[<1][BR_USER][nr-1][0]; // Der Brett/Gruppenname
	    if(sizeof(m)==1)
	    {
		if(!member(plinfo,gruppe) || plinfo[gruppe,PLB_DATE]==-1)
		{
		    if(lower_case(str[1..1])=="b")
			return "Das Brett '" + gruppe +
			    "' ist nicht angemeldet.\n";
                    BRETT_AN(gruppe,0);
	            if(gruppe == "InterMUD")
		        cat("/static/adm/USENET_INFO");
		    return RENEW("Brett '"+gruppe+"' angemeldet.\n",m);
		}
		else
		{
		    if(lower_case(str[1..1])=="n")
			return "Du hast das Brett '" + gruppe +
			    "' bereits angemeldet.\n";
                    BRETT_AB(gruppe);
		    return RENEW("Brett '"+gruppe+"' abgemeldet.\n",m);
		}
	    }
	    else
	    {
                wo = GET_PATH(m);
		if(!member(plinfo[wo[0],PLB_GROUPS],gruppe) ||
                    plinfo[wo[0],PLB_GROUPS][gruppe,PLG_DATE]==-1)
		{
		    if(lower_case(str[1..1])=="b")
			return "Die Gruppe '" + gruppe +
			    "' ist nicht angemeldet.\n";
                    GRUPPE_AN(wo[0],gruppe,0);
		    return RENEW("Gruppe '"+gruppe+"' angemeldet.\n",m);
		}
		else
		{
		    if(lower_case(str[1..1])=="n")
			return "Du hast die Gruppe '" + gruppe +
			    "' bereits angemeldet.\n";
                    GRUPPE_AB(wo[0],gruppe);
		    return RENEW("Gruppe '"+gruppe+"' abgemeldet.\n",m);
		}
	    }
	    // Not reached

        case 'b':  // Artikel beantworten
        case 's':  // Artikel schreiben
            if(sizeof(m)<3) // Nicht in der Brett- oder Gruppenuebersicht
                break;

            if(tmp=cant_write())
                return tmp;
            wo = GET_PATH(m);
            if(!(NEWSD->valid_write(owner, wo[0], wo[1])))
                return "An diese"+(NO_BACK2?"s Brett":" Gruppe")+
                    " darfst Du nicht schreiben.\n";
            if(str[0]=='s') // Artikel schreiben
            {
                write("Bitte Titel eingeben (Abbruch mit '~q')\n");
                input_to("input_titel", INPUT_PROMPT,"Titel: ",m,wo[0..1],0);
                return END_BROWSE;
            }

            // Hier nun beantworten
            if(sizeof(m)==3)
            {
                if(sscanf(str[2..<1],"%d",nr))
                    wo+=({nr});
                else
                {
                    input_to("ask_reply_number", INPUT_PROMPT,
                	"Welchen Artikel willst Du beantworten (Nummer angeben)? ",
			m, wo);
                    return END_BROWSE;
                }
            }
            if(reply_to(m,wo))
                return (sizeof(m)==3)
                    ?"Einen Artikel mit dieser Nummer gibt es nicht.\n"
                    :"Diesen Artikel gibt es inzwischen nicht mehr.\n";
            else
                return END_BROWSE;

        case 'E':
            if(wizp(owner))
            {
                flags["Editor"]=({1,3,3,0})[flags["Editor"]];
                return "Du nutzt nun den "+
                       ({"Mini-ED","ED","Player-ED","XED"})[flags["Editor"]]+
                       ".\n";
            }
            else
            {
                flags["Editor"]=!flags["Editor"];
                return flags["Editor"]
                       ?"Du nutzt nun den ED.\n"
                       :"Du nutzt jetzt den ED nicht mehr.\n";
            }

        case 'e':  // Newsreader beenden
            if(sizeof(m)==4)
                break;        // -> more
        case 'q':
        case 'Q':
            if(!IS_AL)
                zustand|=1;
	    END_CACHE;
    	    return END_BROWSE;

        case 'g':  // Eine neue Gruppe eroeffnen
            if(sizeof(m)!=2) // Nur an einem Brett
                break;

            wo = GET_PATH(m);

            if(!NEWSD->query_brett_owner(wo[0],0,owner))
                return "Du darfst das nicht.\n";
            input_to("ask_for_new_group", INPUT_PROMPT,
    		"Bitte Gruppennamen eingeben: ", m, wo);
            return END_BROWSE;

	case 'f': // Suche nach <Text>
	    if(!wizp(owner))
		break;

            if(!strlen(str=strip(str[2..<1])))
		return "f <Suchtext>\n";
	    
	    wo = GET_PATH(m);
	    if(sizeof(wo)>2)
		wo = wo[0..1];
	
	    apply(#'call_other, NEWS_INDEX, "lookup_artikel", str,
		lambda(({'artikel}), ({#'search_callback, 'artikel, str, quote(m), quote(wo) })),
		0, wo);
		
	    write("Einen Moment bitte, die Artikel werden zusammengesucht.\n");
	    input_to(
		tmp = (:
		    if($2+30<time())
		    {
			write("Ein Fehler ist aufgetreten. Suche abgebrochen.\n");
    			$3[<1][BR_STATUS_BYTE] |= M_NO_FIRST_SCREEN;
			browse($3);
		    }
		    else
		    {
			write("Einen Moment bitte, die Artikel werden zusammengesucht.\n");
			input_to($4,0,$2,$3,$4);
		    }
		:), 0, time(), m, tmp);
	    return END_BROWSE;
	
        case 'i':  // *plonk*
            if(!player_exists(lower_case(tmp=str=strip(str[2..<1]))) &&
	       !member(flags["Ignore"]||([]),lower_case(tmp)))
                tmp=0;
            if(!tmp && sizeof(m)>=3)
            {
                wo = GET_PATH(m);

                if(sizeof(m)==3 && pointerp(tmp=parse_article(str,m,wo)))
		{
		    if(!member(m[2][BR_USER][ART_INFO], tmp[0]))
			return "Es gibt keinen Artikel mit dieser Nummer.\n";
                    wo+=tmp;
		}

                if(sizeof(wo)==3)
                    tmp = m[2][BR_USER][ART_INFO][wo[2],AI_ROOTWAY][0][N_AUTHOR];
                else
                    tmp=0;
            }
            if(!tmp)
            {
                if(!player_exists(lower_case(tmp=str)) &&
		   !member(flags["Ignore"]||([]),lower_case(tmp)))
                {
                    input_to("ask_for_menu", INPUT_PROMPT,
                	"Wen willst Du (nicht mehr) ignorieren? ",
			m, wo||({}),
                    (:
                        if(player_exists(lower_case($1)) ||
			   member(flags["Ignore"]||([]),lower_case($1)))
                            return ({$1});
                        else
                            return "Einen solchen Spieler gibt es nicht.\n";
                    :), #'ignore, 0);
                    return END_BROWSE;
                }
            }
            return ignore(m,({tmp}));

        case 'I': // Ignore-Liste ausgeben
            if(sizeof(flags["Ignore"]))
                return wrap("Du ignorierst " +
                    liste(map(sort_array(m_indices(flags["Ignore"]),#'>),
                    #'capitalize))+".");
            else
                return "Du ignorierst niemanden.\n";

        case 'j': // Uebersicht ueber die Newsgruppen nach Moderator
        case 'J': // Uebersicht ueber alle Newsgruppen

            wo = GET_PATH(m);
            if(str[0]=='J' && strlen(str[2..<1]) && sizeof(wo)<2)
            {
                if(pointerp(tmp = parse_subscribed_group(str[2..<1],m,wo)))
                    wo+=({m[<1][BR_USER][tmp[0]-1][0]});
            }

            txt = get_mod_news_list(str[0]=='j'?
                    (brett=strip(str[1..<1])):0,wo);

            if(stringp(txt)) return txt;

            return ({ ({txt, "Newsliste" + (str[0]=='j'?
                " für "+(strlen(brett)?capitalize(brett):"Dich"):"") +
                " [z,u,b,/<such>] ",
                0, M_AUTO_END|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                DISP_MODS, ([DISP_MENUE: m, DISP_PATH: wo]) }) });

        case 'k': // collapse thread
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
                    wo = wo[0..1] + tmp;
                else if(sizeof(m)==3)
                    return "'k <nr>' um Thread zu- oder aufzuklappen.\n";
		else
		    return "'k [<nr>]' um einen Thread zu- oder aufzuklappen.\n";
            }

	    if(!member(m[2][BR_USER][ART_INFO], wo[2]))
	        return "Einen Thread mit dieser Nummer gibt es nicht.\n";

            if(((flags["Flags"]&1) && !member(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN],wo[2]))
             || member(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED],wo[2]))
            {        // Zusammengeklappt -> Aufklappen
                // Wir muessen auch die Subthreads aufklappen.
                mapping subthreads;
                subthreads=mkmapping( ({wo[2]}) +
                    m[2][BR_USER][ART_INFO][wo[2],AI_SUBLIST]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN]+=subthreads;
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED]-=subthreads;
                return RENEW("Thread bei Artikel "+wo[2]+" aufgeklappt.\n",m);
            }
            else
            {
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN]-=([wo[2]]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED]+=([wo[2]]);
                return RENEW("Thread bei Artikel "+wo[2]+" zugeklappt.\n",m);
            }

        case 'K': // Alles zu-/aufklappen.
            if(sizeof(m)!=3)
                break;

            wo = GET_PATH(m);

            /*
                Standard  |  nix explizit           Teilweise explizit
                          |  auf- oder zu   aufgeklappt  zugeklappt  beides
                --------------------------------------------------------------
                  zu      |  aufklappen      zuklappen   aufklappen  zuklappen
                  auf     |  zuklappen       zuklappen   aufklappen  aufklappen
            */

            if(((flags["Flags"]&1) && !sizeof(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN])) ||
               (!(flags["Flags"]&1) && sizeof(plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED])))
            {
                // Aufklappen
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN]=mkmapping(m[<1][BR_USER][ART_LIST]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED]=([]);
                return RENEW("Alles aufgeklappt.\n",m);
            }
            else
            {   // Zuklappen
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_OPEN]=([]);
                plinfo[wo[0],PLB_GROUPS][wo[1],PLG_COLLAPSED]=mkmapping(m[<1][BR_USER][ART_THREADS]);
                return RENEW("Alles zugeklappt.\n",m);
            }

        case 'l':  // Artikel loeschen
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_articles(str[2..<1],m,wo)))
                    wo = wo[0..1] + tmp;
                else if(sizeof(m)==3)
                {
                    input_to("ask_for_menu", INPUT_PROMPT,
                	"Welchen Artikel willst Du löschen (Nummer angeben)? ",
			m, wo,#'parse_articles,#'delete_article, 0);
                    return END_BROWSE;
                }
		else
		    return "'l [<nr>[-<nr>][, ...]]' um Artikel zu löschen.\n";
            }
            else
                wo[2]=({wo[2]});
            return delete_article(m[0..2],wo);

        case 'L':  // Thread loeschen
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
                    wo = wo[0..1] + tmp;
                else if(sizeof(m)==3)
                {
                    write("Den Thread beginnend mit welchem Artikel willst Du löschen (Nummer angeben)?\n");
	            input_to("ask_for_menu", INPUT_PROMPT, "Nr: ", m, wo,#'parse_article,#'delete_thread, 0);
                    return END_BROWSE;
                }
		else
		    return "'L [<nr>]' um ein Thread zu löschen.\n";
            }

            return delete_thread(m,wo);

        case 'm':  // Alles als ungelesen markieren
            wo = GET_PATH(m);

            if(strlen(str[2..<1]-" "))
            {
                if(sizeof(m)<3)
                {
                    tmp = parse_subscribed_group(str[2..<1],m,wo);
                    if(stringp(tmp))
                        return tmp;
                    else
                        wo+=({m[<1][BR_USER][tmp[0]-1][0]});
                }
                else
                {
                    tmp = parse_articles(str[2..<1],m,wo);
                    if(pointerp(tmp))
                	wo = wo[0..1] + tmp;
		    else if(sizeof(m)==3)
                        return tmp;
                    else
			return "'m [<nr>[-<nr>][, ...]]' um Artikel als ungelesen zu markieren.\n";
                }
            }
            else if(sizeof(wo)<3) // Brett- oder Gruppenuebersicht
            {                    // Vorsichtshalber nachfragen
                input_to("ask_for_menu", INPUT_PROMPT,
            	    "Wirklich alle Artikel" +
                    ({" an allen Brettern", " in allen Gruppen",
                      " in dieser Gruppe"})[sizeof(wo)] +
                    " als ungelesen markieren (Ja/Nein)? Nein\b\b\b\b",
		    m, wo, #'parse_noyes,
                    (:
                        // $1: m, $2: wo
                        mark_all_as_unread($2);
                        return RENEW("Ok.\n",$1);
                    :), 1);
                return END_BROWSE;
            }
            mark_all_as_unread(wo);

            return (sizeof(m)>3)?"Ok.\n":RENEW("Ok.\n",m);

        case 'M':  // Alles als gelesen markieren
            if(sizeof(m)>3) // Nur Brett/Gruppenuebersicht/Artikeluebersicht
                break;

            wo = GET_PATH(m);

            if(strlen(str[2..<1]-" "))
            {
                if(sizeof(m)<3)
                {
                    tmp = parse_subscribed_group(str[2..<1],m,wo);
                    if(stringp(tmp))
                        return tmp;
                    else
                        wo+=({m[<1][BR_USER][tmp[0]-1][0]});
                }
                else
                {
                    tmp = parse_articles(str[2..<1],m,wo);
                    if(pointerp(tmp))
                	wo = wo[0..1] + tmp;
		    else if(sizeof(m)==3)
                        return tmp;
                    else
			return "'M [<nr>[-<nr>][, ...]]' um Artikel als gelesen zu markieren.\n";
                }
            }
            mark_all_as_read(wo);

            return RENEW("Ok.\n",m);

        case 'o':  // Optionsmenue
            return get_option_menu(m);

        case 'O':  // Newsmod anzeigen
            wo = GET_PATH(m);
            if(strlen(str[2..<1]) && sizeof(wo)<2)
            {
                if(pointerp(tmp = parse_subscribed_group(str[2..<1],m,wo)))
                    wo+=({m[<1][BR_USER][tmp[0]-1][0]});
            }
            wo = wo[0..1];
            tmp = ({mapping}) NEWSD->query_owners();

            tmp2 = ({});

            if(sizeof(wo)==0)
                foreach(string my_brett, int datum:plinfo)
                {
                    if(datum>=0)
                        tmp2 += ({ ({my_brett}) });
                }
            else if(sizeof(wo)==1)
                foreach(string my_gruppe, int datum:plinfo[wo[0],PLB_GROUPS])
                {
                    if(datum>=0)
                        tmp2 +=  ({ ({wo[0],my_gruppe}) });
                }
            tmp2=sort_array(tmp2,(:$1[<1]>$2[<1]:));

            for(nr=sizeof(wo);nr>=0;nr--)
                tmp2 = ({ wo[0..nr-1] }) + tmp2;

            txt = ({});
            foreach(string *gr:tmp2)
            {
                string *mods=tmp["/"+implode(gr,"/")];
                if(!mods)
                    continue;

                mods = map(mods,#'capitalize);

                if(!sizeof(wo))
                    txt+=explode(sprintf("%-14s %-#53.4s\n",
                        sizeof(gr)?gr[0]:"Newsadmin",implode(mods,"\n")),
                        "\n")[0..<2];
                else
                    txt+=explode(sprintf("%-31s %-#46.4s\n",
                        sizeof(gr)?implode(gr," "):"Newsadmin",
                        implode(mods,"\n")),"\n")[0..<2];
            }

            return ({ ({txt, "Newsmoderatoren [z,u,b,/<such>] ",
                0, M_AUTO_END|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                DISP_MODS, ([DISP_MENUE: m, DISP_PATH: wo]) }) });

        case 'P':  // Artikel als Post zuschicken.
            if(sizeof(m)<3) //Im Artikel oder inner Artikeluebersicht
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
		{
		    if(!member(m[2][BR_USER][ART_INFO], tmp[0]))
			return "Es gibt keinen Artikel mit dieser Nummer.\n";
                    wo = wo[0..1] + tmp;
		}
                else if(sizeof(m)==3)
                    return "'P <nr>' um Dir den Artikel per Post zu schicken.\n";
		else
                    return "'P [<nr>]' um Dir den Artikel per Post zu schicken.\n";
            }
            tmp = m[2][BR_USER][ART_INFO][wo[2],AI_ROOTWAY][0];
            txt = read_file(({string})NEWSD->query_file_name(wo[0],wo[1],wo[2]));
            if(!txt)
                return "Konnte den Artikel nicht lesen.\n";
            MAILD->send_news(tmp[N_AUTHOR], tmp[N_DATE], tmp[N_TITLE], txt);
            return "Brief wurde abgeschickt.\n";

	case 'r':  // Uebersicht neu ausgeben
	    if(sizeof(m)<4) // Bis zur Artikeluebersicht
		    return renew_menue_more_info(m,2);
	    break;

        case 'R':  // Uebersicht mit umgekehrten y/Y-Flags ausgeben
            if(sizeof(m)<4)
            {
                wo = ({flags["SeeArticle"],flags["SeeGroup"]});
                flags["SeeArticle"] = (flags["SeeArticle"]<=0)?1:0;
                flags["SeeGroup"] = (flags["SeeGroup"]<=0)?1:0;
                tmp = renew_menue_more_info(m,4);
                flags["SeeArticle"] = wo[0]; flags["SeeGroup"] = wo[1];
                return tmp;
            }
            break;
        case 'S':  // Scroll ein-/ausschalten
            flags["Scroll"] ^= 1<<(sizeof(m)-1);
            return "Seitenweise Anzeige " +
                ({"der Brettuebersicht",
                  "der Gruppenübersicht",
                  "der Artikelübersicht",
                  "der Artikel"})[sizeof(m)-1] +
                ((flags["Scroll"] & 1<<(sizeof(m)-1))?" aus":" an")+
                "geschaltet.\n";

        case 't':  // Thread-Baum anzeigen
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
		{
		    if(!member(m[2][BR_USER][ART_INFO], tmp[0]))
			return "Es gibt keinen Thread mit dieser Nummer.\n";
                    wo = wo[0..1] + tmp;
		}
                else if(sizeof(m)==3)
                    return tmp;
		else
		    return "'t [<nr>]' um den Baum eines Threads anzuzeigen.\n";
            }

            tmp = tree2str(m[2][BR_USER][ART_INFO][wo[2],AI_ROOTWAY][0..0],
                m[2][BR_USER][ART_INFO], wo[0], wo[1], 3);

	    if(sizeof(tmp))
	    {
	        tmp = transpose_array(tmp); // Zeilen und Nummern trennen
                return ({ ({tmp[0], "Threadanzeige, Zeile %d von %d "
                    "[z,<,>,u,b,/<such>"+(wizp(this_player())?",w":"")+"] ",
                    0, M_FRAME|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                    DISP_THREAD, ([DISP_MENUE: m, DISP_PATH: wo]) }) });
	    }
            return NOTHING;

        case 'T':  // Thread zusammenfassen
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_article(str[2..<1],m,wo)))
		{
		    if(!member(m[2][BR_USER][ART_INFO], tmp[0]))
			return "Es gibt keinen Thread mit dieser Nummer.\n";
                    wo = wo[0..1] + tmp;
		}
                else if(sizeof(m)==3)
                {
                    write("Den Thread beginnend mit welchem Artikel willst Du zusammenfassen\n");
                    input_to("ask_for_menu", INPUT_PROMPT, "(Nummer angeben)? ",
			m, wo,#'parse_article,#'summarize_thread, 0);
                    return END_BROWSE;
                }
		else
		    return "'T [<nr>]' um einen Thread zusammenzufassen.\n";
            }

            return summarize_thread(m,wo);

        case 'v':  // Zum Vorgaengerartikel
            if(sizeof(m)==4) // Nur in der Artikelansicht
            {
                nr = m[<1][BR_PATH];
                tmp = m[<2][BR_USER][ART_INFO][nr,AI_ROOTWAY]; // Pfad zur Wurzel
                if(sizeof(tmp)>1)
                {
                    nr = tmp[1][N_NUMBER];
                    if(member(m[<2][BR_USER][ART_LIST],nr)>=0)
                        return get_menue_more_info(m[0..2],nr);
                }
                return "Es gibt keinen Bezugsartikel.\n";
            }
            break;

        case 'V': // Artikel an andere Gruppe verschieben
            if(sizeof(m)<3)
                break;
            
            wo = GET_PATH(m);

            if(sizeof(m)==3)
            {
                txt = explode(space(str[2..<1])," ");
                if(sizeof(txt)<2 || !pointerp(tmp=parse_article(txt[0],m,wo)))
                    return "V <Artikelnummer> [Brett:]Gruppe\n";
                wo+=tmp;
                txt = implode(txt[1..<1]," ");
                if(!member(m[2][BR_USER][ART_INFO],wo[2]))
                    return "Es gibt keinen Artikel mit der Nummer "+wo[2]+".\n";
            }
            else
                txt = space(str[2..<1]);
            
            txt = regexplode(txt,"[ :/]")-({" ","",":","/"});
            if(!sizeof(txt) || sizeof(txt)>2)
                return "V "+((sizeof(m)==3)?"<Artikelnummer> ":"")+
                       "[Brett:]Gruppe\n";
            if(sizeof(txt)==1)
                txt = wo[0..0] + txt;
            
            tmp2 = ({});        // Pfad herausfinden
            foreach(mixed art:m[2][BR_USER][ART_INFO][wo[2],1])
                tmp2 = ({art[N_NUMBER]}) + tmp2;
            
            return NEWSD->move_thread(wo[0], wo[1], tmp2, txt[0], txt[1], ({}))
                || RENEW2(wrap("Der Thread wurde nach "+txt[0]+":"+txt[1]+
                    " verschoben."),m[0..2],3);
            
        case 'x':
        case 'X':
            if(sizeof(m)<3) // Artikeluebersicht oder -ansicht
                break;

            if(!sizeof(m[2][BR_USER][ART_LIST]))
                return "Es gibt keine Artikel in dieser Gruppe.\n";
            else
                return get_menue_more_info(m[0..2],(str[0]=='x')
                    ?m[2][BR_USER][ART_LIST][<1]:m[2][BR_USER][ART_LIST][0]);

        case 'y':  // Alle Artikel zeigen
            flags["SeeArticle"] = (flags["SeeArticle"]<=0)?1:0;
            return RENEW(flags["SeeArticle"]
                   ?"Du siehst nun alle Artikel.\n"
                   :"Du siehst jetzt nur noch ungelesene Artikel.\n",m);

        case 'Y':  // Alle Bretter/Gruppen zeigen
            flags["SeeGroup"] = (flags["SeeGroup"]<=0)?1:0;
            return RENEW(flags["SeeGroup"]
                   ?"Du siehst nun alle Bretter und Gruppen.\n"
                   :"Du siehst jetzt nur angemeldete Bretter und Gruppen.\n",m);

        case 'z': //Sonderbehandlung im Artikel
            if(sizeof(m)==4 && (flags["Flags"]&4))
                return get_back_more_info(m[0..2]);
            else
                break;

        case 'Z':  // Artikel zusammenfassen
            if(sizeof(m)<3)
                break;

            wo = GET_PATH(m);

            if(sizeof(m)==3 || strlen(trim(str[2..<1])))
            {
                if(pointerp(tmp=parse_articles(str[2..<1],m,wo)))
                    wo = wo[0..1] + tmp;
                else if(sizeof(m)==3)
                {
                    input_to("ask_for_menu", INPUT_PROMPT,
                	"Welche Artikel willst Du zusammenfasen (Nummern angeben)? ",
			m, wo,#'parse_articles,#'summarize_articles, 0);
                    return END_BROWSE;
                }
		else
		    return "'Z [<nr>[-<nr>][, ...]]' um Artikel zusammenzufasssen.\n";
            }
            else
                wo[2]=({wo[2]});
            return summarize_articles(m,wo);

	case 'C':
	    if(sizeof(m)==3)
	    {
		wo = GET_PATH(m);
		if(wo[0] == "InterMUD" && file_size("/static/adm/CHARTA."+wo[1])>0)
        	    return ({ ({"/static/adm/CHARTA."+wo[1],
			"Charta von " + wo[1] + " %d von %d [z,u,b,/<such>] ",
            		0, M_FRAME|M_AUTO_END|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
			DISP_HELP, ([DISP_MENUE: m]) }) });
	    }
	    break;
	    
        case '?':
            tmp = 0;
	    if(trim(str[2..<1])=="f")
	    {
		tmp = ({
    "Mit 'f <Suchtext>' kann eine Suche über einzelne Bretter/Gruppen oder allen",
    "Artikeln erfolgen. Die Suche findet ohne Beachtung der Groß-/Kleinschreibung",
    "statt. Folgendes ist als Suchtext zulässig:",
    "",
    "   wort                    Es wird nach genau diesem Wort gesucht. Dieses Wort",
    "                           darf nur aus Buchstaben, Zahlen, Punkten und Binde-",
    "                           strichen bestehen.",
    "   wort*                   Es wird nach Wörtern, die mit diesen Buchstaben",
    "                           beginnen, gesucht.",
    "   \"<wort1> <wort2> ...\"   Sucht nach aufeinanderfolgenden Wörtern.",
    "   titel=<suchtext>        Sucht nur im Titel nach dem Suchtext.",
    "   autor=<suchtext>        Sucht nach einem bestimmten Verfasser.",
    "   brett=<suchtext>        Schränkt die Suche auf ein Brett ein.",
    "   gruppe=<suchtext>       Schränkt die Suche auf eine Gruppe ein.",
    "   nr=<suchtext>           Sucht nach einer bestimmten Artikelnummer.",
    "   bezug=<suchtext>        Sucht nach Artikeln, welche sich auf Artikel mit",
    "                           einer bestimmten Nummer beziehen.",
    "   <suche1> OR <suche2>    Artikel, welche auf <suche1> oder <suche2> passen,",
    "                           werden angezeigt.",
    "   <suche1> AND <suche2>   Liefert Artikel, welche beide Suchkritierien",
    "                           erfüllen.",
    "   NOT <suche>             Sucht nach Artikeln, welche nicht auf <suche>",
    "                           passen.",
    "   (<suche>)               Eine Klammerung von Suchausdrücken ist möglich.",
		});
	    }
            else switch(sizeof(m))
            {
                case 1:
                    tmp = ({
    "Hier siehst Du alle (angemeldeten) Bretter.",
    "Mögliche Befehle:",
    "",
    "  q / Q / e   Lesen beenden",
    "  o           Menü mit den Einstellungen aufrufen",
    "  E           ED als Editor verwenden ("+(flags["Editor"]?"ein":"aus")+")",
    "  r           Beschreibung neu ausgeben",
    "  S           Seitenweise anzeigen ("+(flags["Scroll"]&1?"aus":"ein")+")",
    "  ?           Diese Hilfe",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  <nr>        Brett mit dieser Nummer lesen",
    "  n           Das erste Brett lesen, an dem noch ungelesene Artikel sind",
    "  N           Das erste Brett lesen, an dem noch neue Artikel sind",
    "  A           Das erste Brett lesen, an dem noch alte, ungelesene Artikel sind",
    "",
    "  M           Alle Bretter als gelesen markieren",
    "  M <nr>      Das Brett mit dieser Nummer als gelesen markieren",
    "  m           Alle Artikel an allen Brettern als ungelesen markieren",
    "  m <nr>      Alle Artikel des Brettes <nr> als ungelesen markieren",
    "",
    "  an <nr>     Brett mit dieser Nummer anmelden",
    "  ab <nr>     Brett mit dieser Nummer abmelden",
    "  i <name>    Artikel dieses Autors nicht anzeigen bzw. wieder anzeigen",
    "  I           Alle Autoren anzeigen, deren Artikel nicht mehr angezeigt werden",
    "  Y           Abgemeldete Bretter anzeigen ("+(flags["SeeGroup"]>0?"ein":"aus")+")",
    "  R           Beschreibung mit umgekehrtem Y-Flag ausgeben",
    "",
    wizp(this_player())?
    "  f <suche>   Sucht nach bestimmten Artikeln (Hilfe dazu mit '? f')":0,
    wizp(this_player())?"":0,
    "  j <Mod>     Übersicht über die Bretter des Moderators <Mod>",
    "  J <nr>      Übersicht über dieses Brett",
    "  O           Moderatoren aller Bretter anzeigen",
    "  O <nr>      Moderatoren dieses Brettes anzeigen",
    "",
    "Bei 'lies neues pur' werden Dir die Neuigkeiten übrigens nur dann gezeigt,",
    "wenn es wirklich neue bzw. ungelesene Artikel gibt."
                    })-({0});
                    break;

                case 2:
                    wo = GET_PATH(m);
                    tmp = ({
    "Hier siehst Du alle (angemeldeten) Gruppen.",
    "Mögliche Befehle:",
    "",
    "  q / Q / e   Lesen beenden",
    "  z           Zurück zur Brettuebersicht",
    "  o           Menü mit den Einstellungen aufrufen",
    "  E           ED als Editor verwenden ("+(flags["Editor"]?"ein":"aus")+")",
    "  r           Beschreibung neu ausgeben",
    "  S           Seitenweise anzeigen ("+(flags["Scroll"]&2?"aus":"ein")+")",
    "  ?           Diese Hilfe",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  <nr>        Gruppe mit dieser Nummer lesen",
    "  n           Die erste Gruppe lesen, in der noch ungelesene Artikel sind",
    "  N           Die erste Gruppe lesen, in der noch neue Artikel sind",
    "  A           Die erste Gruppe lesen, in der noch alte, ungelesene Artikel sind",
    "",
    "  M           Alle Gruppen als gelesen markieren",
    "  M <nr>      Die Gruppe mit dieser Nummer als gelesen markieren",
    "  m           Alle Artikel an allen Gruppen als ungelesen markieren",
    "  m <nr>      Alle Artikel der Gruppe <nr> als ungelesen markieren",
    "",
    "  an <nr>     Gruppe mit dieser Nummer anmelden",
    "  ab <nr>     Gruppe mit dieser Nummer abmelden",
    "  i <name>    Artikel dieses Autors nicht anzeigen bzw. wieder anzeigen",
    "  I           Alle Autoren anzeigen, deren Artikel nicht mehr angezeigt werden",
    "  Y           Abgemeldete Gruppen anzeigen ("+(flags["SeeGroup"]>0?"ein":"aus")+")",
    "  R           Beschreibung mit umgekehrtem Y-Flag ausgeben",
    ({int})NEWSD->query_brett_owner(wo[0],0,owner)?
    "  g           Neue Gruppe eröffnen":0,
    "",
    wizp(this_player())?
    "  f <suche>   Sucht nach bestimmten Artikeln (Hilfe dazu mit '? f')":0,
    wizp(this_player())?"":0,
    "  j <Mod>     Übersicht über die Gruppen des Moderators <Mod>",
    "  J <nr>      Übersicht über diese Gruppe",
    "  O           Moderatoren aller Gruppen anzeigen",
    "  O <nr>      Moderatoren dieser Gruppe anzeigen",
    "",
    "  +           Nächstes Brett anzeigen",
    "  -           Vorheriges Brett anzeigen",
                    })-({0});
                    break;

                case 3:
                    wo = GET_PATH(m);
                    tmp = ({
    "Hier siehst Du alle Artikel dieser Gruppe.",
    "Mögliche Befehle:",
    "",
    "  q / Q / e   Lesen beenden",
    "  z           Zurück zur Gruppenübersicht",
    "  o           Menü mit den Einstellungen aufrufen",
    "  E           ED als Editor verwenden ("+(flags["Editor"]?"ein":"aus")+")",
    "  r           Beschreibung neu ausgeben",
    "  S           Seitenweise anzeigen ("+(flags["Scroll"]&4?"aus":"ein")+")",
    (wo[0] == "InterMUD" && file_size("/static/adm/CHARTA."+wo[1])>0)?
    "  C           Charta der Grupe anzeigen.":0,
    "  ?           Diese Hilfe",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  <nr>        Artikel mit dieser Nummer lesen",
    "  n           Den ersten ungelesenen Artikel lesen",
    "  N           Den ersten neuen Artikel lesen",
    "  A           Den ersten alten, aber ungelesenen Artikel lesen",
    "  x           Zum letzten Artikel springen",
    "  X           Zum ersten Artikel springen",
    "",
    "  M           Alle Artikel als gelesen markieren",
    "  M <nummern> Diese Artikel als gelesen markieren",
    "  m           Alle Artikel als ungelesen markieren",
    "  m <nummern> Diesen Artikel als ungelesen markieren",
    "",
    "  s           Einen neuen Artikel schreiben",
    "  b <nr>      Diesen Artikel beantworten",
    "  P <nr>      Diesen Artikel als Brief schicken lassen",
    "  l <nummern> Diesen Artikel löschen",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  L <nr>      Diesen Thread löschen":0,
    "  Z <nummern> Eine Zusammenfassung dieser Artikel anzeigen",
    "",
    "  k <nr>      Thread zusammen- bzw. auseinanderklappen",
    "  K           Alle Threads zusammen- bzw. auseinanderklappen",
    "  t <nr>      Den Baum dieses Threads anzeigen",
    "  T <nr>      Eine Zusammenfassung dieses Threads anzeigen",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  V <nr> [Brett:]Gruppe":0,
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "              Diesen Thread in eine andere Gruppe verschieben":0,
    "",
    "  i <name>    Artikel dieses Autors nicht mehr anzeigen bzw. wieder anzeigen",
    "  i <nr>      Artikel des Autors dieses Artikels nicht bzw. wieder anzeigen",
    "  I           Alle Autoren anzeigen, deren Artikel nicht mehr angezeigt werden",
    "  y           Alte Threads anzeigen ("+(flags["SeeArticle"]>0?"ein":"aus")+")",
    "  R           Beschreibung mit umgekehrtem y-Flag ausgeben",
    "",
    wizp(this_player())?
    "  f <suche>   Sucht nach bestimmten Artikeln (Hilfe dazu mit '? f')":0,
    wizp(this_player())?"":0,
    "  O           Moderatoren dieser Gruppe anzeigen",
                   })-({0})+
                   ((flags["Scroll"]&4)?({}):({
    "",
    "  U           Eine halbe Seite nach oben blättern",
    "  D           Eine halbe Seite nach unten blättern",
    "  u           Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang gehen",
    "  >           Ans Ende gehen",
    "  c           Gesamten Rest auf einmal anzeigen",
                    }))+({
    "",
    "  ACHTUNG:    Bei <nummern> kann man eine einzelne Nummer, mehrere durch",
    "              Kommas getrennte Nummern oder einen Bereich mit '<nr>-<nr>'",
    "              angeben. Bei einem Bereich werden alle Artikel angesprochen,",
    "              welche in der Artikelübersicht (NICHT von ihrer Nummer her)",
    "              zwischen den beiden angegebenen Artikeln liegen."
		    });
                    break;

                case 4:
                    wo = GET_PATH(m);
                    tmp = ({
    "Mögliche Befehle:",
    "",
    "  q / Q / e   Lesen beenden",
    "  z           Zurück zur "+(flags["Flags"]&4?"Gruppenübersicht":"Artikelübersicht"),
    "  o           Menü mit den Einstellungen aufrufen",
    "  E           ED als Editor verwenden ("+(flags["Editor"]?"ein":"aus")+")",
    "  r           Text neu ausgeben",
    "  S           Seitenweise anzeigen ("+(flags["Scroll"]&4?"aus":"ein")+")",
    "  ?           Diese Hilfe",
    "  !<befehl>   Befehl als Spieler ausführen",
    "",
    "  n           Den nächsten ungelesenen Artikel lesen",
    "  N           Den nächsten neuen Artikel lesen",
    "  A           Den nächsten alten, aber ungelesenen Artikel lesen",
    "  x           Zum letzten Artikel springen",
    "  X           Zum ersten Artikel springen",
    "  +           Zum nächsten Artikel gehen",
    "  -           Zum vorherigen Artikel gehen",
    "  v           Zum Bezugsartikel gehen",
    "",
    "  m           Diesen Artikel als ungelesen markieren",
    "",
    "  s           Einen neuen Artikel schreiben",
    "  b           Diesen Artikel beantworten",
    "  P           Diesen Artikel als Brief schicken lassen",
    wizp(this_player())?
    "  w <datei>   Als Datei abspeichern":0,
    "  l           Diesen Artikel löschen",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  L           Diesen Thread löschen":0,
    "  t           Den Baum dieses Threads anzeigen",
    "  T           Eine Zusammenfassung dieses Threads anzeigen",
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "  V [Brett:]Gruppe":0,
    ({int})NEWSD->query_brett_owner(wo[0],wo[1],owner)?
    "              Diesen Thread in eine andere Gruppe verschieben":0,
    "",
    "  i           Artikel des Autors dieses Artikels nicht bzw.wieder anzeigen",
    "  i <name>    Artikel dieses Autors nicht mehr anzeigen bzw. wieder anzeigen",
    "  I           Alle Autoren anzeigen, deren Artikel nicht mehr angezeigt werden",
    "",
    wizp(this_player())?
    "  f <suche>   Sucht nach bestimmten Artikeln (Hilfe dazu mit '? f')":0,
    wizp(this_player())?"":0,
    "  O           Moderatoren dieser Gruppe anzeigen",
                   })-({0})+
                   ((flags["Scroll"]&8)?({}):({
    "",
    "  U           Eine halbe Seite nach oben blättern",
    "  D           Eine halbe Seite nach unten blättern",
    "  u           Eine ganze Seite nach oben blättern",
    "  <Enter>     Eine ganze Seite nach unten blättern",
    "  <           An den Anfang des Textes gehen",
    "  >           Ans Ende des Textes gehen",
    "  c           Gesamten restlichen Text auf einmal anzeigen",
    "  <nr>        Zu dieser Zeile springen",
    "  /<text>     Nach diesem Text suchen",
    "  /           Suche wiederholen",
                    }));
                    break;
            }
            if(!tmp)
                return "Es gibt noch keine Hilfe.\n";

            return ({ ({tmp, "Hilfe - Zeile %d von %d [z,u,b,/<such>] ",
                0, M_FRAME|M_AUTO_END|BF_NO_MENUE|(flags["Scroll"]&16?M_SCROLL:0),
                DISP_HELP, ([DISP_MENUE: m]) }) });
	
    }

    return browser::browse_action(str, m);
/*
------------------------------------------------------------------------------
---   Zu definierende Befehle.                                            ----
---   (Bereits implementierte sind mit * gekennzeichnet)                  ----
------------------------------------------------------------------------------
 *  a[n|b] Ab-/Anmelden eines Brettes / einer Gruppe
 *  A	Naechsten alten Artikel
 *  AA  ... ueber Gruppen/Brettgrenzen hinweg
 *  b	Artikel beantworten (in Uebersicht mit Nummer)
    B
 *  c   Rest durchscrollen (more)
    C
 *  B	Eine halbe Seite hoch (auch U, more)
 *  d	Eine halbe Seite runter (more)
 *  D	Eine halbe Seite runter (more)
 *  e	Beenden (more)
 *  E   Editor waehlen (Simple, ed, xed)
 *  f   Suche
    F
 *  g	Neue Gruppe eroeffnen
    G
    h
    H
 *  i   Schreiber (im Artikel oder Name/Artikelnummer angegeben) ignorieren *plonk*
 *  I   Killlist ausgeben
 *  j   Uebersicht ueber Newsgruppen nach Moderator
 *  J   Uebersicht ueber alle Newsgruppen
 *  k   Thread zusammenklappen/auseinanderklappen
 *  K   Alle Threads zusammenklappen (nur in Artikeluebersicht)
 *  l	Artikel loeschen (In Uebersicht mit Nummer)
 *  L   Einen Thread loeschen
 *  m	Als ungelesen markieren (auch fuer ganze Bretter, Gruppen. Dabei Aufklapp-Einstellungen beibehalten)
 *  M	alle Artikel als gelesen markieren (M <nr> auch moeglich)
 *  n	naechster ungelesener Artikel
 *  nn	naechster ungelesener Artikel ueber Gruppen/Brettgrenzen hinweg
 *  N	naechster neuer Artikel
 *  NN  ... ueber Gruppen/Brettgrenzen hinweg
 *  o	Optionsmenue (Editor, Anzeige ungelesener Artikel/Gruppen, Scroll)
 *  O   NewsmOderator anzeigen (auch fuer Spieler)
    p
 *  P	als Brief zuschicken lassen
 *  q	Beenden (more)
 *  Q	Beenden
 *  r	Nochmal ausgeben (more, newsreader)
 *  R   Ausgabe mit umgekehrten Y-Flag
 *  s	Artikel schreiben
 *  S	Seitenweise anzeigen aus/einschalten (Brett/Gruppe/Artikel/Text)
 *  t   Zeigt den vollstaendigen Baum eines Threads an. (Ab diesem Artikel)
 *  T   Zusammenfassung dieses Threads zeigen
 *  u	Eine Seite hoch (more)
 *  U	Eine halbe Seite hoch (more)
 *  v	Zum Bezugsartikel (Vorgaenger)
    V   Thread an anderes Brett, andere Gruppe verschieben.
 *  w   Als Datei speichern (more)
    W
 *  x   Zum letzten Artikel springen
 *  X   Zum ersten Artikel springen
 *  y   Alle Artikel anzeigen
 *  Y   Abgemeldete Bretter/Gruppen anzeigen
 *  z	zurueck
 *  Z	Liste von Artikeln zusammenfassen (Ganzer Thread: T)
 *  <nr>  Menuepunkt auswaehlen (browser), Zur Zeilennummer im Artikel gehen (more)
 *  +	Naechstes Menue / Naechster Artikel (browser)
 *  -   Vorhergehendes Menue / Artikel (browser)
 *  <	An den Anfang des Textes (more)
 *  >	Ans Ende des Textes (more)
 *  /	Suche im aktuellen Menue/Text (more)
 *  ?	Hilfe
 *  ' ' Eine Seite runter (more)

--------------------------------------------------------
Befehle fuer die Zusammenfassung:
 *  L  zugehoerige Artikel loeschen
 *  P  Per Mail schicken
 *  s  Als Artikel ans Brett schreiben
 *  w  Als Datei abspeichern (more)

Befehle fuer die Baumanzeige: (evntl. alle Befehle der Ebene 3?)
 *  A,b,n,N,i,I,O,l,L,m,M,s,t,x,X,Z wie bei Ebene 2
 *  v    Zum Vorgaengerbaum (Baum anzeigen)
 *  T    Zusammenfassung anzeigen
 *  <nr> Artikel anwaehlen

 */
}

/************************************************************************
 *   Zusaetzliche add_actions                                           *
 ************************************************************************/

/*
 * FUNKTION: listnews
 * DEKLARATION: int listnews(string str)
 * BESCHREIBUNG:
 * Die Action "listnews" zeigt zu einem Moderator alle seine Gruppen samt
 * Artikelzahl an. Wurde kein Moderator angenommen, wird this_player()
 * genommen. Bei "alle" werden alle Gruppen angezeigt.
 * VERWEISE:
 * GRUPPEN: news
 */
int listnews(string str)
{
    mixed txt;
    if(!str) str="";
    str=lower_case(strip(str));
    if(str=="alle") str=0;
    txt=get_mod_news_list(str,({}));
    if(stringp(txt))
        write(txt);
    else
        this_player()->more(txt, "Newsliste" +
        (str?" für " +(strlen(str)?capitalize(str):"dich"):"") +
        " [z,u,b,/<such>]", 0, M_AUTO_END);
    return 1;
}

/************************************************************************
 *   Applied Lfuns                                                      *
 ************************************************************************
 *   Die Doku zu diesen Funktionen gibt es in der Enzy.                 *
 ************************************************************************/

private varargs void print_newsreader_options(int quiet)
{
    mixed menue = get_option_menu(0)[<1];
    menue[BR_STATUS_LINE][0] = "Newseinstellungen [q,z,?] ";
    owner->more(menue[BR_MENUE], menue[BR_STATUS_LINE], menue[BR_BEGIN_LINE],
	(menue[BR_STATUS_BYTE] & MORE_MASK) | (quiet?M_NO_FIRST_SCREEN:0),
	"Newsreader-Optionen");
}

nomask int more_action(string str, int line, int max_line, mixed more_id)
{
    if(this_player()!=owner) return NOTHING;
    if(more_id=="Newsreader-Optionen")
    {
	mixed menue;
	if(!str || member(({"","\n","\r"}),trim(str))>=0)
	    return CONTINUE;
	if(strlen(str) && (strlen(str)==1 || str[1]==' '))
	    switch(lower_case(str)[0])
	    {
        	case 'q':
            	    return END_MORE;
        	case 'z':
		    this_player()->start_options_menue();
		    return END_MORE;
		case '?':
		    this_player()->print_options_help(0, "News",
			trim(str[1..<1]),
			(: print_newsreader_options($1) :), 0);
		    return END_MORE;
	    }
	menue = option_action(str, 0);
	if(intp(menue))
	    return menue;
	menue = menue[<1];
	menue[BR_STATUS_LINE][0] = "Newseinstellungen [q,z,?] ";
	owner->more(menue[BR_MENUE], menue[BR_STATUS_LINE], menue[BR_BEGIN_LINE],
	    menue[BR_STATUS_BYTE] & MORE_MASK, "Newsreader-Optionen");
	return END_MORE;
    }
    return ::more_action(str, line, max_line, more_id);
}

/*
 * FUNKTION: query_read_only_news
 * DEKLARATION: varargs string query_read_only_news(string rest, string str)
 * BESCHREIBUNG:
 * Wird von query_read aufgerufen, falls nur dann News angezeigt werden sollen,
 * wenn es wirklich was Neues gibt. Dabei wird dann schon das richtige Brett
 * und die richtige Gruppe mit den Neuigkeiten herausgesucht.
 * VERWEISE:
 * GRUPPEN: news
 */
varargs string query_read_only_news(string rest, string str)
{
    mixed tmp;
    if (!owner)
        return wrap("Nimm Dir doch erstmal "+den()+".");
    if (this_player() != owner)
        return "Das gehört Dir nicht.\n";

    BEGIN_CACHE;
    if(NO_BACK2)
    {
        mixed *gruppen_daten = ({mixed*}) NEWSD->query_dates(flags["Last"][0]);
        int i = member(gruppen_daten, flags["Last"][1]);
        if(i<0)
	{
	    END_CACHE;
            return "Die Gruppe "+flags["Last"][1]+" existiert nicht mehr.\n";
	}
	check_existance(flags["Last"][0..1]);
        if(!check_group_for_new_articles(
            ({flags["Last"][1], gruppen_daten[i+1], (i/2)+1}),
            flags["Last"][0],flags["SeeArticle"]<0?2:3,0))
	    {
		END_CACHE;
                return "Es gibt nichts Neues am Brett "+flags["Last"][1]+".";
	    }
        tmp = flags["Last"];
    }
    else
    {
        mixed *bretter_daten = ({mixed*}) NEWSD->query_dates(0);

        if(!sizeof(bretter_daten))
	{
	    END_CACHE;
            return "Es gibt noch keine Bretter!\n";
	}

        if(NO_BACK1 && sizeof(flags["Last"]))
        {
            int i = member(bretter_daten, flags["Last"][0]);
            if(i<0)
	    {
		END_CACHE;
                return "Das Brett "+flags["Last"][0]+" existiert nicht mehr.\n";
	    }
        }
        
        // Wir haben nun ein Format: ({brettname, datum, brettname, datum,...})
        for(int i=0,int n=1;i<sizeof(bretter_daten);i+=2,n++)
        {
            if(IS_AL || !sizeof(bretter) || member(bretter,bretter_daten[i]))
	    {
                if(!member(plinfo,bretter_daten[i])) // Neues Brett
                {
                    tmp = 1;
                    continue;
                }
                if (check_board_for_new_articles( bretter_daten[i..i+1]+({n}),
                    flags["SeeArticle"]<0?2:3))
                {
                    tmp |= 2;
                    break;
                }
	    }
	}
        if((tmp&2) && !sizeof(flags["Last"]))
            tmp = ({});
        else if(tmp&2) // Schauen, ob das aktuelle Brett auch neu ist.
        {
            int i = member(bretter_daten, flags["Last"][0]);
            if(i<0)
                tmp = ({});
            else
            {
                if(!check_board_for_new_articles(
                    ({flags["Last"][0], bretter_daten[i+1], (i/2)+1}),
                    flags["SeeArticle"]<0?2:3))
                {
                    if(NO_BACK1)
		    {
			END_CACHE;
                        return flags["SeeArticle"]<0
                            ? "Es gibt nichts Neues.\n"
                            : "Es gibt keine ungelesenen Artikel.\n";
		    }
                    tmp=({});
                }
                else if(sizeof(flags["Last"])==1)
                    tmp = flags["Last"];
                else
                {
                    mixed *gruppen_daten = ({mixed*}) NEWSD->query_dates(flags["Last"][0]);
                    i = member(gruppen_daten, flags["Last"][1]);
                    if(i<0)
                        tmp = flags["Last"][0..0];
                    else
                    {
			check_existance(flags["Last"][0..1]);
                        if(!check_group_for_new_articles(
                            ({flags["Last"][1], gruppen_daten[i+1], (i/2)+1}),
                            flags["Last"][0],flags["SeeArticle"]<0?2:3,0))
                                tmp = flags["Last"][0..0];
                        else
                            tmp = flags["Last"];
                    }
                }
            }
        }
        else if(tmp&1)
            tmp = ({});
        else
	{
	    END_CACHE;
            return flags["SeeArticle"]<0
                ? "Es gibt nichts Neues.\n"
                : "Es gibt keine ungelesenen Artikel.\n";
	}
    }
    tmp = tmp[0..0];
    tmp = get_the_menue(tmp);
    if(!intp(tmp) && !browse( tmp ))
    {
	END_CACHE;
	return query_notify_fail();
    }
    return "";
}

varargs string query_read(string rest, string str)
{
    mixed tmp;
    if(lower_case(strip(rest||""))=="pur")
        return query_read_only_news(0,str);
    if (!owner)
        return wrap("Nimm Dir doch erstmal "+den()+".");
    if (this_player() != owner)
        return "Das gehört Dir nicht.\n";

    BEGIN_CACHE;
    tmp = get_the_menue(flags["Last"]);
    if(!intp(tmp) && !browse( tmp ))
    {
	END_CACHE;
	return query_notify_fail();
    }
    return "";
}


string query_read_msg()
{
   if((flags["Flags"]&8) && wizp(this_player()))
       return "";
   return Der(this_player())+" liest die Neuigkeiten.";
}

void init_arg(mixed i)
{
    if(mappingp(i)) flags=i;
    else if(intp(i) && i != 12345 && i != 1)
    {
	if(!intp(i)) i=0;
        flags["Scroll"]= 7;
        flags["Flags"]= 4;
	flags["Editor"]= (i & 2)?1:0;
	flags["SeeArticle"]= (i & 4)?1:0;
	flags["SeeGroup"]= (1 & 8)?1:0;
    }
    set_invis(V_NOLIST);
}

nomask mixed query_auto_load()
{
    if(IS_AL)
        return 12345;
}

void init()
{
    if (owner)
        return;

    if (this_player()==environment())
    {
        owner=this_player();
        set_short(get_genitiv(owner->query_real_cap_name())+" "+query_cap_name());
        if(IS_AL)
	{
            no_back=0;
	    set_invis(V_NOLIST);
	}
        else
            set_invis(V_INVIS);

        plinfo=query_read_artikel();

        if (!plinfo)
	{
            plinfo=m_allocate(1,2);
	    mark_all_as_read(({"Spieler", "+Neues+"}));
            set_read_artikel(plinfo);
        }

        flags = ({mapping})this_player()->query_newsreader_settings() || flags;

        if(wizp(this_player()))
            add_action("listnews","listnews");
	
	owner->add_options_menue("News", #'print_newsreader_options);
    }
}

void create()
{
    set_id(({"newsreader","news","neues","neuigkeiten"}));
    set_long("Die Neuigkeiten aus Magyra. Lies sie doch einfach.");
    set_name("newsreader");
    set_gender("maennlich");
    set_invis(V_NOLIST);
    seteuid(getuid());
    set_weight(0);
    set_no_move_reason ("Dann könntest Du Dich nicht mehr über "
        "die vielen lustigen Artikel amüsieren, das wär doch "
        "schon sehr schade.");
    flags["Last"]=({"Spieler"});
}
